@@ -31,6 +31,7 @@
 #include "bus.h"
 #include "signode.h"
 #include "aitc.h"
+#include "sgstring.h"
 
 #define INTCNTL(base)		((base)+0)
 #define		INTCNTL_ABFLAG	(1<<25)
@@ -157,7 +158,7 @@ update_interrupts(Aitc *ai)
  *      irq line trace, called whenever a change occurs
  * -----------------------------------------------------------
  */
-static int
+static void 
 int_source_change(SigNode *node,int value,void *clientData)
 {
         IrqTraceInfo *ti = (IrqTraceInfo *) clientData;
@@ -169,7 +170,6 @@ int_source_change(SigNode *node,int value,void *clientData)
 		ai->intsrc &= ~((uint64_t)1<<irq);
         }
         update_interrupts(ai);
-        return 0;
 }
 
 /*
@@ -578,12 +578,7 @@ Aitc_New(const char *name)
 {
 	Aitc *ai;
 	int i;
-        ai = (Aitc*)malloc(sizeof(Aitc));
-        if(!ai) {
-                fprintf(stderr,"Out of memory allocating AITC\n");
-                exit(429);
-        }
-        memset(ai,0,sizeof(Aitc));
+        ai = sg_new(Aitc);
 	ai->irqNode = SigNode_New("%s.irq",name);
 	ai->fiqNode = SigNode_New("%s.fiq",name);
 	if(!ai->irqNode || !ai->fiqNode) {
@@ -592,11 +587,7 @@ Aitc_New(const char *name)
 	}
 	ai->nimask = 0x1f;
 	for(i=0;i<64;i++) {
-		IrqTraceInfo *ti = malloc(sizeof(IrqTraceInfo));
-		if(!ti) {
-			fprintf(stderr,"Out of memory\n");
-			exit(1);
-		}
+		IrqTraceInfo *ti = sg_new(IrqTraceInfo);
 		ai->nIntinNode[i] = SigNode_New("%s.nIntSrc%d",name,i);
 		if(!ai->nIntinNode[i]) {
 			fprintf(stderr,"Can not create interrupt node for irq %d\n",i);
