@@ -37,20 +37,21 @@
 #include <fcntl.h>
 #include <sys/file.h>
 
-#include <amdflash.h>
-#include <configfile.h>
-#include <cycletimer.h>
-#include <bus.h>
-#include <signode.h>
-#include <diskimage.h>
+#include "amdflash.h"
+#include "configfile.h"
+#include "cycletimer.h"
+#include "bus.h"
+#include "signode.h"
+#include "diskimage.h"
+#include "sgstring.h"
 
 
 int verbosity = 1;
 
 #if 0
-#define dprintf(verb,x...) { if((verb)>=verbosity) fprintf(stderr,x); }
+#define dbgprintf(verb,x...) { if((verb)>=verbosity) fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 
 /*
@@ -76,13 +77,19 @@ typedef struct FlashType {
 	char *name;
 	int size;
 	int min_sectorsize;
-	uint8_t querystr[0x4c];	
+	uint16_t querystr[0x4c];	
 	AutoSel auto_sel[MAX_AUTO_CODES];
-	uint32_t flags;
+	uint32_t capabilities;
 } FlashType;
 
-#define FLASH_IS_CFI 	 (1)
-#define FLASH_IS_TOPBOOT (2)
+#define CAP_CFI 	(1)
+#define CAP_TOPBOOT 	(2)
+/* CMDSETS */
+#define CAP_LOCK_REG	(4)
+#define CAP_PASSWD	(8)
+#define CAP_PPB		(0x10)
+#define CAP_PPB_LOCK	(0x20)
+#define CAP_DYB		(0x40)
 
 typedef struct AMDFlashBank AMDFlashBank;
 
@@ -102,10 +109,10 @@ typedef struct AMD_Flash {
 	EraseRegion **rmap;
 
 	/* State Variables */
-	int mode;    // current memory mapping mode (Read Array or read registers)
-	int cycle;  // Number of current cycle in multiword cmd sequences
-	int cmd;    // recognized command 
-	int bypass; // Need unlock sequence or not ? 
+	int mode;      /* current memory mapping mode (Read Array or read registers) */
+	int cycle;     /* Number of current cycle in multiword cmd sequences */
+	int cmd;       /* recognized command */
+	int en_cmdset; /* Enabled cmdset (bypass Programm, Lock Register, PPB ... */
 	
 	/*
  	 * Write buffer
@@ -141,9 +148,6 @@ struct AMDFlashBank {
 	AMD_Flash *flash[2];
 };
 
-#define BYPASS_UNLOCKED (1)
-#define BYPASS_LOCKED (0)
-
 /*
  * -----------------------------------------------------------------
  * Calculate Byteaddress from Memory Bus address (16 Bit mode)
@@ -153,7 +157,8 @@ struct AMDFlashBank {
 #define MEMBUS_TO_FLASH_ADDR(bank,addr) ((addr)>>(bank)->addr_shift)
 #define FLASH_ADDR_TO_MEMBUS(bank,addr) ((addr)<<(bank)->addr_shift)
 #define BADDR(x) ((x)<<1)
-#define REG(x)	((x)>>1)
+#define FLASH_ADDR(x)	((x) >> 1)
+#define FLASH_REG(x)	(((x) >> 1) & 0xfff)
 
 
 #define MAP_READ_ARRAY	(0)
@@ -188,15 +193,46 @@ struct AMDFlashBank {
 #define CMD_AUTOSELECT		(1)
 #define CMD_ENTER_SECSI		(5)
 #define CMD_EXIT_SECSI		(6)
-#define CMD_PROGRAMM		(7)
+#define CMD_PROGRAM		(7)
 #define CMD_WRITE_TO_BUF	(8)
 #define CMD_PGM_BUFFER		(9)
 #define CMD_WBUF_ABORT_RESET	(10)
 #define CMD_UNLOCK_BYPASS	(11)
 #define CMD_UNLOCK_BYPASS_PGM   (12)
-#define CMD_UNLOCK_BYPASS_RESET	(13)
+#define CMD_EXIT_CMDSET		(13)
 #define CMD_ERASE		(14) 
 
+/* Sector Protection commands from S29GL-P manual */
+#define	CMD_ENTER_LOCK_REG	(15)
+#define CMD_LOCKBITS_PGM	(16)
+#define CMD_EXIT_LOCK_REG	(18)
+
+#define CMD_ENTER_PW_PROT	(19)
+#define CMD_PW_PGM	(20)
+#define CMD_EXIT_PW_PROT	(23)
+
+#define CMD_ENTER_PPB		(24)
+#define	CMD_PPB_PGM		(25)
+#define CMD_EXIT_PPB		(28)
+
+#define CMD_ENTER_PPB_LOCK	(29)
+#define CMD_PPB_LOCKBIT_SET	(30)
+#define CMD_EXIT_PPB_LOCK	(32)
+
+#define CMD_ENTER_DYB		(33)
+#define CMD_DYB_SET		(34)
+#define CMD_EXIT_DYB		(37)
+
+/* The currently enabled cmdset */
+#define ENCMDSET_NONE		(0)
+#define ENCMDSET_BYPASS 	(1)
+#define ENCMDSET_LOCK_REG	(2)
+#define ENCMDSET_PW_PROT	(3)
+#define ENCMDSET_PPB		(4)
+#define ENCMDSET_PPB_LOCK	(5)
+#define ENCMDSET_DYB		(6)
+
+
 /*
  * ----------------------------------------------------------
  * Flash Types
@@ -223,7 +259,53 @@ flash_types[] = {
 			{0x01,0x4f,0x22da},	/* Device ID */
 			{0x02,0x4f,0x0000},	/* SecSi lowest not locked */
 		},
-		flags: 0 
+		capabilities: 0 
+	},	
+	{
+		
+		name:	"AM29LV160BB",
+		size:	    2*1024*1024,
+		min_sectorsize: 8*1024,	
+		querystr: { 'Q','R','Y', 0x02,0,0x40,0,0,0,0,0,0x27,0x36,0,0, 4, 
+			0,0xa,0,5,0,4,0,0x15, 2,0,0,0,
+			4, 
+			0x00,0x00,0x40,0x00, 
+			0x01,0x00,0x20,0x00, 
+			0x00,0x00,0x80,0x00, 
+			0x1e,0x00,0x00,0x01, 
+			0,0,0,
+			'P','R','I', 0x31,0x30,0,2,1,1,4,0,0,0,0x00,0x00,0
+			,0
+		},
+		auto_sel: { 
+			{0x00,0x43,0x0001},	/* Manufacturer ID */
+			{0x01,0x43,0x2249},	/* Device ID */
+			{0x02,0x43,0x0000},	/* Sector Protect verify */
+		},
+		capabilities: CAP_CFI
+	},	
+	{
+		
+		name:	"AM29LV160BT",
+		size:	    2*1024*1024,
+		min_sectorsize: 8*1024,	
+		querystr: { 'Q','R','Y', 0x02,0,0x40,0,0,0,0,0,0x27,0x36,0,0, 4, 
+			0,0xa,0,5,0,4,0,0x15, 2,0,0,0,
+			4, 
+			0x00,0x00,0x40,0x00, 
+			0x01,0x00,0x20,0x00, 
+			0x00,0x00,0x80,0x00, 
+			0x1e,0x00,0x00,0x01, 
+			0,0,0,
+			'P','R','I', 0x31,0x30,0,2,1,1,4,0,0,0,0x00,0x00,0
+			,0
+		},
+		auto_sel: { 
+			{0x00,0x43,0x0001},	/* Manufacturer ID */
+			{0x01,0x43,0x22c4},	/* Device ID */
+			{0x02,0x43,0x0000},	/* Sector Protect verify */
+		},
+		capabilities: CAP_CFI  | CAP_TOPBOOT
 	},	
 	{
 		
@@ -248,7 +330,7 @@ flash_types[] = {
 			{0x01,0x4f,0x227e},	/* Device ID */
 			{0x03,0x4f,0x0008},	/* SecSi lowest not locked */
 		},
-		flags: FLASH_IS_CFI
+		capabilities: CAP_CFI
 	},	
 	{
 		/* ST Mircoelectronics document 7876 */	
@@ -270,7 +352,7 @@ flash_types[] = {
 			{0x00,0x03,0x0020},	/* Manufacturer ID */
 			{0x01,0x03,0x22cb},	/* Device ID */
 		},
-		flags: FLASH_IS_CFI
+		capabilities: CAP_CFI
 	},	
 	{
 		
@@ -293,7 +375,7 @@ flash_types[] = {
 			{0x01,0x43,0x225b},	/* Device ID */
 			{0x02,0x43,0x0000},	/* Sector unprotected */
 		},
-		flags: FLASH_IS_CFI
+		capabilities: CAP_CFI
 	},	
 	{
 		
@@ -316,9 +398,9 @@ flash_types[] = {
 			{0x01,0x43,0x22da},	/* Device ID */
 			{0x02,0x43,0x0000},	/* Sector unprotected */
 		},
-		flags: FLASH_IS_CFI  | FLASH_IS_TOPBOOT
+		capabilities: CAP_CFI  | CAP_TOPBOOT
 	},	
-	/* Same as SG29GL032M */
+	/* Same as S29GL032M */
 	{
 		name:	"AM29LV320ML",
 		size:	    4*1024*1024,
@@ -341,8 +423,9 @@ flash_types[] = {
 			{0x01,0x4f,0x227e},	/* Device ID */
 			{0x03,0x4f,0x0008},	/* SecSi lowest not locked */
 		},
-		flags: FLASH_IS_CFI
+		capabilities: CAP_CFI
 	},	
+	/* Same as S29GL064A */
 	{
 		name:	"AM29LV640ML",
 		size:	    8*1024*1024,
@@ -360,7 +443,7 @@ flash_types[] = {
 			{0x01,0x4f,0x227e},	/* Device ID */
 			{0x03,0x4f,0x0008},	/* SecSi lowest not locked */
 		},
-		flags: FLASH_IS_CFI
+		capabilities: CAP_CFI
 	},	
 	{
 		name:	"MBM29LV650UE",
@@ -378,7 +461,7 @@ flash_types[] = {
 			{0x02,0x4f,0x0000},	/* Sector Group Protection */
 			{0x03,0x4f,0x0010},	/* Extended ID */
 		},
-		flags: FLASH_IS_CFI
+		capabilities: CAP_CFI
 	},	
 	{
 		name:	"AM29LV128ML",
@@ -397,7 +480,7 @@ flash_types[] = {
 			{0x0e,0x4f,0x2212},	/* Ex Device ID */
 			{0x0f,0x4f,0x2200},	/* Ex2 Device ID */
 		},
-		flags: FLASH_IS_CFI
+		capabilities: CAP_CFI
 	},	
 	{
 		name:	"AM29LV256ML",
@@ -416,11 +499,11 @@ flash_types[] = {
 			{0x01,0x4f,0x227e},	/* Device ID */
 			{0x03,0x4f,0x0008},	/* SecSi lowest not locked */
 		},
-		flags: FLASH_IS_CFI
+		capabilities: CAP_CFI
 	},	
 	{
 		/* No differences found to AM29LV256ML */
-		name:	"SG29GL256MR2",
+		name:	"S29GL256MR2",
 		size:	    32*1024*1024,
 		min_sectorsize: 64*1024,	
 		querystr: { 'Q','R','Y', 0x02,0,0x40,0,0,0,0,0,0x27,0x36,0,0, 7, 
@@ -436,11 +519,11 @@ flash_types[] = {
 			{0x01,0x4f,0x227e},	/* Device ID */
 			{0x03,0x4f,0x0008},	/* SecSi lowest not locked */
 		},
-		flags: FLASH_IS_CFI
+		capabilities: CAP_CFI
 	},	
 	{
 		/* bottom boot (non Uniform) sector version does not really exist ! */
-		name:	"SG29GL256MR4",
+		name:	"S29GL256MR4",
 		size:	    32*1024*1024,
 		min_sectorsize: 64*1024,	
 		querystr: { 'Q','R','Y', 0x02,0,0x40,0,0,0,0,0,0x27,0x36,0,0, 7, 
@@ -462,7 +545,7 @@ flash_types[] = {
 			{0x01,0x4f,0x227e},	/* Device ID */
 			{0x03,0x4f,0x0008},	/* SecSi lowest not locked */
 		},
-		flags: FLASH_IS_CFI
+		capabilities: CAP_CFI
 	},	
 	{
 		/* Warning ! Multiple independent banks not supported */
@@ -487,10 +570,10 @@ flash_types[] = {
 			{0x0e,0x7f,0x2218},
 			{0x0f,0x7f,0x2200},
 		},
-		flags: FLASH_IS_CFI
+		capabilities: CAP_CFI
 	},
 	{
-		name:	"SG29GL128NR2",
+		name:	"S29GL128NR2",
 		size:	    16*1024*1024,
 		min_sectorsize: 128*1024,	
 		querystr: { 'Q','R','Y', 0x02,0,0x40,0,0,0,0,0,0x27,0x36,0,0, 7, 
@@ -506,10 +589,10 @@ flash_types[] = {
 			{0x01,0x4f,0x227e},	/* Device ID */
 			{0x03,0x4f,0x0008},	/* SecSi lowest not locked */
 		},
-		flags: FLASH_IS_CFI
+		capabilities: CAP_CFI
 	},	
 	{
-		name:	"SG29GL256NR2",
+		name:	"S29GL256NR2",
 		size:	    32*1024*1024,
 		min_sectorsize: 128*1024,	
 		querystr: { 'Q','R','Y', 0x02,0,0x40,0,0,0,0,0,0x27,0x36,0,0, 7, 
@@ -525,10 +608,10 @@ flash_types[] = {
 			{0x01,0x4f,0x227e},	/* Device ID */
 			{0x03,0x4f,0x0008},	/* SecSi lowest not locked */
 		},
-		flags: FLASH_IS_CFI
+		capabilities: CAP_CFI
 	},	
 	{
-		name:	"SG29GL512NR2",
+		name:	"S29GL512NR2",
 		size:	    32*1024*1024,
 		min_sectorsize: 128*1024,	
 		querystr: { 'Q','R','Y', 0x02,0,0x40,0,0,0,0,0,0x27,0x36,0,0, 7, 
@@ -544,7 +627,87 @@ flash_types[] = {
 			{0x01,0x4f,0x227e},	/* Device ID */
 			{0x03,0x4f,0x0008},	/* SecSi lowest not locked */
 		},
-		flags: FLASH_IS_CFI
+		capabilities: CAP_CFI
+	},	
+	{
+		name:	"S29GL128GP11TFI010",
+		size:	    16*1024*1024,
+		min_sectorsize: 128*1024,	
+		querystr: { 'Q','R','Y', 0x02,0,0x40,0,0,0,0,0,0x27,0x36,0,0,6, 
+			6,9,0x13,3,5,3,2,0x18, 2,0,6,0,1, 0x7f,0,0,2, 
+				0,0,0,0, 0,0,0,0, 0,0,0,0,  0xffff,0xffff,0xffff,
+			'P','R','I', 0x31,0x33,0x14,2,1,0,8,0,0,2,0xb5,0xc5,5
+			,1
+		},
+		auto_sel: { 
+			{0x00,0x4f,0x0001},	/* Manufacturer ID */
+			{0x01,0x4f,0x227e},	/* Device ID */
+			{0x0e,0x4f,0x2221},	/* Ex Device ID */
+			{0x0f,0x4f,0x2201},	/* Ex2 Device ID */
+			{0x02,0x4f,0x0000},	/* Sector group Prot. verification */
+			{0x03,0x4f,0x0019},	/* SecSi highest not locked */
+		},
+		capabilities: CAP_CFI
+	},	
+	{
+		name:	"S29GL256GP11TFI010",
+		size:	    32*1024*1024,
+		min_sectorsize: 128*1024,	
+		querystr: { 'Q','R','Y', 0x02,0,0x40,0,0,0,0,0,0x27,0x36,0,0,6, 
+			6,9,0x13,3,5,3,2,0x19, 2,0,6,0,1, 0xff,0,0,2, 
+				0,0,0,0, 0,0,0,0, 0,0,0,0,  0xffff,0xffff,0xffff,
+			'P','R','I', 0x31,0x33,0x14,2,1,0,8,0,0,2,0xb5,0xc5,5
+			,1
+		},
+		auto_sel: { 
+			{0x00,0x4f,0x0001},	/* Manufacturer ID */
+			{0x01,0x4f,0x227e},	/* Device ID */
+			{0x0e,0x4f,0x2222},	/* Ex Device ID */
+			{0x0f,0x4f,0x2201},	/* Ex2 Device ID */
+			{0x02,0x4f,0x0000},	/* Sector group Prot. verification */
+			{0x03,0x4f,0x0019},	/* SecSi highest not locked */
+		},
+		capabilities: CAP_CFI | CAP_TOPBOOT
+	},	
+	{
+		name:	"S29GL512GP11TFI010",
+		size:	    64*1024*1024,
+		min_sectorsize: 128*1024,	
+		querystr: { 'Q','R','Y', 0x02,0,0x40,0,0,0,0,0,0x27,0x36,0,0,6, 
+			6,9,0x13,3,5,3,2,0x1a, 2,0,6,0,1, 0xff,1,0,2, 
+				0,0,0,0, 0,0,0,0, 0,0,0,0,  0xffff,0xffff,0xffff,
+			'P','R','I', 0x31,0x33,0x14,2,1,0,8,0,0,2,0xb5,0xc5,5
+			,1
+		},
+		auto_sel: { 
+			{0x00,0x4f,0x0001},	/* Manufacturer ID */
+			{0x01,0x4f,0x227e},	/* Device ID */
+			{0x0e,0x4f,0x2223},	/* Ex Device ID */
+			{0x0f,0x4f,0x2201},	/* Ex2 Device ID */
+			{0x02,0x4f,0x0000},	/* Sector group Prot. verification */
+			{0x03,0x4f,0x0019},	/* SecSi highest not locked */
+		},
+		capabilities: CAP_CFI | CAP_TOPBOOT
+	},	
+	{
+		name:	"S29GL01GP11TFI010",
+		size:	    128*1024*1024,
+		min_sectorsize: 128*1024,	
+		querystr: { 'Q','R','Y', 0x02,0,0x40,0,0,0,0,0,0x27,0x36,0,0,6, 
+			6,9,0x13,3,5,3,2,0x1b, 2,0,6,0,1, 0xff,3,0,2, 
+				0,0,0,0, 0,0,0,0, 0,0,0,0,  0,0,0,
+			'P','R','I', 0x31,0x33,0x14,2,1,0,8,0,0,2,0xb5,0xc5,5
+			,1
+		},
+		auto_sel: { 
+			{0x00,0x4f,0x0001},	/* Manufacturer ID */
+			{0x01,0x4f,0x227e},	/* Device ID */
+			{0x0e,0x4f,0x2228},	/* Ex Device ID */
+			{0x0f,0x4f,0x2201},	/* Ex2 Device ID */
+			{0x02,0x4f,0x0000},	/* Sector group Prot. verification */
+			{0x03,0x4f,0x0019},	/* SecSi highest not locked */
+		},
+		capabilities: CAP_CFI | CAP_TOPBOOT
 	},	
 	{
 		name:	"MX29LV640BU",
@@ -562,7 +725,7 @@ flash_types[] = {
 			{0x02,0x4f,0x0000},	/* Sector Group protect */
 			{0x03,0x4f,0x0008},	/* lowest not factory locked */
 		},
-		flags: FLASH_IS_CFI
+		capabilities: CAP_CFI
 	},	
 	{
 		name: "S29AL016DT", /* Name in datasheet extended by top boot "T" */
@@ -582,7 +745,7 @@ flash_types[] = {
 			{0x00,0x4f,0x0001},
 			{0x01,0x4f,0x22C4},
 		},
-		flags: FLASH_IS_CFI
+		capabilities: CAP_CFI
 	},
 	{
 		name: "S29AL016DB", /* Name in datasheet extended by bootom boot "B" */
@@ -602,7 +765,7 @@ flash_types[] = {
 			{0x00,0x4f,0x0001},
 			{0x01,0x4f,0x2249},
 		},
-		flags: FLASH_IS_CFI
+		capabilities: CAP_CFI
 	},
 }; 
 
@@ -611,7 +774,7 @@ flash_types[] = {
  * Read a cfi register from flash type
  * -----------------------------------------
  */
-static inline uint8_t 
+static inline uint16_t 
 cfi_read(FlashType *ftype,int reg) {
 	if(reg >= 0x10 && reg <= 0x5b) {
 		return ftype->querystr[reg-0x10];
@@ -668,7 +831,7 @@ switch_to_cfimode(AMD_Flash *flash) {
  */
 static void 
 first_cycle_write(AMD_Flash *flash,uint16_t value,uint32_t dev_addr) {
-	uint32_t reg = REG(dev_addr);
+	uint32_t reg = FLASH_REG(dev_addr);
 //	fprintf(stderr,"%s: value %08x, register %08x\n",flash->type->name,value,reg); 
 	if(flash->cmd != CMD_NONE) {
 		fprintf(stderr,"CMD not NONE in first cycle\n");
@@ -684,30 +847,72 @@ first_cycle_write(AMD_Flash *flash,uint16_t value,uint32_t dev_addr) {
 		switch_to_readarray(flash);
 		return;
 	}
-	if((value==0x98) && (reg==0x55)) {
-		if(flash->type->flags & FLASH_IS_CFI) {
+	if((value==0x98) && ((reg==0x55) | (reg == 0x555))) {
+		if(flash->type->capabilities & CAP_CFI) {
 			switch_to_cfimode(flash);
 		}
 		return;
 	} 
-	if((value == 0xa0) &&  (flash->bypass==BYPASS_UNLOCKED)) {
-		flash->cycle++;
-		flash->cmd = CMD_UNLOCK_BYPASS_PGM;
-		return;
+	if((value == 0xa0))  {
+		switch(flash->en_cmdset) {
+			case ENCMDSET_NONE:
+				break;
+
+			case ENCMDSET_BYPASS:
+				flash->cycle++;
+				flash->cmd = CMD_PROGRAM;
+				break;
+
+			case ENCMDSET_LOCK_REG:
+				flash->cycle++;
+				flash->cmd =  CMD_LOCKBITS_PGM;
+				break;
+
+			case ENCMDSET_PW_PROT:
+				flash->cycle++;
+				flash->cmd =  CMD_PW_PGM;
+				break;
+
+			case ENCMDSET_PPB:
+				flash->cycle++;
+				flash->cmd = CMD_PPB_PGM;
+				break;
+
+			case ENCMDSET_PPB_LOCK:
+				flash->cycle++;
+				flash->cmd = CMD_PPB_LOCKBIT_SET;
+				break;
+
+			case ENCMDSET_DYB:	
+				flash->cycle++;
+				flash->cmd = CMD_PPB_LOCKBIT_SET;
+				break;
+
+			default:
+				fprintf(stderr,"AMDFLASH: illegal cmdset selection\n");
+		}
 	}
-	if((value == 0x90) &&  (flash->bypass==BYPASS_UNLOCKED)) {
+	if((value == 0x90) &&  (flash->en_cmdset==ENCMDSET_BYPASS)) {
 		flash->cycle++;
-		flash->cmd = CMD_UNLOCK_BYPASS_RESET;
+		flash->cmd = CMD_EXIT_CMDSET;
 		return;
 	}
 	if((value == 0xb0)) {
+		static int maxmsgs = 5;
 		// ignore suspend resume	
-		fprintf(stderr,"AMD_Flash: Got suspend, but not busy !\n");
+		if(maxmsgs > 0) {
+			fprintf(stderr,"AMD_Flash: Got suspend, but not busy !\n");
+			maxmsgs--;
+		}
 		return;
 	} 
 	if((value == 0x30)) {
+		static int maxmsgs = 5;
 		// ignore suspend resume	
-		fprintf(stderr,"AMD_Flash: Ignore resume because not suspended\n");
+		if(maxmsgs > 0) {
+			fprintf(stderr,"AMD_Flash: Ignore resume because not suspended\n");
+			maxmsgs--;
+		}
 		return;
 	} 
 	{
@@ -734,7 +939,7 @@ programm_word(AMD_Flash *flash,uint16_t value,uint32_t dev_addr) {
 
 static void 
 second_cycle_write(AMD_Flash *flash,uint16_t value,uint32_t dev_addr) {
-	uint32_t reg = REG(dev_addr);
+	uint32_t reg = FLASH_REG(dev_addr);
 	if(flash->cmd == CMD_NONE) {
 		if((reg==0x2aa) && (value==0x55)) {
 			flash->cycle++;
@@ -747,13 +952,13 @@ second_cycle_write(AMD_Flash *flash,uint16_t value,uint32_t dev_addr) {
 		}
 		fprintf(stderr,"Unknown: value %08x to reg %08x in second cycle\n",value,reg);
 		return;
-	} else if(flash->cmd==CMD_UNLOCK_BYPASS_PGM) {
+	} else if(flash->cmd==CMD_PROGRAM) {
 		programm_word(flash,value,dev_addr);
 		flash->cycle=CYCLE_FIRST;
 		flash->cmd=CMD_NONE;
 		return;
-	} else if(flash->cmd == CMD_UNLOCK_BYPASS_RESET) {
-		flash->bypass=BYPASS_LOCKED;
+	} else if(flash->cmd == CMD_EXIT_CMDSET) {
+		flash->en_cmdset=ENCMDSET_NONE;
 		flash->cmd=CMD_NONE;	
 		flash->cycle=CYCLE_FIRST;
 		return;
@@ -766,7 +971,8 @@ second_cycle_write(AMD_Flash *flash,uint16_t value,uint32_t dev_addr) {
 }
 static void 
 third_cycle_write(AMD_Flash *flash,uint16_t value,uint32_t dev_addr) {
-	uint32_t reg = REG(dev_addr);
+	uint32_t reg = FLASH_REG(dev_addr);
+	uint32_t cap = flash->type->capabilities;
 	if(flash->cmd != CMD_NONE) {
 		fprintf(stderr,"Unknown cmd %d in third cycle\n",flash->cmd);
 		return;
@@ -803,7 +1009,7 @@ third_cycle_write(AMD_Flash *flash,uint16_t value,uint32_t dev_addr) {
 			break;
 
 		case 0xa0:
-			flash->cmd = CMD_PROGRAMM;
+			flash->cmd = CMD_PROGRAM;
 			flash->cycle++;
 			break;
 
@@ -815,14 +1021,56 @@ third_cycle_write(AMD_Flash *flash,uint16_t value,uint32_t dev_addr) {
 
 		/* Unlock Bypass */
 		case 0x20:
-			flash->bypass=BYPASS_UNLOCKED;
+			flash->en_cmdset=ENCMDSET_BYPASS;
 			flash->cmd=CMD_NONE;
 			flash->cycle=CYCLE_FIRST;
 			break;
+
+		case 0x40:
+			if(cap & CAP_LOCK_REG) {
+				flash->en_cmdset=ENCMDSET_LOCK_REG;
+				flash->cmd=CMD_NONE;
+				flash->cycle=CYCLE_FIRST;
+			} 
+			break;
+
+		case 0x50:
+			if(cap & CAP_PPB_LOCK) {
+				flash->en_cmdset=ENCMDSET_PPB_LOCK;
+				flash->cmd=CMD_NONE;
+				flash->cycle=CYCLE_FIRST;
+			} 
+			break;
+
+		case 0x60:
+			if(cap & CAP_PASSWD) {
+				flash->en_cmdset=ENCMDSET_PW_PROT;
+				flash->cmd=CMD_NONE;
+				flash->cycle=CYCLE_FIRST;
+			} 
+			break;
+
 		case 0x80:
 			flash->cmd=CMD_ERASE;
 			flash->cycle++;
 			break;
+
+		case 0xC0:
+			if(cap & CAP_PPB) {
+				flash->en_cmdset=ENCMDSET_PPB;
+				flash->cmd=CMD_NONE;
+				flash->cycle=CYCLE_FIRST;
+			} 
+			break;
+
+		case 0xE0:
+			if(cap & CAP_DYB) {
+				flash->en_cmdset=ENCMDSET_DYB;
+				flash->cmd=CMD_NONE;
+				flash->cycle=CYCLE_FIRST;
+			} 
+			break;
+			
 		default:
 			fprintf(stderr,"unknown value %04x in third cycle\n",value);
 	}
@@ -833,7 +1081,7 @@ third_cycle_write(AMD_Flash *flash,uint16_t value,uint32_t dev_addr) {
 static void 
 fourth_cycle_write(AMD_Flash *flash,uint16_t value,uint32_t dev_addr) 
 {
-	uint32_t reg = REG(dev_addr);
+	uint32_t reg = FLASH_REG(dev_addr);
 	uint32_t cmd=flash->cmd;
 	if(cmd==CMD_AUTOSELECT) {
 		if((value==0xf0) || (value == 0xff)) {
@@ -846,7 +1094,7 @@ fourth_cycle_write(AMD_Flash *flash,uint16_t value,uint32_t dev_addr)
 		flash->cmd=CMD_NONE;
 		flash->cycle=CYCLE_FIRST;
 		return;
-	} else if(cmd==CMD_PROGRAMM) {
+	} else if(cmd==CMD_PROGRAM) {
 		programm_word(flash,value,dev_addr);
 		flash->cycle=CYCLE_FIRST;
 		flash->cmd=CMD_NONE;
@@ -882,7 +1130,7 @@ fourth_cycle_write(AMD_Flash *flash,uint16_t value,uint32_t dev_addr)
 static void 
 fifth_cycle_write(AMD_Flash *flash,uint16_t value,uint32_t dev_addr) 
 {
-	uint32_t reg = REG(dev_addr);
+	uint32_t reg = FLASH_REG(dev_addr);
 	int cmd = flash->cmd;
 	if(cmd==CMD_WRITE_TO_BUF) {
 		int i;
@@ -898,10 +1146,10 @@ fifth_cycle_write(AMD_Flash *flash,uint16_t value,uint32_t dev_addr)
 			return;
 		}
 		buf_index = reg & 0xf;
-		flash->wb[buf_index]=value&0xffff;
-		flash->wb_addr[buf_index]=reg;
-		flash->wb_valid[buf_index]=1;
-		flash->wb_ba= reg&~0xf;
+		flash->wb[buf_index] = value & 0xffff;
+		flash->wb_addr[buf_index] = FLASH_ADDR(dev_addr);
+		flash->wb_valid[buf_index] = 1;
+		flash->wb_ba = FLASH_ADDR(dev_addr) & ~UINT32_C(0xf);
 		flash->cycle++;
 		if(flash->wb_wc==0) { // do not decrement
 			//fprintf(stderr,"WB Complete\n");
@@ -958,7 +1206,7 @@ erase_sector(AMD_Flash *flash,uint32_t sa) {
 		/* Misuse of translation Macro */
 		flash->statistics_mem[FLASH_ADDR_TO_MEMBUS(bank,sector)]++;
 	}
-	dprintf("erase sector at 0x%08x, size %d\n",sa,sectorsize);
+	dbgprintf("erase sector at 0x%08x, size %d\n",sa,sectorsize);
 	for(i=0;i<sectorsize;i+=2) {
 		uint32_t addr = FLASH_ADDR_TO_MEMBUS(bank,i+sa);
 		*(uint16_t*)(flash->host_mem+addr)=0xffff;
@@ -985,20 +1233,20 @@ erase_timeout(void *cd) {
 static void 
 sixth_cycle_write(AMD_Flash *flash,uint16_t value,uint32_t dev_addr) 
 {
-	uint32_t reg = REG(dev_addr);
+	uint32_t reg = FLASH_REG(dev_addr);
 	int cmd = flash->cmd;
 	if(cmd==CMD_WRITE_TO_BUF) {
 		int buf_index;
-		if((reg&~0xf) != flash->wb_ba) {
+		if((FLASH_ADDR(dev_addr) & ~UINT32_C(0xf)) != flash->wb_ba) {
 			fprintf(stderr,"outside of writebuffer, addr %08x ba %08x\n",dev_addr,flash->wb_ba);
 			flash->cycle=CYCLE_FIRST;		
 			flash->cmd=CMD_NONE;
 			return;
 		} 
 		buf_index = reg & 0xf;
-		flash->wb[buf_index]=value&0xffff;
-		flash->wb_addr[buf_index]=reg;
-		flash->wb_valid[buf_index]=1;
+		flash->wb[buf_index] = value & 0xffff;
+		flash->wb_addr[buf_index] = FLASH_ADDR(dev_addr);
+		flash->wb_valid[buf_index] = 1;
 		if(--flash->wb_wc==0) {
 			//fprintf(stderr,"WB Complete\n");
 			flash->cmd=CMD_PGM_BUFFER;
@@ -1061,19 +1309,19 @@ sixth_cycle_write(AMD_Flash *flash,uint16_t value,uint32_t dev_addr)
 static void 
 nth_cycle_write(AMD_Flash *flash,uint16_t value,uint32_t dev_addr) 
 {
-	uint32_t reg = REG(dev_addr);
+	uint32_t reg = FLASH_REG(dev_addr);
 	int cmd = flash->cmd;
 	if(cmd==CMD_WRITE_TO_BUF) {
 		int buf_index;
-		if((reg&~0xf) != flash->wb_ba) {
+		if((FLASH_ADDR(dev_addr) & ~UINT32_C(0xf)) != flash->wb_ba) {
 			fprintf(stderr,"outside of writebuffer, addr %08x ba %08x\n",dev_addr,flash->wb_ba);
 			flash->cycle=CYCLE_FIRST;		
 			flash->cmd=CMD_NONE;
 			return;
 		} 
 		buf_index = reg & 0xf;
-		flash->wb[buf_index]=value&0xffff;
-		flash->wb_addr[buf_index]=reg;
+		flash->wb[buf_index] = value & 0xffff;
+		flash->wb_addr[buf_index] = FLASH_ADDR(dev_addr);
 		flash->wb_valid[buf_index]=1;
 		if(--flash->wb_wc==0) {
 			//fprintf(stderr,"WB Complete\n");
@@ -1177,7 +1425,7 @@ fourth_cycle_read(AMD_Flash *flash,uint32_t dev_addr)
 	int i;
 	uint32_t value=0;
 	FlashType *ftype = flash->type;
-	uint32_t reg = REG(dev_addr);
+	uint32_t reg = FLASH_REG(dev_addr);
 //	fprintf(stderr,"fourth cycle reg 0x%08x",reg);
 	if(flash->cmd==CMD_AUTOSELECT) {
 		for(i=0;i<MAX_AUTO_CODES;i++) {
@@ -1224,7 +1472,7 @@ flash_read(void *clientData,uint32_t address,int rqlen) {
 		}
 		return value;
 	} else if((flash->mode == MAP_CFI)) {
-		unsigned int reg = REG((dev_addr ^ bank->endian_xor) & 0xfff);
+		unsigned int reg = FLASH_REG((dev_addr ^ bank->endian_xor) & 0xfff);
 		return cfi_read(flash->type,reg);
 	} else if((flash->mode == MAP_IO)) {
 		if(flash->cycle == CYCLE_FIRST) {
@@ -1351,7 +1599,7 @@ setup_eraseregions(AMD_Flash *flash)
 	int i;
 	int topboot;
 	int n_regions;
-	if(!(ftype->flags & FLASH_IS_CFI)) {
+	if(!(ftype->capabilities & CAP_CFI)) {
 		region = &flash->erase_region[0];
 		region->sectorsize = ftype->min_sectorsize;
 		region->sectors = ftype->size/region->sectorsize;
@@ -1368,7 +1616,7 @@ setup_eraseregions(AMD_Flash *flash)
 		fprintf(stderr,"AMD Flash: Illegal number of Erase Regions: 0x%02x\n",n_regions);
 		exit(4325);
 	}
-	if(ftype->flags & FLASH_IS_TOPBOOT) {
+	if(ftype->capabilities & CAP_TOPBOOT) {
 		topboot=1;
 	} else {
 		topboot=0;
@@ -1436,11 +1684,7 @@ setup_regionmap(AMD_Flash *flash)
 		}
 	}
 	flash->rmap_shift=shift;
-	flash->rmap = malloc(sizeof(*flash->rmap) * flash->rmap_size);
-	if(!flash->rmap) {
-		fprintf(stderr,"Out of memory allocating flash region map\n");
-		exit(9676);
-	}
+	flash->rmap = sg_calloc(sizeof(*flash->rmap) * flash->rmap_size);
 	for(n=0,i=0;i<flash->n_regions;i++) {
 		int entries_per_sector;
 		region = &flash->erase_region[i];
@@ -1469,12 +1713,7 @@ setup_regionmap(AMD_Flash *flash)
  */
 static AMD_Flash *
 AMD_Flash_New(FlashType *ftype,AMDFlashBank *bank,int chip_nr) {
-	AMD_Flash *flash = malloc(sizeof(AMD_Flash));	
-	if(!flash) {
-		fprintf(stderr,"Out of memory\n");
-		exit(634);
-	}
-	memset(flash,0,sizeof(AMD_Flash));
+	AMD_Flash *flash = sg_new(AMD_Flash);	
 
 	flash->type = ftype;
 	flash->bank = bank;
@@ -1484,7 +1723,7 @@ AMD_Flash_New(FlashType *ftype,AMDFlashBank *bank,int chip_nr) {
 	} else {
 		flash->statistics_mem = NULL;
 	}
-	if(ftype->flags & FLASH_IS_CFI) {	
+	if(ftype->capabilities & CAP_CFI) {	
 		flash->cap_bufwrite = (1<<cfi_read(ftype,0x2a)); 
 	}
 	flash->cycle=CYCLE_FIRST;
@@ -1502,7 +1741,7 @@ AMD_Flash_New(FlashType *ftype,AMDFlashBank *bank,int chip_nr) {
  *      Invoked when the Endian Signal Line changes
  * -----------------------------------------------------------
  */
-static int
+static void 
 change_endian(SigNode *node,int value,void *clientData)
 {
         AMDFlashBank *bank = clientData;
@@ -1527,7 +1766,6 @@ change_endian(SigNode *node,int value,void *clientData)
                 fprintf(stderr,"NS9750 Serial: Endian is neither Little nor Big\n");
                 exit(3424);
         }
-        return 0;
 }
 
 /*
@@ -1537,8 +1775,7 @@ change_endian(SigNode *node,int value,void *clientData)
  */
 BusDevice *
 AMDFlashBank_New(const char *flash_name) {
-	AMDFlashBank *bank = malloc(sizeof(AMDFlashBank));
-	char *nodename = alloca(strlen(flash_name)+30);
+	AMDFlashBank *bank = sg_new(AMDFlashBank);
 	int i;
 	int nr_types = sizeof(flash_types)/sizeof(FlashType);
 	uint32_t nr_chips;
@@ -1546,11 +1783,6 @@ AMDFlashBank_New(const char *flash_name) {
 	char *directory;
 	char *flash_type;
 	FlashType *ftype=NULL;
-	if(!bank) {
-		fprintf(stderr,"Out of memory\n");
-		exit(4534);
-	}
-	memset(bank,0,sizeof(AMDFlashBank));
 
         flash_type=Config_ReadVar(flash_name,"type");
         if(!flash_type) {
@@ -1567,7 +1799,7 @@ AMDFlashBank_New(const char *flash_name) {
 		fprintf(stderr,"Flash type %s not found. Available types:\n",flash_type);
 		for(i=0;i<nr_types;i++) {
 			ftype=&flash_types[i];	
-			fprintf(stderr," %-15s - size %dk\n", ftype->name,ftype->size/1024);
+			fprintf(stderr," %-20s - size %dk\n", ftype->name,ftype->size/1024);
 		}
 		exit(1734);
 	}
@@ -1604,11 +1836,7 @@ AMDFlashBank_New(const char *flash_name) {
 		}
 		bank->host_mem=DiskImage_Mmap(bank->disk_image);
 	} else {
-		bank->host_mem=malloc(bank->size);
-		if(!bank->host_mem) {
-			fprintf(stderr,"Out of memory\n");
-			exit(637);
-		}
+		bank->host_mem = sg_calloc(bank->size);
 		memset(bank->host_mem,0xff,bank->size);
 	}
 	directory= Config_ReadVar("global","imagedir");
@@ -1634,10 +1862,9 @@ AMDFlashBank_New(const char *flash_name) {
 	fprintf(stderr,"Flash bank \"%s\" type %s Chips %d writebuf %d\n",
 		flash_name,ftype->name,nr_chips,1<<cfi_read(ftype,0x2a));
 
-	sprintf(nodename,"%s.big_endian",flash_name);
-        bank->big_endianNode = SigNode_New(nodename);
+        bank->big_endianNode = SigNode_New("%s.big_endian",flash_name);
         if(!bank->big_endianNode) {
-                fprintf(stderr,"Can not create Flashbank. EndianNode %s\n",nodename);
+                fprintf(stderr,"Can not create Flashbank. EndianNode\n");
                 exit(3429);
         }
         bank->big_endianTrace = SigNode_Trace(bank->big_endianNode,change_endian,bank);
