@@ -4,7 +4,8 @@
  * Emulation of the ARM CPU, Initialization and 
  * main loop
  *
- * (C) 2004  Lightmaze Solutions AG
+ * (C) 2004 Lightmaze Solutions AG 
+ * (C) 2009 Jochen Karrer
  *   Author: Jochen Karrer
  *
  *
@@ -32,6 +33,8 @@
 #include <fcntl.h>
 #include <string.h>
 #include <setjmp.h>
+#include <errno.h>
+#include <sys/types.h>
 #include <coprocessor.h>
 #include <arm9cpu.h>
 #include <instructions.h>
@@ -45,7 +48,6 @@
 #include <sys/time.h>
 #include <cycletimer.h>
 #include <configfile.h>
-#include <debug.h>
 #include <mainloop_events.h>
 
 #ifdef DEBUG
@@ -57,27 +59,30 @@
 ARM9 gcpu;
 
 uint32_t debugflags=0;
-//uint32_t debugflags = DEBUG_LOG_PC;
 uint32_t mmu_high_vectors=0;
 uint32_t do_alignment_check=0;
 
 /*
  * ----------------------------------------------
- * First the operations for gdb access to the
+ * First the operations for debugger access to the
  * ARM system. GDB expects registers to be in
  * target byteorder
  * ----------------------------------------------
  */
 static int 
-gdb_getreg(void *clientData,uint8_t *data,int index,int maxlen) 
+debugger_getreg(void *clientData,uint8_t *data,uint32_t index,int maxlen) 
 {
 	uint32_t value;
 	if(maxlen < 4)
-		return -1;
+		return -EINVAL;
 	if(index<15) {
 		value = ARM9_ReadReg(index);
 	} else if(index==15) {
-		value = GET_REG_PC - 4; /* ???? No clue if this is Ok */ 
+		if(REG_CPSR & FLAG_T) {
+			value = THUMB_NIA;
+		} else {
+			value = ARM_NIA; 
+		}
 		dbgprintf("Read reg PC %08x\n",value); 
 	} else if(index < 25){
 		value = 0;
@@ -93,8 +98,9 @@ gdb_getreg(void *clientData,uint8_t *data,int index,int maxlen)
 	}
 	return 4;
 }
+
 static void 
-gdb_setreg(void *clientData,const uint8_t *data,int index,int len) 
+debugger_setreg(void *clientData,const uint8_t *data,uint32_t index,int len) 
 {
 	uint32_t value;
 	if(len != 4)
@@ -124,7 +130,7 @@ gdb_setreg(void *clientData,const uint8_t *data,int index,int len)
  * ---------------------------------------------------------------------
  */
 static void 
-gdb_get_bkpt_ins(void *clientData,uint8_t *ins,uint64_t addr,int len) 
+debugger_get_bkpt_ins(void *clientData,uint8_t *ins,uint64_t addr,int len) 
 {
 	if(len == 2) {
 		uint16_t value = 0xbe00;
@@ -144,40 +150,42 @@ gdb_get_bkpt_ins(void *clientData,uint8_t *ins,uint64_t addr,int len)
 }
 
 static int 
-gdb_stop(void *clientData) 
+debugger_stop(void *clientData) 
 {
 
-	gcpu.dbg_mode = DBG_MODE_STOP;	
-	ARM_PostRestartLoop();
+	gcpu.dbg_state = DBG_STATE_STOP;	
+	ARM_SigDebugMode(1);
 	return -1;
 }
 
 static int
-gdb_cont(void *clientData)
+debugger_cont(void *clientData)
 {
 	//fprintf(stderr,"ARM cont\n");
-	gcpu.dbg_mode = DBG_MODE_OFF;
+	gcpu.dbg_state = DBG_STATE_RUNNING;
+	/* Should only be called if there are no breakpoints */
+	ARM_SigDebugMode(0);
 	return 0;
 }
 
 static int 
-gdb_step(void *clientData,uint32_t addr,int use_addr) 
+debugger_step(void *clientData,uint64_t addr,int use_addr) 
 {
 	if(use_addr)  {
-		SET_REG_PC(addr);		
+		ARM_SET_NIA(addr);		
 	}
 	gcpu.dbg_steps = 1;
-	gcpu.dbg_mode = DBG_MODE_STEP;
-
-	ARM_PostRestartLoop();
+	gcpu.dbg_state = DBG_STATE_STEP;
 	return -1;
 }
 
-static int
-gdb_get_status(void *clientData)
+static Dbg_TargetStat 
+debugger_get_status(void *clientData)
 {
-	if(gcpu.dbg_mode == DBG_MODE_STOPPED) {
-		return SIGINT;
+	if(gcpu.dbg_state == DBG_STATE_STOPPED) {
+		return DbgStat_SIGINT;
+	} else if(gcpu.dbg_state == DBG_STATE_RUNNING) {
+		return DbgStat_RUNNING;
 	} else {
 		return -1;
 	}
@@ -189,8 +197,8 @@ gdb_get_status(void *clientData)
  * --------------------------------------------------------------------
  */
 
-static int 
-gdb_getmem(void *clientData,uint8_t *data,uint64_t addr,uint32_t len) 
+static ssize_t 
+debugger_getmem(void *clientData,uint8_t *data,uint64_t addr,uint32_t len) 
 {
 	int count;
 	/* catch exceptions from MMU */
@@ -224,8 +232,8 @@ gdb_getmem(void *clientData,uint8_t *data,uint64_t addr,uint32_t len)
 	return count;
 }
 
-static int 
-gdb_setmem(void *clientData,const uint8_t *data,uint64_t addr,uint32_t len) 
+static ssize_t 
+debugger_setmem(void *clientData,const uint8_t *data,uint64_t addr,uint32_t len) 
 {
 	int count=0;
 	//fprintf(stderr,"ARM readmem\n");
@@ -261,10 +269,11 @@ gdb_setmem(void *clientData,const uint8_t *data,uint64_t addr,uint32_t len)
 
 
 void 
-ARM_set_reg_cpsr(uint32_t val) 
+ARM_set_reg_cpsr(uint32_t new_cpsr) 
 {
-	uint32_t bank = val & 0x1f;
-	gcpu.reg_cpsr=(val);
+	uint32_t bank = new_cpsr & 0x1f;
+	uint32_t diff_cpsr = new_cpsr ^ gcpu.reg_cpsr;
+	gcpu.reg_cpsr = new_cpsr;
 	if(gcpu.reg_bank != bank) {
 		if(likely(gcpu.reg_bank != MODE_FIQ)) {
 			uint32_t *rp;
@@ -302,16 +311,19 @@ ARM_set_reg_cpsr(uint32_t val)
 		}
 		gcpu.signaling_mode = gcpu.reg_bank = bank;
 	}
-	if(unlikely(val & FLAG_I)) {
+	if(unlikely(new_cpsr & FLAG_I)) {
 		gcpu.signal_mask &= ~ARM_SIG_IRQ;	
 	} else {
 		gcpu.signal_mask |= ARM_SIG_IRQ;
 	}
-	if(unlikely(val & FLAG_F)) {
+	if(unlikely(new_cpsr & FLAG_F)) {
 		gcpu.signal_mask &= ~ARM_SIG_FIQ; 
 	} else {
 		gcpu.signal_mask |= ARM_SIG_FIQ;
 	}
+	if(unlikely(diff_cpsr & FLAG_T)) {
+		ARM_PostRestartIdecoder();
+	}
 	gcpu.signals = gcpu.signals_raw & gcpu.signal_mask;
 	if(gcpu.signals) 
 		mainloop_event_pending=1;
@@ -445,13 +457,13 @@ ARM9_InitRegs(ARM9 *arm) {
         arm->regSet[MODE_UNDEFINED] .r14= &arm->r14_und;
         arm->regSet[MODE_FIQ].r14 = &arm->r14_fiq;
 
-        arm->regSet[MODE_USER].pc=&REG_PC;
-        arm->regSet[MODE_SYSTEM].pc=&REG_PC;
-        arm->regSet[MODE_IRQ].pc=&REG_PC;
-        arm->regSet[MODE_SVC].pc=&REG_PC;
-        arm->regSet[MODE_ABORT].pc=&REG_PC;
-        arm->regSet[MODE_UNDEFINED].pc=&REG_PC;
-        arm->regSet[MODE_FIQ].pc=&REG_PC; 	
+        arm->regSet[MODE_USER].pc=&arm->registers[15];
+        arm->regSet[MODE_SYSTEM].pc=&arm->registers[15];
+        arm->regSet[MODE_IRQ].pc=&arm->registers[15];
+        arm->regSet[MODE_SVC].pc=&arm->registers[15];
+        arm->regSet[MODE_ABORT].pc=&arm->registers[15];
+        arm->regSet[MODE_UNDEFINED].pc=&arm->registers[15];
+        arm->regSet[MODE_FIQ].pc=&arm->registers[15]; 	
 
         arm->regSet[MODE_IRQ].spsr = &arm->spsr_irq;
         arm->regSet[MODE_SVC].spsr = &arm->spsr_svc;
@@ -459,6 +471,7 @@ ARM9_InitRegs(ARM9 *arm) {
         arm->regSet[MODE_UNDEFINED].spsr = &arm->spsr_und;
         arm->regSet[MODE_FIQ].spsr = &arm->spsr_fiq; 	
 	/* To avoid segfault if CPU is in illegal mode initialize with some nonsense */
+	/* LPC 2106 ignore mode bit 4 of CPSR! Should be implemented and Tested with other CPU's */
 	for(i=0;i<32;i++) {
 		if(arm->regSet[i].r0==NULL) {
 			arm->regSet[i].r0 = &arm->reg_dummy;
@@ -530,7 +543,7 @@ hello_proc(void *cd) {
 //	CycleTimer_Add(&htimer,10000000000LL,hello_proc,NULL);
 }
 
-static int
+static void 
 irq_change(SigNode *node,int value,void *clientData)
 {
 	if((value == SIG_LOW) || (value == SIG_PULLDOWN)) {
@@ -538,10 +551,9 @@ irq_change(SigNode *node,int value,void *clientData)
 	} else {
 		ARM_UnPostIrq();
 	}
-	return 0;
 }
 
-static int
+static void 
 fiq_change(SigNode *node,int value,void *clientData)
 {
 	if((value == SIG_LOW) || (value == SIG_PULLDOWN)) {
@@ -549,7 +561,58 @@ fiq_change(SigNode *node,int value,void *clientData)
 	} else {
 		ARM_UnPostFiq();
 	}	
-	return 0;
+}
+
+/*
+ *************************************************************************+
+ * Throttle timer for OS not using the halt instruction 
+ *************************************************************************+
+ */
+static void
+arm_throttle(void *clientData)
+{
+	ARM9  *arm = (ARM9 *) clientData;;
+        struct timespec tv_now;
+        uint32_t nsecs;
+        int64_t exp_cpu_cycles,done_cpu_cycles;
+        done_cpu_cycles = CycleCounter_Get() - arm->last_throttle_cycles;
+        arm->cycles_ahead += done_cpu_cycles;
+        do {
+                struct timespec tout;
+                tout.tv_nsec=10000000; /* 10 ms */
+                tout.tv_sec=0;
+
+                clock_gettime(CLOCK_MONOTONIC,&tv_now);
+                nsecs = (tv_now.tv_nsec - arm->tv_last_throttle.tv_nsec) +
+                        (int64_t)1000000000*(tv_now.tv_sec - arm->tv_last_throttle.tv_sec);
+                exp_cpu_cycles = NanosecondsToCycles(nsecs);
+                if(arm->cycles_ahead > exp_cpu_cycles) {
+                        FIO_WaitEventTimeout(&tout);
+                }
+        } while(arm->cycles_ahead > exp_cpu_cycles);
+        arm->cycles_ahead -= exp_cpu_cycles;
+        /*
+         **********************************************************
+         * Forget about catch up if CPU is more than on second
+         * behind to avoid a longer phase of overspeed. Sound
+         * doesn't like it !
+         **********************************************************
+         */
+        if( -arm->cycles_ahead > (CycleTimerRate_Get() >> 2)) {
+                arm->cycles_ahead = 0;
+        }
+        arm->last_throttle_cycles = CycleCounter_Get();
+        arm->tv_last_throttle = tv_now;
+        CycleTimer_Mod(&arm->throttle_timer,CycleTimerRate_Get() / 10);
+        return;
+}
+
+static void
+ARM_ThrottleInit(ARM9 *arm)
+{
+        clock_gettime(CLOCK_MONOTONIC,&arm->tv_last_throttle);
+        arm->last_throttle_cycles = 0;
+        CycleTimer_Add(&arm->throttle_timer,CycleTimerRate_Get() / 25,arm_throttle,arm);
 }
 
 /*
@@ -559,40 +622,39 @@ fiq_change(SigNode *node,int value,void *clientData)
  */
 ARM9* 
 ARM9_New() {
-	char *nodename = alloca(50);
-	int32_t cpu_clock=200000000;
+	uint32_t cpu_clock=200000000;
+	const char *instancename = "arm";
 	ARM9 *arm=&gcpu;
-	Config_ReadInt32(&cpu_clock,"global","cpu_clock");
+	Config_ReadUInt32(&cpu_clock,"global","cpu_clock");
 	fprintf(stderr,"Creating ARM9 CPU with clock %d HZ\n",cpu_clock);
 	memset(arm,0,sizeof(ARM9));
 	IDecoder_New();
 	ARM9_InitRegs(&gcpu); 
 	InitInstructions();
+	ThumbDecoder_New();
 	SET_REG_CPSR(MODE_SVC | FLAG_F | FLAG_I); 
-	CycleTimers_Init(cpu_clock);
+	CycleTimers_Init(instancename,cpu_clock);
 	CycleTimer_Add(&htimer,285000000,hello_proc,NULL);
-	sprintf(nodename,"arm.irq");
-	arm->irqNode = SigNode_New(nodename);
-	sprintf(nodename,"arm.fiq");
-	arm->fiqNode = SigNode_New(nodename);
+	arm->irqNode = SigNode_New("%s.irq",instancename);
+	arm->fiqNode = SigNode_New("%s.fiq",instancename);
 	if(!arm->irqNode || !arm->fiqNode) {
 		fprintf(stderr,"Can not create interrupt nodes for ARM CPU\n");
 		exit(2);
 	}
 	arm->irqTrace = SigNode_Trace(arm->irqNode,irq_change,arm);
 	arm->irqTrace = SigNode_Trace(arm->fiqNode,fiq_change,arm);
-	arm->gdbops.getreg = gdb_getreg;
-	arm->gdbops.setreg = gdb_setreg;
-	arm->gdbops.stop = gdb_stop;
-	arm->gdbops.step = gdb_step;
-	arm->gdbops.cont = gdb_cont;
-	arm->gdbops.get_status = gdb_get_status;
-	arm->gdbops.getmem = gdb_getmem;
-	arm->gdbops.setmem = gdb_setmem;
-	arm->gdbops.step = gdb_step;
-	arm->gdbops.get_bkpt_ins = gdb_get_bkpt_ins;
-	arm->gserv = GdbServer_New(&arm->gdbops,arm);
-	gcpu.signal_mask |= ARM_SIG_RESTART_LOOP;
+	arm->dbgops.getreg = debugger_getreg;
+	arm->dbgops.setreg = debugger_setreg;
+	arm->dbgops.stop = debugger_stop;
+	arm->dbgops.step = debugger_step;
+	arm->dbgops.cont = debugger_cont;
+	arm->dbgops.get_status = debugger_get_status;
+	arm->dbgops.getmem = debugger_getmem;
+	arm->dbgops.setmem = debugger_setmem;
+	arm->dbgops.get_bkpt_ins = debugger_get_bkpt_ins;
+	arm->debugger = Debugger_New(&arm->dbgops,arm);
+	gcpu.signal_mask |= ARM_SIG_RESTART_IDEC | ARM_SIG_DEBUGMODE;
+	ARM_ThrottleInit(arm);
 	return arm;
 }
 
@@ -613,7 +675,7 @@ void dump_stack() {
 	dbgprintf("\n");
 
 }
-UNUSED static void 
+__UNUSED__ static void 
 dump_regs() {
 	int i;
 	for(i=0;i<16;i++) {
@@ -630,28 +692,25 @@ dump_regs() {
 
 
 /*
- * ----------------------------------------------------
- * Generate an exception and call the handler
+ *********************************************************
+ * \fn ARM_Exception(int exception,int nia_offset); 
+ * Generate an exception and call the handler.
+ * \param exception 
  * ----------------------------------------------------
  */
 void 
-ARM9_Exception(int exception,int pc_offset) {
-	uint32_t retaddr=(GET_REG_PC-8)+pc_offset;
+ARM_Exception(ARM_ExceptionID exception,int nia_offset) {
 	uint32_t old_cpsr=REG_CPSR;
 	int new_mode = EX_TO_MODE(exception);
 	uint32_t new_pc = EX_TO_PC(exception);
+	uint32_t retaddr;
 
-#if 0
-	fprintf(stderr,"EXCEPTION %08x at %08x\n",exception,GET_REG_PC-8+pc_offset);
-	exit(1);
-	//ARM_Break();
-#endif
+	retaddr = ARM_NIA + nia_offset;
 
 	if(mmu_high_vectors) {
 		new_pc|=0xffff0000;
 	}
 	/* Save CPSR to SPSR in the bank of the new mode */ 
-
 	/* clear Thumb, set 0-4 to mode , disable IRQ */
 	SET_REG_CPSR((REG_CPSR & (0xffffffe0 & ~FLAG_T)) | new_mode | FLAG_I);
 	if(new_mode==MODE_FIQ) {
@@ -660,9 +719,45 @@ ARM9_Exception(int exception,int pc_offset) {
 	ARM9_WriteReg(old_cpsr,REG_NR_SPSR); 
 	REG_LR=retaddr;
 	/* jump to exception vector address */
-	SET_REG_PC(new_pc);
+	ARM_SET_NIA(new_pc);
 }
 
+static void
+Do_Debug(void) {
+	fprintf(stderr,"one at %08x\n",ARM_NIA);
+	if(likely(gcpu.dbg_state == DBG_STATE_RUNNING)) { 
+		fprintf(stderr,"Debug mode is of, should not be called\n");
+	}  else if(gcpu.dbg_state == DBG_STATE_STEP) {
+		if(gcpu.dbg_steps == 0) {
+			gcpu.dbg_state = DBG_STATE_STOPPED;
+			if(gcpu.debugger) {
+				Debugger_Notify(gcpu.debugger,DbgStat_SIGTRAP);
+			}
+			ARM_RestartIdecoder();
+		} else {
+			gcpu.dbg_steps--;
+			mainloop_event_pending = 1;
+		}
+	} else if(gcpu.dbg_state == DBG_STATE_STOP) {
+		gcpu.dbg_state = DBG_STATE_STOPPED;
+		if(gcpu.debugger) {
+			Debugger_Notify(gcpu.debugger,DbgStat_SIGTRAP);
+		}
+		ARM_RestartIdecoder();
+	} else if(gcpu.dbg_state == DBG_STATE_BREAK) {
+		if(gcpu.debugger) {
+			/* Stop only if the debugger shows a reaction */
+			if(Debugger_Notify(gcpu.debugger,DbgStat_SIGINT) > 0) {
+				gcpu.dbg_state = DBG_STATE_STOPPED;
+				ARM_RestartIdecoder();
+			}
+		}
+		ARM_Exception(EX_PABT,4);
+		gcpu.dbg_state = DBG_STATE_RUNNING;
+	} else {
+		fprintf(stderr,"Unknown restart signal reason %d\n",gcpu.dbg_state);
+	}
+}
 
 /*
  * -----------------------------------------------------------------------
@@ -672,7 +767,7 @@ ARM9_Exception(int exception,int pc_offset) {
  * -----------------------------------------------------------------------
  */
 static inline void 
-CheckSignals() {
+CheckSignals(void) {
 	if(unlikely(mainloop_event_pending)) {
 		mainloop_event_pending = 0;
 		if(mainloop_event_io) {
@@ -680,49 +775,16 @@ CheckSignals() {
 		}
 		if(gcpu.signals) {
 			if(likely(gcpu.signals & ARM_SIG_IRQ)) {
-				ARM9_Exception(EX_IRQ,8); 
+				ARM_Exception(EX_IRQ,4); 
 			}
 			if(unlikely(gcpu.signals & ARM_SIG_FIQ)) {
-				ARM9_Exception(EX_FIQ,8); 
+				ARM_Exception(EX_FIQ,4); 
 			}
-			if(unlikely(gcpu.signals & ARM_SIG_RESTART_LOOP)) {
-				if(likely(gcpu.dbg_mode== DBG_MODE_OFF)) { 
-					ARM_RestartLoop();
-				}  else if(gcpu.dbg_mode == DBG_MODE_STEP) {
-					gcpu.dbg_steps--;
-					if(gcpu.dbg_steps==0) {
-						gcpu.dbg_mode = DBG_MODE_STOPPED;
-						if(gcpu.gserv) {
-							GdbServer_Signal(gcpu.gserv,SIGINT);
-						}
-						ARM_RestartLoop();
-					} else {
-						mainloop_event_pending = 1;
-					}
-				} else if(gcpu.dbg_mode == DBG_MODE_STOP) {
-					gcpu.dbg_mode = DBG_MODE_STOPPED;
-					if(gcpu.gserv) {
-						GdbServer_Signal(gcpu.gserv,SIGINT);
-					}
-					ARM_RestartLoop();
-				} else if(gcpu.dbg_mode == DBG_MODE_BREAK) {
-					if(gcpu.gserv) {
-						if(GdbServer_Signal(gcpu.gserv,SIGINT)>0) {
-							gcpu.dbg_mode = DBG_MODE_STOPPED;
-							ARM_RestartLoop();
-#if 1
-						}   else {
-						// jk
-							gcpu.dbg_mode = DBG_MODE_STOPPED;
-							ARM_RestartLoop();
-#endif
-						}
-					}
-					ARM9_Exception(EX_PABT,8);
-					gcpu.dbg_mode = DBG_MODE_OFF;
-				} else {
-					fprintf(stderr,"Unknown restart signal reason\n");
-				}
+			if(unlikely(gcpu.signals & ARM_SIG_DEBUGMODE)) {
+				Do_Debug();
+			}
+			if(unlikely(gcpu.signals & ARM_SIG_RESTART_IDEC)) {
+				ARM_RestartIdecoder();
 			}
 		}
 	}
@@ -735,7 +797,7 @@ debug_print_instruction(uint32_t icode) {
 			if(debugflags&1) {
 				Instruction *instr;
 				instr=InstructionFind(icode);
-					dbgprintf("Instruction %08x, name %s at %08x\n",icode,instr->name,REG_PC);
+					dbgprintf("Instruction %08x, name %s at %08x\n",icode,instr->name,ARM_NIA);
 					
 			}
 			if(debugflags&2) {
@@ -747,82 +809,21 @@ debug_print_instruction(uint32_t icode) {
 #endif
 }
 
-static uint32_t *circ_buf=NULL;
-static unsigned int circ_wp=0;
-
-#define LOGBUF_ENTRIES (1*1024)
-static inline void 
-circular_log(uint32_t data) {
-	if(unlikely(debugflags & DEBUG_LOG_PC)) {
-		if(circ_buf == NULL) {
-			circ_buf = (uint32_t *)malloc(4*LOGBUF_ENTRIES);
-			if(!circ_buf)  {
-				fprintf(stderr,"Out of memory\n");
-				exit(1);
-			}
-		}
-		circ_buf[circ_wp] = data;	
-		circ_wp = (circ_wp +1) % LOGBUF_ENTRIES;
-	}
-}
-
-#if 0
-static void
-circbuf_dump(int n_entries) {
-	uint32_t index = (circ_wp - n_entries) % LOGBUF_ENTRIES;
-	fprintf(stderr,"dumping PC history\n");
-	do {
-		index = (index +1) % LOGBUF_ENTRIES;
-		fprintf(stderr,"0x%08x\n",circ_buf[index]);
-	} while (index != circ_wp);
-}
-#endif
-
-static inline void 
-log_uint32(uint32_t data) {
-#if 0 
-	circular_log(data);
-	if(data==0xffff0400) {
-		circbuf_dump(128);
-		ARM_Break();
-	}
-
-	static int fd=-1;
-	char *buf = (char*)&data;
-	int count=0;
-	int result;
-	if(unlikely(debugflags & DEBUG_LOG_PC)) {
-		if(fd<0) {
-			fd=open("uint32_logfile",O_RDWR|O_CREAT,0644);
-			if(fd<0) {
-				perror("Can't open logfile");
-				return;
-			}
-		}	
-		while(count!=4) {
-			result=write(fd,buf+count,4-count);
-			if(result>0) { 
-				count+=result;
-			} else {
-				perror("write failed");
-				return;
-			}
-		}
-	}
-#endif
-}
-
 static void
 Thumb_Loop() {
 	ThumbInstructionProc *iproc;
-	fprintf(stderr,"Entering Thumb loop\n");
+	ThumbInstruction *instr;
+	//fprintf(stderr,"Entering Thumb loop\n");
 	setjmp(gcpu.abort_jump);
 	while(1) {
-		ICODE=MMU_IFetch16(REG_PC);
-		REG_PC+=2;
+		CycleCounter+=2;
+		CheckSignals();
+		ICODE=MMU_IFetch16(ARM_NIA);
+		ARM_NIA += 2;
+		instr=ThumbInstruction_Find(ICODE);
+		//fprintf(stderr,"Instruction %08x, name %s at %08x\n",ICODE,instr->name,ARM_NIA);
 		iproc=ThumbInstructionProc_Find(ICODE);
 		iproc();
-		CycleCounter+=2;
 		CycleTimers_Check();
 	}
 }
@@ -838,57 +839,56 @@ ARM9_Loop32() {
 	/* Exceptions use goto (longjmp) */
 	setjmp(gcpu.abort_jump);
 	while(1) {
-		log_uint32(REG_PC);
-		ICODE=MMU_IFetch(REG_PC);
-		REG_PC+=4;
-		iproc=InstructionProcFind(ICODE);
-		debug_print_instruction(ICODE);
-		iproc();
+		CheckSignals();
 		CycleCounter+=6;
+		ICODE=MMU_IFetch(ARM_NIA);
+		ARM_NIA+=4;
+		iproc=InstructionProcFind(ICODE);
+		debug_print_instruction(ICODE);
+		iproc();
 		CycleTimers_Check();
-		CheckSignals();
 
-		log_uint32(REG_PC);
-		ICODE=MMU_IFetch(REG_PC);
-		REG_PC+=4;
+		CheckSignals();
+		ICODE=MMU_IFetch(ARM_NIA);
+		ARM_NIA+=4;
 		iproc=InstructionProcFind(ICODE);
 		debug_print_instruction(ICODE);
 		iproc();
-		CheckSignals();
 
-		log_uint32(REG_PC);
-		ICODE=MMU_IFetch(REG_PC);
-		REG_PC+=4;
+		CheckSignals();
+		ICODE=MMU_IFetch(ARM_NIA);
+		ARM_NIA+=4;
 		iproc=InstructionProcFind(ICODE);
 		debug_print_instruction(ICODE);
 		iproc();
-		CheckSignals();
 	}
 }
 
 void
 ARM9_Run() {
 	uint32_t addr=0;
-	uint32_t gdbwait;
+	uint32_t dbgwait;
 	if(Config_ReadUInt32(&addr,"global","start_address")<0) {
 		addr=0;
 	}
-	if(Config_ReadUInt32(&gdbwait,"global","gdbwait")<0) {
-		gdbwait=0;
+	if(Config_ReadUInt32(&dbgwait,"global","dbgwait")<0) {
+		dbgwait=0;
 	}
-	if(gdbwait) {
-		fprintf(stderr,"CPU is waiting for gdb connection at %08x\n",addr);
-		gcpu.dbg_mode = DBG_MODE_STOPPED;
+	if(dbgwait) {
+		fprintf(stderr,"CPU is waiting for debugger connection at %08x\n",addr);
+		gcpu.dbg_state = DBG_STATE_STOPPED;
+		ARM_SigDebugMode(1);
 	} else {
 		fprintf(stderr,"Starting CPU at %08x\n",addr);
 	}
 	gettimeofday(&gcpu.starttime,NULL);
-	REG_PC=addr;
+	ARM_NIA=addr;
 	/* A long jump to this label redecides which main loop is used  */
-	setjmp(gcpu.restart_loop_jump);
-	gcpu.signals &= ~ARM_SIG_RESTART_LOOP;
+	setjmp(gcpu.restart_idec_jump);
+	gcpu.signals &= ~ARM_SIG_RESTART_IDEC;
+	gcpu.signals_raw &= ~ARM_SIG_RESTART_IDEC;
 	while(1) {
-		if(gcpu.dbg_mode == DBG_MODE_STOPPED) {
+		if(unlikely(gcpu.dbg_state == DBG_STATE_STOPPED)) {
 			struct timespec tout;
                         tout.tv_nsec=0;
                         tout.tv_sec=10000;
