@@ -6,10 +6,10 @@
 #include <idecode.h>
 #include <xy_tree.h>
 #include <sys/time.h>
-#include <gdebug.h>
+#include <debugger.h>
 #include "mainloop_events.h"
 #include "signode.h"
-
+#include "cycletimer.h"
 /*
  * ------------------------------------------------------
  * ARM9_RegPointerSet
@@ -52,16 +52,18 @@ extern uint32_t do_alignment_check;
 #define MODE_SYSTEM	(0x1F)
 #define MODE_MASK	(0x1f)
 
-#define EX_RESET	(MODE_SVC | 0)
-#define EX_UNDEFINED	(MODE_UNDEFINED | (0x00000004<<8))
-#define EX_SWI		(MODE_SVC | (0x00000008<<8))
-#define EX_PABT		(MODE_ABORT|(0x0000000c<<8))
-#define EX_DABT		(MODE_ABORT|(0x00000010<<8))
-#define EX_IRQ		(MODE_IRQ | (0x00000018<<8))
-#define EX_FIQ		(MODE_FIQ | (0x0000001c<<8))
+typedef enum ARM_ExceptionID {
+	EX_RESET =	(MODE_SVC | 0),
+	EX_UNDEFINED =	(MODE_UNDEFINED | (0x00000004<<8)),
+	EX_SWI =	(MODE_SVC | (0x00000008<<8)),
+	EX_PABT =	(MODE_ABORT|(0x0000000c<<8)),
+	EX_DABT =	(MODE_ABORT|(0x00000010<<8)),
+	EX_IRQ =	(MODE_IRQ | (0x00000018<<8)),
+	EX_FIQ =	(MODE_FIQ | (0x0000001c<<8)),
+} ARM_ExceptionID;
 
 #define EX_TO_MODE(ex) ((ex)&0x1f)
-#define EX_TO_PC(ex) ((ex)>>8)
+#define EX_TO_PC(ex)   ((ex)>>8)
 
 #define COND_EQ		(0)
 #define COND_NE 	(1)
@@ -91,6 +93,10 @@ extern uint32_t do_alignment_check;
 #define FLAG_I		(1UL<<7)
 
 typedef struct ARM9 {
+	/* Scratch registers for avoiding arguments to functions  */
+	uint32_t icode;
+	uint32_t am_scratch1, am_scratch2,am_scratch3;
+
 	uint32_t registers[17];
  	uint32_t reg_cpsr;
 	uint32_t reg_bank; /* duplicate of lower 5 Bits of cpsr for fast access */
@@ -114,10 +120,6 @@ typedef struct ARM9 {
 
 	ARM9_RegPointerSet regSet[32];
 
-	/* Scratch registers for avoiding arguments to functions  */
-	uint32_t am_scratch1, am_scratch2,am_scratch3;
-	uint32_t icode;
-
 	/* 
          * --------------------------------------------------------------
          * signaling_mode:
@@ -140,28 +142,36 @@ typedef struct ARM9 {
 	/* Timer Handlers and statistics */
 	struct timeval starttime;
 	jmp_buf abort_jump;
-	jmp_buf restart_loop_jump;
+	jmp_buf restart_idec_jump;
 	
 	/* The GDB Operations */
-	int dbg_mode;
+	int dbg_state;
 	int dbg_steps;
-	GdbServer *gserv;
-	GDebugOps gdbops;
+	Debugger *debugger;
+	DebugBackendOps dbgops;
+
+	/* Throttling cpu to real speed */
+        struct timespec tv_last_throttle;
+        CycleCounter_t last_throttle_cycles;
+        CycleTimer throttle_timer;
+        int64_t cycles_ahead; /* Number of cycles ahead of real cpu */
+
 } ARM9;
 
-#define DBG_MODE_OFF     	(0)
-#define DBG_MODE_STOP		(1)
-#define DBG_MODE_STOPPED	(2)
-#define DBG_MODE_STEP		(3)
-#define DBG_MODE_BREAK		(4)
+#define DBG_STATE_RUNNING     	(0)
+#define DBG_STATE_STOP		(1)
+#define DBG_STATE_STOPPED	(2)
+#define DBG_STATE_STEP		(3)
+#define DBG_STATE_BREAK		(4)
 extern ARM9 gcpu;
 
 /*
  * Bit in field cpu_signals
  */
-#define ARM_SIG_IRQ		(1)  /* Normal Interrupt */
+#define ARM_SIG_IRQ		(1<<0)  /* Normal Interrupt */
 #define ARM_SIG_FIQ		(1<<1)  /* Fast Interrupt */
-#define ARM_SIG_RESTART_LOOP	(1<<2)  /* Something changed in CPU or debugmode */
+#define ARM_SIG_RESTART_IDEC	(1<<2)  /* Something changed in CPU or debugmode */
+#define ARM_SIG_DEBUGMODE	(1<<3)
 #define CPU_REGS (gcpu.regSet)
 
 void ARM_set_reg_cpsr(uint32_t val); 
@@ -169,10 +179,18 @@ void ARM_set_reg_cpsr(uint32_t val);
 #define PC_OFFSET (4)
 #define THUMB_PC_OFFSET (2)
 #define REG_LR	 ((gcpu.registers[14]))
-#define REG_PC  	(gcpu.registers[15])
-#define GET_REG_PC  		(gcpu.registers[15]+PC_OFFSET)
-#define GET_REG_PC_THUMB 	(gcpu.registers[15]+THUMB_PC_OFFSET)
-#define SET_REG_PC(val)	({gcpu.registers[15]=(val);})
+/*
+ **********************************************************
+ * NIA is power PC style: Next instruction address
+ **********************************************************
+ */
+#define ARM_NIA  		(gcpu.registers[15])
+#define ARM_GET_CIA  		(gcpu.registers[15] - PC_OFFSET)
+#define ARM_GET_NNIA  		(gcpu.registers[15] + PC_OFFSET)
+#define THUMB_NIA 		(gcpu.registers[15])
+#define THUMB_GET_NNIA 		(gcpu.registers[15] + THUMB_PC_OFFSET)
+
+#define ARM_SET_NIA(val)	({gcpu.registers[15]=(val);})
 #define REG_CPSR      (gcpu.reg_cpsr)
 
 #define SET_REG_CPSR(val) ARM_set_reg_cpsr(val); 
@@ -209,8 +227,12 @@ Thumb_ReadReg(int nr) {
 }
 
 static inline uint32_t
-Thumb_ReadLowReg(int nr) {
-	return gcpu.registers[nr];
+Thumb_ReadHighReg(int nr) {
+        if(likely(nr!=15)) {
+                return gcpu.registers[nr];
+        } else {
+                return gcpu.registers[15]+THUMB_PC_OFFSET;
+        }
 }
 
 static inline void 
@@ -246,7 +268,8 @@ ARM9_WriteReg(uint32_t val,int nr) {
  * ------------------------------------------------------
  * Register access for instructions which don't use
  * the current register set but explicitely specify
- * the bank
+ * the bank.
+ * Do not call 
  * ------------------------------------------------------
  */
 static inline uint32_t
@@ -264,6 +287,7 @@ ARM9_ReadRegBank(int nr,int bank)
 		}
 	}
 }
+
 static inline void
 ARM9_WriteRegBank(uint32_t val,int nr,int bank) 
 {
@@ -310,8 +334,20 @@ ARM_UnPostFiq() {
 
 
 static inline void 
-ARM_PostRestartLoop() {
-	gcpu.signals_raw |= ARM_SIG_RESTART_LOOP;
+ARM_PostRestartIdecoder() {
+	gcpu.signals_raw |= ARM_SIG_RESTART_IDEC;
+        gcpu.signals = gcpu.signals_raw & gcpu.signal_mask;
+	if(gcpu.signals)
+		mainloop_event_pending = 1;
+}
+
+static inline void 
+ARM_SigDebugMode(value) {
+	if(value) {
+		gcpu.signals_raw |= ARM_SIG_DEBUGMODE;
+	} else {
+		gcpu.signals_raw &= ~ARM_SIG_DEBUGMODE;
+	}
         gcpu.signals = gcpu.signals_raw & gcpu.signal_mask;
 	if(gcpu.signals)
 		mainloop_event_pending = 1;
@@ -319,15 +355,15 @@ ARM_PostRestartLoop() {
 
 static inline void 
 ARM_Break() {
-	gcpu.dbg_mode = DBG_MODE_BREAK;
-	ARM_PostRestartLoop(); 
+	gcpu.dbg_state = DBG_STATE_BREAK;
+	ARM_PostRestartIdecoder(); 
 }
 
-void ARM9_Exception(int exception,int pc_offset);
+void ARM_Exception(ARM_ExceptionID exception,int nia_offset);
 
 static inline void
-ARM_RestartLoop() {
-	longjmp(gcpu.restart_loop_jump,1);
+ARM_RestartIdecoder() {
+	longjmp(gcpu.restart_idec_jump,1);
 }
 
 #endif
