@@ -26,10 +26,11 @@
 #include <stdint.h>
 #include <string.h>
 #include <unistd.h>
-#include <bus.h>
-#include <signode.h>
-#include <configfile.h>
-#include <at91_aic.h>
+#include "bus.h"
+#include "signode.h"
+#include "configfile.h"
+#include "at91_aic.h"
+#include "sgstring.h"
 
 #define AIC_SMR(base,n) ((base)+0x04*(n))
 #define		SMR_SRCTYPE_MASK	(3<<5)
@@ -183,7 +184,7 @@ update_interrupts(AT91Aic *aic)
 	update_ipr(aic);
 	if(aic->ipr & aic->imr) { /* & ~1 for fiq ? */
 		irq = highest_priority_irq(aic,&maxlevel);
-		if((maxlevel > aic->curplvl) && (!aic->interrupt_posted)) {
+		if((maxlevel > aic->curplvl)) {
 			interrupt = 1;
 		}
 	}
@@ -234,7 +235,7 @@ srctype_translate(uint32_t srctype,int irq)
  * v0
  * -----------------------------------------------------------
  */
-static int
+static void 
 int_source_change(SigNode *node,int value,void *clientData)
 {
         IrqTraceInfo *ti = (IrqTraceInfo *) clientData;
@@ -270,7 +271,7 @@ int_source_change(SigNode *node,int value,void *clientData)
 		}
         }
         update_interrupts(aic);
-        return 0;
+        return;
 }
 
 /*
@@ -365,6 +366,8 @@ ivr_read(void *clientData,uint32_t address,int rqlen)
 		fprintf(stderr,"IVR read with no new irq\n");
 		return aic->svr[irq];
 	}
+
+	/* If read from a debugger do not modify the state of the Interruptcontroller */
 	if(!(aic->dcr & DCR_PROT)) {
 		push_current_on_stack(aic);
 		aic->curplvl = aic->smr[irq] & SMR_PRIOR_MASK;	
@@ -732,13 +735,8 @@ AT91Aic_UnMap(void *owner,uint32_t base,uint32_t mask)
 BusDevice *
 AT91Aic_New(const char *name)
 {
-        AT91Aic *aic = malloc(sizeof(AT91Aic));
+        AT91Aic *aic = sg_new(AT91Aic);
 	int i;
-        if(!aic) {
-                fprintf(stderr,"Out of memory for AT91 Usart\n");
-                exit(1);
-        }
-        memset(aic,0,sizeof(AT91Aic));
         aic->irqOut = SigNode_New("%s.irq",name);
         aic->fiqOut = SigNode_New("%s.fiq",name);
         if(!aic->irqOut || !aic->fiqOut) {
@@ -749,12 +747,9 @@ AT91Aic_New(const char *name)
 	SigNode_Set(aic->fiqOut,SIG_HIGH);
 	aic->interrupt_posted = 0;
 	aic->curplvl = -1;
+	aic->stackptr = 0;
 	for(i=0;i<32;i++) {
-		IrqTraceInfo *ti = malloc(sizeof(IrqTraceInfo));
-                if(!ti) {
-                        fprintf(stderr,"Out of memory\n");
-                        exit(1);
-                }
+		IrqTraceInfo *ti = sg_new(IrqTraceInfo);
 		ti->irqnr = i;
 		ti->aic = aic;
 		aic->traceInfo[i]  = ti;
