@@ -29,19 +29,18 @@
 #include <string.h>
 #include <stdint.h>
 #include <errno.h>
-#include <linux-tap.h>
-#include <ns9750_eth.h>
-#include <ns9750_timer.h>
-#include <fio.h>
-#include <bus.h>
-#include <phy.h>
-#include <signode.h>
-#include <cycletimer.h>
+#include "linux-tap.h"
+#include "fio.h"
+#include "bus.h"
+#include "phy.h"
+#include "signode.h"
+#include "cycletimer.h"
+#include "sgstring.h"
 
 #if 0
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 
 
@@ -384,7 +383,7 @@ input_event(void *cd,int mask)
 static void
 enable_receiver(AT91Emac *emac) {
         if(!emac->receiver_is_enabled && (emac->ether_fd >=0)) {
-                dprintf("AT91Emac: enable receiver\n");
+                dbgprintf("AT91Emac: enable receiver\n");
                 //fprintf(stderr,"AT91Emac: enable receiver\n");
                 FIO_AddFileHandler(&emac->input_fh,emac->ether_fd,FIO_READABLE,input_event,emac);
                 emac->receiver_is_enabled=1;
@@ -395,7 +394,7 @@ static void
 disable_receiver(AT91Emac *emac)
 {
         if(emac->receiver_is_enabled) {
-        	dprintf("AT91Emac: disable receiver\n");
+        	dbgprintf("AT91Emac: disable receiver\n");
         	//fprintf(stderr,"AT91Emac: disable receiver\n");
                 FIO_RemoveFileHandler(&emac->input_fh);
                 emac->receiver_is_enabled=0;
@@ -1222,12 +1221,7 @@ AT91Emac_RegisterPhy(BusDevice *dev,PHY_Device *phy,unsigned int phy_addr)
 BusDevice *
 AT91Emac_New(const char *name) 
 {
-	AT91Emac *emac = malloc(sizeof(AT91Emac));
-	if(!emac) {
-		fprintf(stderr,"out of memory allocating AT91Emac\n");
-		exit(1);
-	}
-	memset(emac,0,sizeof(*emac));
+	AT91Emac *emac = sg_new(AT91Emac);
 	emac->ether_fd = Net_CreateInterface(name);
         fcntl(emac->ether_fd,F_SETFL,O_NONBLOCK);
 	emac->irqNode = SigNode_New("%s.irq",name);
