@@ -27,11 +27,12 @@
 #include <stdint.h>
 #include <string.h>
 #include <unistd.h>
-#include <bus.h>
-#include <fio.h>
-#include <signode.h>
-#include <configfile.h>
-#include <at91_pio.h>
+#include "bus.h"
+#include "fio.h"
+#include "signode.h"
+#include "configfile.h"
+#include "at91_pio.h"
+#include "sgstring.h"
 
 #define	PIO_PER(base)	((base) + 0x00)
 #define PIO_PDR(base)	((base) + 0x04)
@@ -77,6 +78,7 @@ typedef struct Edge_TraceInfo {
 
 struct AT91Pio {
 	BusDevice bdev;
+	char *name;
 	uint32_t psr;
 	uint32_t osr;
 	uint32_t ifsr;
@@ -129,10 +131,13 @@ update_interrupt(AT91Pio *pio)
  * ----------------------------------------------------------------------
  */
 static void
-psr_update(AT91Pio *pio) 
+psr_update(AT91Pio *pio,uint32_t diff) 
 {
 	int i;
 	for(i=0;i<32;i++) {
+		if(((diff >> i) & 1) == 0) {
+			continue;
+		}
 		/* First Link data then the output enable to guarantee stability */
 		if(pio->psr & (1<<i)) {
 			SigNode_RemoveLink(pio->poe[i],pio->noea[i]);
@@ -158,8 +163,10 @@ static void
 per_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 {
 	AT91Pio *pio = (AT91Pio *) clientData;
-	pio->psr |= value;	
-	psr_update(pio);
+	uint32_t newvalue = pio->psr | value;
+	uint32_t diff = pio->psr ^ newvalue;
+	pio->psr = newvalue;	
+	psr_update(pio,diff);
 }
 
 static uint32_t
@@ -173,8 +180,10 @@ static void
 pdr_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 {
 	AT91Pio *pio = (AT91Pio *) clientData;
-	pio->psr &= ~value;	
-	psr_update(pio);
+	uint32_t newvalue = pio->psr & ~value;
+	uint32_t diff = pio->psr ^ newvalue;
+	pio->psr = newvalue;	
+	psr_update(pio,diff);
 }
 
 static uint32_t
@@ -368,7 +377,7 @@ odsr_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 	}
 }
 
-static int 
+static void 
 pdsr_update(SigNode *node,int value,void *clientData)
 {
 	Edge_TraceInfo *ti = (Edge_TraceInfo *) clientData;
@@ -381,7 +390,6 @@ pdsr_update(SigNode *node,int value,void *clientData)
 	}
 	pio->isr |= (1<<nr);
 	update_interrupt(pio);
-	return 0;
 }
 
 
@@ -461,7 +469,7 @@ imr_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
  * This is the or gate: It is invoked by change of noeb and noea
  */
 
-static int 
+static void 
 noe_update(SigNode *node,int value,void *clientData) 
 {
 	OR_TraceInfo *ti = (OR_TraceInfo *) clientData;
@@ -475,17 +483,19 @@ noe_update(SigNode *node,int value,void *clientData)
 			SigNode_Link(pio->dout[nr],pio->pad[nr]);
 		}
 	}
-	return 0;
 }
 /*
  * The mdsr 
  */
 
 static void 
-mdsr_update(AT91Pio *pio) 
+mdsr_update(AT91Pio *pio,uint32_t diff) 
 {
 	int i;
 	for(i=0;i<32;i++) {
+		if(((diff >> i) & 1) == 0) {
+			continue;
+		}
 		/* First connect data, then noeb, to guarantee stable value */ 
 		if(pio->mdsr & (1<<i)) {
 			SigNode_RemoveLink(pio->gnd,pio->noeb[i]);
@@ -512,8 +522,10 @@ static void
 mder_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 {
 	AT91Pio *pio  = (AT91Pio *) clientData;
-	pio->mdsr |= value;
-	mdsr_update(pio);
+	uint32_t newvalue = pio->mdsr | value;
+	uint32_t diff = pio->mdsr ^ newvalue;
+	pio->mdsr = newvalue;
+	mdsr_update(pio,diff);
 }
 
 static uint32_t
@@ -527,8 +539,10 @@ static void
 mddr_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 {
 	AT91Pio *pio  = (AT91Pio *) clientData;
-	pio->mdsr &= ~value;
-	mdsr_update(pio);
+	uint32_t newvalue = pio->mdsr &~ value;
+	uint32_t diff = newvalue ^ pio->mdsr;
+	pio->mdsr = newvalue;
+	mdsr_update(pio,diff);
 }
 
 static uint32_t
@@ -600,10 +614,13 @@ pusr_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
         fprintf(stderr,"AT91Pio: write location 0x%08x not implemented\n",address);
 }
 static void
-absr_update(AT91Pio *pio) 
+absr_update(AT91Pio *pio,uint32_t diff) 
 {
 	int i;
 	for(i=0;i<32;i++) {
+		if(((diff >> i) & 1) == 0) {
+			continue;
+		}
 		if(pio->absr & (1<<i)) {
 			SigNode_RemoveLink(pio->paoe[i],pio->poe[i]);
 			SigNode_RemoveLink(pio->pao[i],pio->po[i]);
@@ -628,8 +645,10 @@ static void
 asr_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 {
 	AT91Pio *pio = (AT91Pio *) clientData;
-	pio->absr &= ~value;
-	absr_update(pio);
+	uint32_t newvalue = pio->absr & ~value;
+	uint32_t diff = pio->absr ^ newvalue;
+	pio->absr = newvalue;
+	absr_update(pio,diff);
 }
 
 static uint32_t
@@ -643,8 +662,10 @@ static void
 bsr_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 {
 	AT91Pio *pio = (AT91Pio *) clientData;
-	pio->absr |= value;
-	absr_update(pio);
+	uint32_t newvalue = pio->absr | value;
+	uint32_t diff = pio->absr ^ newvalue;
+	pio->absr = newvalue;
+	absr_update(pio,diff);
 }
 
 static uint32_t
@@ -780,12 +801,13 @@ BusDevice *
 AT91Pio_New(const char *name)
 {
 	int i;
-	AT91Pio *pio = (AT91Pio *) malloc(sizeof(*pio));
-	if(!pio) {
-		fprintf(stderr,"Out of memory allocating AT91Pio module\n");
+	struct timeval tv;
+	AT91Pio *pio = sg_new(AT91Pio);
+	pio->name = sg_strdup(name);
+	if(!pio->name) {
+		fprintf(stderr,"Out of memory allocating AT91Pio name\n");
 		exit(1);
 	}
-	memset(pio,0,sizeof(*pio));
 	pio->irqNode = SigNode_New("%s.irq",name);
 	pio->gnd = SigNode_New("%s.gnd",name);
 	if(!pio->irqNode || !pio->gnd) {
@@ -836,6 +858,47 @@ AT91Pio_New(const char *name)
 		SigNode_Set(pio->pullup[i],SIG_PULLUP);
 	}
 	SigNode_Set(pio->irqNode,SIG_PULLDOWN);
+
+	/* 
+	 * All registers have initial value 0 
+	 * So call all disable register writes
+	 */
+	per_write(pio,0xffffffff,0,4);
+	pdr_write(pio,0xffffffff,0,4);
+
+	oer_write(pio,0xffffffff,0,4);
+	odr_write(pio,0xffffffff,0,4);
+
+	ifer_write(pio,0xffffffff,0,4);
+	ifdr_write(pio,0xffffffff,0,4);
+
+	sodr_write(pio,0xffffffff,0,4);
+	codr_write(pio,0xffffffff,0,4);
+
+	idr_write(pio,0xffffffff,0,4);
+
+	mder_write(pio,0xffffffff,0,4);
+	mddr_write(pio,0xffffffff,0,4);
+
+	puer_write(pio,0xffffffff,0,4);
+	pudr_write(pio,0xffffffff,0,4);
+
+	bsr_write(pio,0xffffffff,0,4);
+	asr_write(pio,0xffffffff,0,4);
+
+	owdr_write(pio,0xffffffff,0,4);
+	/* Lets have the Peripheral A and B random values */
+        gettimeofday(&tv,NULL);
+#if 0
+        srand48(tv.tv_usec + (tv.tv_sec<<20));
+	for(i=0;i<32;i++) {
+		if(lrand48() > 1<<30) {
+			SigNode_Set(pio->pao[i],SIG_LOW);
+		} else  {
+			SigNode_Set(pio->pao[i],SIG_HIGH);
+		}
+	}
+#endif
 	pio->bdev.first_mapping=NULL;
         pio->bdev.Map=AT91Pio_Map;
         pio->bdev.UnMap=AT91Pio_UnMap;
