@@ -33,6 +33,7 @@
 #include "cycletimer.h"
 #include "clock.h"
 #include "at91_pmc.h"
+#include "sgstring.h"
 
 #define PMC_SCER(base)		((base)+0x00)
 #define PMC_SCDR(base)		((base)+0x04)
@@ -678,12 +679,7 @@ AT91Pmc_UnMap(void *owner,uint32_t base,uint32_t mask)
 BusDevice *
 AT91Pmc_New(const char *name)
 {
-	AT91Pmc *pmc = malloc(sizeof(*pmc));	
-	if(!pmc) {
-		fprintf(stderr,"Out of memory allocating AT91Pmc\n");
-		exit(1);
-	}
-	memset(pmc,0,sizeof(*pmc));	
+	AT91Pmc *pmc = sg_new(AT91Pmc);
 	pmc->irqNode = SigNode_New("%s.irq",name);
 	if(!pmc->irqNode) {
 		fprintf(stderr,"Failed to create irq line for AT91 PMC\n");
