@@ -33,7 +33,6 @@
 #include <fio.h>
 #include <bus.h>
 #include <configfile.h>
-#include <debugserver.h>
 #include <loader.h>
 #include <i2c_serdes.h>
 #include <m24cxx.h>
@@ -42,9 +41,11 @@
 #include <sram.h>
 #include <pc16550.h>
 #include <pl190_irq.h>
+#include "clock.h"
 #include <lpc2106_timer.h>
 #include <lpc2106_gpio.h>
 #include <lpc2106_scb.h>
+#include "devices/lcd_hd44780/lcd_hd44780.h"
 
 /*
  * ---------------------------------------------------------------
@@ -54,7 +55,7 @@
  * ---------------------------------------------------------------
  */
 static void
-create_signal_links()
+create_signal_links(void)
 {
 	SigName_Link("arm.irq","vic.irq");
 	SigName_Link("arm.fiq","vic.fiq");
@@ -62,12 +63,42 @@ create_signal_links()
 	SigName_Link("timer1.irq","vic.nVICINTSOURCE5");
 	SigName_Link("uart0.irq","vic.nVICINTSOURCE6");
 	SigName_Link("uart1.irq","vic.nVICINTSOURCE7");
+	/* Connect the LCD */
+	SigName_Link("gpio.P0.4","lcd0.D4");
+	SigName_Link("gpio.P0.5","lcd0.D5");
+	SigName_Link("gpio.P0.6","lcd0.D6");
+	SigName_Link("gpio.P0.7","lcd0.D7");
+
+	SigName_Link("gpio.P0.8","lcd0.RS");
+	SigName_Link("gpio.P0.9","lcd0.RW");
+	SigName_Link("gpio.P0.10","lcd0.E");
+	
+
 }
+
+static void
+create_clock_links(void)
+{
+	Clock_Link("arm.clk","scb.cclk");
+	Clock_Link("uart0.clk","scb.pclk");
+	Clock_Link("uart1.clk","scb.pclk");
+	Clock_Link("timer0.pclk","scb.pclk");
+	Clock_Link("timer1.pclk","scb.pclk");
+}
+
 static int
-board_armee_create()
+board_armee_create(void)
 {
         ArmCoprocessor *copro;
 	BusDevice *dev;
+	FbDisplay *display = NULL;
+        Keyboard *keyboard = NULL;
+
+	FbDisplay_New("lcd0",&display,&keyboard,NULL,NULL);
+        if(!display) {
+                fprintf(stderr,"LCD creation failed\n");
+		exit(1);
+        }
 
         Bus_Init(MMU_InvalidateTlb,4*1024);
         ARM9_New();
@@ -90,17 +121,20 @@ board_armee_create()
 	Mem_AreaAddMapping(dev,0xE0008000,0x20,MEM_FLAG_WRITABLE | MEM_FLAG_READABLE);
 
 	dev = PC16550_New("uart0",2);
-	Mem_AreaAddMapping(dev,0xE000C000,0x20,MEM_FLAG_WRITABLE | MEM_FLAG_READABLE);
+	Mem_AreaAddMapping(dev,0xE000C000,0x40,MEM_FLAG_WRITABLE | MEM_FLAG_READABLE);
 
 	dev = PC16550_New("uart1",2);
-	Mem_AreaAddMapping(dev,0xE0010000,0x20,MEM_FLAG_WRITABLE | MEM_FLAG_READABLE);
+	Mem_AreaAddMapping(dev,0xE0010000,0x40,MEM_FLAG_WRITABLE | MEM_FLAG_READABLE);
 
-	LPC2106_GpioNew("gpio");
+	LPC2106_GpioNew("gpio.P0");
 
 	dev = LPC2106_ScbNew("scb");
 	Mem_AreaAddMapping(dev,0xE01FC000,0x200,MEM_FLAG_WRITABLE | MEM_FLAG_READABLE);
 
+	HD44780_LcdNew("lcd0", display);
+
 	create_signal_links();
+	create_clock_links();
 	return 0;
 }
 
@@ -112,9 +146,21 @@ board_armee_run(Board *bd) {
 #define DEFAULTCONFIG \
 "[global]\n" \
 "start_address: 0\n"\
+"cpu_clock: 58982400\n"\
+"oscillator: 14745600\n" \
+"\n" \
 "[iram]\n" \
 "size: 64k\n" \
 "\n" \
+"[lcd0]\n" \
+"backend: rfbserver\n" \
+"host: 127.0.0.1\n" \
+"port: 5901\n" \
+"width: 310\n" \
+"height: 75\n" \
+"start: vncviewer localhost:5901\n" \
+"exit_on_close: 1\n" \
+"\n" \
 "[loader]\n" \
 "load_address: 0x0\n"\
 "\n" \
