@@ -33,7 +33,9 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <fcntl.h>
-#include <bus.h>
+#include "sgstring.h"
+#include "bus.h"
+#include "loader.h"
 
 Bus *MainBus;
 /*
@@ -349,14 +351,7 @@ Mem_Map2LvlBlock(uint32_t addr,uint8_t *host_mem,uint32_t flags)
 	if(flags & MEM_FLAG_READABLE) {
 		slvl_map_r = twoLevelMMap.flvlmap_read[index];
 		if(unlikely(!slvl_map_r)) {
-			slvl_map_r = twoLevelMMap.flvlmap_read[index]=malloc(sizeof(uint8_t*)*twoLevelMMap.scnd_lvl_sz);
-			if(unlikely(!slvl_map_r)) {
-				fprintf(stderr,"Out of memory in Map2LvlBlock\n");
-				exit(4324);
-			} else {
-				memset(slvl_map_r,0,sizeof(uint8_t*)*twoLevelMMap.scnd_lvl_sz);
-				/* assert  use count of 0 */
-			}
+			slvl_map_r = twoLevelMMap.flvlmap_read[index]=sg_calloc(sizeof(uint8_t*)*twoLevelMMap.scnd_lvl_sz);
 		}  
 		twoLevelMMap.flvl_map_read_use_count[index]++;
                	slvl_map_r[sl_index]=host_mem;
@@ -364,13 +359,7 @@ Mem_Map2LvlBlock(uint32_t addr,uint8_t *host_mem,uint32_t flags)
 	if(flags & MEM_FLAG_WRITABLE) {
 		slvl_map_w = twoLevelMMap.flvlmap_write[index];
 		if(unlikely(!slvl_map_w)) {
-			slvl_map_w = twoLevelMMap.flvlmap_write[index]=malloc(sizeof(uint8_t*)*twoLevelMMap.scnd_lvl_sz);
-			if(unlikely(!slvl_map_w)) {
-				fprintf(stderr,"Out of memory in Map2LvlBlock\n");
-				exit(4329);
-			} else {
-				memset(slvl_map_w,0,sizeof(uint8_t*)*twoLevelMMap.scnd_lvl_sz);
-			}
+			slvl_map_w = twoLevelMMap.flvlmap_write[index]=sg_calloc(sizeof(uint8_t*)*twoLevelMMap.scnd_lvl_sz);
 		}  
 		twoLevelMMap.flvl_map_write_use_count[index]++;
         	slvl_map_w[sl_index]=host_mem;
@@ -417,11 +406,7 @@ Mem_UnMap2LvlBlock(uint32_t addr)
 void 
 Mem_AreaAddMapping(BusDevice *bdev,uint32_t base,uint32_t mapsize,uint32_t flags) 
 {
-	MemMapping *mapping=malloc(sizeof(MemMapping));
-	if(!mapping) {
-		fprintf(stderr,"Out of memory allocating MemMaping\n");
-		exit(837);
-	}
+	MemMapping *mapping=sg_new(MemMapping);
 	mapping->next=bdev->first_mapping;
 	bdev->first_mapping=mapping;
 	mapping->base=base;
@@ -712,6 +697,7 @@ IOH_Find(uint32_t address) {
 		index = (address & IOH_SLVL_MASK) >> IOH_SLVL_SHIFT;		
 		return slvl_map[index];
 	} 
+	//fprintf(stderr,"JK: Addr %08x no IOH slevel %p, index %d\n",address,slvl_map,index);
 	return NULL;
 }
 
@@ -750,12 +736,7 @@ IOH_New(uint32_t cpu_addr,IOReadProc *readproc,IOWriteProc *writeproc,void *clie
 		fprintf(stderr,"Bug, more than one IO-Handler for address %08x\n",cpu_addr);
 		exit(4324);
 	}
-	h=malloc(sizeof(IOHandler));
-	if(!h) {
-		perror("Out of memory allocating IOHandler\n");	
-		exit(97);
-	}
-	memset(h,0,sizeof(IOHandler));
+	h=sg_new(IOHandler);
 	h->cpu_addr=cpu_addr;
 	h->clientData=clientData;
 	h->readproc=readproc;
@@ -806,12 +787,7 @@ static inline IOHandler *
 allocate_iohandler(uint32_t addr,IOReadProc *readproc,IOWriteProc *writeproc,int swap_endian,void *clientData) 
 {
 	IOHandler *h;
-	h=malloc(sizeof(IOHandler));
-	if(!h) {
-		perror("Out of memory allocating IOHandler\n");	
-		exit(97);
-	}
-	memset(h,0,sizeof(IOHandler));
+	h=sg_new(IOHandler);
 	h->cpu_addr=addr;
 	h->clientData=clientData;
 	h->readproc=readproc;
@@ -839,18 +815,13 @@ IOH_Map2LvlBlock(uint32_t addr,IOHandler *h)
 	int sl_index; 
 	sl_map = iohandlerFlvlMap[fl_index]; 
 	if(!sl_map) {
-		sl_map = iohandlerFlvlMap[fl_index] = malloc(sizeof(uint8_t*)*IOH_SLVL_SZ);
-		if(!sl_map) {
-			fprintf(stderr,"Out of memory in IOH_Map2LvlBlock\n");
-			exit(4724);
-		} else {
-			memset(sl_map,0,sizeof(uint8_t*)*IOH_SLVL_SZ);
-			ioh_flvl_use_count[fl_index]++;
-		}
+		//fprintf(stderr,"JK: This is a new FLEVEL\n");
+		sl_map = iohandlerFlvlMap[fl_index] = sg_calloc(sizeof(uint8_t*)*IOH_SLVL_SZ);
 	} 
 	sl_index = (addr & IOH_SLVL_MASK)>>IOH_SLVL_SHIFT;
 	if(!sl_map[sl_index]) {
 		sl_map[sl_index] = h;
+		ioh_flvl_use_count[fl_index]++;
 	} else {
 		fprintf(stderr,"There is already a handler for IO-Region 0x%08x\n",addr);
 		exit(3246);
@@ -937,7 +908,7 @@ IOH_UnMap2LvlBlock(uint32_t addr) {
 void
 IOH_DeleteRegion(uint32_t addr,uint32_t length) 
 {
-	//fprintf(stderr,"Delete Region %08x\n",addr);
+	// fprintf(stderr,"JK Delete Region %08x size 0x%08x\n",addr,length);
 	while(length) {
 		if(!(addr&IOH_MAP_BLOCKMASK) && (length >= IOH_MAP_BLOCKSIZE)) {
 			IOH_UnMapBlock(addr);
@@ -1198,17 +1169,6 @@ IO_Read8(uint32_t addr)
 	return h->readproc(h->clientData,addr,1);
 }
 
-static void *
-ckalloc(size_t size) 
-{
-	void *mem=malloc(size);
-	if(!mem) {
-		fprintf(stderr,"Out of memory\n");
-		exit(423);
-	}
-	memset(mem,0,size);
-	return mem;
-}
 
 static struct Bus mainBus = {
 	.read32 = Bus_Read32,
@@ -1220,6 +1180,43 @@ static struct Bus mainBus = {
 	.write8 = Bus_Write8,
 	.writeblock = Bus_Write
 };
+
+/*
+ * -----------------------------------------------------
+ * The interface to the loader
+ * -----------------------------------------------------
+ */
+static int
+load_to_bus(void *clientData,uint32_t addr,uint8_t *buf,unsigned int count,int flags)
+{
+        while(count) {
+                /* This is wrong (outdated) because of smaller mappings */
+                uint32_t len=MEM_MAP_BLOCKSIZE-(addr&MEM_MAP_BLOCKMASK);
+                uint32_t offs = addr&(MEM_MAP_BLOCKMASK);
+                uint8_t *host_mem = Bus_GetHVAWrite(addr & ~MEM_MAP_BLOCKMASK);
+                if(!host_mem) {
+			host_mem = Bus_GetHVARead(addr &  ~MEM_MAP_BLOCKMASK);
+                }
+                if(!host_mem) {
+                        fprintf(stderr,"Bus: Cannot load record to memory at 0x%08x\n",addr);
+                        return -1;
+                }
+                if(len>count)
+                        len=count;
+                if(flags & LOADER_FLAG_SWAP32) {
+                        int i;
+                        for(i=0;i<len;i++) {
+                                uint8_t *dst = host_mem+((offs+i)^3);
+                                *dst = buf[i];
+                        }
+                } else  {
+                        memcpy(host_mem+offs,buf,len);
+                }
+                count-=len;
+        }
+        return 0;
+}
+
 /*
  * --------------------------------------------------------------
  * Create and Initialize memory and IO-Handler maps and Hashes
@@ -1246,17 +1243,18 @@ Bus_Init(InvalidateCallback *invalidate,uint32_t min_memblocksize)
 	twoLevelMMap.scnd_lvl_mask =  (~0 << i) & ~twoLevelMMap.frst_lvl_mask; 
 	twoLevelMMap.scnd_lvl_sz = (1<<(twoLevelMMap.frst_lvl_shift-twoLevelMMap.scnd_lvl_shift));
 
-	iohandlerHash=ckalloc(sizeof(IOHandler*)*IOH_HASH_SIZE);
-	mem_map_read=ckalloc(sizeof(uint8_t*)*MEM_MAP_ENTRIES);
-	mem_map_write=ckalloc(sizeof(uint8_t*)*MEM_MAP_ENTRIES);
+	iohandlerHash=sg_calloc(sizeof(IOHandler*)*IOH_HASH_SIZE);
+	mem_map_read=sg_calloc(sizeof(uint8_t*)*MEM_MAP_ENTRIES);
+	mem_map_write=sg_calloc(sizeof(uint8_t*)*MEM_MAP_ENTRIES);
 
-	twoLevelMMap.flvlmap_read=ckalloc(sizeof(uint8_t*)*twoLevelMMap.frst_lvl_sz);
-	twoLevelMMap.flvlmap_write=ckalloc(sizeof(uint8_t*)*twoLevelMMap.frst_lvl_sz);
-	twoLevelMMap.flvl_map_read_use_count=ckalloc(sizeof(*twoLevelMMap.flvl_map_read_use_count)*twoLevelMMap.frst_lvl_sz);
-	twoLevelMMap.flvl_map_write_use_count=ckalloc(sizeof(*twoLevelMMap.flvl_map_read_use_count)*twoLevelMMap.frst_lvl_sz);
+	twoLevelMMap.flvlmap_read=sg_calloc(sizeof(uint8_t*)*twoLevelMMap.frst_lvl_sz);
+	twoLevelMMap.flvlmap_write=sg_calloc(sizeof(uint8_t*)*twoLevelMMap.frst_lvl_sz);
+	twoLevelMMap.flvl_map_read_use_count=sg_calloc(sizeof(*twoLevelMMap.flvl_map_read_use_count)*twoLevelMMap.frst_lvl_sz);
+	twoLevelMMap.flvl_map_write_use_count=sg_calloc(sizeof(*twoLevelMMap.flvl_map_read_use_count)*twoLevelMMap.frst_lvl_sz);
 
-	iohandlerMap=ckalloc(sizeof(IOHandler *)*IOH_MAP_ENTRIES);
-	iohandlerFlvlMap=ckalloc(sizeof(IOHandler **)*IOH_FLVL_SZ);
+	iohandlerMap=sg_calloc(sizeof(IOHandler *)*IOH_MAP_ENTRIES);
+	iohandlerFlvlMap=sg_calloc(sizeof(IOHandler **)*IOH_FLVL_SZ);
 	MainBus = &mainBus; 
+	Loader_RegisterBus("bus",load_to_bus,NULL);
 	fprintf(stderr,"MemMap and IO-Handler Hash initialized\n");
 }
