@@ -25,12 +25,30 @@
 
 #ifndef BYTEORDER_H
 #define BYTEORDER_H
-#include <byteswap.h>
 
+/* These defines were found with cpp -dM,
+ * see echo | cpp -dM for example */
+#if defined(__FreeBSD__)
+
+#include <sys/endian.h>
+#define swap64(x)		bswap64(x)
+#define swap32(x)		bswap32(x)
+#define swap16(x)		bswap16(x)
+
+#elif defined(linux)
+
+#include <byteswap.h>
 #define swap64(x)		__bswap_64(x)
 #define swap32(x)		__bswap_32(x)
 #define swap16(x)		__bswap_16(x)
 
+#else
+
+#error Unknown architecture
+
+#endif
+
+
 enum en_endianness {
 	en_LITTLE_ENDIAN = 0,
 	en_BIG_ENDIAN = 1,
@@ -42,15 +60,15 @@ enum en_endianness {
 #define host32_to_be(x)		(x)	
 #define host16_to_be(x)		(x)	
 
-#define host32_to_le(x)		__bswap_32(x)	
-#define host16_to_le(x)		__bswap_16(x)
+#define host32_to_le(x)		swap32(x)	
+#define host16_to_le(x)		swap16(x)
 
 #define be32_to_host(x)		(x)
 #define be16_to_host(x)		(x)	
 
 
-#define le32_to_host(x)		__bswap_32(x)
-#define	le16_to_host(x)		__bswap_16(x)
+#define le32_to_host(x)		swap32(x)
+#define	le16_to_host(x)		swap16(x)
 
 /*
  * ------------------------------------------
@@ -61,18 +79,18 @@ enum en_endianness {
 #define TARGET_BYTEORDER en_BIG_ENDIAN
 #define NON_TARGET_BYTEORDER en_LITTLE_ENDIAN
 #define	target64_to_host(x) 	(x)
-#define	target64_to_le(x)   	__bswap_64(x)
-#define	le64_to_target(x)   	__bswap_64(x)
+#define	target64_to_le(x)   	swap64(x)
+#define	le64_to_target(x)   	swap64(x)
 #define host64_to_target(x)	(x)	
 
 #define	target32_to_host(x) 	(x)
-#define	target32_to_le(x)   	__bswap_32(x)
-#define	le32_to_target(x)   	__bswap_32(x)
+#define	target32_to_le(x)   	swap32(x)
+#define	le32_to_target(x)   	swap32(x)
 #define host32_to_target(x)	(x)	
 
 #define	target16_to_host(x) 	(x)
-#define	target16_to_le(x)   	__bswap_16(x)
-#define	le16_to_target(x)   	__bswap_16(x)
+#define	target16_to_le(x)   	swap16(x)
+#define	le16_to_target(x)   	swap16(x)
 #define host16_to_target(x)	(x)	
 
 /*
@@ -83,18 +101,18 @@ enum en_endianness {
 #else
 #define TARGET_BYTEORDER en_LITTLE_ENDIAN
 #define NON_TARGET_BYTEORDER en_BIG_ENDIAN
-#define		target64_to_host(x) __bswap_64(x)
-#define		host64_to_target(x) __bswap_64(x)
+#define		target64_to_host(x) swap64(x)
+#define		host64_to_target(x) swap64(x)
 
-#define 	target32_to_host(x) __bswap_32(x)
+#define 	target32_to_host(x) swap32(x)
 #define 	target32_to_le(x)   (x)
 #define		le32_to_target(x)   (x)
-#define		host32_to_target(x) __bswap_32(x)
+#define		host32_to_target(x) swap32(x)
 
-#define 	target16_to_host(x) __bswap_16(x)
+#define 	target16_to_host(x) swap16(x)
 #define 	target16_to_le(x)   (x)
 #define		le16_to_target(x)   (x)
-#define		host16_to_target(x) __bswap_16(x)
+#define		host16_to_target(x) swap16(x)
 
 
 #endif /* TARGET_BIG_ENDIAN */
@@ -110,14 +128,14 @@ enum en_endianness {
 #define host32_to_le(x)		(x)	
 #define	host16_to_le(x)		(x)
 
-#define host32_to_be(x)		__bswap_32(x)	
-#define host16_to_be(x)		__bswap_16(x)	
+#define host32_to_be(x)		swap32(x)	
+#define host16_to_be(x)		swap16(x)	
 
 #define le32_to_host(x)		(x)
 #define le16_to_host(x)		(x)
 
-#define be32_to_host(x)		__bswap_32(x)
-#define be16_to_host(x)		__bswap_16(x)
+#define be32_to_host(x)		swap32(x)
+#define be16_to_host(x)		swap16(x)
 
 /*
  * ------------------------------------------
@@ -127,20 +145,20 @@ enum en_endianness {
 #if TARGET_BIG_ENDIAN
 #define TARGET_BYTEORDER en_BIG_ENDIAN
 #define NON_TARGET_BYTEORDER en_LITTLE_ENDIAN
-#define	target64_to_host(x) 	__bswap_64(x)
-#define	target64_to_le(x)   	__bswap_64(x)
-#define	le64_to_target(x)   	__bswap_64(x)
-#define host64_to_target(x)	__bswap_64(x)	
+#define	target64_to_host(x) 	swap64(x)
+#define	target64_to_le(x)   	swap64(x)
+#define	le64_to_target(x)   	swap64(x)
+#define host64_to_target(x)	swap64(x)	
 
-#define	target32_to_host(x) 	__bswap_32(x)
-#define	target32_to_le(x)   	__bswap_32(x)
-#define	le32_to_target(x)   	__bswap_32(x)
-#define host32_to_target(x)	__bswap_32(x)	
+#define	target32_to_host(x) 	swap32(x)
+#define	target32_to_le(x)   	swap32(x)
+#define	le32_to_target(x)   	swap32(x)
+#define host32_to_target(x)	swap32(x)	
 
-#define	target16_to_host(x) 	__bswap_16(x)
-#define	target16_to_le(x)   	__bswap_16(x)
-#define	le16_to_target(x)   	__bswap_16(x)
-#define host16_to_target(x)	__bswap_16(x)	
+#define	target16_to_host(x) 	swap16(x)
+#define	target16_to_le(x)   	swap16(x)
+#define	le16_to_target(x)   	swap16(x)
+#define host16_to_target(x)	swap16(x)	
 
 /*
  * ----------------------------------------------
