@@ -15,9 +15,11 @@
 #define unlikely(x)     x 
 #endif
 #ifdef __GNUC__
-#define UNUSED __attribute__((unused))
+#define __UNUSED__ __attribute__((unused))
+#define __NORETURN__ __attribute__((noreturn))
 #else
 #define UNUSED
+#define __NORETURN__
 #endif
 
 #endif
