@@ -37,6 +37,7 @@
 #include "linux-tap.h"
 #include "m93c46.h"
 #include "cs8900.h"
+#include "sgstring.h"
 
 #define IO_RXTXDATA0(base)	((base) + 0x00)
 #define IO_RXTXDATA1(base)	((base) + 0x02)
@@ -1187,8 +1188,13 @@ mem_testctl_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 static uint32_t
 mem_txlength_read(void *clientData,uint32_t address,int rqlen)
 {
-	fprintf(stderr,"CS8900: TXLENGTH is writeonly\n");
-	return 0;
+        CS8900 *cs = (CS8900 *) clientData;
+	static int warned = 0;
+	if(!warned) {
+		fprintf(stderr,"CS8900: reading from writeonly register TXLENGTH\n");
+		warned = 1;
+	}
+	return cs->txlength;
 }
 
 static void
@@ -1980,12 +1986,7 @@ CS8900_New(const char *devname)
 	CS8900 *cs;
 	int i;
 	char *eepromname = alloca(strlen(devname)+20);
-        cs = (CS8900*)malloc(sizeof(CS8900));
-        if(!cs) {
-                fprintf(stderr,"Out of memory allocating CS8900\n");
-                exit(429);
-        }
-        memset(cs,0,sizeof(CS8900));
+        cs = sg_new(CS8900);
 	for(i=0;i<4;i++) {
 		cs->intrqNode[i] = SigNode_New("%s.intrq%d",devname,i);
 		if(!cs->intrqNode[i]) {
