@@ -32,20 +32,20 @@
 #include <string.h>
 #include <stdint.h>
 #include <errno.h>
-#include <linux-tap.h>
-#include <ns9750_eth.h>
-#include <ns9750_timer.h>
-#include <fio.h>
-#include <bus.h>
-#include <phy.h>
-#include <signode.h>
-#include <m93c46.h>
-#include <cycletimer.h>
+#include "linux-tap.h"
+#include "fio.h"
+#include "bus.h"
+#include "phy.h"
+#include "signode.h"
+#include "m93c46.h"
+#include "cycletimer.h"
+#include "devices/dm9000/dm9000.h"
+#include "sgstring.h"
 
 #if 0
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 
 #define DM_NCR 		(0)
@@ -412,7 +412,7 @@ rxfifo_put_packet(DM9000 *dm,uint8_t *buf,int count)
 	int pktsize;
 	int i;
 	uint32_t rp,wp;
-	dprintf("Emu: Put a packet with len %d\n",count);
+	dbgprintf("Emu: Put a packet with len %d\n",count);
 	rp = dm->rxfifo_rp;
 	wp = dm->rxfifo_wp;
 	if(rp<=wp) {
@@ -454,7 +454,7 @@ rxfifo_put_packet(DM9000 *dm,uint8_t *buf,int count)
 			break;
 		case IOMODE_16:
 			if(dm->rxfifo_wp & 1) {
-				dprintf("IOMODE16 padding\n");
+				dbgprintf("IOMODE16 padding\n");
 				rxfifo_putc(dm,0xff);
 			}
 			break;
@@ -464,7 +464,7 @@ rxfifo_put_packet(DM9000 *dm,uint8_t *buf,int count)
 			fprintf(stderr,"DM9000: Illegal iomode %d\n",iomode);
 	}
 	dm->sram[dm->rxfifo_wp]=0; /* Declare next packet as not yet received */
-	dprintf("next packet will be at %d\n",dm->rxfifo_wp);
+	dbgprintf("next packet will be at %d\n",dm->rxfifo_wp);
 	return;
 }
 
@@ -575,7 +575,7 @@ clear_pktsrc(DM9000 *dm) {
 static void
 enable_receiver(DM9000 *dm) {
         if(!dm->receiver_is_enabled && (dm->ether_fd >=0)) {
-                dprintf("DM9000: enable receiver\n");
+                dbgprintf("DM9000: enable receiver\n");
                 FIO_AddFileHandler(&dm->input_fh,dm->ether_fd,FIO_READABLE,input_event,dm);
                 dm->receiver_is_enabled=1;
         }
@@ -584,7 +584,7 @@ enable_receiver(DM9000 *dm) {
 static void
 disable_receiver(DM9000 *dm) 
 {
-        dprintf("DM9000: disable receiver\n");
+        dbgprintf("DM9000: disable receiver\n");
         if(dm->receiver_is_enabled) {
                 FIO_RemoveFileHandler(&dm->input_fh);
                 dm->receiver_is_enabled=0;
@@ -677,7 +677,7 @@ transmit_packet(DM9000 *dm)
 	uint8_t iomode;
 	char packet[1600];
 	if(len>1600) {
-		dprintf("packet to big: %d bytes\n",len);
+		dbgprintf("packet to big: %d bytes\n",len);
 		return;
 	}
 	if((len+dm->txfifo_rp) > 3*1024) {
@@ -1022,7 +1022,7 @@ epcr_write(DM9000 *dm,uint32_t value,uint8_t index,int rqlen)
 			phy = &dm->iphy;
 		}
 		if(!phy) {
-			dprintf("DM9000 EPCR: Phy not found\n");
+			dbgprintf("DM9000 EPCR: Phy not found\n");
 			return;
 		}
 		/* This is not true in real device ! On
@@ -1165,7 +1165,7 @@ static void
 par_write(DM9000 *dm,uint32_t value,uint8_t index,int rqlen) 
 {
 	unsigned int n = index - 0x10;	
-	dprintf("PAR write 0x%02x to n %d,rqlen %d\n",value,n,rqlen);
+	dbgprintf("PAR write 0x%02x to n %d,rqlen %d\n",value,n,rqlen);
 	if(n>6) {
 		fprintf(stderr,"DM9000 emulator bug in par_write\n");
 		return;
@@ -1274,7 +1274,7 @@ rwpah_read(DM9000 *dm,uint8_t index,int rqlen)
 static void
 rwpah_write(DM9000 *dm,uint32_t value,uint8_t index,int rqlen) 
 {
-        dprintf("DM9000 RWPAH is not writable\n");
+        dbgprintf("DM9000 RWPAH is not writable\n");
 }
 
 static uint32_t
@@ -1330,19 +1330,19 @@ chipr_read(DM9000 *dm,uint8_t index,int rqlen)
 static void
 chipr_write(DM9000 *dm,uint32_t value,uint8_t index,int rqlen) 
 {
-        dprintf("DM9000 CHIPR is not writable\n");
+        dbgprintf("DM9000 CHIPR is not writable\n");
 }
 
 static uint32_t
 smcr_read(DM9000 *dm,uint8_t index,int rqlen)
 {
-        dprintf("Not implemented 0x%02x\n",index);
+        dbgprintf("Not implemented 0x%02x\n",index);
         return 0;
 }
 static void
 smcr_write(DM9000 *dm,uint32_t value,uint8_t index,int rqlen)
 {
-        dprintf("Not implemented 0x%02x\n",index);
+        dbgprintf("Not implemented 0x%02x\n",index);
 }
 
 /*
@@ -1377,7 +1377,7 @@ tcscr_read(DM9000 *dm,uint8_t index,int rqlen)
 static void
 tcscr_write(DM9000 *dm,uint32_t value,uint8_t index,int rqlen)
 {
-        dprintf("DM9000: Checksum control register not implemented\n",index);
+        dbgprintf("DM9000: Checksum control register not implemented\n",index);
 }
 
 /*
@@ -1394,7 +1394,7 @@ rcscsr_read(DM9000 *dm,uint8_t index,int rqlen)
 static void
 rcscsr_write(DM9000 *dm,uint32_t value,uint8_t index,int rqlen)
 {
-        dprintf("DM9000: Not implemented 0x%02x\n",index);
+        dbgprintf("DM9000: Not implemented 0x%02x\n",index);
 }
 /*
  * ----------------------------------------------------------
@@ -1410,7 +1410,7 @@ mpar_read(DM9000 *dm,uint8_t index,int rqlen)
 static void
 mpar_write(DM9000 *dm,uint32_t value,uint8_t index,int rqlen)
 {
-        dprintf("DM9000: Not implemented 0x%02x\n",index);
+        dbgprintf("DM9000: Not implemented 0x%02x\n",index);
 }
 
 /*
@@ -1428,7 +1428,7 @@ ledcr_read(DM9000 *dm,uint8_t index,int rqlen)
 static void
 ledcr_write(DM9000 *dm,uint32_t value,uint8_t index,int rqlen)
 {
-        dprintf("DM9000: Not implemented 0x%02x\n",index);
+        dbgprintf("DM9000: Not implemented 0x%02x\n",index);
 }
 static uint32_t
 buscr_read(DM9000 *dm,uint8_t index,int rqlen)
@@ -1480,7 +1480,7 @@ sccr_read(DM9000 *dm,uint8_t index,int rqlen)
 static void
 sccr_write(DM9000 *dm,uint32_t value,uint8_t index,int rqlen)
 {
-        dprintf("DM9000: Not implemented 0x%02x\n",index);
+        dbgprintf("DM9000: Not implemented 0x%02x\n",index);
 }
 static uint32_t
 rsccr_read(DM9000 *dm,uint8_t index,int rqlen)
@@ -1491,7 +1491,7 @@ rsccr_read(DM9000 *dm,uint8_t index,int rqlen)
 static void
 rsccr_write(DM9000 *dm,uint32_t value,uint8_t index,int rqlen)
 {
-        dprintf("DM9000: Not implemented 0x%02x\n",index);
+        dbgprintf("DM9000: Not implemented 0x%02x\n",index);
 }
 
 
@@ -1714,7 +1714,7 @@ isr_write(DM9000 *dm,uint32_t value,uint8_t index,int rqlen)
 {
 	uint32_t clearbits = (value &  0xf);
 	dm->isr  &= ~clearbits;
-	dprintf("Isr: cleared bits 0x%08x, isr 0x%08x\n",clearbits,dm->isr);
+	dbgprintf("Isr: cleared bits 0x%08x, isr 0x%08x\n",clearbits,dm->isr);
 	update_interrupts(dm);
 }
 
@@ -1936,7 +1936,7 @@ iphy_write(PHY_Device *phy,uint16_t value,int reg)
 	switch(reg) {
 		case	DMP_CONTROL:
 			dm->control=value;
-			dprintf("write to phy reg 0x%02x value 0x%04x\n",reg,value);
+			dbgprintf("write to phy reg 0x%02x value 0x%04x\n",reg,value);
 			break;	
 
 		case	DMP_STATUS:
@@ -1996,14 +1996,8 @@ epy_access_done(void *clientData)
 
 BusDevice *
 DM9000_New(const char *devname,int register_spacing) {
-        DM9000 *dm = (DM9000*)malloc(sizeof(DM9000));
+        DM9000 *dm = sg_new(DM9000);
 	char *epromname = (char*)alloca(strlen(devname) + 20);
-	char *irqrequest_name = (char*)alloca(strlen(devname)+20);
-        if(!dm) {
-                fprintf(stderr,"Out of memory \n");
-                exit(480);
-        }
-        memset(dm,0,sizeof(DM9000));
         dm->ether_fd = Net_CreateInterface(devname);
         fcntl(dm->ether_fd,F_SETFL,O_NONBLOCK);
 
@@ -2014,10 +2008,9 @@ DM9000_New(const char *devname,int register_spacing) {
         dm->bdev.hw_flags=MEM_FLAG_WRITABLE|MEM_FLAG_READABLE;
 	create_registers(dm);
 
-	sprintf(irqrequest_name,"%s.irq",devname);
-        dm->irqNode = SigNode_New(irqrequest_name);
+        dm->irqNode = SigNode_New("%s.irq",devname);
         if(!dm->irqNode) {
-                fprintf(stderr,"DM9000: Can not create IrqRequest Node %s\n",irqrequest_name);
+                fprintf(stderr,"DM9000: Can not create IrqRequest Node for dev %s\n",devname);
 		exit(3425);
         }
 	dm->buswidth = 2;
