@@ -37,6 +37,7 @@
 #include <sys/mman.h>
 #include <sys/types.h>
 #include "diskimage.h"
+#include "sgstring.h"
 
 /*
  * ------------------------------------------------------------------------
@@ -49,8 +50,8 @@ diskimage_fill(DiskImage *di,uint8_t emptyval) {
 	int i;
 	uint8_t buf[1024];
 	int64_t count; /* must be signed because file might be oversized */
-	off64_t pos;
-	if((pos=lseek64(di->fd,0,SEEK_END))==(off_t)-1) {
+	off_t pos;
+	if((pos=lseek(di->fd,0,SEEK_END))==(off_t)-1) {
 		perror("lseek64 on flashfile failed");
 		return -1;
 	}
@@ -80,19 +81,19 @@ diskimage_fill(DiskImage *di,uint8_t emptyval) {
 
 static int 
 diskimage_sparse_fill(DiskImage *di,uint8_t emptyval) {
-	off64_t pos;
+	off_t pos;
 	if(di->size < 1) {
 		fprintf(stderr,"Sparse diskimage is < 1 Byte\n");
 		return -1;
 	}
-	if((pos=lseek64(di->fd,0,SEEK_END))==-1) {
+	if((pos=lseek(di->fd,0,SEEK_END))==-1) {
 		perror("lseek64 on flashfile failed");
 		return -1;
 	}
 	if(pos >= di->size) {
 		return 0;
 	}
-	if((pos=lseek64(di->fd,di->size-1,SEEK_SET)) != (di->size-1)) {
+	if((pos=lseek(di->fd,di->size-1,SEEK_SET)) != (di->size-1)) {
 		perror("lseek64 on sparse file failed");
 		return -1;
 	}
@@ -108,12 +109,7 @@ DiskImage *
 DiskImage_Open(const char *name,uint64_t size,int flags) {
 	DiskImage *di;
 	uint8_t emptyval;
-	di = malloc(sizeof(DiskImage));
-	if(!di) {
-		fprintf(stderr,"out of memory\n");
-		exit(14);
-	}
-	memset(di,0,sizeof(DiskImage));
+	di = sg_new(DiskImage);
 	di->size = size;
 	if(flags & DI_RDWR) {
 		if(flags & (DI_CREAT_FF | DI_CREAT_00)) {
@@ -143,12 +139,14 @@ DiskImage_Open(const char *name,uint64_t size,int flags) {
 	}
 	if(flags & DI_SPARSE) {
 		if(diskimage_sparse_fill(di,emptyval) < 0) {
+			fprintf(stderr,"Filling the sparse diskimage failed\n");
 			close(di->fd);
 			free(di);
 			return NULL;
 		}
 	} else {
 		if(diskimage_fill(di,emptyval) < 0) {
+			fprintf(stderr,"Filling the diskimage failed\n");
 			close(di->fd);
 			free(di);
 			return NULL;
@@ -158,11 +156,11 @@ DiskImage_Open(const char *name,uint64_t size,int flags) {
 }
 
 int
-DiskImage_Read(DiskImage *di,off64_t ofs,uint8_t *buf,int count) 
+DiskImage_Read(DiskImage *di,off_t ofs,uint8_t *buf,int count) 
 {
 	int result;
 	int cnt;
-	if(lseek64(di->fd,ofs,SEEK_SET) != ofs) {
+	if(lseek(di->fd,ofs,SEEK_SET) != ofs) {
 		return -EINVAL;
 	}
 	for(cnt=0;cnt < count;) {
@@ -180,11 +178,11 @@ DiskImage_Read(DiskImage *di,off64_t ofs,uint8_t *buf,int count)
 	
 }
 int
-DiskImage_Write(DiskImage *di,off64_t ofs,const uint8_t *buf,int count) 
+DiskImage_Write(DiskImage *di,off_t ofs,const uint8_t *buf,int count) 
 {
 	int result;
 	int cnt;
-	if(lseek64(di->fd,ofs,SEEK_SET) != ofs) {
+	if(lseek(di->fd,ofs,SEEK_SET) != ofs) {
 		return -EINVAL;
 	}
 	for(cnt=0;cnt < count;) {
