@@ -219,7 +219,7 @@ lp_close(void *dev)
 	}
 }
 
-UNUSED static void
+__UNUSED__ static void
 dump_printdata(UsbTransaction *ta) 
 {
 	int i;
@@ -579,7 +579,7 @@ usbrq_synch_frame(UsbEndpoint *ep,UsbSetupBuffer *sb)
  * -------------------------------------------------------------
  */
 
-static int
+static void 
 PowerPress(struct SigNode * node,int value, void *clientData)
 {
 	if(value == SIG_LOW) {
@@ -587,10 +587,9 @@ PowerPress(struct SigNode * node,int value, void *clientData)
 	} else {
 		fprintf(stderr,"DJ460 Power Button released\n");
 	}
-	return 0;
 }
 
-static int
+static void 
 CancelPress(struct SigNode * node,int value, void *clientData)
 {
 	if(value == SIG_LOW) {
@@ -598,10 +597,9 @@ CancelPress(struct SigNode * node,int value, void *clientData)
 	} else {
 		fprintf(stderr,"DJ460 Cancel Button released\n");
 	}
-	return 0;
 }
 
-static int
+static void 
 ResumePress(struct SigNode * node,int value, void *clientData)
 {
 	if(value == SIG_LOW) {
@@ -609,10 +607,9 @@ ResumePress(struct SigNode * node,int value, void *clientData)
 	} else {
 		fprintf(stderr,"DJ460 Resume Button released\n");
 	}
-	return 0;
 }
 
-static int
+static void 
 DoorDetect(struct SigNode * node,int value, void *clientData)
 {
 	if(value == SIG_LOW) {
@@ -620,7 +617,6 @@ DoorDetect(struct SigNode * node,int value, void *clientData)
 	} else {
 		fprintf(stderr,"DJ460 Door closed\n");
 	}
-	return 0;
 }
 
 /*
@@ -633,7 +629,7 @@ DoorDetect(struct SigNode * node,int value, void *clientData)
 void 
 DJet460_New(const char *name,UsbDevice **udevRet /*, ParportPrinter *pprn */) 
 {
-	DJet460 *dj = sg_calloc(DJet460); 
+	DJet460 *dj = sg_new(DJet460); 
 	UsbDevice *udev;
 	UsbEndpoint *ep0;
 	int i;
