@@ -40,6 +40,7 @@
 #include "bus.h"
 #include "configfile.h"
 #include "dram.h"
+#include "sgstring.h"
 
 /* all times in nanoseconds, all clocks in cycles */
 typedef struct DRamTiming {
@@ -428,12 +429,7 @@ DRam_New(char *dram_name) {
 		fprintf(stderr,"DRAM bank \"%s\" not present\n",dram_name);
 		return NULL;
 	}
-	dram = malloc(sizeof(DRam));
-	if(!dram) {
-		fprintf(stderr,"Out of memory\n");
-		exit(556);
-	}
-	memset(dram,0,sizeof(DRam));
+	dram = sg_new(DRam);
 	dram->type = drtype;
 	dram->state = SDRSTAT_POWERON;
 	if(drtype) {
@@ -443,11 +439,7 @@ DRam_New(char *dram_name) {
 		/* Skip DRAM initialisation */
 		dram->cycletype = SDRCYC_NORMAL;
 	}
-	dram->host_mem=malloc(size);
-	if(!dram->host_mem) {
-		fprintf(stderr,"Can not allocate memory for DRAM emulation\n");
-		exit(546);
-	}
+	dram->host_mem = sg_calloc(size);
 	memset(dram->host_mem,0xff,size);
 	dram->size=size;
 	dram->bdev.first_mapping=NULL;
