@@ -32,9 +32,9 @@
 #include "signode.h"
 #include "configfile.h"
 #include "diskimage.h"
-#include "spi.h"
 #include "ds1305.h"
 #include "cycletimer.h"
+#include "sgstring.h"
 
 #define DIR_NONE (1)
 #define DIR_IN (2)
@@ -75,7 +75,6 @@
 #define DS1305_IMAGESIZE (DS1305_TIMEOFSSIZE + DS1305_RAMSIZE + DS1305_MAGICSIZE)
 
 typedef struct DS1305 {
-	SPI_Device spidev;
 	int cpol; /* clock polarity */
 	int spi_bits;
 	SigNode *sclkNode;
@@ -880,7 +879,7 @@ sdo_shiftout(DS1305 *ds)
  * 	The signal trace procedure invoked when the SPI-Clock line changes 
  * --------------------------------------------------------------------------
  */
-static int
+static void 
 SpiClk(SigNode *node,int value,void *clientData) 
 {
 	DS1305 *ds = (DS1305*) clientData;
@@ -897,7 +896,7 @@ SpiClk(SigNode *node,int value,void *clientData)
 			sdo_shiftout(ds);
 		}
 	}
-	return 0;
+	return;
 }
 
 
@@ -906,7 +905,7 @@ SpiClk(SigNode *node,int value,void *clientData)
  * State change of Chip enable line
  * -----------------------------------------------------------
  */
-static int
+static void 
 SpiCe(SigNode *node,int value,void *clientData) 
 {
 	DS1305 *ds = (DS1305*) clientData;
@@ -935,7 +934,6 @@ SpiCe(SigNode *node,int value,void *clientData)
 			ds->sclkTrace = NULL;
 		}
 	}
-	return 0;
 }
 /*
  * -------------------------------------------------------------------------------
@@ -944,17 +942,12 @@ SpiCe(SigNode *node,int value,void *clientData)
  * Creates also a Pullup resistor because I'm to lazy to do it elsewhere
  * -------------------------------------------------------------------------------
  */
-SPI_Device *
+void
 DS1305_New(const char *name) 
 {
 	char *dirname;
 	char *imagename;
-	DS1305 *ds = malloc(sizeof(*ds));
-	if(!ds) {
-		fprintf(stderr,"Out of memory allocating Dallas realtime clock DS1305\n");
-		exit(1);
-	}
-	memset(ds,0,sizeof(*ds));
+	DS1305 *ds = sg_new(DS1305);
 	ds->sclkNode = SigNode_New("%s.sclk",name);
 	ds->sdiNode = SigNode_New("%s.sdi",name);
 	ds->sdoNode = SigNode_New("%s.sdo",name);
@@ -985,5 +978,4 @@ DS1305_New(const char *name)
         }
 	fprintf(stderr,"DS1305 created with time offset %lld\n",(long long)ds->time_offset);
 	update_time(ds);
-	return &ds->spidev;
 }
