@@ -33,11 +33,12 @@
 #include "ds1337.h"
 #include "configfile.h"
 #include "diskimage.h"
+#include "sgstring.h"
 
 #if 0
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 
 #define DS_STATE_ADDR (0)
@@ -711,7 +712,7 @@ ds1337_read(void *dev,uint8_t *data)
 {
 	DS1337 *ds = dev;
 	*data = read_reg(ds,ds->reg_address);
-	dprintf("DS1337 read 0x%02x from %04x\n",*data,ds->reg_address);
+	dbgprintf("DS1337 read 0x%02x from %04x\n",*data,ds->reg_address);
 	ds->reg_address = (ds->reg_address + 1) & 0xf;
 	if(ds->reg_address == 0) {
 		update_time(ds);
@@ -728,11 +729,11 @@ static int
 ds1337_write(void *dev,uint8_t data) {
 	DS1337 *ds = dev;
 	if(ds->state==DS_STATE_ADDR) {
-		dprintf("DS1337 Addr 0x%02x\n",data);
+		dbgprintf("DS1337 Addr 0x%02x\n",data);
 		ds->reg_address = data & 0xf;
 		ds->state = DS_STATE_DATA;
 	} else if(ds->state==DS_STATE_DATA) {
-		dprintf("DS1337 Write 0x%02x to %04x\n",data,ds->reg_address);
+		dbgprintf("DS1337 Write 0x%02x to %04x\n",data,ds->reg_address);
 		write_reg(ds,data,ds->reg_address);	
 		ds->reg_address=((ds->reg_address+1)&0xf) | (ds->reg_address & ~0xf);
 	}
@@ -742,7 +743,7 @@ ds1337_write(void *dev,uint8_t data) {
 static int
 ds1337_start(void *dev,int i2c_addr,int operation) {
 	DS1337 *ds = dev;
-	dprintf("ds1337 start\n");
+	dbgprintf("ds1337 start\n");
 	ds->state = DS_STATE_ADDR;
 	ds->direction = operation;
 	update_time(ds); 
@@ -753,7 +754,7 @@ ds1337_start(void *dev,int i2c_addr,int operation) {
 static void 
 ds1337_stop(void *dev) {
 	DS1337 *ds = dev;
-	dprintf("ds1337 stop\n");
+	dbgprintf("ds1337 stop\n");
 	if(ds->direction == I2C_WRITE) {
 		if(ds->time_changed_flag) {
 			ds->time_offset = diff_systime(ds);
@@ -779,15 +780,10 @@ static I2C_SlaveOps ds1337_ops = {
 
 I2C_Slave *
 DS1337_New(char *name) {
-	DS1337 *ds = malloc(sizeof(DS1337)); 
+	DS1337 *ds = sg_new(DS1337); 
 	char *dirname,*imagename;
 	
 	I2C_Slave *i2c_slave;
-	if(!ds) {
-		fprintf(stderr,"Out of memory\n");
-		exit(8152);
-	}
-	memset(ds,0,sizeof(DS1337));
 	dirname=Config_ReadVar("global","imagedir");
 	if(dirname) {
                 imagename = alloca(strlen(dirname) + strlen(name) + 20);
