@@ -32,9 +32,9 @@
 #include "signode.h"
 #include "configfile.h"
 #include "diskimage.h"
-#include "spi.h"
 #include "ds1305.h"
 #include "cycletimer.h"
+#include "sgstring.h"
 
 #if 0
 #define dbgprintf(x...) { fprintf(stderr,x); }
@@ -100,7 +100,6 @@
 #define DS3234_IMAGESIZE (512)
 
 typedef struct DS3234 {
-	SPI_Device spidev;
 	int cpol; /* clock polarity */
 	int spi_bits;
 	SigNode *sclkNode;
@@ -1028,7 +1027,7 @@ sdo_shiftout(DS3234 *ds)
  * 	The signal trace procedure invoked when the SPI-Clock line changes 
  * --------------------------------------------------------------------------
  */
-static int
+static void 
 SpiClk(SigNode *node,int value,void *clientData) 
 {
 	DS3234 *ds = (DS3234*) clientData;
@@ -1045,7 +1044,6 @@ SpiClk(SigNode *node,int value,void *clientData)
 			sdo_shiftout(ds);
 		}
 	}
-	return 0;
 }
 
 
@@ -1054,7 +1052,7 @@ SpiClk(SigNode *node,int value,void *clientData)
  * State change of Chip enable line
  * -----------------------------------------------------------
  */
-static int
+static void 
 SpiCs(SigNode *node,int value,void *clientData) 
 {
 	DS3234 *ds = (DS3234*) clientData;
@@ -1083,7 +1081,6 @@ SpiCs(SigNode *node,int value,void *clientData)
 			ds->sclkTrace = NULL;
 		}
 	}
-	return 0;
 }
  /*
   * ---------------------------------------------------------------------------------
@@ -1108,17 +1105,12 @@ DS3234_SetPorValues(DS3234 *ds) {
  * Creates also a Pullup resistor because I'm to lazy to do it elsewhere
  * -------------------------------------------------------------------------------
  */
-SPI_Device *
+void
 DS3234_New(const char *name) 
 {
 	char *dirname;
 	char *imagename;
-	DS3234 *ds = malloc(sizeof(*ds));
-	if(!ds) {
-		fprintf(stderr,"Out of memory allocating Dallas realtime clock DS3234\n");
-		exit(1);
-	}
-	memset(ds,0,sizeof(*ds));
+	DS3234 *ds = sg_new(DS3234);
 	ds->sclkNode = SigNode_New("%s.sclk",name);
 	ds->sdiNode = SigNode_New("%s.sdi",name);
 	ds->sdoNode = SigNode_New("%s.sdo",name);
@@ -1150,5 +1142,4 @@ DS3234_New(const char *name)
 	CycleTimer_Add(&ds->TemperatureUpdateTimer,MillisecondsToCycles(5000),temp_meass_timer,ds);
 	fprintf(stderr,"DS3234 created with time offset %lld\n",(unsigned long long)ds->time_offset);
 	update_time(ds);
-	return &ds->spidev;
 }
