@@ -1,5 +1,7 @@
 #ifndef __FBDISPLAY
 #define	__FBDISPLAY
+#include "keyboard.h"
+#include "sound.h"
 
 typedef struct FbUpdateRequest {
 	unsigned int offset;	/* Start address relative to framebuffer start */
@@ -34,8 +36,42 @@ typedef struct FbDisplay {
 	void (*setFbFormat)(struct FbDisplay *,FbFormat *);	
 	int (*fbUpdateRequest)(struct FbDisplay *,FbUpdateRequest *);
 	int (*fbCtrlMsg)(struct FbDisplay *,FbCtrlMsg *);
+	char *name;
+	uint32_t width;
+	uint32_t height;
 } FbDisplay;
 
+/*
+ ***********************************************************
+ * Get the width of the display
+ ***********************************************************
+ */
+static inline uint32_t
+FbDisplay_Width(FbDisplay *disp) 
+{
+	return disp->width;
+}
+/*
+ ***********************************************************
+ * Get the height of the display
+ ***********************************************************
+ */
+static inline uint32_t
+FbDisplay_Height(FbDisplay *disp) 
+{
+	return disp->height;
+}
+/*
+ ***********************************************************
+ * Get the name of a display
+ * Eventually needed by the user to parse the config file
+ ***********************************************************
+ */
+static inline char *
+FbDisplay_Name(FbDisplay *disp) 
+{
+	return disp->name;
+}
 /*
  * --------------------------------------------------------------------
  * FB_Display constructor needs to be called with information about 
@@ -75,4 +111,7 @@ FbDisplay_ControlMessage(FbDisplay *disp,FbCtrlMsg *msg)
 		return -1;
 	}
 }
+
+void FbDisplay_New(const char *name,FbDisplay **display,Keyboard **keyboard,void **mouse,SoundDevice **sdev);
+
 #endif /* __FBSIDPLAY */
