@@ -32,23 +32,26 @@
 #include <stdarg.h>
 #include <string.h>
 #include <errno.h>
+#include <sys/types.h>
 #include <sys/socket.h>
-#include <netinet/ip.h>
-#include <netinet/in.h>
 #include <arpa/inet.h>
+#include <netinet/in.h>
+#include <netinet/in_systm.h>
+#include <netinet/ip.h>
 #include <netinet/tcp.h>
 #include <unistd.h>
 #include <fcntl.h>
-#include <fio.h>
-#include <configfile.h>
-#include <gdebug.h>
 #include <sys/time.h>
+#include <fio.h>
+#include "configfile.h"
+#include "gdebug.h"
+#include "sgstring.h"
 
 
 #if 0
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 
 #define CMDBUF_SIZE (512)
@@ -56,15 +59,17 @@
 typedef struct BreakPoint {
 	uint64_t addr;
 	int len;
+	int type;
 	uint8_t backup[8];
 	struct BreakPoint *next;
 } BreakPoint;
 
+typedef struct GdbServer GdbServer;
 typedef struct GdbSession {
 	int fd;
 	int rfh_is_active;
 	FIO_FileHandler rfh;
-	GDebugOps *dbgops;
+	DebugBackendOps *dbgops;
 	GdbServer *gserv;
 
 	char cmdbuf[CMDBUF_SIZE];
@@ -77,9 +82,10 @@ typedef struct GdbSession {
 } GdbSession;
 
 struct GdbServer {
+	Debugger debugger;
 	int sockfd;
 	FIO_TcpServer gserv;
-	GDebugOps *dbgops;
+	DebugBackendOps *dbgops;
 	void *backend;
 	GdbSession *first_gsess;
 };
@@ -139,8 +145,9 @@ gsess_reply(GdbSession *gsess,const char *format,...) {
 	int i;
 	count = sprintf(reply,"$");
         va_start (ap, format);
-        count+=vsnprintf(reply+count,1024-count-4,format,ap);
+        count+=vsnprintf(reply+count,sizeof(reply)- count - 4,format,ap);
         va_end(ap);
+	dbgprintf("Reply \"%s\"\n",reply);
 	for(i=1;i<count;i++) {
 		chksum += reply[i];
 	}
@@ -187,11 +194,18 @@ unlink_breakpoint(GdbSession *gsess,uint64_t addr,int len)
 	}
 	return NULL;
 }
+
+/*
+ * ------------------------------------------------------
+ * delete_breakpoints
+ * 	Delete all breakpoints belonging to a session
+ * ------------------------------------------------------
+ */
 static void 
-remove_breakpoints(GdbSession *gsess) 
+delete_breakpoints(GdbSession *gsess) 
 {
 	BreakPoint *cursor;
-	GDebugOps *dbgops = gsess->dbgops;
+	DebugBackendOps *dbgops = gsess->dbgops;
 	if(!dbgops->setmem)
 		return;
 	while(gsess->bkpt_head) {
@@ -203,78 +217,94 @@ remove_breakpoints(GdbSession *gsess)
 	}
 }
 
-void
-gsess_del(GdbSession *gsess) {
+/*
+ * --------------------------------------------
+ * gsess_terminate
+ * 	Terminate a gdb session
+ * --------------------------------------------
+ */
+static void
+gsess_terminate(GdbSession *gsess) {
         uint8_t c;
 	GdbServer *gserv = gsess->gserv;
         if(gsess->rfh_is_active) {
                 gsess->rfh_is_active=0;
-                dprintf("remove the filehandler\n");
+                dbgprintf("remove the filehandler\n");
                 FIO_RemoveFileHandler(&gsess->rfh);
         }
         shutdown(gsess->fd,SHUT_RDWR);
         while(read(gsess->fd,&c,1)>0) {
-                dprintf("READEMPTY %02x\n",c);
+                dbgprintf("READEMPTY %02x\n",c);
         }
         if(gsess->fd>2) {
-                dprintf("close %d\n",gsess->fd);
+                dbgprintf("close %d\n",gsess->fd);
                 if(close(gsess->fd)<0) {
                         perror("close fd failed");
                 } else {
                         gsess->fd=-1;
                 }
         } else {
-                dprintf("Do not close %d\n",gsess->fd);
+                dbgprintf("Do not close %d\n",gsess->fd);
         }
 	if(gserv->first_gsess == gsess) {
 		gserv->first_gsess = NULL;
 	}
-	remove_breakpoints(gsess);
+	delete_breakpoints(gsess);
         free(gsess);
 }
 
-#define MAXREGS (30)
+#define MAXREGS (40)
 static void
 gsess_getregs(GdbSession *gsess) 
 {
 	char reply[MAXREGS*8+1];
 	int i,j;
-	GDebugOps *dbgops = gsess->dbgops;
+	int len;
+	int bytecnt;
+	DebugBackendOps *dbgops = gsess->dbgops;
 	if(!dbgops->getreg) {
 		gsess_reply(gsess,"00000000");
 	} else {
+		bytecnt=0;
 		for(i=0;i<MAXREGS;i++) {
 			uint8_t value[4];
-			if(dbgops->getreg(gsess->backend,value,i,4)<=0) {
+			len = dbgops->getreg(gsess->backend,value,i,4);
+			if(len <= 0) {
 				break;	
 			}
-			for(j=0;j<4;j++) {
-				sprintf(reply+8*i+j*2,"%02x",value[j]);
+			for(j = 0; j < len; j++) {
+				bytecnt += sprintf(reply+bytecnt,"%02x",value[j]);
 			}
 		}
-		*(reply+i*8) = 0;
-		dprintf("Got %d registers, %s\n",i,reply);
+		*(reply+bytecnt) = 0;
+		dbgprintf("Got %d registers, byted %d, %s\n",i,bytecnt,reply);
 		gsess_reply(gsess,reply);
 	}
 }
+
 static void
 gsess_getreg(GdbSession *gsess,int index) 
 {
 	char reply[10];
 	int j;
-	GDebugOps *dbgops = gsess->dbgops;
+	int len;
+	int bytecnt;
+	DebugBackendOps *dbgops = gsess->dbgops;
 	if(!dbgops->getreg) {
 		gsess_reply(gsess,"00000000");
 	} else {
 		uint8_t value[4];
-		if(dbgops->getreg(gsess->backend,value,index,4)<=0) {
-			
+		bytecnt = 0;
+		len = dbgops->getreg(gsess->backend,value,index,4);
+		if(len <= 0) {
+			fprintf(stderr,"GDEBUG: Can not get: R%d\n",index);
+				
 		}
-		for(j=0;j<4;j++) {
-			sprintf(reply+j*2,"%02x",value[j]);
+		for(j = 0;j < len;j++) {
+			bytecnt += sprintf(reply+bytecnt,"%02x",value[j]);
 		}
-		*(reply+8) = 0;
-		dprintf("registers %d, %s\n",index,reply);
+		*(reply+bytecnt) = 0;
+		dbgprintf("GDEBUG: R%d, %s\n",index,reply);
 		gsess_reply(gsess,reply);
 	}
 }
@@ -286,7 +316,7 @@ gsess_getmem(GdbSession *gsess,uint64_t addr,uint32_t len)
 	uint8_t buf[32];
 	int i;
 	int result;
-	GDebugOps *dbgops = gsess->dbgops;
+	DebugBackendOps *dbgops = gsess->dbgops;
 	if(len > 256) {
 		len=256;
 	}
@@ -319,7 +349,7 @@ gsess_getmem(GdbSession *gsess,uint64_t addr,uint32_t len)
 			addr+=1;
 		}
 		reply[writep] = 0;
-		dprintf("count %d gm reply \"%s\"\n",writep, reply);
+		dbgprintf("count %d gm reply \"%s\"\n",writep, reply);
 		gsess_reply(gsess,reply);
 	}
 }
@@ -328,21 +358,24 @@ static void
 gsess_getstatus(GdbSession *gsess) 
 {
 	int result;
-	GDebugOps *dbgops = gsess->dbgops;
+	DebugBackendOps *dbgops = gsess->dbgops;
 	if(!dbgops->get_status) {
 		gsess_reply(gsess,"S00");
 	} else {
 		result = dbgops->get_status(gsess->backend);
-		if(result>=0) {
+		if(result <= DbgStat_GDBEND) {
 			gsess_reply(gsess,"S%02x",result);
+		} else if(result == DbgStat_RUNNING) {
+			gsess_reply(gsess,"OK");
 		}		
 	}
 }
+
 static void
 gsess_stop(GdbSession *gsess) 
 {
 	int result;
-	GDebugOps *dbgops = gsess->dbgops;
+	DebugBackendOps *dbgops = gsess->dbgops;
 	if(!dbgops->stop) {
 		gsess_reply(gsess,"S00");
 	} else {
@@ -355,12 +388,16 @@ gsess_stop(GdbSession *gsess)
 
 /*
  * ------------------------------------------
- * Returns the number of sessions
+ * GdbServer_NotifyStatus
+ *	Tell the gdb about some event
+ * 	sig is a unix signal number 
+ *	SIG_INT if stopped
  * ------------------------------------------
  */
 int
-GdbServer_Signal(GdbServer *gserv,int sig) 
+GdbServer_Notify(void *_gserv,Dbg_TargetStat sig) 
 {
+	GdbServer *gserv = (GdbServer *) _gserv;
 	GdbSession *gsess = gserv->first_gsess;
 	if(!gsess) {
 		return 0;	
@@ -373,7 +410,7 @@ static void
 gsess_cont(GdbSession *gsess) 
 {
 	int result;
-	GDebugOps *dbgops = gsess->dbgops;
+	DebugBackendOps *dbgops = gsess->dbgops;
 	if(!dbgops->cont) {
 
 	} else {
@@ -387,14 +424,16 @@ gsess_cont(GdbSession *gsess)
 static void
 gsess_step(GdbSession *gsess,uint32_t addr,int use_addr) 
 {
-	GDebugOps *dbgops = gsess->dbgops;
+	DebugBackendOps *dbgops = gsess->dbgops;
 	int result;
 	if(!dbgops->step) {
 		gsess_reply(gsess,"S00");
 	} else {
 		result = dbgops->step(gsess->backend,addr,use_addr);
-		if(result>=0) {
+		if(result >= 0) {
 			gsess_reply(gsess,"S%02x",result);
+		} else if(result == DbgStat_OK) {
+			gsess_reply(gsess,"OK");
 		}		
 	}
 }
@@ -406,9 +445,9 @@ hexparse(const char *str,uint8_t *retval,int maxbytes) {
         for(i=0;i<maxbytes;i++) {
 		for(nibble=0;nibble<2;nibble++) {
                 	char c = *str;
-			if(c>='0' && c<='9') {
+			if((c >= '0') && (c <= '9')) {
 				val =(c - '0');
-			} else if(c>='A' && c<='F') {
+			} else if((c >= 'A') && (c <= 'F')) {
 				val =(c - 'A'+10);
 			} else if(c>='a' && c<='f') {
 				val =(c - 'a'+10);
@@ -433,7 +472,7 @@ gsess_setmem(GdbSession *gsess,char *data,int maxlen)
 	uint32_t len;
 	int readlen;
 	uint32_t readp=0;
-	GDebugOps *dbgops = gsess->dbgops;
+	DebugBackendOps *dbgops = gsess->dbgops;
 	if(!dbgops->setmem) {
 		gsess_reply(gsess,"E00");
 	}
@@ -470,7 +509,7 @@ gsess_setmem(GdbSession *gsess,char *data,int maxlen)
 static void
 gsess_setreg(GdbSession *gsess,char *data)
 {
-	GDebugOps *dbgops = gsess->dbgops;
+	DebugBackendOps *dbgops = gsess->dbgops;
 	int reg; 
 	uint8_t value[8];
 	int count;
@@ -478,7 +517,7 @@ gsess_setreg(GdbSession *gsess,char *data)
 		gsess_reply(gsess,"E00");
 		return;
 	}
-	if(sscanf(data,"%d=",&reg)!=1) {
+	if(sscanf(data,"%x=",&reg)!=1) {
 		gsess_reply(gsess,"E00"); // fix the error number 
 		return;
 	}
@@ -490,16 +529,6 @@ gsess_setreg(GdbSession *gsess,char *data)
 		gsess_reply(gsess,"E00");
 		return;
 	}
-#if 1
-	{
-		int i;
-		fprintf(stderr,"hexparsed %d bytes: ",count);
-		for(i=0;i<4;i++) {
-			fprintf(stderr,"%02x ",value[i]);	
-		}
-		fprintf(stderr,"\n");	
-	}
-#endif
 	dbgops->setreg(gsess->backend,value,reg,count);
 	gsess_reply(gsess,"OK");
 }
@@ -517,7 +546,7 @@ gsess_setreg(GdbSession *gsess,char *data)
 static void
 gsess_add_breakpoint(GdbSession *gsess,char *cmd,int cmdlen) 
 {
-	GDebugOps *dbgops = gsess->dbgops;
+	DebugBackendOps *dbgops = gsess->dbgops;
 	BreakPoint *bkpt;
 	int type;
 	uint32_t addr; 
@@ -540,28 +569,32 @@ gsess_add_breakpoint(GdbSession *gsess,char *cmd,int cmdlen)
 	dbgops->get_bkpt_ins(gsess->backend,bkpt_ins,addr,len);
 	switch(type) {
 		case 0:
+			/* The software breakpoint */
 			bkpt = find_breakpoint(gsess,addr,len);
 			if(bkpt)  {
 				fprintf(stderr,"Breakpoint already exists\n");
 				gsess_reply(gsess,"E00");
 				return;
 			}
-			bkpt = malloc(sizeof(*bkpt));
-			memset(bkpt,0,sizeof(*bkpt));
+			bkpt = sg_new(BreakPoint);
 			bkpt->addr = addr;
 			bkpt->len = len;
 			bkpt->next = gsess->bkpt_head;
+			bkpt->type = type;	
 			gsess->bkpt_head = bkpt;
 			dbgops->getmem(gsess->backend,bkpt->backup,addr,len);
 			dbgops->setmem(gsess->backend,bkpt_ins,addr,len);
 			gsess_reply(gsess,"OK");
 			break;
+		case 1:	
+			/* The hardware breakpoint */
 		default:
+			fprintf(stderr,
+			  "softgun gdb interface does not support breakpoint type %d\n",type);
+			fprintf(stderr,"cmd %s\n",cmd);
 			gsess_reply(gsess,"");
 			break;
 	}
-	fprintf(stderr,"softgun gdb interface does not support breakpoints\n");
-	fprintf(stderr,"cmd %s\n",cmd);
 }
 
 /*
@@ -573,7 +606,7 @@ gsess_add_breakpoint(GdbSession *gsess,char *cmd,int cmdlen)
 static void
 gsess_remove_breakpoint(GdbSession *gsess,char *cmd,int cmdlen) 
 {
-	GDebugOps *dbgops = gsess->dbgops;
+	DebugBackendOps *dbgops = gsess->dbgops;
 	BreakPoint *bkpt;
 	int type;
 	uint32_t addr; 
@@ -602,15 +635,17 @@ gsess_remove_breakpoint(GdbSession *gsess,char *cmd,int cmdlen)
 			}
 			dbgops->setmem(gsess->backend,bkpt->backup,addr,len);
 			free(bkpt);
+			fprintf(stderr,"remove breakpoint %s\n",cmd);
 			gsess_reply(gsess,"OK");
 			break;
 			
+		case 1:
 		default:
+			fprintf(stderr,"softgun gdb interface does not support breakpoints of type %d\n",type);
+			fprintf(stderr,"cmd %s\n",cmd);
 			gsess_reply(gsess,"");
 			break;
 	}
-	fprintf(stderr,"softgun gdb interface does not support breakpoints\n");
-	fprintf(stderr,"cmd %s\n",cmd);
 }
 
 /*
@@ -700,7 +735,7 @@ gsess_execute_cmd(GdbSession *gsess)
 			break;
 			
 		default:
-			dprintf("unknown cmd \'%c\'\n",cmd[0]);
+			dbgprintf("unknown cmd \'%c\'\n",cmd[0]);
 			gsess_output(gsess,"$#00",4); 
 			break;
 	}
@@ -716,7 +751,7 @@ gsess_execute_cmd(GdbSession *gsess)
 static int
 feed_state_machine (GdbSession *gsess,uint8_t c) {
 	int result = 0;
-	dprintf("%c",c);
+	dbgprintf("%c",c);
 	switch(gsess->cmdstate) 
 	{
 		case CMDSTATE_WAIT_START:
@@ -725,14 +760,15 @@ feed_state_machine (GdbSession *gsess,uint8_t c) {
 				gsess->cmdstate = CMDSTATE_WAIT_DATA;
 				gsess->csum = 0;
 			} else if(c == '+') {
-				dprintf("Got acknowledge\n");
+				dbgprintf("Got acknowledge\n");
 			} else if(c == '-') {
-				dprintf("Got NACK\n");
+				dbgprintf("Got NACK\n");
 			} else if(c == 0x03) {
 				gsess_stop(gsess);
-				dprintf("received abort\n");
+				fprintf(stderr,"received abort\n");
+				dbgprintf("received abort\n");
 			} else {
-				dprintf("received unknown byte %02x in START state\n",c);
+				dbgprintf("received unknown byte %02x in START state\n",c);
 			}
 			break;
 
@@ -779,7 +815,7 @@ feed_state_machine (GdbSession *gsess,uint8_t c) {
 				fprintf(stderr,"illegal byte %02x in chksum\n",c);	
 				break;
 			}
-			dprintf("received complete command, csum %02x\n",gsess->csum); 
+			dbgprintf("received complete command, csum %02x\n",gsess->csum); 
 			if(gsess->csum == 0) {
 				print_timestamp("Input"); 
 				gsess_output(gsess,"+",1);
@@ -808,19 +844,25 @@ gsess_input(void *cd,int mask) {
         uint8_t c;
         while((result=read(gsess->fd,&c,1))==1) {
                 if(feed_state_machine (gsess, c)<0) {
-                	gsess_del(gsess);
+                	gsess_terminate(gsess);
                         return 0;
                 }
         }
         if((result<0) && (errno==EAGAIN))  {
                 return 0;
         } else {
-                dprintf("Connection lost\n");
-                gsess_del(gsess);
+                dbgprintf("Connection lost\n");
+                gsess_terminate(gsess);
                 return 0;
         }
 }
 
+/*
+ * -----------------------------------------------------------
+ *  gserv_accept
+ * 	Accept a new connection from on a network socket
+ * -----------------------------------------------------------
+ */
 
 static void
 gserv_accept(int fd,char *host,unsigned short port,void *clientData) 
@@ -833,12 +875,7 @@ gserv_accept(int fd,char *host,unsigned short port,void *clientData)
 		close(fd);
 		return;
 	}
-	gsess = malloc(sizeof(GdbSession));	
-	if(!gsess) {
-		fprintf(stderr,"Out of memory allocating GDB session\n");
-		exit(344);
-	}
-	memset(gsess,0,sizeof(GdbSession));
+	gsess = sg_new(GdbSession);	
 	gserv->first_gsess = gsess;
 	gsess->fd = fd;
 	gsess->gserv = gserv;
@@ -853,29 +890,28 @@ gserv_accept(int fd,char *host,unsigned short port,void *clientData)
 	gsess->rfh_is_active = 1;
 }
 
-GdbServer *
-GdbServer_New(GDebugOps *dbgops,void *backend)
+Debugger *
+GdbServer_New(DebugBackendOps *dbgops,void *backend)
 {
         int fd;
 	int port;
         GdbServer *gserv;
+	Debugger *debugger;
 	char *host=Config_ReadVar("gdebug","host");
 	if(!host || (Config_ReadInt32(&port,"gdebug","port")<0)) {
 		fprintf(stderr,"GDB server is not configured\n");	
 		return NULL;
 	}
-        gserv=malloc(sizeof(GdbServer));
-        if(!gserv) {
-		fprintf(stderr,"Out of memory for GDB server\n");
-                return NULL;
-	}
-	memset(gserv,0,sizeof(GdbServer));
+        gserv = sg_new(GdbServer);
+	debugger = &gserv->debugger;
+	debugger->implementor = gserv;
+	debugger->notifyStatus = GdbServer_Notify;
 	gserv->dbgops = dbgops;
 	gserv->backend = backend;
         if((fd=FIO_InitTcpServer(&gserv->gserv,gserv_accept,gserv,host,port))<0) {
-                free(gserv);
+                sg_free(gserv);
                 return NULL;
         }
         fprintf(stderr,"GDB server listening on host \"%s\" port %d\n",host,port);
-	return gserv;
+	return debugger;
 }
