@@ -32,19 +32,21 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
-#include <signode.h>
-#include <i2c_serdes.h>
-#include <i2c.h>
-#include <compiler_extensions.h>
-#include <cycletimer.h>
+#include "signode.h"
+#include "i2c_serdes.h"
+#include "i2c.h"
+#include "compiler_extensions.h"
+#include "cycletimer.h"
+#include "sgstring.h"
 
 #if 0
-#define dprintf(x...) { fprintf(stderr,x);fflush(stderr);}
+#define dbgprintf(x...) { fprintf(stderr,x);fflush(stderr);}
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 
 #define I2C_TIME_INFINITE (~0)
+#define SAT_U32(x) ((x) > ~((uint32_t)0) ? ~((uint32_t)0) : x)
 
 /* 
  * ------------------------------------
@@ -156,13 +158,13 @@ invalidate_timing(I2C_Timing *timing) {
 
 static void 
 dump_timing(I2C_Timing *timing) {
-	fprintf(stderr,"T_HDSTA %d\n",timing->t_hdsta);
-	fprintf(stderr,"T_LOW   %d\n",timing->t_low);
-	fprintf(stderr,"T_HIGH  %d\n",timing->t_high);
-	fprintf(stderr,"T_SUSTA %d\n",timing->t_susta);
-	fprintf(stderr,"T_SUDAT %d\n",timing->t_sudat);
-	fprintf(stderr,"T_SUSTO %d\n",timing->t_susto);
-	fprintf(stderr,"T_BUF   %d\n",timing->t_buf);
+	fprintf(stderr,"T_HDSTA %u\n",timing->t_hdsta);
+	fprintf(stderr,"T_LOW   %u\n",timing->t_low);
+	fprintf(stderr,"T_HIGH  %u\n",timing->t_high);
+	fprintf(stderr,"T_SUSTA %u\n",timing->t_susta);
+	fprintf(stderr,"T_SUDAT %u\n",timing->t_sudat);
+	fprintf(stderr,"T_SUSTO %u\n",timing->t_susto);
+	fprintf(stderr,"T_BUF   %u\n",timing->t_buf);
 }
 /*
  * -----------------------------------------------------------------
@@ -225,6 +227,7 @@ update_t_hdsta(I2C_SerDes *serdes,int line)
 {
 	CycleCounter_t now = CycleCounter_Get();	
 	int64_t nsecs = CyclesToNanoseconds(now -  serdes->sda_change_time);
+	nsecs = SAT_U32(nsecs);
 	if(nsecs < serdes->timing.t_hdsta) {
 		serdes->timing.t_hdsta = nsecs;
 	}
@@ -234,7 +237,8 @@ update_t_low(I2C_SerDes *serdes,int line)
 {
 	CycleCounter_t now = CycleCounter_Get();	
 	int64_t nsecs = CyclesToNanoseconds(now -  serdes->scl_change_time);
-		if(nsecs < serdes->timing.t_low) {
+	nsecs = SAT_U32(nsecs);
+	if(nsecs < serdes->timing.t_low) {
 		serdes->timing.t_low = nsecs;
 	}
 }
@@ -244,6 +248,7 @@ update_t_high(I2C_SerDes *serdes,int line)
 {
 	CycleCounter_t now = CycleCounter_Get();	
 	int64_t nsecs = CyclesToNanoseconds(now -  serdes->scl_change_time);
+	nsecs = SAT_U32(nsecs);
 	if(nsecs < serdes->timing.t_high) {
 		serdes->timing.t_high = nsecs;
 	}
@@ -258,16 +263,18 @@ update_t_susta(I2C_SerDes *serdes,int line)
 {
 	CycleCounter_t now = CycleCounter_Get();	
 	int64_t nsecs = CyclesToNanoseconds(now -  serdes->scl_change_time);
+	nsecs = SAT_U32(nsecs);
 	if(nsecs < serdes->timing.t_susta) {
 		serdes->timing.t_susta = nsecs;
 	}
 }
 
-UNUSED static void 
+__UNUSED__ static void 
 update_t_hddat_max(I2C_SerDes *serdes,int line) 
 {
 	CycleCounter_t now = CycleCounter_Get();	
 	int64_t nsecs = CyclesToNanoseconds(now -  serdes->scl_change_time);
+	nsecs = SAT_U32(nsecs);
 	if(nsecs > serdes->timing.t_hddat_max) {
 		serdes->timing.t_hddat_max = nsecs;
 	}
@@ -278,6 +285,7 @@ update_t_sudat(I2C_SerDes *serdes,int line)
 {
 	CycleCounter_t now = CycleCounter_Get();	
 	int64_t nsecs = CyclesToNanoseconds(now -  serdes->sda_change_time);
+	nsecs = SAT_U32(nsecs);
 	if(nsecs < serdes->timing.t_sudat) {
 		serdes->timing.t_sudat = nsecs;
 	}
@@ -287,6 +295,7 @@ update_t_susto(I2C_SerDes *serdes,int line)
 {
 	CycleCounter_t now = CycleCounter_Get();	
 	int64_t nsecs = CyclesToNanoseconds(now -  serdes->scl_change_time);
+	nsecs = SAT_U32(nsecs);
 	if(nsecs < serdes->timing.t_susto) {
 		serdes->timing.t_susto = nsecs;
 	}
@@ -296,6 +305,7 @@ update_t_buf(I2C_SerDes *serdes,int line)
 {
 	CycleCounter_t now = CycleCounter_Get();	
 	int64_t nsecs = CyclesToNanoseconds(now -  serdes->sda_change_time);
+	nsecs = SAT_U32(nsecs);
 	if(nsecs < serdes->timing.t_buf) {
 		serdes->timing.t_buf = nsecs;
 	}
@@ -413,7 +423,7 @@ I2C_SerDesFeed(I2C_SerDes *serdes,int pinstate)
 	int oldscl=serdes->oldpinstate&I2C_SCL;
 	int oldsda=serdes->oldpinstate&I2C_SDA;
 
-	dprintf("feed  %02x, prev %02x, state %d n %s\n",pinstate,serdes->oldpinstate,serdes->state,serdes->name);
+	dbgprintf("feed  %02x, prev %02x, state %d n %s\n",pinstate,serdes->oldpinstate,serdes->state,serdes->name);
 	if(oldscl && !scl && serdes->stretch_scl) {
 		serdes->outpinstate = I2C_SDA;
 		return;
@@ -423,7 +433,7 @@ I2C_SerDesFeed(I2C_SerDes *serdes,int pinstate)
 		reset_serdes(serdes);
 		update_t_buf(serdes,__LINE__);
 		update_t_susta(serdes,__LINE__);
-		dprintf("Start Condition\n");
+		dbgprintf("Start Condition\n");
 		serdes->state=I2C_STATE_ADDR;
 		return;
 	}
@@ -453,7 +463,7 @@ I2C_SerDesFeed(I2C_SerDes *serdes,int pinstate)
 			slave->devops->stop(slave->dev);
 		}
 		reset_serdes(serdes);
-		dprintf("Stop Condition\n");
+		dbgprintf("Stop Condition\n");
 		return;
 	}
 	switch(serdes->state) {
@@ -476,7 +486,7 @@ I2C_SerDesFeed(I2C_SerDes *serdes,int pinstate)
 				if(sda) {
 					serdes->inbuf|=1;
 				}			
-				dprintf("inbuf 0x%02x\n",serdes->inbuf);
+				dbgprintf("inbuf 0x%02x\n",serdes->inbuf);
 				if(serdes->bitcount==8) {
 					I2C_Slave *slave;
 					serdes->address = serdes->inbuf >> 1;
@@ -486,7 +496,7 @@ I2C_SerDesFeed(I2C_SerDes *serdes,int pinstate)
 							break;
 						}
 					}
-					dprintf("I2C-Address %02x slave %p\n",serdes->inbuf>>1,slave);	
+					dbgprintf("I2C-Address %02x slave %p\n",serdes->inbuf>>1,slave);	
 					if(slave) {
 						int result;
 						if(!check_t_buf(serdes,slave)) {
@@ -535,7 +545,7 @@ I2C_SerDesFeed(I2C_SerDes *serdes,int pinstate)
 			if(oldscl && !scl) {
 				update_t_high(serdes,__LINE__);
 				/* release the lines */
-				//dprintf(stderr,"Release the lines for the getting ACK/NACK\n");
+				//dbgprintf(stderr,"Release the lines for the getting ACK/NACK\n");
 				serdes->outpinstate = I2C_SCL | I2C_SDA; 
 			} else if(!oldscl && scl) {
 				I2C_Slave *slave=serdes->active_slave;
@@ -547,7 +557,7 @@ I2C_SerDesFeed(I2C_SerDes *serdes,int pinstate)
 				}
 				if(sda) {
 					/* Last was not acknowledged, so read nothing  */
-					dprintf("Not acked\n");
+					dbgprintf("Not acked\n");
 					serdes->state=I2C_STATE_WAIT;
 					/* 
  					 * -----------------------------------------------------------
@@ -589,18 +599,18 @@ I2C_SerDesFeed(I2C_SerDes *serdes,int pinstate)
 				break;
 			} else if(!oldscl && scl) {
 				update_t_low(serdes,__LINE__);
-				dprintf("goto write state addr %02x\n",serdes->address);
+				dbgprintf("goto write state addr %02x\n",serdes->address);
 				serdes->state=I2C_STATE_WRITE;
 				serdes->bitcount=0;
 				serdes->inbuf=0;
 				break;
 			}
-	//		dprintf("I2C ack write addr %02x input %02x return %02x\n",serdes->address,pinstate,serdes->outpinstate);
+	//		dbgprintf("I2C ack write addr %02x input %02x return %02x\n",serdes->address,pinstate,serdes->outpinstate);
 			break;
 		
 		case I2C_STATE_READ:
 			// We change output after falling edge of scl
-			//dprintf(stderr,"read: bitcount %d, scl %d\n",serdes->bitcount,scl);
+			//dbgprintf(stderr,"read: bitcount %d, scl %d\n",serdes->bitcount,scl);
 			if(oldscl && !scl) {
 				I2C_Slave *slave=serdes->active_slave;
 				update_t_high(serdes,__LINE__);
@@ -629,7 +639,7 @@ I2C_SerDesFeed(I2C_SerDes *serdes,int pinstate)
 			} else if(!oldscl && scl) {
 				update_t_low(serdes,__LINE__);
 				if(serdes->bitcount==0) {
-					//dprintf(stderr,"goto ack receive\n");
+					//dbgprintf(stderr,"goto ack receive\n");
 					serdes->state=I2C_STATE_ACK_READ;
 				}
 			}
@@ -791,13 +801,13 @@ schedule_outputs(I2C_SerDes *serdes)
  * This trace of the signal node is invoked whenever SDA line changes 
  * -------------------------------------------------------------------
  */
-static int 
+static void 
 I2C_SdaChange(SigNode *node,int value,void *clientData) 
 {
 	I2C_SerDes *serdes = clientData;
 	int pinstate = serdes->oldpinstate;
 	if((value == SIG_HIGH)) {
-		dprintf("feed SDA_H on %s\n",SigName(node));
+		dbgprintf("feed SDA_H on %s\n",SigName(node));
 		pinstate = pinstate | I2C_SDA;	
 	} else if (value == SIG_LOW) {
 		pinstate = pinstate & ~I2C_SDA;	
@@ -808,17 +818,16 @@ I2C_SdaChange(SigNode *node,int value,void *clientData)
 	serdes->sda_change_time = CycleCounter_Get();
 	schedule_outputs(serdes);
 	serdes->oldpinstate = pinstate;
-	return 0;
 }
 
-static int 
+static void 
 I2C_SclChange(SigNode *node,int value,void *clientData) 
 {
 	I2C_SerDes *serdes = clientData;
 	int pinstate = serdes->oldpinstate;
 	if((value == SIG_HIGH)) {
 		pinstate = pinstate | I2C_SCL;	
-		dprintf("feed SCL_H on %s\n",SigName(node));
+		dbgprintf("feed SCL_H on %s\n",SigName(node));
 	} else if (value == SIG_LOW) {
 		pinstate = pinstate & ~I2C_SCL;	
 	} else {
@@ -828,7 +837,6 @@ I2C_SclChange(SigNode *node,int value,void *clientData)
 	serdes->scl_change_time = CycleCounter_Get();
 	schedule_outputs(serdes);
 	serdes->oldpinstate = pinstate;
-	return 0;
 }
 
 void
@@ -858,12 +866,7 @@ SerDes_UnstretchScl(I2C_SerDes *serdes)
 I2C_SerDes *
 I2C_SerDesNew(const char *name) 
 {
-	I2C_SerDes *serdes = malloc(sizeof(I2C_SerDes));
-	if(!serdes) {
-		fprintf(stderr,"No memory for I2C-Deserializer\n");
-		exit(234);
-	}	
-	memset(serdes,0,sizeof(I2C_SerDes));
+	I2C_SerDes *serdes = sg_new(I2C_SerDes);
 	serdes->scl = SigNode_New("%s.scl",name);
 	if(!serdes->scl) {
 		exit(2763);
@@ -892,7 +895,7 @@ I2C_SerDesNew(const char *name)
 	SigName_Link(SigName(serdes->scl),SigName(serdes->scl_pullup));
 	SigName_Link(SigName(serdes->sda),SigName(serdes->sda_pullup));
 	serdes->oldpinstate = I2C_SDA | I2C_SCL;
-	serdes->name = strdup(name);
+	serdes->name = sg_strdup(name);
 
 	fprintf(stderr,"I2C Serializer/Deserializer \"%s\" created\n",name);
 	return serdes;
