@@ -27,9 +27,9 @@
 
 
 #if 0
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 
 #include <fcntl.h>
@@ -44,6 +44,7 @@
 #include "cycletimer.h"
 #include "signode.h"
 #include "i82559.h"
+#include "sgstring.h"
 
 #define MODIFY_MASKED(v,mod,mask) { (v) = ((v) & ~(mask)) | ((mod)&(mask)); }
 
@@ -193,10 +194,6 @@ typedef struct CBH {
 } CBH;
 
 /* Command block header access */
-static void
-CBH_fetch() {
-
-}
 
 static uint32_t	
 scb_status_read(void *clientData,uint32_t address,int rqlen) 
@@ -570,7 +567,7 @@ i82559_unmap_mmio(I82559 *ixx,uint32_t base) {
  */
 static void
 change_pci_command(I82559 *ixx,uint16_t value,uint16_t mask) {
-        dprintf("Change PCI command from %04x to %04x, mask %04x\n",ixx->pci_command,value,mask);
+        dbgprintf("Change PCI command from %04x to %04x, mask %04x\n",ixx->pci_command,value,mask);
         MODIFY_MASKED(ixx->pci_command,value,mask);
         // register/register iospace and mmio
 	i82559_unmap_mmio(ixx,ixx->pci_bar0);
@@ -620,7 +617,7 @@ I82559_ConfigWrite(uint32_t value,uint32_t mask,uint8_t reg,PCI_Function *pcifun
 			}
                         break;
                 case 0x5:
-        //              dprintf("Write bar 1 to %08x\n",value);
+        //              dbgprintf("Write bar 1 to %08x\n",value);
 			if(ixx->pci_command & 1) {
                         	i82559_unmap_io(ixx,ixx->pci_bar1);
 			}
@@ -694,13 +691,13 @@ I82559_ConfigRead(uint32_t *value,uint8_t reg,PCI_Function *pcifunc)
                 case 4:
                         // IO-Type Base Address with Window of 256 bytes
                         *value = ixx->pci_bar0;
-//                      dprintf("read bar 0 %08x\n",*value);
+//                      dbgprintf("read bar 0 %08x\n",*value);
                         break;
 
                 case 5:
                         // MMIO Type
                         *value = ixx->pci_bar1 | 1;
-//                      dprintf("read bar 1 %08x\n",*value);
+//                      dbgprintf("read bar 1 %08x\n",*value);
                         break;
                 case 6:
                         *value = ixx->pci_bar2;
@@ -749,7 +746,7 @@ I82559_ConfigRead(uint32_t *value,uint8_t reg,PCI_Function *pcifunc)
                         break;
 
         }
-//      dprintf("I82559-Config Read reg %d val %08x\n",reg*4, *value);
+//      dbgprintf("I82559-Config Read reg %d val %08x\n",reg*4, *value);
         return 0;
 }
 
@@ -757,13 +754,7 @@ I82559_ConfigRead(uint32_t *value,uint8_t reg,PCI_Function *pcifunc)
 PCI_Function *
 I82559_New(const char *devname,PCI_Function *bridge,int dev_nr,int bus_irq) 
 {
-	char *nodename = alloca(strlen(devname)+30);
-	I82559 *ixx = malloc(sizeof(I82559));
-	if(!ixx) {
-		fprintf(stderr,"Out of memory allocating i82559\n");
-		exit(5460);
-	}
-	memset(ixx,0,sizeof(*ixx));
+	I82559 *ixx = sg_new(I82559);
 	ixx->ether_fd = Net_CreateInterface(devname);
         fcntl(ixx->ether_fd,F_SETFL,O_NONBLOCK);
 
@@ -790,10 +781,9 @@ I82559_New(const char *devname,PCI_Function *bridge,int dev_nr,int bus_irq)
         ixx->pcifunc.dev=dev_nr;
         ixx->pcifunc.owner=ixx;
         ixx->bridge=bridge;
-	sprintf(nodename,"%s.irq",devname);
-	ixx->irqNode = SigNode_New(nodename);
+	ixx->irqNode = SigNode_New("%s.irq",devname);
 	if(!ixx->irqNode) {
-		fprintf(stderr,"i82559: can not create IRQ-Line %s\n",nodename); 
+		fprintf(stderr,"i82559: can not create IRQ-Line for %s\n",devname); 
 		exit(342);
 	}
         PCI_FunctionRegister(bridge,&ixx->pcifunc);
