@@ -29,9 +29,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <arm9cpu.h>
-#include <idecode.h>
-#include <instructions.h>
+#include "arm9cpu.h"
+#include "idecode.h"
+#include "instructions.h"
+#include "sgstring.h"
 
 #define MAX_INSTRUCTIONS (200000)
 
@@ -48,7 +49,7 @@ Instruction *alloc_instruction() {
 	if(alloc_pointer<MAX_INSTRUCTIONS) {
 		return imem+alloc_pointer++;
 	} else{
-		return malloc(sizeof(Instruction));
+		return sg_new(Instruction);
 	}
 }
 
@@ -264,12 +265,8 @@ IDecoder_New() {
 	IDecoder *dec;
 	int sum=0;
 	int validcount=0;
-	idecoder=dec=malloc(sizeof(IDecoder));
-	imem=malloc(sizeof(Instruction)*MAX_INSTRUCTIONS);
-	if(!imem) {
-		fprintf(stderr,"OOM\n");
-		exit(1234);
-	}
+	idecoder = dec = sg_new(IDecoder);
+	imem = sg_calloc(sizeof(Instruction)*MAX_INSTRUCTIONS);
 	memset(dec,0,sizeof(IDecoder));
 	for(i=0;i<=INSTR_INDEX_MAX;i++) {
 		for(cursor=instrlist;cursor->mask;cursor++) {
@@ -314,7 +311,7 @@ IDecoder_New() {
 		validcount+=statistics[i];
 		fprintf(stderr," %d",statistics[i]);
 	}
-	iProcTab=malloc(sizeof(InstructionProc *)*(INSTR_INDEX_MAX+1));
+	iProcTab = sg_calloc(sizeof(InstructionProc *)*(INSTR_INDEX_MAX+1));
 	if(!iProcTab) {
 		fprintf(stderr,"Out of Memory");
 		exit(378);
