@@ -33,6 +33,7 @@
 #include <string.h> 
 #include <fcntl.h> 
 #include "ihex.h"
+#include "sgstring.h"
 
 typedef struct HR_Context { 
 	XY_IHexDataHandler *dataCB;
@@ -231,27 +232,25 @@ XY_LoadIHexFile(const char *filename,XY_IHexDataHandler *callback,void *clientDa
         if(!file) {
                 return -1;
         }
-	ctxt = malloc(sizeof(HR_Context));
-	if(!ctxt) {
-		fprintf(stderr,"Out of memory\n");
-		exit(3240);
-	}
-	memset(ctxt,0,sizeof(HR_Context));
+	ctxt = sg_new(HR_Context);
 	ctxt->file = file;
 	ctxt->dataCB = callback;
 	ctxt->dataCB_clientData = clientData;
         while(!feof(file)) {
                 int count;
-                if((count = read_record(ctxt,&addr,buf))<0) {
+                if((count = read_record(ctxt,&addr,buf)) < 0) {
 			fprintf(stderr,"can not read record %d\n",count);
 			return -1;
                 }
-		if(callback(addr,buf,count,clientData) <0) {
-			return -1;
+		if(count > 0) {
+			if(callback(addr,buf,count,clientData) < 0) {
+				return -1;
+			}
 		}
-		total+= count;
+		total += count;
         }
         fclose(file);
+	sg_free(ctxt);
         return total;
 }
 
