@@ -30,13 +30,14 @@
 #include <unistd.h>
 #include <sys/ioctl.h>
 #include <sys/fcntl.h>
-#include <bus.h>
-#include <fio.h>
-#include <signode.h>
-#include <imx21_crm.h>
-#include <configfile.h>
-#include <clock.h>
-#include <cycletimer.h>
+#include "bus.h"
+#include "fio.h"
+#include "signode.h"
+#include "imx21_crm.h"
+#include "configfile.h"
+#include "clock.h"
+#include "cycletimer.h"
+#include "sgstring.h"
 
 
 //#define CRM_BASE_ADDR (0x10027000)
@@ -591,8 +592,8 @@ update_perdivs(IMX_Crm *crm) {
 	} else {
 		Clock_MakeDerived(crm->ssi2clk,crm->ssi2inclk,0,1);
 	}
-	/*	fprintf(stderr,"** perclk1: freq %f\n",Clock_Freq(crm->perclk1)); 	*/
-	/*	fprintf(stderr,"** mpll_clk: freq %f\n",Clock_Freq(crm->mpll_clk)); 	*/
+	/*	fprintf(stderr,"** perclk1: freq %d\n",Clock_Freq(crm->perclk1)); 	*/
+	/*	fprintf(stderr,"** mpll_clk: freq %d\n",Clock_Freq(crm->mpll_clk)); 	*/
 }
 
 static uint32_t
@@ -1226,14 +1227,10 @@ dump_proc(void *cd)
 BusDevice *
 IMX21Crm_New(const char *name) 
 {
-	IMX_Crm *crm = (IMX_Crm*)malloc(sizeof(IMX_Crm));
+	IMX_Crm *crm = sg_new(IMX_Crm);
 	uint32_t osc32_freq;
-	if(!crm) {
-		fprintf(stderr,"Out of memory allocating crm\n");
-		exit(1);
-	}
-	memset(crm,0,sizeof(IMX_Crm));
-	crm->name = strdup(name);
+
+	crm->name = sg_strdup(name);
 	if(!crm->name) {
 		fprintf(stderr,"Out of memory for CRM name\n");
 		exit(1);
