@@ -28,16 +28,17 @@
 #include <unistd.h>
 #include <sys/ioctl.h>
 #include <sys/fcntl.h>
-#include <bus.h>
-#include <imx21_cspi.h>
-#include <signode.h>
-#include <cycletimer.h>
-#include <clock.h>
+#include "bus.h"
+#include "imx21_cspi.h"
+#include "signode.h"
+#include "cycletimer.h"
+#include "clock.h"
+#include "sgstring.h"
 
 #if 0
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 
 
@@ -232,7 +233,7 @@ rxdata_read(void *clientData,uint32_t address,int rqlen)
 	int rxcount;
 	uint32_t intreg;
 	uint32_t dmareg;
-	dprintf("RXDATA read 0x%08x\n",value);
+	dbgprintf("RXDATA read 0x%08x\n",value);
 	if(cspi->rxfifo_rp < cspi->rxfifo_wp) {
 		cspi->rxfifo_rp++;
 		rxcount = RXFIFO_COUNT(cspi); 
@@ -420,7 +421,7 @@ rxfifo_put(IMX21Cspi *cspi)
 	if(rxcount < RXFIFO_SIZE) {
 		cspi->rxfifo[cspi->rxfifo_wp % RXFIFO_SIZE] = cspi->in_shiftreg & mask;
 		cspi->rxfifo_wp++; rxcount ++;
-		dprintf("rxfifo_put %08x\n", cspi->in_shiftreg & mask);
+		dbgprintf("rxfifo_put %08x\n", cspi->in_shiftreg & mask);
 		intreg = cspi->intreg;
 		dmareg = cspi->dmareg;
 		intreg |= CSPI_INT_RR; /* receive ready */
@@ -742,10 +743,10 @@ run_programm(void *clientData)
 		if(result == PGM_DONE) {
 			/* Try to refill the shiftreg */
 			if(txfifo_to_shiftreg(cspi,1) <= 0) {
-				dprintf("Nothing to refill\n");
+				dbgprintf("Nothing to refill\n");
 				break;
 			}
-			dprintf("Refilled\n");
+			dbgprintf("Refilled\n");
 		} else if (result == PGM_SLEEP) {
 			break;
 		}
@@ -787,10 +788,10 @@ ctrl_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 	uint32_t diff;
 	diff = cspi->controlreg ^ value;
 	cspi->controlreg = (cspi->controlreg & CSPI_CTRL_XCH) | (value & 0x00ffffff);
-	dprintf("%s: Controlreg write %08x, modified to %08x\n",cspi->name,value,cspi->controlreg);
+	dbgprintf("%s: Controlreg write %08x, modified to %08x\n",cspi->name,value,cspi->controlreg);
 	update_spiclk(cspi);
 	if(diff & value  & CSPI_CTRL_XCH) {
-		dprintf("Start XCH\n");
+		dbgprintf("Start XCH\n");
 		txfifo_to_shiftreg(cspi,0);
 		run_programm(cspi);
 	}
@@ -949,14 +950,9 @@ IMXCspi_Map(void *owner,uint32_t base,uint32_t mask,uint32_t flags)
 BusDevice *
 IMX21_CspiNew(const char *name) 
 {
-	IMX21Cspi *cspi = malloc(sizeof(IMX21Cspi));	
+	IMX21Cspi *cspi = sg_new(IMX21Cspi);	
 	int i;
-	if(!cspi) {
-		fprintf(stderr,"out of memory allocating CSPI\n");
-		exit(1);
-	}	
-	memset(cspi,0,sizeof(IMX21Cspi));
-	cspi->name = strdup(name);
+	cspi->name = sg_strdup(name);
 	cspi->irqNode = SigNode_New("%s.irq",name);
 	if(!cspi->irqNode) {
 		fprintf(stderr,"Can not create interrupt request line for CSPI\n");
