@@ -30,16 +30,17 @@
 #include <unistd.h>
 #include <sys/ioctl.h>
 #include <sys/fcntl.h>
-#include <bus.h>
-#include <fio.h>
-#include <signode.h>
-#include <imx21_dmac.h>
-#include <configfile.h>
+#include "bus.h"
+#include "fio.h"
+#include "signode.h"
+#include "imx21_dmac.h"
+#include "configfile.h"
+#include "sgstring.h"
 
 #if 0
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 
 #define DMA_CH_BASE(base,x)  ((base) + 0x080 + 0x040 * (x))
@@ -223,19 +224,19 @@ update_interrupts(IMX21Dmac *dmac)
 	int i;
 	uint16_t ints = (dmac->disr|dmac->dbtosr|dmac->drtosr|dmac->dsesr|dmac->dbosr) 
 			& ~dmac->dimr; 
-	dprintf("DMAC: update ints %04x\n",ints);
+	dbgprintf("DMAC: update ints %04x\n",ints);
 	for(i=0;i<16;i++) {
 		DMAChan *chan = &dmac->dmachan[i];
 		if(ints & (1<<i)) {
 			if(!chan->interrupt_posted) {
 				chan->interrupt_posted = 1;
-				dprintf("Post an DMA int chan %d\n",chan->nr);
+				dbgprintf("Post an DMA int chan %d\n",chan->nr);
 				SigNode_Set(chan->irqNode,SIG_LOW);
 			}
 		} else {
 			if(chan->interrupt_posted) {
 				chan->interrupt_posted = 0;
-				dprintf("UnPost an DMA int chan %d\n",chan->nr);
+				dbgprintf("UnPost an DMA int chan %d\n",chan->nr);
 				SigNode_Set(chan->irqNode,SIG_HIGH);
 			}
 		}
@@ -379,7 +380,7 @@ do_dma(DMAChan *chan)
 	chan->stop_dma = 0;
 	int count = 0;
 	//int burst_len = chan->blr & 0x3f; 
-	dprintf("Entered DO-DMA\n");
+	dbgprintf("Entered DO-DMA\n");
 	smod = (chan->ccr & CCR_SMOD_MASK);
 	dmod = (chan->ccr & CCR_DMOD_MASK);
 	switch(chan->ccr & CCR_SSIZ_MASK) {
@@ -432,7 +433,7 @@ do_dma(DMAChan *chan)
 		dmac->disr |= (1<<chan->nr);
 		update_interrupts(dmac);
 	}
-	dprintf("**************** Leave Do DMA with CCNR %d of CNTR %d, disr 0x%04x, chan %d ssiz %d dsiz %d\n",chan->ccnr,chan->cntr,dmac->disr,chan->nr,ssiz,dsiz);
+	dbgprintf("**************** Leave Do DMA with CCNR %d of CNTR %d, disr 0x%04x, chan %d ssiz %d dsiz %d\n",chan->ccnr,chan->cntr,dmac->disr,chan->nr,ssiz,dsiz);
 }
 
 static void
@@ -445,24 +446,23 @@ check_dma(DMAChan *chan)
 	}
 }
 
-static int
+static void 
 change_dmareq(SigNode *node,int value,void *clientData)
 {
 	DMAChan *chan = (DMAChan *) clientData;
-	dprintf("DMAC: request line changed to state %d\n",value);
+	dbgprintf("DMAC: request line changed to state %d\n",value);
 	if(value == SIG_LOW) {
 		do_dma(chan);
 	} else {
 		chan->stop_dma = 1;
 	}
-	return 0;
 }
 
 static void
 disable_dmareq(DMAChan *chan) 
 {
 	if(chan->dmaReqTrace) {
-		dprintf("DMAC: UnTracing dma request line\n");
+		dbgprintf("DMAC: UnTracing dma request line\n");
 		SigNode_Untrace(chan->currReqLine,chan->dmaReqTrace);
 		chan->dmaReqTrace = NULL;
 	}
@@ -472,7 +472,7 @@ enable_dmareq(DMAChan *chan)
 {
 	IMX21Dmac *dmac = chan->dmac;
 	if(chan->currReqLine != dmac->dummyHighNode) {
-		dprintf("DMAC: Tracing dma request line\n");
+		dbgprintf("DMAC: Tracing dma request line\n");
 		chan->dmaReqTrace = SigNode_Trace(chan->currReqLine,change_dmareq,chan);
 		check_dma(chan);
 	}
@@ -488,7 +488,7 @@ update_dmareq(DMAChan *chan)
 	if(chan->ccr & (CCR_REN && CCR_CEN)) {
 		enable_dmareq(chan);
 	} else {
-		dprintf("disable dmareq in %d\n",__LINE__);
+		dbgprintf("disable dmareq in %d\n",__LINE__);
 		disable_dmareq(chan);	
 	}
 }
@@ -885,6 +885,12 @@ blr_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
         return;
 }
 
+/*
+ * ---------------------------------------------------------------
+ * Request Timeout / Bus Utilization control register
+ * Currently has no effect in emulation
+ * ---------------------------------------------------------------
+ */
 
 static uint32_t
 rtor_bucr_read(void *clientData,uint32_t address,int rqlen)
@@ -906,7 +912,6 @@ rtor_bucr_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 	} else {
         	chan->bucr = value;
 	}
-        fprintf(stderr,"IMX21 DMAC: rtor/bucr not implemented\n");
         return;
 }
 
@@ -993,13 +998,8 @@ IMXDmac_Map(void *owner,uint32_t base,uint32_t mask,uint32_t mapflags)
 
 BusDevice *
 IMX21_DmacNew(const char *name) {
-        IMX21Dmac *dmac = malloc(sizeof(IMX21Dmac));
+        IMX21Dmac *dmac = sg_new(IMX21Dmac);
 	int i;
-        if(!dmac) {
-                fprintf(stderr,"Out of memory for IMX USB OTG\n");
-                exit(1);
-        }
-        memset(dmac,0,sizeof(IMX21Dmac));
 	dmac->dummyHighNode = SigNode_New("%s.high",name);
 	if(!dmac->dummyHighNode) {
 		fprintf(stderr,"Can not create dummy high node for DMAC\n");
