@@ -28,9 +28,10 @@
 #include <unistd.h>
 #include <sys/ioctl.h>
 #include <sys/fcntl.h>
-#include <bus.h>
-#include <signode.h>
+#include "bus.h"
+#include "signode.h"
 #include "imx21_eim.h"
+#include "sgstring.h"
 
 #define EIM_CS0U(base)		((base) + 0)
 #define 	CSxU_SP			(1<<31)
@@ -373,12 +374,7 @@ IMX21_EimNew(const char *name)
 {
 	IMX21Eim *eim;
 	int i;
-	eim = malloc(sizeof(IMX21Eim));
-        if(!eim) {
-                fprintf(stderr,"Out of memory for IMX21 EIM\n");
-                exit(1);
-        }
-        memset(eim,0,sizeof(IMX21Eim));
+	eim = sg_new(IMX21Eim);
 	eim->bdev.first_mapping=NULL;
         eim->bdev.Map=IMXEim_Map;
         eim->bdev.UnMap=IMXEim_Unmap;
