@@ -36,11 +36,12 @@
 #include <unistd.h>
 #include <sys/ioctl.h>
 #include <sys/fcntl.h>
-#include <bus.h>
-#include <fio.h>
-#include <signode.h>
-#include <imx21_gpio.h>
-#include <configfile.h>
+#include "bus.h"
+#include "fio.h"
+#include "signode.h"
+#include "imx21_gpio.h"
+#include "configfile.h"
+#include "sgstring.h"
 
 //define IMX_GPIO_BASE              (0x15000 + IMX_IO_BASE)
 
@@ -1034,7 +1035,7 @@ IMX21Gpio_Unmap(void *owner,uint32_t base,uint32_t mask)
         IOH_Delete32(GPIO_PMASK(base));
 }
 
-static int
+static void 
 gin_trace_proc(struct SigNode * node,int value, void *clientData)
 {
         Gin_TraceInfo *ti = clientData;
@@ -1058,7 +1059,6 @@ gin_trace_proc(struct SigNode * node,int value, void *clientData)
 			}
 			break;
 	}
-	return 0;
 }
 
 
@@ -1101,7 +1101,7 @@ create_port(GpioPort *port,int index,const char *modulename)
 		ti->port = port;
 		ti->index = i;
 		if(pin_info->pin_name) {
-			port->cpuPinNode[i] = SigNode_New(pin_info->pin_name);
+			port->cpuPinNode[i] = SigNode_New("%s",pin_info->pin_name);
 			port->ginTrace[i] = SigNode_Trace(port->cpuPinNode[i],gin_trace_proc,ti);
 		}
 		port->gpioNode[i] = SigNode_New("%s.%d",portname,i);
@@ -1113,7 +1113,7 @@ create_port(GpioPort *port,int index,const char *modulename)
 	}
 }
 
-UNUSED static int
+__UNUSED__ static int
 pin_trace(struct SigNode * node,int value, void *clientData)
 {
 	fprintf(stderr,"Pin changed to %d\n",value);
@@ -1125,12 +1125,7 @@ IMX21_GpioNew(const char *modulename)
 {
         IMX21Gpio *gpio;
 	int i;
-        gpio = malloc(sizeof(IMX21Gpio));
-        if(!gpio) {
-                fprintf(stderr,"Out of memory for IMX GPIO module\n");
-                exit(1);
-        }
-        memset(gpio,0,sizeof(IMX21Gpio));
+        gpio = sg_new(IMX21Gpio);
         gpio->irqNode = SigNode_New("%s.irq",modulename); // irq output
         if(!gpio->irqNode) {
                 fprintf(stderr,"Can not create irq node\n");
