@@ -37,12 +37,13 @@
 #include <string.h>
 #include <unistd.h>
 #include <sys/fcntl.h>
-#include <bus.h>
-#include <fio.h>
-#include <signode.h>
-#include <imx21_i2c.h>
-#include <configfile.h>
-#include <cycletimer.h>
+#include "bus.h"
+#include "fio.h"
+#include "signode.h"
+#include "imx21_i2c.h"
+#include "configfile.h"
+#include "cycletimer.h"
+#include "sgstring.h"
 
 #define IADR(base) ((base)+0)
 #define IFDR(base) ((base)+4)
@@ -186,7 +187,7 @@ scl_timeout(void *clientData)
  *      remove the timer and continue running the interpreter
  * -------------------------------------------------------------------
  */
-static int
+static void 
 scl_released(SigNode *node,int value,void *clientData)
 {
         IMX21I2c *i2c = (IMX21I2c *)clientData;
@@ -195,7 +196,6 @@ scl_released(SigNode *node,int value,void *clientData)
                 CycleTimer_Remove(&i2c->ndelayTimer);
                 run_interpreter(clientData);
         }
-        return 0;
 }
 
 /*
@@ -210,6 +210,9 @@ execute_instruction(IMX21I2c *i2c) {
                 fprintf(stderr,"i.MX21 I2C: corrupt I2C script\n");
                 return RET_INTERP_ERROR;
         }
+	if(i2c->ip == i2c->icount) {
+		return RET_DONE;
+	}
         icode = i2c->code[i2c->ip++];
         switch(icode&0xff000000) {
                 case INSTR_SDA_H:
@@ -529,7 +532,7 @@ mscript_write_byte(IMX21I2c * i2c,uint8_t data)
 static void
 start_script(IMX21I2c *i2c) {
         if(CycleTimer_IsActive(&i2c->ndelayTimer)) {
-                fprintf(stderr,"NS-I2C Emulator bug: Starting script which already runs\n");
+                fprintf(stderr,"imx i2c emulator bug: Starting script which already runs\n");
                 return;
         }
         CycleTimer_Add(&i2c->ndelayTimer,0,run_interpreter,i2c);
@@ -541,7 +544,7 @@ start_script(IMX21I2c *i2c) {
  * Monitor bus Status (start/stop conditions)
  * --------------------------------------------------------
  */
-static int
+static void 
 sda_trace_proc(struct SigNode * node,int value, void *clientData)
 {
         IMX21I2c *i2c = (IMX21I2c*)clientData;
@@ -562,7 +565,6 @@ sda_trace_proc(struct SigNode * node,int value, void *clientData)
                         CycleTimer_Add(&i2c->ndelayTimer,cycles,run_interpreter,i2c);
                 }
         }
-        return 0;
 }
 
 
@@ -704,12 +706,7 @@ IMXI2c_Map(void *owner,uint32_t base,uint32_t mask,uint32_t mapflags)
 BusDevice *
 IMX21_I2cNew(const char *name) {
 	I2C_Timing *timing;
-        IMX21I2c *i2c = malloc(sizeof(IMX21I2c));
-        if(!i2c) {
-                fprintf(stderr,"Out of memory for IMX21 SD-Card controller\n");
-                exit(1);
-        }
-        memset(i2c,0,sizeof(IMX21I2c));
+        IMX21I2c *i2c = sg_new(IMX21I2c);
 	timing = &i2c->i2c_timing;
        	i2c->irqNode = SigNode_New("%s.irq",name);
         if(!i2c->irqNode) {
