@@ -30,15 +30,16 @@
 #include <termios.h>
 #include <sys/ioctl.h>
 #include <sys/fcntl.h>
-#include <bus.h>
-#include <fio.h>
-#include <signode.h>
-#include <imx21_lcdc.h>
-#include <rfbserver.h>
-#include <configfile.h>
-#include <cycletimer.h>
-#include <fbdisplay.h>
-#include <clock.h>
+#include "bus.h"
+#include "fio.h"
+#include "signode.h"
+#include "imx21_lcdc.h"
+#include "rfbserver.h"
+#include "configfile.h"
+#include "cycletimer.h"
+#include "fbdisplay.h"
+#include "clock.h"
+#include "sgstring.h"
 
 #if 0
 #define dbgprintf(x...) { fprintf(stderr,x); }
@@ -1102,12 +1103,11 @@ IMXLcdc_Map(void *owner,uint32_t base,uint32_t mask,uint32_t flags)
 BusDevice *
 IMX21_LcdcNew(const char *name,FbDisplay *display)
 {
-	IMXLcdc *lcdc = (IMXLcdc *) malloc(sizeof(IMXLcdc));
-	if(!lcdc) {
-		fprintf(stderr,"Out of memory for LCDC\n");
+	IMXLcdc *lcdc = sg_new(IMXLcdc);
+	if(!display) {
+		fprintf(stderr,"i.MX21 LCD controller requires a valid display\n");
 		exit(1);
 	}
-	memset(lcdc,0,sizeof(IMXLcdc));
 	lcdc->irqNode = SigNode_New("%s.irq",name);
 	if(!lcdc->irqNode) {
 		fprintf(stderr,"LCDC: creation of irq node failed\n");
