@@ -28,13 +28,14 @@
 #include <unistd.h>
 #include <sys/ioctl.h>
 #include <sys/fcntl.h>
-#include <bus.h>
-#include <fio.h>
-#include <imx21_pwm.h>
-#include <signode.h>
-#include <configfile.h>
-#include <clock.h>
-#include <cycletimer.h>
+#include "bus.h"
+#include "fio.h"
+#include "imx21_pwm.h"
+#include "signode.h"
+#include "configfile.h"
+#include "clock.h"
+#include "cycletimer.h"
+#include "sgstring.h"
 
 #define PWMC(base) ((base)+0x0)
 #define		PWMC_HCTR	(1<<18)
@@ -178,12 +179,7 @@ BusDevice *
 IMX21_PwmNew(const char *name) 
 {
 	IMX21Pwm *pwm;
-	pwm = malloc(sizeof(*pwm));
-	if(!pwm) {
-		fprintf(stderr,"Out of memory error allocating IMX21Pwm\n");
-		exit(1);
-	}
-	memset(pwm,0,sizeof(*pwm));
+	pwm = sg_new(IMX21Pwm);
 	pwm->pwmc = 0x000000a0;
 	pwm->pwmp = 0xfffe;
 	pwm->pwmcnt = 0;
