@@ -29,14 +29,15 @@
 #include <time.h>
 #include <sys/ioctl.h>
 #include <sys/fcntl.h>
-#include <bus.h>
-#include <fio.h>
-#include <signode.h>
-#include <imx21_rtc.h>
-#include <configfile.h>
-#include <clock.h>
-#include <cycletimer.h>
-#include <diskimage.h>
+#include "bus.h"
+#include "fio.h"
+#include "signode.h"
+#include "imx21_rtc.h"
+#include "configfile.h"
+#include "clock.h"
+#include "cycletimer.h"
+#include "diskimage.h"
+#include "sgstring.h"
 
 
 #define RTC_HOURMIN(base) 	((base) + 0x00)
@@ -379,12 +380,7 @@ BusDevice *
 IMX21_RtcNew(const char *name)
 {
 	char *dirname,*imagename;
-        IMX21Rtc *rtc = (IMX21Rtc*)malloc(sizeof(IMX21Rtc));
-        if(!rtc) {
-                fprintf(stderr,"Out of memory allocating rtc\n");
-                exit(1);
-        }
-        memset(rtc,0,sizeof(IMX21Rtc));
+        IMX21Rtc *rtc = sg_new(IMX21Rtc);
 	rtc->irqNode = SigNode_New("%s.irq",name);
         if(!rtc->irqNode) {
                 fprintf(stderr,"i.MX21 RTC: can not create interrupt line\n");
@@ -397,12 +393,12 @@ IMX21_RtcNew(const char *name)
                 rtc->disk_image = DiskImage_Open(imagename,4,DI_RDWR | DI_CREAT_00);
                 if(!rtc->disk_image) {
                         fprintf(stderr,"Failed to open PCF8563 time offset file\n");
-                        rtc->time_offset = malloc(4);
+                        rtc->time_offset = sg_calloc(4);
                 } else {
                         rtc->time_offset = DiskImage_Mmap(rtc->disk_image);
                 }
         } else {
-                rtc->time_offset = malloc(4);
+                rtc->time_offset = sg_calloc(4);
         }
 	rtc->bdev.first_mapping=NULL;
         rtc->bdev.Map=IMXRtc_Map;
