@@ -29,19 +29,20 @@
 #include <unistd.h>
 #include <sys/ioctl.h>
 #include <sys/fcntl.h>
-#include <bus.h>
-#include <fio.h>
-#include <signode.h>
-#include <imx21_sdhc.h>
-#include <configfile.h>
-#include <mmcard.h>
-#include <cycletimer.h>
-#include <clock.h>
+#include "bus.h"
+#include "fio.h"
+#include "signode.h"
+#include "imx21_sdhc.h"
+#include "configfile.h"
+#include "mmcard.h"
+#include "cycletimer.h"
+#include "clock.h"
+#include "sgstring.h"
 
 #if 0
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 #define NEW
 
@@ -140,6 +141,8 @@ typedef struct IMX21Sdhc {
 	CycleTimer cmd_delay_timer;
 	CycleTimer write_delay_timer;
 	CycleTimer data_delay_timer;
+	/* If clock is not running one Timer can be made waiting for clock */
+	CycleTimer *forClockWaitingTimer;
 	uint16_t str_stp_clk;	
 	uint16_t status;
 	uint16_t clk_rate;
@@ -180,14 +183,13 @@ onecount(const uint32_t value)
         return ones;
 }
 
-static uint32_t 
+static void 
 update_clock(IMX21Sdhc *sdhc) 
 {
 	uint32_t clk_divider = sdhc->clk_rate & 0xf;
 	uint32_t clk_prescaler = (sdhc->clk_rate >> 4) & 0xfff;
 	uint32_t divider;
 	uint32_t prescaler;
-	uint32_t freq;
 	if((onecount(clk_prescaler) > 1)) {
 		fprintf(stderr,"%s: Illegal Clock Prescaler 0x%03x\n",sdhc->name,clk_prescaler);
 	}
@@ -200,10 +202,9 @@ update_clock(IMX21Sdhc *sdhc)
 		prescaler = 1;
 	}
 	divider = (clk_divider+1) * (prescaler);
+	/* Bad: should check if clock is enabled at all */
 	Clock_MakeDerived(sdhc->sdclk,sdhc->inclk,1,divider);
-	freq = Clock_Freq(sdhc->sdclk);
 	//fprintf(stderr,"%s: DIVIDER %d value 0x%08x rate %d\n",sdhc->name,divider,sdhc->clk_rate,  freq);
-	return freq;
 }
 
 static void
@@ -239,10 +240,10 @@ update_interrupt(IMX21Sdhc *sdhc)
 		interrupt = 1;
 	}
 	if(interrupt) {
-		dprintf("Interrupt status %08x int_status %08x int_cntl %08x\n",sdhc->status,sdhc->int_status,sdhc->int_cntl);
+		dbgprintf("Interrupt status %08x int_status %08x int_cntl %08x\n",sdhc->status,sdhc->int_status,sdhc->int_cntl);
 		SigNode_Set(sdhc->irqNode,SIG_LOW);
 	} else {
-		dprintf("No Interrupt status %08x int_status %08x int_cntl %08x\n",sdhc->status,sdhc->int_status,sdhc->int_cntl);
+		dbgprintf("No Interrupt status %08x int_status %08x int_cntl %08x\n",sdhc->status,sdhc->int_status,sdhc->int_cntl);
 		SigNode_Set(sdhc->irqNode,SIG_HIGH);
 	}	
 }
@@ -316,7 +317,17 @@ str_stp_clk_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 		sdhc_reset(sdhc);
 	}
 	if(value & STR_STP_CLK_START_CLK) {
-		sdhc->status |= STATUS_CARD_BUS_CLK_RUN; 
+		if(!(sdhc->status & STATUS_CARD_BUS_CLK_RUN)) {
+			sdhc->status |= STATUS_CARD_BUS_CLK_RUN; 
+			if(sdhc->forClockWaitingTimer == &sdhc->data_delay_timer) {
+				CycleTimer_Mod(&sdhc->data_delay_timer,MicrosecondsToCycles(UDELAY_DATA));
+			} else if(sdhc->forClockWaitingTimer == &sdhc->cmd_delay_timer) {
+				CycleTimer_Mod(&sdhc->cmd_delay_timer,MicrosecondsToCycles(UDELAY_CMD));
+			} else if(sdhc->forClockWaitingTimer == &sdhc->write_delay_timer) {
+				CycleTimer_Mod(&sdhc->write_delay_timer,MicrosecondsToCycles(UDELAY_WRITE));
+			}
+			sdhc->forClockWaitingTimer = NULL;
+		}
 	} else if(value & STR_STP_CLK_STOP_CLK) {
 		/* Clock run should be disabled delayed (after transmission) */
 		sdhc->status &= ~(STATUS_CARD_BUS_CLK_RUN | STATUS_TIME_OUT_READ 
@@ -435,6 +446,7 @@ static void
 do_transfer_write(void *clientData) 
 {
 	IMX21Sdhc *sdhc = (IMX21Sdhc *) clientData;
+	MMC_DataBlock dataBlock;
 	uint8_t buf[64];
 	int count;  
 	int result;
@@ -476,7 +488,9 @@ do_transfer_write(void *clientData)
 	}
 	if(!sdhc->card)
 		return;
-	result = MMCard_Write(sdhc->card,buf,count);
+	dataBlock.data = buf;
+	dataBlock.datalen = count;
+	result = MMCard_Write(sdhc->card,&dataBlock);
 	/* What to do with result ??? */
 	if(sdhc->transfer_count == rqlen) {
 		/* 
@@ -512,9 +526,11 @@ do_transfer_write(void *clientData)
  * ----------------------------------------------------------------
  */
 static int 
-receive_data(void *dev,uint8_t *buf,int len) 
+receive_data(void *dev,MMC_DataBlock *dataBlock) 
 {
 	IMX21Sdhc *sdhc = (IMX21Sdhc *)dev;
+	uint8_t *buf = dataBlock->data;
+	int len = dataBlock->datalen;
 	int i;
 	/* Read direction */
 	int fifobytes;
@@ -548,7 +564,7 @@ receive_data(void *dev,uint8_t *buf,int len)
 		fprintf(stderr,"Bug in %s line %d\n",__FILE__,__LINE__);
 		exit(1);
 	}
-	dprintf("mmc read result %d bytes\n",result); 
+	dbgprintf("mmc read result %d bytes\n",result); 
 	if(len > 0 && (len <= fifobytes)) {
 		sdhc->status &= ~STATUS_APPL_BUFF_FE;
 		for(i=0;i<len;i+=2) {
@@ -587,7 +603,7 @@ static void
 do_data_delayed(void *clientData) {
 	IMX21Sdhc *sdhc = (IMX21Sdhc*) clientData;
 	if(sdhc->cmd_dat_cont & CMD_DAT_CONT_DATA_ENABLE) {
-		dprintf("Data involved in transfer\n");
+		dbgprintf("Data involved in transfer\n");
 		if((sdhc->cmd_dat_cont & CMD_DAT_CONT_WRITE)) {
 			/* should check fifo full before ? */
 			// if(full)do_transfer_write(sdhc);
@@ -611,7 +627,7 @@ do_cmd_delayed(void *clientData)
 	uint32_t arg;
 	int i;
 	if(!sdhc->card) {
-		dprintf("No card found in slot\n");
+		dbgprintf("No card found in slot\n");
 		sdhc->status |=  STATUS_END_CMD_RESP; 
 		sdhc->status |= STATUS_TIME_OUT_RESP; 
 		sdhc->int_status |= STATUS_END_CMD_RESP;
@@ -619,7 +635,7 @@ do_cmd_delayed(void *clientData)
 		return;
 	}
 	arg = sdhc->argl | ((uint32_t)sdhc->argh<<16);
-	dprintf("Start cmd 0x%02x \n",sdhc->cmd);
+	dbgprintf("Start cmd 0x%02x \n",sdhc->cmd);
 	result = MMCard_DoCmd(sdhc->card,sdhc->cmd,arg,&resp);
 	if(result != MMC_ERR_NONE) {
 		sdhc->status |=  STATUS_END_CMD_RESP; 
@@ -643,10 +659,10 @@ do_cmd_delayed(void *clientData)
 		for(i=1;i<17;i+=2) {
 			sdhc->res_fifo[sdhc->res_fifo_wp % RES_FIFO_SIZE] =
 				(resp.data[i] << 8) | (resp.data[i+1]<<0);
-			dprintf("%04x ",sdhc->res_fifo[sdhc->res_fifo_wp % RES_FIFO_SIZE] );
+			dbgprintf("%04x ",sdhc->res_fifo[sdhc->res_fifo_wp % RES_FIFO_SIZE] );
 			sdhc->res_fifo_wp++;
 		}
-		dprintf("\n");
+		dbgprintf("\n");
 	} else if (rformat == CMD_DAT_CONT_NO_RESPONSE) {
 		/* put nothing to fifo */
 	} else {
@@ -663,8 +679,13 @@ do_cmd_delayed(void *clientData)
 	 */
 	if(sdhc->cmd_dat_cont & CMD_DAT_CONT_DATA_ENABLE) {
 		sdhc->transfer_count = 0;
-		if((sdhc->cmd_dat_cont & CMD_DAT_CONT_WRITE)) {
-			CycleTimer_Mod(&sdhc->data_delay_timer,MicrosecondsToCycles(UDELAY_DATA));
+		if(sdhc->cmd_dat_cont & CMD_DAT_CONT_WRITE) {
+			if(sdhc->status & STATUS_CARD_BUS_CLK_RUN) {
+				CycleTimer_Mod(&sdhc->data_delay_timer,MicrosecondsToCycles(UDELAY_DATA));
+			} else {
+				//fprintf(stderr,"Delay Data transaction until clock is running\n");
+				sdhc->forClockWaitingTimer = &sdhc->data_delay_timer;
+			}
 		} 
 	}
 }
@@ -682,8 +703,12 @@ cmd_dat_cont_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 		fprintf(stderr,"SDHC: Response fifo not empty cmd %d, prev %d, rp %d, wp %d\n",sdhc->cmd,sdhc->prev_cmd,(int)sdhc->res_fifo_rp,(int)sdhc->res_fifo_wp);
 		sdhc->res_fifo_rp = sdhc->res_fifo_wp;
 	}
-	/* Bad ! should wait until clock is started */ 
-	CycleTimer_Mod(&sdhc->cmd_delay_timer,MicrosecondsToCycles(UDELAY_CMD));
+	if(sdhc->status & STATUS_CARD_BUS_CLK_RUN) {
+		//fprintf(stderr,"Clock already running on start cmd %02x\n",sdhc->cmd);
+		CycleTimer_Mod(&sdhc->cmd_delay_timer,MicrosecondsToCycles(UDELAY_CMD));
+	} else {
+		sdhc->forClockWaitingTimer = &sdhc->cmd_delay_timer;
+	}
         return;
 }
 
@@ -876,7 +901,7 @@ res_fifo_read(void *clientData,uint32_t address,int rqlen)
 	if(sdhc->res_fifo_rp  < sdhc->res_fifo_wp) {
 		sdhc->res_fifo_rp++;
 	}
-	dprintf("res fifo read 0x%04x\n",value);
+	dbgprintf("res fifo read 0x%04x\n",value);
         return value;
 }
 
@@ -907,7 +932,7 @@ buffer_access_read(void *clientData,uint32_t address,int rqlen)
 		if(sdhc->buf_fifo_rp == sdhc->buf_fifo_wp) {
 			sdhc->status = sdhc->status | STATUS_APPL_BUFF_FE;
 			sdhc->int_status = sdhc->int_status | STATUS_APPL_BUFF_FE;
-			dprintf("Release DMA request\n");
+			dbgprintf("Release DMA request\n");
 			SigNode_Set(sdhc->nDmaReq,SIG_HIGH);
 			update_interrupt(sdhc);
 		}
@@ -932,10 +957,15 @@ buffer_access_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 		if((sdhc->buf_fifo_wp - sdhc->buf_fifo_rp) == fifosize) {
 			sdhc->status = sdhc->status | STATUS_APPL_BUFF_FF;
 			sdhc->int_status = sdhc->int_status | STATUS_APPL_BUFF_FF;
-			dprintf("Stop the DMA request because of full fifo\n");
+			dbgprintf("Stop the DMA request because of full fifo\n");
 			SigNode_Set(sdhc->nDmaReq,SIG_HIGH);
 			update_interrupt(sdhc);
-			CycleTimer_Mod(&sdhc->write_delay_timer,MicrosecondsToCycles(UDELAY_WRITE));
+			if(sdhc->status & STATUS_CARD_BUS_CLK_RUN) {
+				CycleTimer_Mod(&sdhc->write_delay_timer,MicrosecondsToCycles(UDELAY_WRITE));
+			} else {
+				//fprintf(stderr,"Delay WRITE transaction until clock is running\n");
+				sdhc->forClockWaitingTimer = &sdhc->write_delay_timer;
+			}
 		}
 	} else {
 		fprintf(stderr,"Sdhc: Buffer access fifo write overflow\n");
@@ -1013,14 +1043,9 @@ IMX21Sdhc_RemoveCard(BusDevice *dev,MMCard *card)
 
 BusDevice *
 IMX21_SdhcNew(const char *name) {
-	IMX21Sdhc *sdhc = malloc(sizeof(IMX21Sdhc));
-        if(!sdhc) {
-                fprintf(stderr,"Out of memory for IMX21 SD-Card controller\n");
-                exit(1);
-        }
-        memset(sdhc,0,sizeof(IMX21Sdhc));
+	IMX21Sdhc *sdhc = sg_new(IMX21Sdhc);
 	sdhc->irqNode = SigNode_New("%s.irq",name);
-	sdhc->name = strdup(name);
+	sdhc->name = sg_strdup(name);
 	if(!sdhc->irqNode) {
 		fprintf(stderr,"i.MX21 SDHC: can not create interrupt line\n");
 		exit(1);
@@ -1039,7 +1064,7 @@ IMX21_SdhcNew(const char *name) {
         sdhc->bdev.hw_flags=MEM_FLAG_WRITABLE|MEM_FLAG_READABLE;
 	sdhc->inclk = Clock_New("%s.inclk",name);
 	sdhc->sdclk = Clock_New("%s.sdclk",name);
-	/* Documentation (big piece of shit) has contradictions about clock source for SD-Clk */
+	/* Documentation has contradictions about clock source for SD-Clk */
 	Clock_SetFreq(sdhc->inclk,66500013.419/2); /* Should be connected to clock module ipg_perclk or perclk2  */
 	CycleTimer_Init(&sdhc->cmd_delay_timer,do_cmd_delayed,sdhc);
 	CycleTimer_Init(&sdhc->data_delay_timer,do_data_delayed,sdhc);
