@@ -28,9 +28,10 @@
 #include <unistd.h>
 #include <sys/ioctl.h>
 #include <sys/fcntl.h>
-#include <bus.h>
-#include <signode.h>
+#include "bus.h"
+#include "signode.h"
 #include "imx21_sdrc.h"
+#include "sgstring.h"
 
 #define SDRC_SDCTL0(base)	((base)+0)
 #define SDRC_SDCTL1(base)	((base)+0x4)
@@ -171,12 +172,7 @@ BusDevice *
 IMX21_SdrcNew(const char *name,BusDevice *dram0,BusDevice *dram1)
 {
         IMX21Sdrc *sdrc;
-        sdrc = malloc(sizeof(*sdrc));
-        if(!sdrc) {
-                fprintf(stderr,"Out of memory for IMX21 SDRC\n");
-                exit(1);
-        }
-        memset(sdrc,0,sizeof(*sdrc));
+        sdrc = sg_new(IMX21Sdrc);
 	sdrc->dram[0] = dram0;
 	sdrc->dram[1] = dram1;
 	sdrc->sdctl[0] = 0x01000300;
