@@ -28,14 +28,16 @@
 #include <unistd.h>
 #include <sys/ioctl.h>
 #include <sys/fcntl.h>
-#include <bus.h>
-#include <fio.h>
-#include <signode.h>
-#include <clock.h>
-#include <imx21_uart.h>
-#include <serial.h>
-#include <configfile.h>
-#include <senseless.h>
+#include "bus.h"
+#include "fio.h"
+#include "signode.h"
+#include "clock.h"
+#include "imx21_uart.h"
+#include "serial.h"
+#include "configfile.h"
+#include "senseless.h"
+#include "cycletimer.h"
+#include "sgstring.h"
 
 #define URR(base)	((base) + 0x0)
 #define		URR_CHARRDY	(1<<15)
@@ -180,6 +182,7 @@
 #define TX_FIFO_SIZE (32)
 #define TX_FIFO_MASK (TX_FIFO_SIZE-1)
 #define TX_FIFO_COUNT(ser) ((ser)->txfifo_wp-(ser)->txfifo_rp)
+#define TX_FIFO_RELEASED(ser) ((ser)->txfifo_releasep-(ser)->txfifo_rp)
 #define TX_FIFO_ROOM(ser) (TX_FIFO_SIZE-TX_FIFO_COUNT(ser))
 #define TX_FIFO_WIDX(iuart) ((iuart)->txfifo_wp & TX_FIFO_MASK)
 #define TX_FIFO_RIDX(iuart) ((iuart)->txfifo_rp & TX_FIFO_MASK)
@@ -204,7 +207,9 @@ typedef struct IMX_Uart {
 
 	uint8_t txfifo[TX_FIFO_SIZE];
 	int64_t txfifo_wp;
+	int64_t txfifo_releasep; 
         int64_t txfifo_rp;
+	int nsecs_per_char;
 
 	uint16_t rxfifo[RX_FIFO_SIZE];
 	int64_t rxfifo_wp;
@@ -214,6 +219,7 @@ typedef struct IMX_Uart {
         SigNode *irqNode;
 	SigNode *rxDmaReqNode;
 	SigNode *txDmaReqNode;
+	CycleTimer baudTimer;
 } IMX_Uart;
 
 
@@ -358,12 +364,12 @@ serial_input(void *cd) {
 static void 
 serial_output(void *cd) {
         IMX_Uart *iuart=cd;
-        int fill;
-        while(TX_FIFO_COUNT(iuart) > 0) {
+        int released;
+        while(TX_FIFO_RELEASED(iuart) > 0) {
                 int count,len;
-                fill=TX_FIFO_COUNT(iuart);
-                len=fill;
-                if((TX_FIFO_RIDX(iuart)+fill) > TX_FIFO_SIZE) {
+                released=TX_FIFO_RELEASED(iuart);
+                len=released;
+                if((TX_FIFO_RIDX(iuart)+released) > TX_FIFO_SIZE) {
                         len = TX_FIFO_SIZE - TX_FIFO_RIDX(iuart);
                 }
                 count=SerialDevice_Write(iuart->port,&iuart->txfifo[TX_FIFO_RIDX(iuart)],len);
@@ -377,6 +383,40 @@ serial_output(void *cd) {
         return;
 }
 
+static void
+update_rx(IMX_Uart *iuart) 
+{
+	if(iuart->ucr2 & UCR2_RXEN) {
+		int room = RX_FIFO_ROOM(iuart);
+		if(room > 0) {
+			if(!(iuart->ucr3 & UCR3_RXDMUXSEL)) {
+				fprintf(stderr,"Driver bug, starting rx with DMUXSEL 0\n");
+			} else {
+				SerialDevice_StartRx(iuart->port);
+			}
+		}
+	} else {
+		SerialDevice_StopRx(iuart->port);
+	}
+}
+
+/*
+ * ------------------------------------------------------------------
+ * This releases a char in the txfifo for beeing sent
+ * ------------------------------------------------------------------
+ */
+static void
+IUart_ReleaseChar(void *clientData)
+{
+	IMX_Uart *iuart = (IMX_Uart *)clientData;
+	if(iuart->txfifo_releasep < iuart->txfifo_wp) {
+		iuart->txfifo_releasep++;
+		SerialDevice_StartTx(iuart->port);
+	} 	
+	if(iuart->txfifo_releasep < iuart->txfifo_wp) {
+		CycleTimer_Mod(&iuart->baudTimer,NanosecondsToCycles(iuart->nsecs_per_char));
+	}
+}
 
 static uint32_t
 urr_read(void *clientData,uint32_t address,int rqlen)
@@ -389,9 +429,9 @@ urr_read(void *clientData,uint32_t address,int rqlen)
 		iuart->rxfifo_rp++; 
 		update_interrupts(iuart);
 		update_rxdma(iuart);
-		/* Maybe rx was stopped because fifo was full */
-		if((count == RX_FIFO_SIZE) && (iuart->ucr2 & UCR2_RXEN)) { 
-			SerialDevice_StartRx(iuart->port); 
+		/* Maybe rx was stopped because fifo was full, so reenable it */
+		if((count == RX_FIFO_SIZE)) { 
+			update_rx(iuart);
 		}
 	} else {
 		urr = urr &~ URR_CHARRDY;
@@ -428,7 +468,9 @@ utr_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 	iuart->txfifo[TX_FIFO_WIDX(iuart)] = value;	
 	iuart->txfifo_wp += 1;
 	if(iuart->ucr2 & UCR2_TXEN) {
-		SerialDevice_StartTx(iuart->port);
+		if(!CycleTimer_IsActive(&iuart->baudTimer)) {
+			CycleTimer_Mod(&iuart->baudTimer,NanosecondsToCycles(iuart->nsecs_per_char));
+		}
 	} else {
 		fprintf(stderr,"i.MX21 UART: Writing to TX register of %s while disabled\n",iuart->name );
 	}
@@ -469,27 +511,19 @@ ucr2_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 	iuart->ucr2 = value | UCR2_SRST;
 	if(diff & UCR2_TXEN) {
 		if(value & UCR2_TXEN) {
-			SerialDevice_StartTx(iuart->port);
+			if(!CycleTimer_IsActive(&iuart->baudTimer)) {
+				CycleTimer_Mod(&iuart->baudTimer,NanosecondsToCycles(iuart->nsecs_per_char));
+			}
 		} else {
-			SerialDevice_StopTx(iuart->port);
+			CycleTimer_Remove(&iuart->baudTimer);
 		}
 	}
 	if(diff & UCR2_RXEN) {
-		if(value & UCR2_RXEN) {
-			int room = RX_FIFO_ROOM(iuart);
-			if(room > 0) {
-				if(!(iuart->ucr3 & UCR3_RXDMUXSEL)) {
-					fprintf(stderr,"Driver bug, starting rx with DMUXSEL 0\n");
-				} else {
-					SerialDevice_StartRx(iuart->port);
-				}
-			}
-		} else {
-			SerialDevice_StopRx(iuart->port);
-		}
+		update_rx(iuart);
 	}
 	if(!(value & UCR2_SRST)) {
 		iuart->txfifo_wp  =  iuart->txfifo_rp = 0; 
+		iuart->txfifo_releasep = 0;
 		iuart->rxfifo_wp =   iuart->rxfifo_rp = 0; 
 		iuart->usr1 = USR1_RXDS | USR1_TRDY;
 		iuart->usr2 = USR2_TXFE | USR2_TXDC;
@@ -548,7 +582,11 @@ static void
 ucr3_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 {
 	IMX_Uart *iuart = (IMX_Uart *) clientData;
+	uint32_t diff = value ^ iuart->ucr3;
 	iuart->ucr3 = value;
+	if((diff & UCR3_RXDMUXSEL)) {
+		update_rx(iuart);
+	}
         return;
 }
 
@@ -669,7 +707,7 @@ usr2_read(void *clientData,uint32_t address,int rqlen)
 		usr2 |= USR2_DCDDELT;
 	}
 	iuart->usr2 = usr2;
-	if((usr2 & USR2_TXFE) && (~usr2 & USR2_RDR)) {
+	if(!(usr2 & USR2_TXFE) || (~usr2 & USR2_RDR)) {
 		Senseless_Report(150);
 	}
 	return iuart->usr2;
@@ -754,6 +792,15 @@ ubmr_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 	cmd.opcode = UART_OPC_SET_BAUDRATE;
 	cmd.arg = baudrate;
 	SerialDevice_Cmd(iuart->port,&cmd);
+	if(baudrate) {
+		int nsecs = 10000000000LL / baudrate;
+		if((nsecs > 5000) && (iuart->nsecs_per_char != nsecs)) {
+			iuart->nsecs_per_char = nsecs;
+			if(CycleTimer_IsActive(&iuart->baudTimer)) {
+				CycleTimer_Mod(&iuart->baudTimer,NanosecondsToCycles(iuart->nsecs_per_char));
+			}
+		}
+	}
         return;
 }
 
@@ -816,7 +863,7 @@ uts_read(void *clientData,uint32_t address,int rqlen)
 		iuart->uts |= UTS_TXFULL;
 	}
 	//fprintf(stderr,"rx_room %d tx_room %d,rx_fill %d tx_fill %d uts %08x\n",rx_room,tx_room,rx_fill,tx_fill,iuart->uts);
-	if((iuart->uts & UTS_TXEMPTY) && (iuart->uts & UTS_RXEMPTY)) {
+	if(!(iuart->uts & UTS_TXEMPTY) || (iuart->uts & UTS_RXEMPTY)) {
 		Senseless_Report(100);
 	}
 	return iuart->uts;
@@ -896,12 +943,7 @@ IMXUart_UnMap(void *owner,uint32_t base,uint32_t mask)
 BusDevice *
 IMXUart_New(const char *name) 
 {
-	IMX_Uart *iuart = malloc(sizeof(IMX_Uart));
-	if(!iuart) {
-		fprintf(stderr,"Out of memory for IMX Uart\n");
-		exit(1);
-	}
-	memset(iuart,0,sizeof(IMX_Uart));
+	IMX_Uart *iuart = sg_new(IMX_Uart);
 	iuart->irqNode = SigNode_New("%s.irq",name);
 	iuart->rxDmaReqNode = SigNode_New("%s.rx_dmareq",name);
 	iuart->txDmaReqNode = SigNode_New("%s.tx_dmareq",name);
@@ -919,9 +961,12 @@ IMXUart_New(const char *name)
         iuart->bdev.UnMap=IMXUart_UnMap;
         iuart->bdev.owner=iuart;
         iuart->bdev.hw_flags=MEM_FLAG_WRITABLE|MEM_FLAG_READABLE;
-	iuart->name = strdup(name);
+	iuart->name = sg_strdup(name);
 	iuart->port = Uart_New(name,serial_input,serial_output,NULL,iuart);
 	iuart->in_clk = Clock_New("%s.clk",name);
+	iuart->nsecs_per_char = 1000000;
+	CycleTimer_Init(&iuart->baudTimer,IUart_ReleaseChar,iuart);
+
 	/* Clock should come from perclk1 of CRM module */
 
 	iuart->refclk = Clock_New("%s.refclk",name);
