@@ -27,11 +27,13 @@
 #include <stdlib.h>
 #include <stdint.h>
 #include <string.h>
-#include <bus.h>
-#include <signode.h>
-#include <cycletimer.h>
-#include <imx_timer.h>
-#include <senseless.h>
+#include "bus.h"
+#include "signode.h"
+#include "cycletimer.h"
+#include "imx_timer.h"
+#include "senseless.h"
+#include "sgstring.h"
+
 /* default base is 0x10003000/4000/5000 for imx21 */
 #define TCTL(base) 	((base)+0)
 #define		TCTL_SWR		(1<<15)
@@ -234,7 +236,7 @@ update_cmp_event_timer(IMX_Timer *itmr)
 	if(!divider) {
 		return;
 	}
-	ccyc_until_event = divider * tcyc_until_event - itmr->saved_cpu_cycles; 	
+	ccyc_until_event = (uint64_t)divider * tcyc_until_event - itmr->saved_cpu_cycles; 	
 	//fprintf(stderr,"Cyc until %lld divider %d tcyc_until %d saved %lld\n",ccyc_until_event,divider,tcyc_until_event,itmr->saved_cpu_cycles);
 	if(CycleTimer_IsActive(&itmr->event_timer)) {
 		CycleTimer_Mod(&itmr->event_timer,ccyc_until_event);
@@ -251,7 +253,7 @@ do_event(void *clientData)
         update_cmp_event_timer(itmr);
 }
 
-static int
+static void 
 capture(SigNode *node,int value,void *clientData)
 {
 	IMX_Timer *itmr = (IMX_Timer *) clientData;
@@ -277,7 +279,6 @@ capture(SigNode *node,int value,void *clientData)
 			itmr->tcr = itmr->tcn;	
 			break;
 	}
-	return 0;
 }
 
 /*
@@ -484,12 +485,7 @@ IMXTimer_Map(void *owner,uint32_t base,uint32_t mask,uint32_t flags)
 BusDevice *
 IMXTimer_New(char *name)
 {
-	IMX_Timer *itmr = malloc(sizeof(IMX_Timer));
-	if(!itmr) {
-		fprintf(stderr,"Out of memory for IMX_Timer\n");
-		exit(7);
-	}
-	memset(itmr,0,sizeof(IMX_Timer));
+	IMX_Timer *itmr = sg_new(IMX_Timer);
 	itmr->irqNode = SigNode_New("%s.irq",name);
 	if(!itmr->irqNode) {
 		fprintf(stderr,"Cannot create irqNode for \"%s\"\n",name);
