@@ -29,23 +29,24 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <instructions.h>
-#include <arm9cpu.h>
-#include <mmu.h>
-#include <compiler_extensions.h>
-#include <debug.h>
+#include "instructions.h"
+#include "arm9cpu.h"
+#include "mmu.h"
+#include "compiler_extensions.h"
+#include "sgstring.h"
+#include "sglib.h"
 
 #if defined(__i386__) || defined (__i486__) || defined(__i586__)
-#define USE_ASM 1 
+#define USE_ASM 0 
 #else 
 #define USE ASM 0
 #endif
 
 #ifdef DEBUG
-#define dprintf(x...) { if(unlikely(debugflags&DEBUG_INSTRUCTIONS)) { fprintf(stderr,x); fflush(stderr); } }
+#define dbgprintf(x...) { if(unlikely(debugflags&DEBUG_INSTRUCTIONS)) { fprintf(stderr,x); fflush(stderr); } }
 #else
-#define dprintf(x...) 
-//define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) 
+//define dbgprintf(x...) { fprintf(stderr,x); }
 #endif
 #define ISNEG(x) ((x)&(1<<31))
 #define ISNOTNEG(x) (!((x)&(1<<31)))
@@ -55,6 +56,7 @@ static uint32_t sub_flagmap[4096];
 
 #define testbit(bit_nr,value) (value&(1<<bit_nr))
 
+#if USE_ASM
 static uint32_t 
 asm_sub(uint32_t *diff_32, uint32_t op1_32, uint32_t op2_32)
 {
@@ -101,6 +103,7 @@ asm_cmp(uint32_t op1, uint32_t op2)
     );
     return sub_flagmap[flags&0xfff];
 }
+#endif /* USE_ASM */
 
 #define IA32_FLAG_C (1<<0)
 #define IA32_FLAG_Z (1<<6)
@@ -113,93 +116,67 @@ InAPrivilegedMode(void) {
 	return (ARM_SIGNALING_MODE!=MODE_USER);
 }
 
-/*
- * ------------------------------
- * Count ones in a bitfield
- * ------------------------------
- */
-
-static unsigned int
-onecount_slow(const uint32_t value) 
-{
-	uint32_t val=value;
-	int ones=0;
-	while(val) {
-		if(val&1)
-			ones++;
-		val>>=1;
-	}
-	return ones;
-}
-
-static uint8_t onecount_map[256];
-
-static void 
-init_onecount_map() 
-{
-	int j; 
-	for(j=0; j<256; ++j) {
-		 onecount_map[j] = onecount_slow(j);
-	}
-}
-
-static inline int 
-onecount(uint32_t w) 
-{
-	int c=0; 
-	while (w) {
-		c += onecount_map[w&255]; 
-		w >>= 8;
-	}
-	return c;
-}
-
 /*
  * ------------------------------------
  * Helper functions for add and sub
  * ------------------------------------
  */
 static inline uint32_t
-sub_carry(uint32_t op1,uint32_t op2,uint32_t result) {
-	if( ((ISNEG(op1) && ISNOTNEG(op2)) 
-	  || (ISNEG(op1) && ISNOTNEG(result))
-	  || (ISNOTNEG(op2) && ISNOTNEG(result)))) {
-			return FLAG_C;
-	} else { 
-		return 0;
-	}
+sub_carry(uint32_t Rn,uint32_t Rm, uint32_t R)
+{
+        if((Rn < (Rn - Rm)) || ((Rn - Rm) < R)) {
+                return 0;
+        } else {
+                return FLAG_C; 
+        }
+}
+
+static inline uint32_t
+sub_carry_nocarry(uint32_t Rn,uint32_t Rm, uint32_t R)
+{
+        if(Rn < R) {
+                return 0;
+        } else {
+                return FLAG_C; 
+        }
 }
 
 static inline uint32_t
 add_carry(uint32_t op1,uint32_t op2,uint32_t result) {
-	if( ((ISNEG(op1) && ISNEG(op2)) 
-	  || (ISNEG(op1) && ISNOTNEG(result))
-	  || (ISNEG(op2) && ISNOTNEG(result)))) {
-			return FLAG_C;
-	} else { 
+	if(((op1+op2) < op1) || ((op1+op2) > result)) {
+		return FLAG_C;
+	} else {
 		return 0;
 	}
 }
 
+/*
+ * Carry flag calculation for addition without use of carryflag
+ * for calcualtion of result
+ */
+#if 0
+#define add_carry_nocarry add_carry
+#else
+static inline uint32_t
+add_carry_nocarry(uint32_t op1,uint32_t op2,uint32_t result) {
+	if(result < op1) {
+		return FLAG_C;
+	} else {
+		return 0;
+	}
+}
+#endif
+
 static inline uint32_t
 sub_overflow(uint32_t op1,uint32_t op2,uint32_t result) {
-        if ((ISNOTNEG (op1) && ISNEG (op2) && ISNEG (result)) || 
-  	  (ISNEG (op1) && ISNOTNEG (op2) && ISNOTNEG (result))) {
-		return FLAG_V;
-	} else {
-		return 0;
-	}
+ 	return (((op1 & ~op2 & ~result) | (~op1 & op2 & result)) >> 3) & FLAG_V;
 }
 
 static inline uint32_t
-add_overflow(uint32_t op1,uint32_t op2,uint32_t result) {
-  	if ((ISNEG (op1) && ISNEG (op2) && ISNOTNEG (result))
-          || (ISNOTNEG (op1) && ISNOTNEG (op2) && ISNEG (result))) {
-		return FLAG_V;
-	} else {
-		return 0;
-	}
+add_overflow(int32_t op1,int32_t op2,int32_t result) {
+	return (((op1 & op2 & ~result) | (~op1 & ~op2 & result)) >> 3) & FLAG_V;
 }
+
 /*
  * -----------------------------------------------------------
  * Conditons map allows a fast lookup if condition is true 
@@ -209,8 +186,7 @@ add_overflow(uint32_t op1,uint32_t op2,uint32_t result) {
  * -----------------------------------------------------------
  */
 
-#define CM_TYPE char 
-CM_TYPE *condition_map;
+char *ARM_ConditionMap;
 #define CONDITION_INDEX(icode,cpsr) ((((icode)>>28)&0xf) | (((cpsr)>>24)&0xf0))
 #define CONDITION_UNINDEX_CPSR(index) (((index)&0xf0)<<24)
 #define CONDITION_UNINDEX_ICODE(index) (((index)&0xf)<<28)
@@ -220,19 +196,14 @@ check_condition(const uint32_t icode) {
 	if(likely((icode&0xf0000000)==0xe0000000)) {
 		return 1;
 	} else {
-		return condition_map[CONDITION_INDEX(icode,REG_CPSR)];
+		return ARM_ConditionMap[CONDITION_INDEX(icode,REG_CPSR)];
 	}
 }
 
 static inline void 
 init_condition_map(void) {
 	int i;
-	CM_TYPE *m=condition_map=(CM_TYPE *)malloc(256*sizeof(*m));
-	if(!m) {
-		fprintf(stderr,"Out of memory\n");
-		exit(243);
-	}
-	memset(condition_map,0,256);
+	char *m=ARM_ConditionMap=(char *)sg_calloc(256*sizeof(*m));
 	for(i=0;i<256;i++) {
 		uint32_t cpsr=CONDITION_UNINDEX_CPSR(i);
 		uint32_t icode=CONDITION_UNINDEX_ICODE(i);
@@ -355,7 +326,7 @@ am1_rorx() {
 	uint32_t Rm;
 	int rm=icode&0xf;
 	Rm=ARM9_ReadReg(rm);
-	AM_SCRATCH1 = Rm>>1 | ((REG_CPSR & FLAG_C) << (31-FLAG_C_SHIFT));
+	AM_SCRATCH1 = (Rm >> 1) | ((REG_CPSR & FLAG_C) << (31-FLAG_C_SHIFT));
 	return (Rm & 1) << FLAG_C_SHIFT;
 }
 		
@@ -490,12 +461,12 @@ am1_lsli() {
 	uint32_t Rm;
 	uint32_t shift_imm = (icode>>7)&0x1f;
 	Rm=ARM9_ReadReg(icode & 0xf);
-	if(shift_imm==0) {
+	if(shift_imm == 0) {
 		AM_SCRATCH1 = Rm;
 		return REG_CPSR & FLAG_C;
 	} else {
-		AM_SCRATCH1 = Rm<<shift_imm;
-		if(Rm&(1<<(32-shift_imm))) {
+		AM_SCRATCH1 = Rm << shift_imm;
+		if(Rm & (1 << (32-shift_imm))) {
 			return FLAG_C;
 		} else {
 			return 0;
@@ -570,8 +541,8 @@ am1_rori() {
 static  uint32_t
 am1_undefined() {
 	uint32_t icode=ICODE;
-	fprintf(stderr,"Unknown addressing Mode 1, icode %08x at %08x\n",icode,GET_REG_PC);
-	ARM9_Exception(EX_UNDEFINED,4);
+	fprintf(stderr,"Unknown addressing Mode 1, icode %08x at %08x\n",icode,ARM_GET_NNIA);
+	ARM_Exception(EX_UNDEFINED,0);
 	return 0;
 }
 
@@ -586,7 +557,7 @@ admode1_create_mapping()
 {
 	int i;
 	uint32_t icode;
-	am1_map=(Am1_Proc **)malloc(sizeof(Am1_Proc *)*2048);
+	am1_map=(Am1_Proc **)sg_calloc(sizeof(Am1_Proc *)*2048);
 	for(i=0;i<2048;i++) {
 		icode=AM1_UNINDEX(i);
 		if((icode&0x0e000000)==0x02000000) {
@@ -636,7 +607,6 @@ static inline uint32_t
 get_data_processing_operand(uint32_t icode) 
 {
 	Am1_Proc *proc=am1_map[AM1_INDEX(icode)];	
-	//Am1_Proc *proc=*(Am1_Proc **)(((unsigned long)am1_map)+AM1_OFFSET(icode));	
 	return proc();
 }
 
@@ -654,7 +624,7 @@ am2_imm_post(uint32_t *new_Rn) {
 	uint32_t Rn=*new_Rn;
 	uint32_t U;
 	uint32_t offset=icode & 0xfff;
-	dprintf("Immediate postindexed %08x\n",icode);	
+	dbgprintf("Immediate postindexed %08x\n",icode);	
 	addr = Rn; 
 	U=testbit(23,icode);
 	if(U) {
@@ -852,7 +822,7 @@ am2_imm_pre(uint32_t *new_Rn) {
 	uint32_t Rn=*new_Rn;
 	uint32_t U;
 	uint32_t offset=icode & 0xfff;
-	dprintf("Immediate preindexed %08x\n",icode);	
+	dbgprintf("Immediate preindexed %08x\n",icode);	
 	U=testbit(23,icode);
 	if(U) {
 		Rn=Rn+offset;
@@ -946,8 +916,8 @@ am2_sreg_pre(uint32_t *new_Rn) {
 } 
 static  uint32_t
 am2_undefined(uint32_t *new_Rn) {
-	fprintf(stderr,"Unknown addressing mode in instruction %08x at %08x\n",ICODE,GET_REG_PC);	
-	ARM9_Exception(EX_UNDEFINED,4);
+	fprintf(stderr,"Unknown addressing mode in instruction %08x at %08x\n",ICODE,ARM_GET_NNIA);	
+	ARM_Exception(EX_UNDEFINED,0);
 	return 0;
 }
 
@@ -962,7 +932,7 @@ admode2_create_mapping()
 {
 	int i;
 	uint32_t icode;
-	am2_map=(Am2_Proc **)malloc(sizeof(Am2_Proc *)*8192);
+	am2_map=(Am2_Proc **)sg_calloc(sizeof(Am2_Proc *)*8192);
 	for(i=0;i<8192;i++) {
 		icode=AM2_UNINDEX(i);
 		if(( icode & 0x0f000000) == 0x04000000) { // allow also for ldrbt
@@ -1157,7 +1127,7 @@ static uint32_t
 am3_undef()
 {
 	fprintf(stderr,"Undefined addressing mode 3\n");
-	ARM9_Exception(EX_UNDEFINED,4);
+	ARM_Exception(EX_UNDEFINED,0);
 	AM3_UPDATE_RN = 0;
 	return 0;
 }
@@ -1171,11 +1141,7 @@ void
 admode3_create_mapping() 
 {
 	int i;
-	am3_map=(Am3_Proc**)malloc(sizeof(Am3_Proc *)*256);
-	if(!am3_map) {
-		fprintf(stderr,"Out of memory\n");
-		exit(462);
-	}
+	am3_map=(Am3_Proc**)sg_calloc(sizeof(Am3_Proc *)*256);
 	for(i=0;i<256;i++) {
 		uint32_t icode=AM3_UNINDEX(i);
 		if((icode&0x0f600090)==	0x01400090) {
@@ -1226,7 +1192,7 @@ i_lsm() {
 	if(!check_condition(icode)) {
 		return;
 	}
-	ones=onecount(icode&0xffff);
+	ones=SGLib_OnecountU32(icode&0xffff);
 	rn=(icode>>16)&0xf;
 
 	/* Unaligned LDM/STM ignores last two bits (A4-35 DDI100) */
@@ -1259,16 +1225,16 @@ i_lsm() {
 	L=testbit(20,icode); // Load/Store
 	S=testbit(22,icode); // Modify Flags ?
 	if(S && L && (icode&(1<<15))) {
-		bank=ARM_BANK;
+		bank = ARM_BANK;
 	}  else if(S) {
-		bank=MODE_USER;	
+		bank = MODE_USER;	
 	} else {
 		// default bank is current bank
-		bank=ARM_BANK;	
+		bank = ARM_BANK;	
 	}
 	if(L) {
 		uint32_t value[16];
-		if(bank==ARM_BANK) {
+		if(bank == ARM_BANK) {
 			for(i=0;i<15;i++) {
 				if(icode&(1<<i)) {
 					value[i]=MMU_Read32(start_address);
@@ -1359,7 +1325,7 @@ i_lsm() {
 		}	
         }
 	CycleCounter+=(ones<<1);
-	dprintf("Done LSM addr %08x L %d\n",start_address, L ? 1:0); 
+	dbgprintf("Done LSM addr %08x L %d\n",start_address, L ? 1:0); 
 }
 
 void 
@@ -1406,7 +1372,7 @@ i_adc(){
 			REG_CPSR=cpsr;
 		}
 	}
-	dprintf("ADC result op1 %08x,op2 %08x, result %08x\n",op1,op2,result); 
+	dbgprintf("ADC result op1 %08x,op2 %08x, result %08x\n",op1,op2,result); 
 }
 #if USE_ASM
 void 
@@ -1416,7 +1382,7 @@ i_add(){
         uint32_t S;
         uint32_t Rn,op2,result;
         if(!check_condition(icode)) {
-                dprintf("ADD: condition not true\n");
+                dbgprintf("ADD: condition not true\n");
                 return;
         }
         rd=(icode>>12)&0xf;
@@ -1445,7 +1411,7 @@ i_add(){
                 result=Rn+op2;
         	ARM9_WriteReg(result,rd);
         }
-        dprintf("ADD result op1 %08x,op2 %08x, result %08x\n",Rn,op2,result);
+        dbgprintf("ADD result op1 %08x,op2 %08x, result %08x\n",Rn,op2,result);
 }
 #else
 void 
@@ -1455,7 +1421,7 @@ i_add(){
 	uint32_t S;
 	uint32_t op1,op2,result;
 	if(!check_condition(icode)) {
-		dprintf("ADD: condition not true\n"); 
+		dbgprintf("ADD: condition not true\n"); 
 		return;
 	}
 	rd=(icode>>12)&0xf;
@@ -1463,19 +1429,18 @@ i_add(){
 	op1=ARM9_ReadReg(rn);
 	get_data_processing_operand(icode);	
 	op2 = AM_SCRATCH1;
-	result=op1+op2;
+	result = op1 + op2;
 	ARM9_WriteReg(result,rd);
 	S=testbit(20,icode);
 	if(S) {
-#if 1
 		uint32_t cpsr=REG_CPSR;
 		cpsr&= ~(FLAG_N | FLAG_Z | FLAG_C |FLAG_V);
-		if(result==0) {
-			cpsr|=FLAG_Z;
+		if(result == 0) {
+			cpsr |= FLAG_Z;
 		} else if(ISNEG(result)) {
-			cpsr|= FLAG_N;
+			cpsr |= FLAG_N;
 		}
-		cpsr |= add_carry(op1,op2,result);
+		cpsr |= add_carry_nocarry(op1,op2,result);
 		cpsr |= add_overflow(op1,op2,result);
 		if(rd==15) {
 			if(MODE_HAS_SPSR) {
@@ -1486,9 +1451,8 @@ i_add(){
 		} else {
 			REG_CPSR=cpsr;
 		}
-#endif
 	}
-	dprintf("ADD result op1 %08x,op2 %08x, result %08x\n",op1,op2,result); 
+	dbgprintf("ADD result op1 %08x,op2 %08x, result %08x\n",op1,op2,result); 
 }
 #endif
 void 
@@ -1527,24 +1491,20 @@ i_and(){
 			REG_CPSR=cpsr;
 		}
 	}
-	dprintf("AND result op1 %08x,op2 %08x, result %08x\n",Rn,op2,result); 
+	dbgprintf("AND result op1 %08x,op2 %08x, result %08x\n",Rn,op2,result); 
 }
 
 void 
 i_b(){
 	uint32_t icode = ICODE;
-	uint32_t signed_immed;
+	int32_t signed_immed;
 	if(!check_condition(icode)) {
 		return;
 	}
-	if(icode&0x00800000) {
-		signed_immed= (icode & 0x00ffffff) | 0xff000000;
-	} else {
-		signed_immed=icode & 0x00ffffff;
-	}
-	signed_immed <<= 2;
-	SET_REG_PC(GET_REG_PC+signed_immed);
+	signed_immed = ((int32_t)(icode << 8)) >> 6;
+	ARM_SET_NIA(ARM_GET_NNIA+signed_immed);
 }
+
 void 
 i_bl(){
 	uint32_t icode = ICODE;
@@ -1552,15 +1512,11 @@ i_bl(){
 	if(!check_condition(icode)) {
 		return;
 	}
-	if(icode&0x00800000) {
-		signed_immed= (icode & 0x00ffffff) | 0xff000000;
-	} else {
-		signed_immed=icode & 0x00ffffff;
-	}
-	signed_immed <<= 2;
-	REG_LR=GET_REG_PC - 4;	
-	SET_REG_PC(GET_REG_PC+signed_immed);	
+	signed_immed = ((int32_t)(icode << 8)) >> 6;
+	REG_LR = ARM_GET_NNIA - 4;	
+	ARM_SET_NIA(ARM_GET_NNIA + signed_immed);	
 }
+
 void 
 i_bbl(){
 	uint32_t icode = ICODE;
@@ -1569,17 +1525,12 @@ i_bbl(){
 	if(!check_condition(icode)) {
 		return;
 	}
-	if(icode&0x00800000) {
-		signed_immed= (icode & 0x00ffffff) | 0xff000000;
-	} else {
-		signed_immed=icode & 0x00ffffff;
-	}
-	signed_immed <<= 2;
+	signed_immed = ((int32_t)(icode << 8)) >> 6;
 	L=testbit(24,icode); 
 	if(L) {
-		REG_LR=GET_REG_PC-4;	
+		REG_LR=ARM_GET_NNIA-4;	
 	}
-	SET_REG_PC(GET_REG_PC+signed_immed);	
+	ARM_SET_NIA(ARM_GET_NNIA + signed_immed);	
 }
 void 
 i_bic() {
@@ -1617,7 +1568,7 @@ i_bic() {
 			REG_CPSR=cpsr;
 		}
 	}
-	dprintf("BIC result op1 %08x,op2 %08x, result %08x\n",Rn,op2,result); 
+	dbgprintf("BIC result op1 %08x,op2 %08x, result %08x\n",Rn,op2,result); 
 }
 
 void 
@@ -1630,17 +1581,13 @@ i_blx1() {
 	uint32_t icode = ICODE;
 	int32_t offset; 
 	uint32_t h = (icode >> 23) & 2;
-	if(icode & (1<<23)) {
-		offset = (icode & 0xffffff) | 0xff000000;
-	} else {
-		offset = (icode & 0xffffff);
-	}
-	offset = (offset << 2) | h;
-	REG_LR=GET_REG_PC-4;	
+	offset = ((int32_t)(icode << 8)) >> 6;
+	offset |= h;
+	REG_LR=ARM_GET_NNIA-4;	
 	REG_CPSR |= FLAG_T;  
-	SET_REG_PC(GET_REG_PC+offset);
+	ARM_SET_NIA(ARM_GET_NNIA+offset);
 	/* Enter Thumb */
-	ARM_RestartLoop();
+	ARM_RestartIdecoder();
 }
 
 void 
@@ -1653,18 +1600,18 @@ i_blx2bx()
 	if(!check_condition(icode)) {
 		return;
 	}
-	rm = icode&0xf;
+	rm = icode & 0xf;
 	Rm=ARM9_ReadReg(rm);
+	L = testbit(5,icode);
+	if(L) {
+		REG_LR = ARM_NIA;	
+	}
+	ARM_SET_NIA(Rm & 0xfffffffe);
 	if(Rm & 1) {
 		/* Enter Thumb mode */
 		REG_CPSR |= FLAG_T;  
-		ARM_RestartLoop();
+		ARM_RestartIdecoder();
 	}
-	L = testbit(5,icode);
-	if(L) {
-		REG_LR=GET_REG_PC-4;	
-	}
-	SET_REG_PC(Rm & 0xfffffffe);
 	
 }
 
@@ -1679,14 +1626,14 @@ i_cdp() {
 	cp = (icode >> 8) &0xf;
 	copro=gcpu.copro[cp];
 	if(!copro) {	
-		dprintf("CDP: No ArmCoprocessor %d\n",cp);
-		ARM9_Exception(EX_UNDEFINED,4);
+		dbgprintf("CDP: No ArmCoprocessor %d\n",cp);
+		ARM_Exception(EX_UNDEFINED,0);
 	} else {
 		if(copro->cdp) {
 			copro->cdp(copro,icode);
 		} else {
 			fprintf(stderr,"ArmCoprocessor %d does not support CDP\n",cp);
-			ARM9_Exception(EX_UNDEFINED,4);
+			ARM_Exception(EX_UNDEFINED,0);
 		}
 	}
 }
@@ -1714,7 +1661,7 @@ i_clz() {
 		count=32;
 	}
 	ARM9_WriteReg(count,rd);
-	dprintf("CLZ of %08x gives %d\n",Rm,count);
+	dbgprintf("CLZ of %08x gives %d\n",Rm,count);
 //	fprintf(stderr,"Not tested clz\n"); exit(2);
 }
 void 
@@ -1729,17 +1676,16 @@ i_cmn() {
 	rd=(icode>>12)&0xf;
 	rn=(icode>>16)&0xf;
 	Rn=ARM9_ReadReg(rn);
-	cpsr&= ~(FLAG_N | FLAG_Z | FLAG_C |FLAG_V);
+	cpsr &= ~(FLAG_N | FLAG_Z | FLAG_C |FLAG_V);
 	get_data_processing_operand(icode);	
 	op2 = AM_SCRATCH1;
-	result=Rn+op2;
-	if(!result) {
-		cpsr|=FLAG_Z;
-	} 
+	result = Rn + op2;
 	if(ISNEG(result)) {
 		cpsr|= FLAG_N;
-	}
-	cpsr|=add_carry(Rn,op2,result);
+	} else if(result == 0) {
+		cpsr|=FLAG_Z;
+	} 
+	cpsr|=add_carry_nocarry(Rn,op2,result);
 	cpsr|=add_overflow(Rn,op2,result);
 	REG_CPSR=cpsr;
 }
@@ -1762,7 +1708,7 @@ i_cmp() {
         cpsr = (cpsr &  ~(FLAG_N | FLAG_Z | FLAG_C |FLAG_V) )
                | new_flags;
 	REG_CPSR=cpsr;
-	dprintf("CMP result op1 %08x,op2 %08x, cpsr %08x\n",Rn,op,cpsr); 
+	dbgprintf("CMP result op1 %08x,op2 %08x, cpsr %08x\n",Rn,op,cpsr); 
 }
 #else
 void 
@@ -1779,7 +1725,7 @@ i_cmp() {
 	Rn=ARM9_ReadReg(rn);
 	get_data_processing_operand(icode);	
 	op = AM_SCRATCH1;
-	result=Rn-op;
+	result = Rn - op;
 	cpsr&= ~(FLAG_N | FLAG_Z | FLAG_C |FLAG_V);
 	if(result==0) {
 		cpsr|=FLAG_Z;
@@ -1787,10 +1733,10 @@ i_cmp() {
 	if(ISNEG(result)) {
 		cpsr|= FLAG_N;
 	}
-	cpsr|=sub_carry(Rn,op,result);
-	cpsr|=sub_overflow(Rn,op,result);
+	cpsr |= sub_carry_nocarry(Rn,op,result);
+	cpsr |= sub_overflow(Rn,op,result);
 	REG_CPSR=cpsr;
-	dprintf("CMP result op1 %08x,op2 %08x, result %08x cpsr %08x\n",Rn,op,result,cpsr); 
+	dbgprintf("CMP result op1 %08x,op2 %08x, result %08x cpsr %08x\n",Rn,op,result,cpsr); 
 }
 #endif
 void 
@@ -1829,7 +1775,7 @@ i_eor() {
 			REG_CPSR=cpsr;
 		}
 	}
-	dprintf("EOR result op1 %08x,op2 %08x, result %08x\n",op1,op2,result); 
+	dbgprintf("EOR result op1 %08x,op2 %08x, result %08x\n",op1,op2,result); 
 }
 void 
 i_ldc() {
@@ -1842,17 +1788,17 @@ i_ldc() {
 	cp = (icode >> 8) &0xf;
 	copro=gcpu.copro[cp];
 	if(!copro) {	
-		dprintf("LDC: No ArmCoprocessor %d\n",cp);
-		ARM9_Exception(EX_UNDEFINED,4);
+		dbgprintf("LDC: No ArmCoprocessor %d\n",cp);
+		ARM_Exception(EX_UNDEFINED,0);
 	} else {
 		if(copro->ldc) {
 			copro->ldc(copro,icode);
 		} else {
 			fprintf(stderr,"ArmCoprocessor %d does not support LDC\n",cp);
-			ARM9_Exception(EX_UNDEFINED,4);
+			ARM_Exception(EX_UNDEFINED,0);
 		}
 	}
-	dprintf("address %08x, instruction %08x\n",GET_REG_PC,icode);
+	dbgprintf("address %08x, instruction %08x\n",ARM_GET_NNIA,icode);
 }
 
 static inline void 
@@ -1876,7 +1822,7 @@ i_strwub(){
 		value=ARM9_ReadReg(rd);
 		if(unlikely(addr&3)) {
 			MMU_AlignmentException(addr);
-			fprintf(stderr,"Warning: unaligned str icode %08x, addr %08x, pc %08x\n",icode,addr,GET_REG_PC);
+			fprintf(stderr,"Warning: unaligned str icode %08x, addr %08x, pc %08x\n",icode,addr,ARM_GET_NNIA);
 			addr=addr&~3;
 		}
 		MMU_Write32(value,addr);
@@ -1888,7 +1834,7 @@ i_strwub(){
 	if(Rn_new!=Rn) {
 		ARM9_WriteReg(Rn_new,rn);
 	}
-	dprintf("Stored value %08x from register %d to addr %08x\n",value,rd,addr);
+	dbgprintf("Stored value %08x from register %d to addr %08x\n",value,rd,addr);
 }
 
 inline void 
@@ -1909,7 +1855,7 @@ i_ldr() {
 		unsigned int shift;
 		uint32_t memval;
 		MMU_AlignmentException(addr);
-		fprintf(stderr,"Warning: Unaligned ldr  from %08x to reg %d at PC %08x\n",addr,rd,GET_REG_PC-8); 
+		fprintf(stderr,"Warning: Unaligned ldr  from %08x to reg %d at PC %08x\n",addr,rd,ARM_GET_NNIA-8); 
 		memval = MMU_Read32(addr&~3);
 		shift=(addr&3)<<3;
 		value = (memval >> shift) | (memval << (32-shift));
@@ -1922,11 +1868,11 @@ i_ldr() {
 		if(unlikely(value & 1)) {
 			/* Goto Thumb mode */
 			REG_CPSR |= FLAG_T;  
-			SET_REG_PC(value&0xfffffffe);	
+			ARM_SET_NIA(value&0xfffffffe);	
 			ARM9_WriteReg(Rn_new,rn);
-			ARM_RestartLoop();
+			ARM_RestartIdecoder();
 		} else {
-			SET_REG_PC(value&0xfffffffc);	
+			ARM_SET_NIA(value&0xfffffffc);	
 		}
 	}
 	if(Rn_new!=Rn) {
@@ -1979,7 +1925,7 @@ i_ldrh(){
 	addr = addr_mode3();
 	if(unlikely(addr&1)) {
 		MMU_AlignmentException(addr);
-		fprintf(stderr,"Unaligned ldrh at 0x%08x rd %d gives unpredictable results !\n",GET_REG_PC-8,rd);
+		fprintf(stderr,"Unaligned ldrh at 0x%08x rd %d gives unpredictable results !\n",ARM_GET_NNIA-8,rd);
 		return;
 	}
 	value=MMU_Read16(addr);
@@ -1989,7 +1935,7 @@ i_ldrh(){
 		int rn = (icode>>16)&0xf;
 		ARM9_WriteReg(AM3_NEW_RN,rn);
 	}
-	dprintf("LDRH from %08x got %04x to Register %d, now %08x\n",addr,value,rd,Rd);
+	dbgprintf("LDRH from %08x got %04x to Register %d, now %08x\n",addr,value,rd,Rd);
 }
 
 void 
@@ -2011,7 +1957,7 @@ i_ldrsb(){
 		int rn = (icode>>16)&0xf;
 		ARM9_WriteReg(AM3_NEW_RN,rn);
 	}
-	dprintf("LDRSB\n");
+	dbgprintf("LDRSB\n");
 }
 
 void 
@@ -2038,7 +1984,7 @@ i_ldrsh(){
 		int rn = (icode>>16)&0xf;
 		ARM9_WriteReg(AM3_NEW_RN,rn);
 	}
-	dprintf("LDRSH\n");
+	dbgprintf("LDRSH\n");
 }
 void 
 i_ldrt() {
@@ -2062,8 +2008,8 @@ i_mcr() {
 		Rd=ARM9_ReadReg(rd);
 		copro->mcr(copro,icode,Rd);
 	} else {		
-		dprintf("ArmCoprocessor %d MCR operation not found\n",cp_num); 
-		ARM9_Exception(EX_UNDEFINED,4);
+		dbgprintf("ArmCoprocessor %d MCR operation not found\n",cp_num); 
+		ARM_Exception(EX_UNDEFINED,0);
 	}
 }
 
@@ -2096,7 +2042,7 @@ i_mla() {
 		}
 		REG_CPSR=cpsr;
 	}
-	dprintf("MLA\n"); 
+	dbgprintf("MLA\n"); 
 }
 
 
@@ -2115,7 +2061,7 @@ i_mov() {
 	cpsr&= ~(FLAG_N | FLAG_Z | FLAG_C);
 	cpsr |= get_data_processing_operand(icode);	
 	Rd = AM_SCRATCH1;
-	dprintf("MOV value %08x to reg %d \n",Rd,rd);
+	dbgprintf("MOV value %08x to reg %d \n",Rd,rd);
 	ARM9_WriteReg(Rd,rd);
 	S=testbit(20,icode);
 	if(S) {
@@ -2123,7 +2069,7 @@ i_mov() {
 			if(MODE_HAS_SPSR) {
 				SET_REG_CPSR(REG_SPSR);
 			} else {
-				fprintf(stderr,"Mode has no spsr in line %d pc %08x icode %08x\n",__LINE__,GET_REG_PC,icode);
+				fprintf(stderr,"Mode has no spsr in line %d pc %08x icode %08x\n",__LINE__,ARM_GET_NNIA,icode);
 			}
 		} else {	
 			if(!Rd) {
@@ -2154,20 +2100,20 @@ i_mrc() {
 		Rd=copro->mrc(copro,icode);
 		if(rd==15) {
 			REG_CPSR=(REG_CPSR&0x0fffffff) | (Rd & 0xf0000000); 
-			dprintf("Rd %08x, new cpsr %08x at %08x\n",Rd,REG_CPSR,GET_REG_PC);
+			dbgprintf("Rd %08x, new cpsr %08x at %08x\n",Rd,REG_CPSR,ARM_GET_NNIA);
 		} else {
 			ARM9_WriteReg(Rd,rd);
 		}
-		dprintf("MRC returned %08x to register %d\n",Rd,rd);
+		dbgprintf("MRC returned %08x to register %d\n",Rd,rd);
 	} else {		
-		dprintf("MRC: ArmCoprocessor %d not found\n",cp_num); 
-		ARM9_Exception(EX_UNDEFINED,4);
+		dbgprintf("MRC: ArmCoprocessor %d not found\n",cp_num); 
+		ARM_Exception(EX_UNDEFINED,0);
 	}
 }
 void 
 i_mrc2() {
 	fprintf(stderr,"No coprocessor supporting mrc2\n");
-	ARM9_Exception(EX_UNDEFINED,4);
+	ARM_Exception(EX_UNDEFINED,0);
 }
 void 
 i_mrs() {
@@ -2191,7 +2137,7 @@ i_mrs() {
 		Rd=REG_CPSR;
 		ARM9_WriteReg(Rd,rd);
 	}
-	dprintf("MRS  to %d\n",rd); 
+	dbgprintf("MRS  to %d\n",rd); 
 }
 void 
 i_msr() {
@@ -2237,7 +2183,7 @@ i_msr() {
 			cpsr=(cpsr&0x00ffffff) |(operand &0xff000000);
 		}
 		SET_REG_CPSR(cpsr);
-		dprintf("msrr new CPSR %08x\n",REG_CPSR); 
+		dbgprintf("msrr new CPSR %08x\n",REG_CPSR); 
 	} else {
 		if(MODE_HAS_SPSR) {
 			uint32_t spsr=REG_SPSR;
@@ -2254,12 +2200,12 @@ i_msr() {
 				spsr=(spsr&0x00ffffff) |(operand &0xff000000);
 			}
 			REG_SPSR=spsr;
-			dprintf("msrr new SPSR %08x\n",REG_SPSR); 
+			dbgprintf("msrr new SPSR %08x\n",REG_SPSR); 
 		} else {
-			fprintf(stderr,"Mode %02x has no spsr in line %d, pc %08x icode %08x\n",ARM_BANK,__LINE__,GET_REG_PC,icode);
+			fprintf(stderr,"Mode %02x has no spsr in line %d, pc %08x icode %08x\n",ARM_BANK,__LINE__,ARM_GET_NNIA,icode);
 		}
 	}
-	dprintf("MSR  \n"); 
+	dbgprintf("MSR  \n"); 
 }
 void 
 i_msri(){
@@ -2298,7 +2244,7 @@ i_mul(){
 		}
 		REG_CPSR=cpsr;
 	}
-	dprintf("MUL\n"); 
+	dbgprintf("MUL\n"); 
 }
 void 
 i_mvn(){
@@ -2316,7 +2262,7 @@ i_mvn(){
 	cpsr |= get_data_processing_operand(icode);	
 	value = AM_SCRATCH1;
 	value^=0xffffffff;
-	dprintf("MVN value %08x to reg %d \n",value,rd);
+	dbgprintf("MVN value %08x to reg %d \n",value,rd);
 	ARM9_WriteReg(value,rd);
 	S=testbit(20,icode);
 	if(S) {
@@ -2373,7 +2319,7 @@ i_orr(){
 			REG_CPSR=cpsr;
 		}
 	}
-	dprintf("orr result op1 %08x,op2 %08x, result %08x\n",Rn,op2,result); 
+	dbgprintf("orr result op1 %08x,op2 %08x, result %08x\n",Rn,op2,result); 
 }
 #if USE_ASM
 void 
@@ -2407,11 +2353,11 @@ i_rsb(){
 		} else {	
 			REG_CPSR=cpsr;
 		}
-		dprintf("RSB result op1 %08x,op2 %08x, result %08x flags %08x\n",Rn,op,result,cpsr); 
+		dbgprintf("RSB result op1 %08x,op2 %08x, result %08x flags %08x\n",Rn,op,result,cpsr); 
 	} else {
 		result=op-Rn;
 		ARM9_WriteReg(result,rd);
-		dprintf("RSB result op1 %08x,op2 %08x, result %08x\n",Rn,op,result); 
+		dbgprintf("RSB result op1 %08x,op2 %08x, result %08x\n",Rn,op,result); 
 	}
 }
 #else
@@ -2431,7 +2377,7 @@ i_rsb() {
 	cpsr&= ~(FLAG_N | FLAG_Z | FLAG_C |FLAG_V);
 	get_data_processing_operand(icode);	
 	op = AM_SCRATCH1;
-	result=op-Rn;
+	result = op - Rn;
 	ARM9_WriteReg(result,rd);
 	S=testbit(20,icode);	
 	if(S) {
@@ -2441,7 +2387,7 @@ i_rsb() {
 		if(ISNEG(result)) {
 			cpsr|= FLAG_N;
 		}
-		cpsr |= sub_carry(op,Rn,result);
+		cpsr |= sub_carry_nocarry(op,Rn,result);
 		cpsr |= sub_overflow(op,Rn,result);
 		if(rd==15) {
 			if(MODE_HAS_SPSR) {
@@ -2452,9 +2398,9 @@ i_rsb() {
 		} else {	
 			REG_CPSR=cpsr;
 		}
-		dprintf("RSB result op1 %08x,op2 %08x, result %08x flags %08x\n",Rn,op,result,cpsr); 
+		dbgprintf("RSB result op1 %08x,op2 %08x, result %08x flags %08x\n",Rn,op,result,cpsr); 
 	} else {
-		dprintf("RSB result op1 %08x,op2 %08x, result %08x\n",Rn,op,result); 
+		dbgprintf("RSB result op1 %08x,op2 %08x, result %08x\n",Rn,op,result); 
 	}
 }
 #endif
@@ -2489,8 +2435,8 @@ i_rsc() {
 		if(ISNEG(result)) {
 			cpsr|= FLAG_N;
 		}
-		cpsr|=sub_carry(shifter_operand,Rn,result);
-		cpsr|=sub_overflow(shifter_operand,Rn,result);
+		cpsr |= sub_carry(shifter_operand,Rn,result);
+		cpsr |= sub_overflow(shifter_operand,Rn,result);
 		if(rd==15) {
 			if(MODE_HAS_SPSR) {
 				SET_REG_CPSR(REG_SPSR);
@@ -2545,7 +2491,7 @@ i_sbc() {
 			REG_CPSR=cpsr;
 		}
 	}
-	dprintf("Warning untested SBC\n"); 
+	dbgprintf("Warning untested SBC\n"); 
 }
 void 
 i_smlal() {
@@ -2584,7 +2530,7 @@ i_smlal() {
 		}
 		REG_CPSR=cpsr;
 	}
-	dprintf("SMLAL\n"); 
+	dbgprintf("SMLAL\n"); 
 		
 }
 void 
@@ -2622,7 +2568,7 @@ i_smull() {
 		}
 		REG_CPSR=cpsr;
 	}
-	dprintf("SMULL %d*%d is %lld\n",Rm,Rs,result); 
+	dbgprintf("SMULL %d*%d is %lld\n",Rm,Rs,result); 
 }
 void 
 i_stc() {
@@ -2635,21 +2581,21 @@ i_stc() {
 	cp = (icode>>8)&0xf;
 	copro=gcpu.copro[cp];
 	if(!copro) {	
-		dprintf("STC: No ArmCoprocessor %d\n",cp);
-		ARM9_Exception(EX_UNDEFINED,4);
+		dbgprintf("STC: No ArmCoprocessor %d\n",cp);
+		ARM_Exception(EX_UNDEFINED,0);
 	} else {
 		if(copro->stc) {
 			copro->stc(copro,icode);
 		} else {
 			fprintf(stderr,"ArmCoprocessor %d does not support STC\n",cp);
-			ARM9_Exception(EX_UNDEFINED,4);
+			ARM_Exception(EX_UNDEFINED,0);
 		} 
 	}
 }
 
 void 
 i_str(){
-//	fprintf(stderr,"icode %08x at addr %08x\n",icode,GET_REG_PC);
+//	fprintf(stderr,"icode %08x at addr %08x\n",icode,ARM_GET_NNIA);
 	return i_strwub();
 }
 void 
@@ -2675,7 +2621,7 @@ i_strh(){
 	addr = addr_mode3();
 	if(unlikely(addr&1)) {
 		MMU_AlignmentException(addr);
-		fprintf(stderr,"Unaligned strh at %08x rd %d gives unpredictable results\n",GET_REG_PC-8,rd);
+		fprintf(stderr,"Unaligned strh at %08x rd %d gives unpredictable results\n",ARM_GET_NNIA-8,rd);
 		return;
 	}
 	Rd=ARM9_ReadReg(rd);
@@ -2684,7 +2630,7 @@ i_strh(){
 		int rn = (icode>>16)&0xf;
 		ARM9_WriteReg(AM3_NEW_RN,rn);
 	}
-	dprintf("STRH to value %04x to %08x\n",Rd,addr);
+	dbgprintf("STRH to value %04x to %08x\n",Rd,addr);
 }
 
 void 
@@ -2708,7 +2654,7 @@ i_sub(){
 	Rn=ARM9_ReadReg(rn);
 	get_data_processing_operand(icode);	
 	op = AM_SCRATCH1;
-	result=Rn-op;
+	result = Rn - op;
 	ARM9_WriteReg(result,rd);
 	S=testbit(20,icode);	
 	if(S) {
@@ -2727,17 +2673,17 @@ i_sub(){
 			if(ISNEG(result)) {
 				cpsr|= FLAG_N;
 			}
-			cpsr|=sub_carry(Rn,op,result);
-			cpsr|=sub_overflow(Rn,op,result);
+			cpsr |= sub_carry_nocarry(Rn,op,result);
+			cpsr |= sub_overflow(Rn,op,result);
 			REG_CPSR=cpsr;
 		}
 	}
-	dprintf("SUB result op1 %08x,op2 %08x, result %08x\n",Rn,op,result); 
+	dbgprintf("SUB result op1 %08x,op2 %08x, result %08x\n",Rn,op,result); 
 }
 
 void 
 i_swi() {
-	ARM9_Exception(EX_SWI,4);
+	ARM_Exception(EX_SWI,0);
 }
 
 void 
@@ -2765,7 +2711,7 @@ i_swp(){
 	}
 	MMU_Write32(Rm,Rn);	
 	ARM9_WriteReg(temp,rd);
-	dprintf("SWP\n"); 
+	dbgprintf("SWP\n"); 
 }
 
 void 
@@ -2788,7 +2734,7 @@ i_swpb() {
 	MMU_Write8(Rm,Rn);	
 	Rd=val;
 	ARM9_WriteReg(Rd,rd);
-	dprintf("SWPB\n"); 
+	dbgprintf("SWPB\n"); 
 }
 void 
 i_teq() {
@@ -2812,7 +2758,7 @@ i_teq() {
 		cpsr|= FLAG_N;
 	}
 	REG_CPSR=cpsr;
-	dprintf("TEQ op1 %08x,op2 %08x, result %08x\n",Rn,op2,alu_out); 
+	dbgprintf("TEQ op1 %08x,op2 %08x, result %08x\n",Rn,op2,alu_out); 
 }
 void 
 i_tst() {
@@ -2836,7 +2782,7 @@ i_tst() {
 		cpsr |= FLAG_N;
 	}
 	REG_CPSR=cpsr;
-	dprintf("tst %08x and %08x result %08x\n",Rn,op2,result); 
+	dbgprintf("tst %08x and %08x result %08x\n",Rn,op2,result); 
 }
 void 
 i_umlal() {
@@ -2874,7 +2820,7 @@ i_umlal() {
 		}
 		REG_CPSR=cpsr;
 	}
-	dprintf("UMLAL\n"); 
+	dbgprintf("UMLAL\n"); 
 }
 void 
 i_umull() {
@@ -2909,7 +2855,7 @@ i_umull() {
 		}
 		REG_CPSR=cpsr;
 	}
-	dprintf("UMULL\n"); 
+	dbgprintf("UMULL\n"); 
 }
 
 /* 
@@ -2927,9 +2873,9 @@ i_ldrd() {
 		return;
 	}
 	fprintf(stderr,"Warning: untested instruction LDRD\n");
-	rd1=icode>>12&0xf;
-	rd2=rd1|1;
-	if(rd1&1) {
+	rd1 = (icode >> 12) & 0xf;
+	rd2 = rd1 | 1;
+	if(rd1 & 1) {
 		fprintf(stderr,"LDRD: only even numbered destination registers are allowed\n");
 		return;
 	}
@@ -2943,25 +2889,25 @@ i_ldrd() {
 	ARM9_WriteReg(Rd1,rd1);
 	ARM9_WriteReg(Rd2,rd2);
 	if(AM3_UPDATE_RN) {
-		int rn = (icode>>16)&0xf;
+		int rn = (icode >> 16) & 0xf;
 		ARM9_WriteReg(AM3_NEW_RN,rn);
 	}
 }
 void 
 i_mcrr()  {
 	fprintf(stderr,"No coprocessor supporting MCRR\n"); 
-	ARM9_Exception(EX_UNDEFINED,4);
+	ARM_Exception(EX_UNDEFINED,0);
 }
 void 
 i_mrrc() {
 	fprintf(stderr,"No coprocessor supporting MRRC\n"); 
-	ARM9_Exception(EX_UNDEFINED,4);
+	ARM_Exception(EX_UNDEFINED,0);
 }
 
 void 
 i_pld() 
 {
-	dprintf("PLD: Cache preload instruction: Emulator does nothing\n");
+	dbgprintf("PLD: Cache preload instruction: Emulator does nothing\n");
 }
 
 /*
@@ -2977,7 +2923,7 @@ i_qadd() {
 	int32_t Rn,Rm;
 	int64_t result;
 	if(!check_condition(icode)) {
-		dprintf("QADD: condition not true\n"); 
+		dbgprintf("QADD: condition not true\n"); 
 		return;
 	}
 	rn=(icode>>16)&0xf;
@@ -2995,7 +2941,7 @@ i_qadd() {
 		REG_CPSR=REG_CPSR|FLAG_Q;
 	}
 	ARM9_WriteReg(result,rd);
-	dprintf("QADD result op1 %08x,op2 %08x, result %08llx\n",Rn,Rm,result); 
+	dbgprintf("QADD result op1 %08x,op2 %08x, result %08llx\n",Rn,Rm,result); 
 }
 
 void 
@@ -3005,7 +2951,7 @@ i_qdadd() {
 	int32_t Rn,Rm;
 	int64_t dRn,result;
 	if(!check_condition(icode)) {
-		dprintf("QDADD: condition not true\n"); 
+		dbgprintf("QDADD: condition not true\n"); 
 		return;
 	}
 	rn=(icode>>16)&0xf;
@@ -3031,7 +2977,7 @@ i_qdadd() {
 		REG_CPSR=REG_CPSR|FLAG_Q;
 	}
 	ARM9_WriteReg(result,rd);
-	dprintf("QDADD result op1 %08x,op2 %08x, result %08llx\n",Rn,Rm,result); 
+	dbgprintf("QDADD result op1 %08x,op2 %08x, result %08llx\n",Rn,Rm,result); 
 }
 void 
 i_qdsub() {
@@ -3040,7 +2986,7 @@ i_qdsub() {
 	int32_t Rn,Rm;
 	int64_t dRn,result;
 	if(!check_condition(icode)) {
-		dprintf("QDSUB: condition not true\n"); 
+		dbgprintf("QDSUB: condition not true\n"); 
 		return;
 	}
 	fprintf(stderr,"Warning: untested instruction QDSUB\n");
@@ -3066,7 +3012,7 @@ i_qdsub() {
 		REG_CPSR=REG_CPSR|FLAG_Q;
 	}
 	ARM9_WriteReg(result,rd);
-	dprintf("QDSUB result op1 %08x,op2 %08x, result %08llx\n",Rn,Rm,result); 
+	dbgprintf("QDSUB result op1 %08x,op2 %08x, result %08llx\n",Rn,Rm,result); 
 }
 
 void 
@@ -3076,7 +3022,7 @@ i_qsub() {
 	int32_t Rn,Rm;
 	int64_t result;
 	if(!check_condition(icode)) {
-		dprintf("QSUB: condition not true\n"); 
+		dbgprintf("QSUB: condition not true\n"); 
 		return;
 	}
 	rn=(icode>>16)&0xf;
@@ -3094,7 +3040,7 @@ i_qsub() {
 		REG_CPSR=REG_CPSR|FLAG_Q;
 	}
 	ARM9_WriteReg(result,rd);
-	dprintf("QADD result op1 %08x,op2 %08x, result %08llx\n",Rn,Rm,result); 
+	dbgprintf("QADD result op1 %08x,op2 %08x, result %08llx\n",Rn,Rm,result); 
 
 }
 
@@ -3107,7 +3053,7 @@ i_smlaxy() {
 	int64_t result;
 	int x,y;
 	if(!check_condition(icode)) {
-		dprintf("SLMAXY: condition not true\n"); 
+		dbgprintf("SLMAXY: condition not true\n"); 
 		return;
 	}
 	rn=(icode>>12)&0xf;
@@ -3148,7 +3094,7 @@ i_smlalxy() {
 	int64_t result;
 	int x,y;
 	if(!check_condition(icode)) {
-		dprintf("SMLALXY: condition not true\n"); 
+		dbgprintf("SMLALXY: condition not true\n"); 
 		return;
 	}
 	rm=icode&0xf;
@@ -3190,7 +3136,7 @@ i_smlawy() {
 	int64_t result;
 	int y;
 	if(!check_condition(icode)) {
-		dprintf("SMLAWY: condition not true\n"); 
+		dbgprintf("SMLAWY: condition not true\n"); 
 		return;
 	}
 	fprintf(stderr,"Warning: untested instruction SMLAW<y>\n");
@@ -3233,10 +3179,10 @@ i_smulxy() {
 	int64_t result;
 	int x,y;
 	if(!check_condition(icode)) {
-		dprintf("SMULXY: condition not true\n"); 
+		dbgprintf("SMULXY: condition not true\n"); 
 		return;
 	}
-	/* fprintf(stderr,"SMUL<x><y> at %08x\n",GET_REG_PC); */
+	/* fprintf(stderr,"SMUL<x><y> at %08x\n",ARM_GET_NNIA); */
 	rm=icode&0xf;
 	rs=(icode>>8)&0xf;
 	rd=(icode>>16)&0xf;
@@ -3267,7 +3213,7 @@ i_smulwy() {
 	int64_t result;
 	int y;
 	if(!check_condition(icode)) {
-		dprintf("SMULWY: condition not true\n"); 
+		dbgprintf("SMULWY: condition not true\n"); 
 		return;
 	}
 	fprintf(stderr,"Warning: untested instruction SMULW<y>\n");
@@ -3323,14 +3269,13 @@ i_strd() {
 void 
 i_und() {
 	uint32_t icode = ICODE;
-        fprintf(stderr,"Instruction not found for icode %08x at pc %08x\n",icode,GET_REG_PC);
-	ARM9_Exception(EX_UNDEFINED,4);
+        fprintf(stderr,"Instruction not found for icode %08x at pc %08x\n",icode,ARM_GET_NNIA);
+	ARM_Exception(EX_UNDEFINED,0);
 }
 
 void 
 InitInstructions() {
 	int i;
-	init_onecount_map(); 
 	for(i=0;i<4096;i++) {
 		uint32_t flags=0;
 		if(i&IA32_FLAG_C) {
