@@ -27,6 +27,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
+#include <inttypes.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -38,7 +39,18 @@
 #include "configfile.h"
 #include "loader.h"
 
-#define FLAG_SWAP32 (2)
+/* Should be a linked list with many namepaces, but for now one is enough */
+
+LoadProc *firstLoadProc = NULL;
+void *firstLoadProcClientData = NULL;
+
+int  
+Loader_RegisterBus(const char *name,LoadProc *proc,void *clientData)
+{
+	firstLoadProc = proc;
+	firstLoadProcClientData = clientData;
+	return 0;
+}
 
 /*
  * -----------------------------------------------------
@@ -47,35 +59,10 @@
  * by using the address in the Read-memory Map. 
  * -----------------------------------------------------
  */
-static int 
+static inline int 
 write_to_bus(uint32_t addr,uint8_t *buf,unsigned int count,int flags) 
 {
-	while(count) {
-		/* This is wrong (outdated) because of smaller mappings */
-                uint32_t len=MEM_MAP_BLOCKSIZE-(addr&MEM_MAP_BLOCKMASK);
-                uint32_t offs = addr&(MEM_MAP_BLOCKMASK);
-                uint8_t *host_mem= Bus_GetHVAWrite(addr & ~MEM_MAP_BLOCKMASK);
-                if(!host_mem) {
-                	host_mem= Bus_GetHVARead(addr &  ~MEM_MAP_BLOCKMASK);
-                }
-                if(!host_mem) {
-                        fprintf(stderr,"Loader: Cannot write record to memory at 0x%08x\n",addr);
-                        return -1;
-                }
-		if(len>count)
-			len=count;
-		if(flags & FLAG_SWAP32) {
-			int i;	
-			for(i=0;i<len;i++) {
-				uint8_t *dst = host_mem+((offs+i)^3);	
-				*dst = buf[i];
-			}
-		} else  {
-			memcpy(host_mem+offs,buf,len);
-		}
-		count-=len;
-	}
-	return 0;
+	return firstLoadProc(firstLoadProcClientData,addr,buf,count,flags);
 }
 
 typedef struct LoaderInfo {
@@ -94,8 +81,13 @@ static int
 write_srec_to_bus(uint32_t addr,uint8_t *buf,int count,void *clientData) 
 {
 	LoaderInfo *li = (LoaderInfo *)clientData;
+	if(count <= 0) {
+		return 0;
+	}
 	if((addr < li->region_start) || ((addr + count - 1) > li->region_end)) {
 		fprintf(stderr,"S-Record is outside of memory region\n");
+		fprintf(stderr,"Region from 0x%08" PRIx64 " to 0x%08" PRIx64 "\n",li->region_start,li->region_end);
+		fprintf(stderr,"S-Record at 0x%08x\n",addr);
 		return -1;
 	} else {
 		write_to_bus(addr,buf,count,li->flags);
@@ -110,9 +102,9 @@ Load_SRecords(char *filename,uint32_t startaddr,int flags,uint64_t region_size)
 	li.flags = flags;
 	li.region_start = startaddr;
 	if(region_size > 0) {
-		li.region_end = startaddr + region_size -1;
+		li.region_end = startaddr + region_size - 1;
 	} else {
-		li.region_end = ~0ULL;
+		li.region_end = ~UINT64_C(0);
 	}
 	return XY_LoadSRecordFile(filename,write_srec_to_bus,&li);
 }
@@ -127,6 +119,9 @@ static int
 write_ihex_to_bus(uint32_t addr,uint8_t *buf,int count,void *cd) 
 {
 	LoaderInfo *li = (LoaderInfo *)cd;
+	if(count <= 0) {
+		return 0;
+	}
 	if((addr < li->region_start) || ((addr + count - 1) > li->region_end)) {
 		fprintf(stderr,"Ihex: Record at 0x%08x is outside of memory_region\n",addr);
 		return -1;
@@ -145,7 +140,7 @@ Load_IHex(char *filename,uint32_t startaddr,int flags,uint64_t region_size)
 	if(region_size > 0) {
 		li.region_end = startaddr + region_size -1;
 	} else {
-		li.region_end = ~0ULL;
+		li.region_end = ~UINT64_C(0);
 	}
 	fprintf(stderr,"Loading Intel Hex Record file \"%s\"\n",filename);
         return XY_LoadIHexFile(filename,write_ihex_to_bus,&li);;
@@ -210,12 +205,14 @@ Load_AutoType(char *filename,uint32_t load_addr,uint64_t region_size)  {
 	int len = strlen(filename);
 	if(Config_ReadUInt32(&swap,"loader","swap32") >=0) {
 		if(swap) {
-			flags=FLAG_SWAP32;
+			flags=LOADER_FLAG_SWAP32;
 		}
 	}
 	fprintf(stderr,"Loading %s to 0x%08x flags %d\n",filename,load_addr,flags);
 	if((len>=5) && (!strcmp(filename+len-5,".srec"))){
 		return Load_SRecords(filename,load_addr,flags,region_size);
+	} else if((len>=4) && (!strcmp(filename+len-4,".s19"))){
+		return Load_SRecords(filename,load_addr,flags,region_size);
 	} else if((len>=4) && (!strcmp(filename+len-4,".hex"))){
 		return Load_IHex(filename,load_addr,flags,region_size);
 	} else {
