@@ -31,7 +31,9 @@
 #include <string.h>
 #include "bus.h"
 #include "signode.h"
+#include "clock.h"
 #include "cycletimer.h"
+#include "sgstring.h"
 
 /* Register definitions */
 #define TMR_IR		(0x00)
@@ -107,6 +109,7 @@ typedef struct CaptureTraceInfo {
 
 typedef struct LPCTimer {
 	BusDevice bdev;
+	Clock_t *clk_pclk;
 	uint32_t ir;
 	uint32_t tc;
 	uint32_t tcr;
@@ -115,6 +118,7 @@ typedef struct LPCTimer {
 	uint32_t mcr;
 	uint32_t emr;
 	CycleCounter_t last_timer_update;
+	CycleCounter_t saved_cycles;
 	CycleTimer event_timer;
 	uint32_t mr[4];
 	uint32_t ccr;
@@ -207,14 +211,17 @@ actualize_timer(LPCTimer *tmr)
 	int i;
 	CycleCounter_t cpu_cycles,pc_cycles,tc_cycles;
 	uint64_t old_tc,new_tc;
+	double clkdiv = Clock_Freq(tmr->clk_pclk) / CycleTimerRate_Get();
 	if(!(tmr->tcr &	TCR_ENA) || (tmr->tcr &	TCR_RST)) {
 		return;
 	}
 	cpu_cycles = CycleCounter_Get() - tmr->last_timer_update;	
 	tmr->last_timer_update = CycleCounter_Get();
-	pc_cycles = cpu_cycles; /* ????? Check this pclk */
-	tc_cycles = (tmr->pc + pc_cycles) / ((uint64_t)tmr->pr+1);
-	tmr->pc = tmr->pc + pc_cycles % ((uint64_t)tmr->pr+1);	
+	pc_cycles = (tmr->saved_cycles + cpu_cycles) / clkdiv; 
+	tmr->saved_cycles -= pc_cycles * clkdiv;
+
+	tc_cycles = (tmr->pc + pc_cycles) / ((uint64_t)tmr->pr + 1);
+	tmr->pc = (tmr->pc + pc_cycles) % ((uint64_t)tmr->pr + 1);	
 	old_tc = tmr->tc;
 	new_tc = tmr->tc = tmr->tc + tc_cycles;
 	for(i=0;i<4;i++) {
@@ -262,10 +269,10 @@ update_events(LPCTimer *tmr)
 			enable_timer = 1;
 		}
 	}
-	/* scaling factor for pclk ???? */
 	sleep_cycles = (uint64_t)mindiff * ((uint64_t)tmr->pr+1) - tmr->pc;  
+	sleep_cycles *= CycleTimerRate_Get() / Clock_Freq(tmr->clk_pclk);
 	if(enable_timer) {
-		CycleTimer_Add(&tmr->event_timer,sleep_cycles,do_event,tmr);
+		CycleTimer_Mod(&tmr->event_timer,sleep_cycles);
 	} else {
 		CycleTimer_Remove(&tmr->event_timer);
 	}
@@ -287,7 +294,7 @@ do_event(void *clientData)
  *      triggers an interrupt
  * -----------------------------------------------------------
  */
-static int
+static void 
 capture(SigNode *node,int value,void *clientData)
 {
 	CaptureTraceInfo *ti = (CaptureTraceInfo *) clientData;
@@ -317,7 +324,6 @@ capture(SigNode *node,int value,void *clientData)
 		}
 	}
 	tmr->oldCapStatus[index] = value;
-	return 0;
 }
 
 /*
@@ -370,7 +376,7 @@ tcr_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 		tmr->pc = 0;	
 	}
 	update_events(tmr);
-        //fprintf(stderr,"timer register %08x is not implemented\n",address);
+        fprintf(stderr,"TCR register write %08x\n",value);
         return;
 }
 
@@ -620,13 +626,8 @@ Tmr_UnMap(void *owner,uint32_t base,uint32_t mask)
 BusDevice *
 LPC2106Timer_New(const char *name) 
 {
-	LPCTimer *tmr = malloc(sizeof(LPCTimer));
+	LPCTimer *tmr = sg_new(LPCTimer);
 	int i;
-	if(!tmr) {
-		fprintf(stderr,"Out of memory allocating timer\n");
-		exit(7);
-	}
-	memset(tmr,0,sizeof(LPCTimer));
 	tmr->irqNode = SigNode_New("%s.irq",name);
 	if(!tmr->irqNode) {
 		fprintf(stderr,"Cannot create irqNode\n");	
@@ -652,6 +653,8 @@ LPC2106Timer_New(const char *name)
         tmr->bdev.UnMap=Tmr_UnMap;
         tmr->bdev.owner=tmr;
         tmr->bdev.hw_flags=MEM_FLAG_WRITABLE|MEM_FLAG_READABLE;
+	tmr->clk_pclk = Clock_New("%s.pclk",name);
+	CycleTimer_Init(&tmr->event_timer,do_event,tmr);
 	fprintf(stderr,"LPC2106 Timer \"%s\" created\n",name);
 	return &tmr->bdev;	
 }
