@@ -37,11 +37,12 @@
 #include "configfile.h"
 #include "cycletimer.h"
 #include "diskimage.h"
+#include "sgstring.h"
 
 #if 0
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 
 #define EE_STATE_ADDR0 (0)
@@ -202,7 +203,7 @@ static int
 m24cxx_write(void *dev,uint8_t data) {
 	M24Cxx *m24 = dev;
 	if(m24->state==EE_STATE_ADDR0) {
-		dprintf("M24Cxx HiAddr 0x%02x\n",data);
+		dbgprintf("M24Cxx HiAddr 0x%02x\n",data);
 		if(m24->addrlen==1) {
 			m24->mem_address = data   & (m24->size - 1);
 			m24->state = EE_STATE_DATA;
@@ -211,7 +212,7 @@ m24cxx_write(void *dev,uint8_t data) {
 			m24->state = EE_STATE_ADDR1;
 		}
 	} else if (m24->state==EE_STATE_ADDR1) {
-		dprintf("M24Cxx LoAddr 0x%02x\n",data);
+		dbgprintf("M24Cxx LoAddr 0x%02x\n",data);
 		m24->mem_address = (m24->mem_address | data) & (m24->size-1);
 		m24->state = EE_STATE_DATA;
 	} else if(m24->state==EE_STATE_DATA) {
@@ -219,7 +220,7 @@ m24cxx_write(void *dev,uint8_t data) {
 		uint32_t addr = m24->mem_address +  
 			((m24->current_i2c_addr & (m24->nr_addresses-1))<<(8*m24->addrlen));
 		addr=addr & (m24->size - 1);
-		dprintf("M24Cxx Write 0x%02x to %04x\n",data,m24->mem_address);
+		dbgprintf("M24Cxx Write 0x%02x to %04x\n",data,m24->mem_address);
 
 		wc = SigNode_Val(m24->write_control);
 		if(((addr & m24->wc_mask) != m24->wc_value) || 
@@ -229,7 +230,7 @@ m24cxx_write(void *dev,uint8_t data) {
 			m24->writebuffer_dirty |= 1<<(addr & (m24->pagesize-1));
 			m24->mem_address=((m24->mem_address+1) % m24->pagesize) | (m24->mem_address & ~(m24->pagesize-1));
 		} else {
-			dprintf("Write control is %d\n",wc);
+			dbgprintf("Write control is %d\n",wc);
 			return I2C_NACK; 
 		}
 	}
@@ -251,7 +252,7 @@ m24cxx_read(void *dev,uint8_t *data)
 	addr=addr & (m24->size - 1);
 	*data=m24->data[addr];
 	m24->mem_address=(m24->mem_address + 1)&(m24->size - 1) & ((1<<(8*m24->addrlen))-1);
-	dprintf("M24Cxx read 0x%02x from %04x\n",*data,m24->mem_address);
+	dbgprintf("M24Cxx read 0x%02x from %04x\n",*data,m24->mem_address);
 	return I2C_DONE;
 };
 
@@ -266,7 +267,7 @@ static int
 m24cxx_start(void *dev,int i2c_addr,int operation) {
 	M24Cxx *m24 = dev;
 	// may be repeated start ?
-	dprintf("m24cxx start\n");
+	dbgprintf("m24cxx start\n");
 	if(CycleCounter_Get() < m24->busy_until) {
 		return I2C_NACK;
 	}
@@ -286,7 +287,7 @@ static void
 m24cxx_stop(void *dev) {
 	M24Cxx *m24 = dev;
 	int i;
-	dprintf("m24cxx stop\n");
+	dbgprintf("m24cxx stop\n");
 	m24->state =  EE_STATE_ADDR0; 
 	if(m24->writebuffer_dirty) {
 		for(i=0;i<m24->pagesize;i++) {
@@ -314,22 +315,15 @@ static I2C_SlaveOps m24_ops = {
  */
 I2C_Slave *
 M24Cxx_New(const char *typename,const char *name) {
-	M24Cxx *m24 = malloc(sizeof(M24Cxx)); 
-	char *writecontrol_name = alloca(strlen(name)+20);
+	M24Cxx *m24 = sg_new(M24Cxx); 
 	char *dirname;
 	EEPromType *type;
 	I2C_Slave *i2c_slave;
-	if(!m24) {
-		fprintf(stderr,"Out of memory\n");
-		exit(8152);
-	}
-	memset(m24,0,sizeof(M24Cxx));
 	i2c_slave = &m24->i2c_slave;
 	i2c_slave->devops = &m24_ops; 
 	i2c_slave->dev = m24;
 	i2c_slave->speed = I2C_SPEED_FAST;
-	sprintf(writecontrol_name,"%s.wc",name);
-	m24->write_control = SigNode_New(writecontrol_name);
+	m24->write_control = SigNode_New("%s.wc",name);
 	if(!m24->write_control) {
 		fprintf(stderr,"Warning, can not create WriteControl Line for Serial eeprom %s\n",name);
 		exit(1);
@@ -360,11 +354,7 @@ M24Cxx_New(const char *typename,const char *name) {
 		} 
 		m24->data = DiskImage_Mmap(m24->disk_image);
 	} else {
-		m24->data = malloc(type->size);
-		if(!m24->data) {
-			fprintf(stderr,"Out of memory\n");
-			exit(3260);
-		}
+		m24->data = sg_calloc(type->size);
 		memset(m24->data,0xff,m24->size);
 	}
 	fprintf(stderr,"%s I2C-EEProm \"%s\" created.\n",typename,name);
