@@ -32,7 +32,7 @@ typedef struct MMCard MMCard;
 #define MMC_ERR_FAILED  4
 #define MMC_ERR_INVALID 5
 
-#define MMCARD_MAX_RESPLEN 18 /* 16 + CRC == 17, 1 for word alignment because of fifos  */
+#define MMCARD_MAX_RESPLEN 32 /* 16 + CRC == 17, 1 for word alignment because of fifos  */
 
 typedef struct 
 MMCResponse {
@@ -40,6 +40,12 @@ MMCResponse {
 	uint8_t data[MMCARD_MAX_RESPLEN];
 } MMCResponse;
 
+typedef struct MMC_DataBlock {
+	uint8_t *data;
+	int datalen;
+	uint16_t crc[4]; /* one per dat line */
+} MMC_DataBlock;
+
 #define CARD_TYPE_MMC		(2)
 #define CARD_TYPE_MMCPLUS 	(3)
 #define CARD_TYPE_SD  		(4)
@@ -49,7 +55,7 @@ int MMCard_DoCmd(MMCard *card,uint32_t cmd,uint32_t arg,MMCResponse *resp);
 void MMCard_Delete(MMCard *card);
 MMCard * MMCard_New(const char *name);
 
-typedef int MMCard_DataSink(void *dev,uint8_t *buf,int count); 
+typedef int MMCard_DataSink(void *dev,MMC_DataBlock *); 
 void MMCard_AddListener(MMCard *,void *dev,int maxpkt,MMCard_DataSink *);
 void MMCard_RemoveListener(MMCard *,void *dev);
 
@@ -59,7 +65,11 @@ void MMCard_RemoveListener(MMCard *,void *dev);
  * retval: number of Bytes, -errcode on error
  * ------------------------------------------------------------------
  */
-int MMCard_Write(MMCard *card,const uint8_t *buf,int count);
+int MMCard_Write(MMCard *card,MMC_DataBlock *);
+int MMCard_Read(MMCard *card,uint8_t *buf,int count);
+
+
+void MMCard_GotoSpi(MMCard *card);
 
 #define _MMCARD_H
 #endif
