@@ -201,3 +201,25 @@
 
 #define OCR_NOTBUSY	(1<<31)
 #define OCR_CCS		(1<<30) /* Card capacity status */
+
+/*
+ * Definitions for SPI mode
+ */
+
+#define SPIR1_IDLE		(1 << 0)
+#define SPIR1_ERASE_RESET	(1 << 1)
+#define SPIR1_ILLEGAL_CMD	(1 << 2)
+#define SPIR1_COM_CRC_ERR	(1 << 3)
+#define	SPIR1_ERASE_SEQ_ERR	(1 << 4)
+#define SPIR1_ADDR_ERR		(1 << 5)
+#define SPIR1_PARAM_ERR		(1 << 6)
+
+#define SPIR2_CARD_LCKD		(1<<0)
+#define SPIR2_WPERSKIPLCK_FAIL	(1<<1)
+#define	SPIR2_ERROR		(1<<2)
+#define SPIR2_CC_ERR		(1<<3)
+#define SPIR2_CARD_ECC_FAIL	(1<<4)
+#define SPIR2_WP_VIOLATION	(1<<5)
+#define SPIR2_ERASE_PARAM	(1<<6)
+#define SPIR2_OOR_CSD_OVER	(1<<7)
+
