@@ -33,14 +33,14 @@
 #include <sys/time.h>
 #include <sys/types.h>
 
-#include <arm9cpu.h>
-#include <cycletimer.h>
-#include <mmu.h>
-#include <bus.h>
-#include <compiler_extensions.h>
+#include "arm9cpu.h"
+#include "cycletimer.h"
+#include "mmu.h"
+#include "bus.h"
+#include "compiler_extensions.h"
 #include "fio.h"
-#include "debug.h"
 #include "mainloop_events.h"
+#include "sgstring.h"
 
 typedef void   McrProc(void *clientData,uint32_t icode,uint32_t value);
 typedef uint32_t MrcProc(void *clientData,uint32_t icode);
@@ -82,9 +82,9 @@ static uint32_t domain_access_reg;
 SystemCopro *gmmu;
 
 #ifdef DEBUG
-#define dprintf(x...) { if(unlikely(debugflags & DEBUG_MMU)) { fprintf(stderr,x); } }
+#define dbgprintf(x...) { if(unlikely(debugflags & DEBUG_MMU)) { fprintf(stderr,x); } }
 #else 
-#define dprintf(x...) 
+#define dbgprintf(x...) 
 #endif
 
 #define MMU_FAR  (gmmu->far)
@@ -192,7 +192,7 @@ update_byteorder(SystemCopro *mmu)
 		mmu_byte_addr_xor = 0x3;
 		mmu_word_addr_xor = 0x2;
 		SigNode_Set(mmu->endianNode,SIG_HIGH);
-		fprintf(stderr,"pc %08x\n",GET_REG_PC);
+		fprintf(stderr,"pc %08x\n",ARM_GET_NNIA);
 	} else {
 		fprintf(stderr,"MMU: Byteorder is now LE\n");
 		mmu_byte_addr_xor = 0;
@@ -243,11 +243,11 @@ MMU_translation_fault_section(uint32_t access_type,uint32_t far) {
 	if(likely(!MMU_DEBUGMODE)) {
 		if(access_type&MMU_ACCESS_IFETCH) {
 			MMU_IFSR=(MMU_IFSR&~FSR_FS_MASK) | FS_TRANS_SECTION;
-			ARM9_Exception(EX_PABT,8);
+			ARM_Exception(EX_PABT,4);
 		} else {
 			MMU_FAR=far;  
 			MMU_DFSR=(MMU_DFSR&~FSR_FS_MASK) | FS_TRANS_SECTION;
-			ARM9_Exception(EX_DABT,8);
+			ARM_Exception(EX_DABT,4);
 		}
 	}
 	longjmp(gcpu.abort_jump,1);
@@ -259,11 +259,11 @@ MMU_translation_fault_page(uint32_t access_type,uint32_t far,uint32_t domain) {
 	if(likely(!MMU_DEBUGMODE)) {
 		if(access_type&MMU_ACCESS_IFETCH) {
 			MMU_IFSR=(MMU_IFSR&~(FSR_FS_MASK|FSR_FS_DOMAIN_MASK)) | FS_TRANS_PAGE |(domain<<4);
-			ARM9_Exception(EX_PABT,8);
+			ARM_Exception(EX_PABT,4);
 		} else {
 			MMU_FAR=far;  
 			MMU_DFSR=(MMU_DFSR&~(FSR_FS_MASK|FSR_FS_DOMAIN_MASK)) | FS_TRANS_PAGE |(domain<<4);
-			ARM9_Exception(EX_DABT,8);
+			ARM_Exception(EX_DABT,4);
 		}	
 	}
 	longjmp(gcpu.abort_jump,1);
@@ -276,15 +276,15 @@ MMU_translation_fault_page(uint32_t access_type,uint32_t far,uint32_t domain) {
  */ 
 static inline void
 MMU_permission_fault_page(uint32_t access_type,uint32_t far,uint32_t domain) {
-//	dprintf("Permission fault page at %04x\n",access_type);
+//	dbgprintf("Permission fault page at %04x\n",access_type);
 	if(likely(!MMU_DEBUGMODE)) {
 		if(access_type & MMU_ACCESS_IFETCH) {
 			MMU_IFSR=(MMU_IFSR&~(FSR_FS_MASK|FSR_FS_DOMAIN_MASK)) | FS_PERM_PAGE |(domain<<4);
-			ARM9_Exception(EX_PABT,8);
+			ARM_Exception(EX_PABT,4);
 		} else {
 			MMU_FAR=far;  
 			MMU_DFSR=(MMU_DFSR&~(FSR_FS_MASK|FSR_FS_DOMAIN_MASK)) | FS_PERM_PAGE |(domain<<4);
-			ARM9_Exception(EX_DABT,8);
+			ARM_Exception(EX_DABT,4);
 		}	
 	}
 	longjmp(gcpu.abort_jump,1);
@@ -292,16 +292,16 @@ MMU_permission_fault_page(uint32_t access_type,uint32_t far,uint32_t domain) {
 
 static inline void 
 MMU_permission_fault_section(uint32_t access_type,uint32_t far,uint32_t domain) {
-//	dprintf("fault section %08x ptb %08x,atype %d pc: %08x\n",far,ttbl_base,access_type,REG_PC);
-//	dprintf("Permission fault section at %04x\n",access_type);
+//	dbgprintf("fault section %08x ptb %08x,atype %d pc: %08x\n",far,ttbl_base,access_type,REG_PC);
+//	dbgprintf("Permission fault section at %04x\n",access_type);
 	if(likely(!MMU_DEBUGMODE)) {
 		if(access_type & MMU_ACCESS_IFETCH) {
 			MMU_IFSR=(MMU_IFSR & ~(FSR_FS_MASK|FSR_FS_DOMAIN_MASK)) | FS_PERM_SECTION |(domain<<4);
-			ARM9_Exception(EX_PABT,8);
+			ARM_Exception(EX_PABT,4);
 		} else {
 			MMU_FAR=far;  
 			MMU_DFSR=(MMU_DFSR&~(FSR_FS_MASK|FSR_FS_DOMAIN_MASK)) | FS_PERM_SECTION |(domain<<4);
-			ARM9_Exception(EX_DABT,8);
+			ARM_Exception(EX_DABT,4);
 		}
 	}
 	longjmp(gcpu.abort_jump,1);
@@ -322,7 +322,7 @@ MMU_AlignmentException(uint32_t far) {
 		return;
 	MMU_FAR=far;
 	MMU_DFSR=(MMU_DFSR & ~(FSR_FS_MASK)) | FS_ALIGN_EX;
-	ARM9_Exception(EX_DABT,8);
+	ARM_Exception(EX_DABT,4);
 	longjmp(gcpu.abort_jump,1);
 }
 
@@ -462,7 +462,7 @@ MMU_InvalidateTlb() {
  * Dump all registers of CPU
  * ----------------------------------
  */
-UNUSED static void 
+__UNUSED__ static void 
 dump_regs() {
         int i;
         for(i=0;i<16;i++) {
@@ -528,7 +528,7 @@ dump_mctrl(SystemCopro *mmu,FILE *file) {
  * Dump complete Pagetable
  * ------------------------------------
  */
-UNUSED static void 
+__UNUSED__ static void 
 pgtables_dump(SystemCopro *mmu) 
 {
 	char filename[50];
@@ -580,7 +580,7 @@ static inline void check_cachebuff(uint32_t pgd,uint32_t vaddr,uint32_t access_t
 {
 #ifdef DEBUG
 	if((debugflags&DEBUG_MMU_CB) && ((pgd & 0xc) !=0xc)) {
-		fprintf(stderr,"page not cache and bufferable at %08x atype %08x\n",GET_REG_PC-8,access_type);
+		fprintf(stderr,"page not cache and bufferable at %08x atype %08x\n",ARM_GET_NNIA-8,access_type);
 	} 
 #endif
 }
@@ -624,7 +624,7 @@ translate_address(uint32_t addr,uint32_t access_type) {
 						domain_shift=(flpd>>4)&0x1e;
 						ap = (slpd >> (4+((addr&0xc000)>>13)))&3;
 						if(unlikely(!check_permission(ap,access_type,domain_shift))) {
-							dprintf("Large Coarse Page Permission fault\n");
+							dbgprintf("Large Coarse Page Permission fault\n");
 							MMU_permission_fault_page(access_type,addr,(flpd>>5)&0xf);
 						}
 						taddr=(slpd&0xffff0000) | (addr&0xffff);
@@ -940,7 +940,7 @@ ctrl_write(void *clientData,uint32_t icode,uint32_t value)
 	SystemCopro *mmu = (SystemCopro *) clientData;
         uint32_t crm=icode&0xf;
 	uint32_t diff;
-	dprintf("Setting MMU Control Register to %08x\n",value);
+	dbgprintf("Setting MMU Control Register to %08x\n",value);
 	if(crm!=0) {
 		fprintf(stderr,"MMU: CRM is not zero(%d) when writing CTRL\n",crm);
 		return;
@@ -987,7 +987,7 @@ mtbase_write(void *clientData,uint32_t icode,uint32_t value)
 	SystemCopro *mmu = (SystemCopro *) clientData;
 	mmu->mtbase = value;
 	ttbl_base = value;
-	dprintf("Setting Page table base to %08x\n",value);
+	dbgprintf("Setting Page table base to %08x\n",value);
 	if(value & 0x3fff) {
 		fprintf(stderr,"Bad page table base 0x%08x\n",value);
 	}
@@ -1006,7 +1006,7 @@ mdac_write(void *clientData,uint32_t icode,uint32_t value)
 {
 	SystemCopro *mmu = (SystemCopro *) clientData;
 	uint32_t diff;
-	dprintf("Load Domain Access Control Register with %08x\n",value);
+	dbgprintf("Load Domain Access Control Register with %08x\n",value);
 	diff = mmu->mdac ^ value;
 	mmu->mdac = value;
 	domain_access_reg = value;
@@ -1094,8 +1094,12 @@ mcctrl_write(void *clientData,uint32_t icode,uint32_t value)
 		last_halt=&mmu->last_halt;
 		clock_gettime(CLOCK_MONOTONIC,&tv_now);
 		nsecs = tv_now.tv_nsec-last_halt->tv_nsec + (int64_t)1000000000*(tv_now.tv_sec - last_halt->tv_sec);
-		mmu->saved_cycles=mmu->saved_cycles - (CycleTimerRate_Get()/1000000) * (nsecs/1000) +
+		mmu->saved_cycles= mmu->saved_cycles 
+			/* Subtract the  cycles which should have simulated since the last halt */
+			- (CycleTimerRate_Get()/1000000) * (nsecs/1000) +
+			/* Add the cycles which really have been simulated since last halt */
 			(CycleCounter_Get() - mmu->last_halt_cycles);
+
 		*last_halt=tv_now;
 
 		/* 
@@ -1136,7 +1140,7 @@ mcctrl_write(void *clientData,uint32_t icode,uint32_t value)
 			*last_halt=tv_now;
 		} 
 	} else {
-		dprintf("Ignore Cache settings\n");
+		dbgprintf("Ignore Cache settings\n");
 	}
 	return;
 }
@@ -1154,7 +1158,7 @@ mtlbctrl_write(void *clientData,uint32_t icode,uint32_t value)
 {
 	//fprintf(stderr,"inv op2 %08x crm %08x,data %08x\n",opcode_2,crm,data);
 	invalidate_tlb();
-	dprintf("Invalidate TLB\n");
+	dbgprintf("Invalidate TLB\n");
 	return;
 
 }
@@ -1251,7 +1255,7 @@ MMUmrc(ArmCoprocessor *copro,uint32_t icode) {
 	SystemCopro *mmu = copro->owner;
         uint32_t crn=(icode>>16)&0xf;
 	if(ARM_SIGNALING_MODE==MODE_USER) {
-		ARM9_Exception(EX_UNDEFINED,4);
+		ARM_Exception(EX_UNDEFINED,0);
 		return 0;
 	}
 
@@ -1278,7 +1282,7 @@ MMUmcr(ArmCoprocessor *copro,uint32_t icode,uint32_t data)
         uint32_t crn=(icode>>16)&0xf;
 
 	if(ARM_SIGNALING_MODE==MODE_USER) {
-		ARM9_Exception(EX_UNDEFINED,4);
+		ARM_Exception(EX_UNDEFINED,0);
 		return;
 	}
 	if(mmu->mcrProc[crn]) {
@@ -1303,22 +1307,15 @@ MMU_Create(const char *name,int endian,uint32_t mmu_type)
 	SystemCopro * mmu;
 //	uint32_t variant = mmu_type & 0xffff;
 	uint32_t type = mmu_type & 0xffff0000;
-	char *nodename = alloca(strlen(name)+30);
-	gmmu=mmu=malloc(sizeof(SystemCopro));
-	if(!mmu) {
-		fprintf(stderr,"Out of memory\n");
-		exit(3);
-	}
-	memset(mmu,0,sizeof(*mmu));
+	gmmu = mmu = sg_new(SystemCopro);
 	mmu->ctrl=0x50078;
 	mmu->iclck = 0x0000fff0;
 	mmu->dclck = 0x0000fff0;
 	fprintf(stderr,"- Create MMU Coprocessor\n");
 
-	sprintf(nodename,"%s.endian",name);
-        mmu->endianNode = SigNode_New(nodename);
+        mmu->endianNode = SigNode_New("%s.endian",name);
         if(!mmu->endianNode) {
-                fprintf(stderr,"Can not create MMU. EndianNode %s\n",nodename);
+                fprintf(stderr,"Can not create MMU. EndianNode \n");
                 exit(3429);
         }
 	if(endian == en_BIG_ENDIAN) {
