@@ -33,11 +33,12 @@
 #include <ns9750_bbus.h>
 #include <ns9750_bbdma.h>
 #include "signode.h"
+#include "sgstring.h"
 
 #if 1
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 
 
@@ -347,7 +348,7 @@ load_descriptor(BufDescr *bd,uint32_t addr)
 	uint32_t src = addr;
 	int i;
 	uint32_t *data = (uint32_t *) bd;
-	dprintf("load descriptor from 0x%08x\n",addr);
+	dbgprintf("load descriptor from 0x%08x\n",addr);
 	for(i=0;i<4;i++) {
 		*data=Bus_Read32(src);
 		//Bus_Read(data,src,4);
@@ -440,7 +441,7 @@ update_interrupt(BBusDMA_Channel *chan)
  *	from peripheral
  * -------------------------------------------------------------
  */
-UNUSED static void 
+__UNUSED__ static void 
 RetireFbwDescriptor(BBusDMA_Channel *chan) 
 {
 	BufDescr *descr=&chan->bufDescr;
@@ -489,7 +490,7 @@ BBDMA_Fbw(BBusDMA_Channel *chan,uint8_t *buf,int count)
 	int transferlen; 
 	
 	if(!(chan->dmactrl & CTRL_CE)) {
-		// dprintf("DMA Channel %d is not enabled, ignoring data\n");
+		// dbgprintf("DMA Channel %d is not enabled, ignoring data\n");
 		return 0;
 	}
 	if((chan->dmactrl & CTRL_MODE_MASK) != CTRL_MODE_FLY_BY_WRITE) {
@@ -532,13 +533,13 @@ DMA_Read(BBusDMA_Channel *chan,uint8_t *buf,int maxlen)
 	int count; 
 	
 	if((chan->dmactrl & CTRL_CE) == 0) {
-		dprintf("DMA Channel %d is not enabled, reading from nowhere\n",chan->nr);
+		dbgprintf("DMA Channel %d is not enabled, reading from nowhere\n",chan->nr);
 		return 0;
 	}
 	if((chan->dmactrl & CTRL_CA) == 0) {
 		chan->dmactrl = chan->dmactrl & ~CTRL_CE;
 		chan->dmastatier |= STATIER_CAIP;
-		dprintf("DMA Channel %d abort bit is still set\n",chan->nr);
+		dbgprintf("DMA Channel %d abort bit is still set\n",chan->nr);
 		update_interrupt(chan);
 		return 0;
 	}
@@ -552,7 +553,7 @@ DMA_Read(BBusDMA_Channel *chan,uint8_t *buf,int maxlen)
 	bytes_remaining = buflen - chan->bytes_done;
 	if(!(descr->flags_status & BD_FLAG_FULL)) {
 		//chan->dmactrl &= ~CTRL_CE; /* ???? Netos sets it itself to 0 in tx_dma_int */
-		dprintf("Buffer not full\n");
+		dbgprintf("Buffer not full\n");
 		chan->dmastatier |= STATIER_NRIP;	
 		update_interrupt(chan);
 		dump_descriptor(descr);
@@ -636,27 +637,27 @@ static void
 do_fbr(BBusDMA_Channel *chan) {
 	uint8_t buf[64];
 	if(SigNode_Val(chan->FbrDmaReq)==SIG_LOW) {
-		dprintf("do_fbr: No request\n");
+		dbgprintf("do_fbr: No request\n");
 	}
 	while(SigNode_Val(chan->FbrDmaReq)==SIG_HIGH) {
 		int count;
 		int usedcount;
 		if(!(chan->dmactrl & CTRL_CE)) {
-			dprintf("do_fbr(%d): Channel not enabled\n",chan->nr);
+			dbgprintf("do_fbr(%d): Channel not enabled\n",chan->nr);
 			return;
 		}
 		fprintf(stderr,"do_fbr\n");
 		if(!chan->FbrProc) {
-			dprintf("do_fbr: No Fbr callback\n");
+			dbgprintf("do_fbr: No Fbr callback\n");
 			return;
 		}	
 		count = DMA_Read(chan,buf,64);
 		if(count<=0) {
-			dprintf("DMA_Read: failed with %d\n",count);
+			dbgprintf("DMA_Read: failed with %d\n",count);
 			return;
 		}
 		usedcount=chan->FbrProc(chan,buf,count,chan->FbrClientData);
-		dprintf("FbrProc transfered %d bytes\n",usedcount);
+		dbgprintf("FbrProc transfered %d bytes\n",usedcount);
 		if(usedcount<=0) {
 			return;
 		}
@@ -670,7 +671,7 @@ do_fbr(BBusDMA_Channel *chan) {
  * This function might be invoked recursively !
  * -----------------------------------------------------------
  */
-static int
+static void 
 fbr_dma_request_proc(struct SigNode * node,int value, void *clientData)
 {
 	BBusDMA_Channel *chan = clientData;
@@ -678,13 +679,11 @@ fbr_dma_request_proc(struct SigNode * node,int value, void *clientData)
 		fprintf(stderr,"got tx dma request\n");
 		do_fbr(chan);
         } else if((value==SIG_LOW)) {
-		fprintf(stderr,"removed tx dma request %s\n",node->name);
+		fprintf(stderr,"removed tx dma request %s\n",SigName(node));
 
         } else {
 		fprintf(stderr,"Illegal value %d for DMA request node\n",value);
 	}
-//      fprintf(stderr,"trace val %d, gpio %d\n",value,gpio);
-        return 0;
 }
 
 
@@ -696,7 +695,7 @@ fbr_dma_request_proc(struct SigNode * node,int value, void *clientData)
 static uint32_t 
 bdp_read(void *clientData,uint32_t address,int rqlen) {
 	BBusDMA_Channel *chan=clientData;	
-	dprintf("bbdma bdp read:  0x%08x\n",chan->fbdp);
+	dbgprintf("bbdma bdp read:  0x%08x\n",chan->fbdp);
 	return chan->fbdp;
 }
 
@@ -704,7 +703,7 @@ static void
 bdp_write(void *clientData,uint32_t value,uint32_t address,int rqlen)  {
 	BBusDMA_Channel *chan=clientData;	
 	chan->fbdp=value;
-	dprintf("bbdma bdp write:  0x%08x\n",chan->fbdp);
+	dbgprintf("bbdma bdp write:  0x%08x\n",chan->fbdp);
 	return;
 }
 
@@ -719,7 +718,7 @@ dmactrl_read(void *clientData,uint32_t address,int rqlen)
 {
 	BBusDMA_Channel *chan=clientData;	
 	chan->dmactrl =  (chan->dmactrl & ~0x3ff & ~0xfc00) | (chan->byte_index & 0x3ff);
-	dprintf("bbdmactrl(%d) read: 0x%08x\n",chan->nr,chan->dmactrl);
+	dbgprintf("bbdmactrl(%d) read: 0x%08x\n",chan->nr,chan->dmactrl);
 	return chan->dmactrl;
 }
 
@@ -760,7 +759,7 @@ dmactrl_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 			}
 		}
 	}
-	dprintf("bbdmactrl(%d) write: 0x%08x is now 0x%08x\n",chan->nr,value,chan->dmactrl);
+	dbgprintf("bbdmactrl(%d) write: 0x%08x is now 0x%08x\n",chan->nr,value,chan->dmactrl);
 	return;
 }
 
@@ -781,7 +780,7 @@ dmastatier_read(void *clientData,uint32_t address,int rqlen)
 	chan->dmastatier |= (uint16_t)(descr->buflen-chan->bytes_done);
 	chan->dmastatier |= (((uint32_t)descr->flags_status) >> 12) & 
 		(STATIER_WRAP | STATIER_IDONE | STATIER_LAST | STATIER_FULL); 
-	dprintf("bbdma statier(%d) read: 0x%08x\n",chan->nr,chan->dmastatier);
+	dbgprintf("bbdma statier(%d) read: 0x%08x\n",chan->nr,chan->dmastatier);
 	return chan->dmastatier;
 }
 
@@ -791,7 +790,7 @@ dmastatier_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 	BBusDMA_Channel *chan=clientData;	
 	uint32_t clearmask = ~(value & 0xf0000000);
 	uint32_t overtakemask = 0x7f00000;
-	dprintf("bbdma statier(%d) write 0x%08x\n",chan->nr,value);
+	dbgprintf("bbdma statier(%d) write 0x%08x\n",chan->nr,value);
 	chan->dmastatier &= clearmask;
 	chan->dmastatier = (chan->dmastatier & ~overtakemask) | (value & overtakemask);
 	update_interrupt(chan);
@@ -836,12 +835,7 @@ BBusDMA_UnMap(void *owner,uint32_t base,uint32_t mask) {
 BBusDMACtrl *
 NS9750_BBusDMA_New(char *name) {
 	int i;
-	char *nodename=alloca(strlen(name)+30);
-	BBusDMACtrl *bbdma=malloc(sizeof(BBusDMACtrl));
-	if(!bbdma) {
-		exit(1785);
-	}
-	memset(bbdma,0,sizeof(BBusDMACtrl));
+	BBusDMACtrl *bbdma = sg_new(BBusDMACtrl);
 	bbdma->bdev.first_mapping=NULL;
         bbdma->bdev.Map=BBusDMA_Map;
         bbdma->bdev.UnMap=BBusDMA_UnMap;
@@ -855,17 +849,15 @@ NS9750_BBusDMA_New(char *name) {
 		chan->spec=spec;
 
 		if(spec->cap & CAP_FBR) {
-			sprintf(nodename,"%s.%d.FbrDmaReq",name,i);
-			chan->FbrDmaReq = SigNode_New(nodename);
+			chan->FbrDmaReq = SigNode_New("%s.%d.FbrDmaReq",name,i);
 			if(!(chan->FbrDmaReq)) {
-				fprintf(stderr,"Can not create FBR DMA request node \"%s\"\n",nodename);
+				fprintf(stderr,"Can not create FBR DMA request node for \"%s\"\n",name);
 				exit(4327);
 			}
 			SigNode_Trace(chan->FbrDmaReq,fbr_dma_request_proc,chan);
 		}
 		if(spec->cap & CAP_FBW) {
-			sprintf(nodename,"%s.%d.FbwDmaGnt",name,i);
-			chan->FbwDmaGnt = SigNode_New(nodename);
+			chan->FbwDmaGnt = SigNode_New("%s.%d.FbwDmaGnt",name,i);
 			if(!(chan->FbwDmaGnt)) {
 				fprintf(stderr,"Can not create FBW DMA request node for channel %d\n",i);
 				exit(4325);
