@@ -33,11 +33,12 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#include <ns9750_timer.h>
-#include <ns9750_bbus.h>
-#include <signode.h>
-#include <bus.h>
-#include <configfile.h>
+#include "ns9750_timer.h"
+#include "ns9750_bbus.h"
+#include "signode.h"
+#include "bus.h"
+#include "configfile.h"
+#include "sgstring.h"
 
 #define GPIO_MODE(bbus,n) (((bbus)->gpiocfg[(n)>>3] >> (((n)&7)<<2)) & 0xf)
 
@@ -381,7 +382,7 @@ bbu_update_gpio(BBus *bb,unsigned int n) {
 	}
 }
 
-static int 
+static void 
 irq_trace_proc(struct SigNode *node,int value,void *clientData) 
 {
 	Irq_TraceInfo *ti = clientData;	
@@ -392,7 +393,6 @@ irq_trace_proc(struct SigNode *node,int value,void *clientData)
 	} else if(value == SIG_HIGH) {
 		BBus_UnPostIRQ(irq);
 	}	
-	return 0;
 }
 
 /*
@@ -402,7 +402,7 @@ irq_trace_proc(struct SigNode *node,int value,void *clientData)
  * 	changes. The Change will be reported in the gpiostat registers 
  * ---------------------------------------------------------------------
  */
-static int
+static void 
 gpio_trace_proc(struct SigNode * node,int value, void *clientData) 
 {
 	GPIO_TraceInfo *ti = clientData;	
@@ -417,8 +417,6 @@ gpio_trace_proc(struct SigNode * node,int value, void *clientData)
 	} else if((value == SIG_PULLDOWN) ||(value==SIG_LOW)) {
 		bb->gpiostat[index] &= ~mask;
 	}
-//	fprintf(stderr,"trace val %d, gpio %d\n",value,gpio);
-	return 0;
 }
 
 
@@ -806,16 +804,11 @@ NS9xxx_BBusDel(BusDevice *dev) {
  */
 BusDevice *
 NS9xxx_BBusNew(char *mode,char *devname) {
-	BBus *bb = malloc(sizeof(BBus));
+	BBus *bb = sg_new(BBus);
 	char **rst_nodes;
 	char **endian_nodes;
 	char str[80];
 	int i;
-	if(!bb) {
-		fprintf(stderr,"Out of memory \n");
-		exit(0);
-	}
-	memset(bb,0,sizeof(BBus));
 	bbus=bb;
 	if(!strcmp(mode,"NS9750")) {
 		rst_nodes=ns9750_reset_nodes;
