@@ -37,6 +37,7 @@
 #include <phy.h>
 #include "byteorder.h"
 #include "signode.h"
+#include "sgstring.h"
 
 #if 0
 #define simulate_errors 1
@@ -45,9 +46,9 @@
 #endif
 
 #if 0
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 #define RSTSTT_ON_RD(eth) ((eth)->egcr2 & EGCR2_AUTOZ)
 
@@ -294,7 +295,7 @@ typedef struct NS9750eth {
  *      Invoked when the Data-Endian Signal Line changes
  * -------------------------------------------------------
  */
-static int
+static void
 change_dataendian(SigNode *node,int value,void *clientData)
 {
         NS9750eth *eth = clientData;
@@ -306,7 +307,6 @@ change_dataendian(SigNode *node,int value,void *clientData)
                 fprintf(stderr,"NS9750 Eth: Data endian is neither Little nor Big\n");
                 exit(3424);
         }
-        return 0;
 }
 
 static void
@@ -360,12 +360,12 @@ update_tx_interrupt(NS9750eth *eth)
 			int result;
 			eth->txint_posted=1;
 			result=Sysco_PostIrq(NS9750_IRQ_ETH_TRANSMIT);
-			dprintf("Posted a Transmit Interrupt result %d\n",result);
+			dbgprintf("Posted a Transmit Interrupt result %d\n",result);
 		} 
 	} else {
 		if(eth->txint_posted) {
 			eth->txint_posted=0;
-			dprintf("UnPoste einen Transmit Interrupt\n");
+			dbgprintf("UnPoste einen Transmit Interrupt\n");
 			Sysco_UnPostIrq(NS9750_IRQ_ETH_TRANSMIT);
 		} 
 	}	
@@ -467,7 +467,7 @@ update_rx_statistics(NS9750eth *eth,int len,int type) {
  */
 static void
 EthResetRx(NS9750eth *eth) {
-	dprintf("Reset RX\n");
+	dbgprintf("Reset RX\n");
 	/* EGCR1_RXINIT is not reset on RX-Reset in real device */
 	/* RXPTRS are not reset on EGCR1_ERX in real device */
 	eth->rxfifo_count=0;
@@ -570,7 +570,7 @@ void
 save_rx_descriptor(NS9750eth *eth,RBDescriptor *rbd,uint32_t addr) {
 	Bus_Write32(rbd->blength,addr+4);
 	Bus_Write32(rbd->flags,addr+0xc);
-	dprintf("wrote packet length of %d rbd to %08x\n",rbd->blength,addr);
+	dbgprintf("wrote packet length of %d rbd to %08x\n",rbd->blength,addr);
 }
 
 void
@@ -578,7 +578,7 @@ load_rx_descriptor(NS9750eth *eth,RBDescriptor *rbd,uint32_t addr) {
 	rbd->source=Bus_Read32(addr);
 	rbd->blength=Bus_Read32(addr+4);
 	rbd->flags=Bus_Read32(addr+0xc);
-	dprintf("loaded rx descriptor from %08x\n",addr);
+	dbgprintf("loaded rx descriptor from %08x\n",addr);
 
 }
 
@@ -592,7 +592,7 @@ fill_rxfifo(NS9750eth *eth) {
         int result;
         char buf[2048];
         if(eth->rxfifo_count) {
-                dprintf("Dropping paket because rxfifo is full !\n");
+                dbgprintf("Dropping paket because rxfifo is full !\n");
 		eth->eintr |= IR_RXOVFL_DATA;	
 		update_rx_interrupt(eth);
                 return read(eth->ether_fd,buf,2048);
@@ -630,14 +630,14 @@ receive(NS9750eth *eth) {
 	for(i=0;i<4;i++) {
 		RBDescriptor *rbd=&eth->int_rbdescr[i];
 		if(rbd->flags & RB_FULL) {
-			dprintf("rbd %d is full\n",i);
+			dbgprintf("rbd %d is full\n",i);
 			continue;
 		}
 		if(rbd->blength<bus_size) {
 			continue;
 		}
 		if(!(rbd->flags & RB_ENA)) {
-			dprintf("rbd %d not enabled\n",i);
+			dbgprintf("rbd %d not enabled\n",i);
 			continue;
 		}
 		if(rbd->blength >= bus_size) {
@@ -704,7 +704,7 @@ receive(NS9750eth *eth) {
 			return;
 		}
 	} 
-	dprintf("NS9750_Eth: No empty slot for received packet\n");
+	dbgprintf("NS9750_Eth: No empty slot for received packet\n");
 }
 
 /*
@@ -757,39 +757,39 @@ update_receiver_state(NS9750eth *eth)
 {
 	int enable=1;
 	if(!(eth->egcr1 & EGCR1_ERX)) {
-		dprintf("MAC1 RX is in Reset\n");
+		dbgprintf("MAC1 RX is in Reset\n");
 		enable=0;
 	}
 	if(!(eth->egcr1 & EGCR1_ERXDMA)) {
-		dprintf("ERXDMA is disabled\n");
+		dbgprintf("ERXDMA is disabled\n");
 		enable=0;
 	}
 	if(eth->egcr1 & EGCR1_MAC_HRST)  {
 		enable=0;
 	}
 	if(eth->mac1 & MAC1_SRST) {
-		dprintf("MAC1_SRST\n");
+		dbgprintf("MAC1_SRST\n");
 		enable=0;	
 	}
 	if(eth->mac1 & MAC1_RPERFUN) {
-		dprintf("MAC1_RPERFUN\n");
+		dbgprintf("MAC1_RPERFUN\n");
 		enable=0;	
 	}
 	if(eth->mac1 & MAC1_RPEMCSR) {
-		dprintf("MAC1_RPEMCSE\n");
+		dbgprintf("MAC1_RPEMCSE\n");
 		enable=0;	
 	}
 	if(!(eth->mac1 & MAC1_RXEN)) {
-		dprintf("!MAC1_RXEN\n");
+		dbgprintf("!MAC1_RXEN\n");
 		enable=0;	
 	}
 	if(eth->egcr1 & EGCR1_ERXINIT) {
-		dprintf("ERXININT\n");
+		dbgprintf("ERXININT\n");
 		enable=0;
 	}
 	if(enable) {
 		if(!eth->receiver_is_enabled && (eth->ether_fd >=0)) {
-			dprintf("ns9750 eth: enable receiver\n");
+			dbgprintf("ns9750 eth: enable receiver\n");
 			FIO_AddFileHandler(&eth->input_fh,eth->ether_fd,FIO_READABLE,input_event,eth);
 			eth->receiver_is_enabled=1;
 		}
@@ -816,7 +816,7 @@ update_transmitter_state(NS9750eth *eth)
 		enable=0;
 	}
 	eth->tx_enabled=enable;
-	dprintf("egcr is %08x, tx_enabled is %d\n",eth->egcr1,enable);
+	dbgprintf("egcr is %08x, tx_enabled is %d\n",eth->egcr1,enable);
 }
 
 /*
@@ -833,9 +833,9 @@ preload_rx_descriptors(NS9750eth *eth) {
 		rxdesc_addr=eth->rxptr[i];						
 		load_rx_descriptor(eth,rbd,rxdesc_addr);	
 		eth->rxoff[i]=0; 
-		dprintf("preload %d, from %08x, source %08x,length %08x\n",i,rxdesc_addr,rbd->source,rbd->blength);
+		dbgprintf("preload %d, from %08x, source %08x,length %08x\n",i,rxdesc_addr,rbd->source,rbd->blength);
 	}
-	dprintf("Preloaded RX-Descriptors\n");
+	dbgprintf("Preloaded RX-Descriptors\n");
 }
 
 /*
@@ -867,7 +867,7 @@ transmit_frame(NS9750eth *eth)
 	TBDescriptor tbd;
 	eth->etsr=0;
 	load_tx_descriptor(eth,&tbd,eth->txoff>>2);
-	dprintf("called transmit frame txoff %04x\n",eth->txoff);
+	dbgprintf("called transmit frame txoff %04x\n",eth->txoff);
 	eth->txfifo_count=0;
 	while(1) {
 		int i;
@@ -875,15 +875,15 @@ transmit_frame(NS9750eth *eth)
 			eth->txsptr=eth->txoff>>2;
 			eth->eintr |= IR_TX_IDLE;
 			update_tx_interrupt(eth);
-			dprintf("not full src %08x, flags %08x\n",tbd.source,tbd.flags);
+			dbgprintf("not full src %08x, flags %08x\n",tbd.source,tbd.flags);
 			return RET_NONE_AVAILABLE;
 		}		
 		if(tbd.flags & TB_IRQ) {
-			dprintf("setze einen BUFC Interrupt\n");
+			dbgprintf("setze einen BUFC Interrupt\n");
 			eth->eintr |= IR_TX_BUFC;
 		}
 		if(eth->txfifo_count+tbd.blength>2047) {
-			dprintf("TX-Packet is to big\n");	
+			dbgprintf("TX-Packet is to big\n");	
 			eth->etsr |= ETSR_TXAJ;
 			break;
 		}
@@ -908,16 +908,16 @@ transmit_frame(NS9750eth *eth)
 		tbd.flags &= ~TB_FULL;
 		save_tx_descriptor_flags(eth,&tbd,eth->txoff>>2);
 		if(tbd.flags & TB_WRAP) {
-			dprintf("TB_WRAP\n");
+			dbgprintf("TB_WRAP\n");
 			eth->txoff=0;
 		} else {
 			eth->txoff=(eth->txoff+16)&0x3ff;
-			dprintf("New TB off %08x\n",eth->txoff);
+			dbgprintf("New TB off %08x\n",eth->txoff);
 		}
 		if(tbd.flags & TB_LAST) {
 			int count=0;
 			int len=eth->txfifo_count;
-			dprintf("Send the packet\n");
+			dbgprintf("Send the packet\n");
 			do  {
 				int result;
 				result=write(eth->ether_fd,((char*)eth->txfifo)+count,len-count);
@@ -926,7 +926,7 @@ transmit_frame(NS9750eth *eth)
 				} else {
 					if(errno!=EAGAIN) {
 						eth->etsr &= ~ETSR_TXOK; 
-						dprintf("NS9750eth: error writing to Linux TAP fd\n");
+						dbgprintf("NS9750eth: error writing to Linux TAP fd\n");
 					   	return RET_ERROR;
 					}
 				}
@@ -935,7 +935,7 @@ transmit_frame(NS9750eth *eth)
 			update_tx_statistics(eth,len,0 /* type */); 
 			break;
 		} else {
-			dprintf("Not last: flags %08x\n",tbd.flags);
+			dbgprintf("Not last: flags %08x\n",tbd.flags);
 		}
 		load_tx_descriptor(eth,&tbd,eth->txoff>>2);
 		if(!(tbd.flags & TB_FULL)) {
@@ -1027,7 +1027,7 @@ egcr2_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 			eth->txoff=eth->txrptr<<2;
 			transmit(eth);
 		} else {
-			dprintf("clear error flag without error\n");
+			dbgprintf("clear error flag without error\n");
 		}
 	}
 	if(value&EGCR2_CLRCNT) {
@@ -1098,7 +1098,7 @@ mac1_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 		return;
 	}
 	eth->mac1=value;
-	dprintf("New mac1 %08x\n",value);
+	dbgprintf("New mac1 %08x\n",value);
 	update_receiver_state(eth);
 	update_mii_state(eth,0);
 	return;
@@ -1248,7 +1248,7 @@ mcmd_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 		uint16_t result=0xffff;
 		phy=eth->phy[nr_phy];
 		if(!phy) {
-			dprintf("No phy with address %d\n",nr_phy);
+			dbgprintf("No phy with address %d\n",nr_phy);
 		} else {
 			PHY_ReadRegister(phy,&result,reg_nr);
 		}
@@ -1298,7 +1298,7 @@ mwtd_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	}
 	phy=eth->phy[nr_phy];
 	if(!phy) {
-		dprintf("No phy with address %d\n",nr_phy);
+		dbgprintf("No phy with address %d\n",nr_phy);
 		return;
 	}
 	PHY_WriteRegister(phy,value,reg_nr);
@@ -1321,7 +1321,7 @@ mrdd_read(void *clientData,uint32_t address,int rqlen) {
 		unsigned int reg_nr = (eth->madr &0x1f);
 		phy=eth->phy[nr_phy];
 		if(!phy) {
-			dprintf("No phy with address %d\n",nr_phy);
+			dbgprintf("No phy with address %d\n",nr_phy);
 		}
 		PHY_ReadRegister(phy,&result,reg_nr);
 		eth->mrdd=result;
@@ -2316,13 +2316,7 @@ NS9750_EthRegisterPhy(BusDevice *dev,PHY_Device *phy,unsigned int phy_addr)
 
 BusDevice *
 NS9750_EthInit(const char *devname) {
-	char *nodename = alloca(strlen(devname)+30);
-        NS9750eth *eth = malloc(sizeof(NS9750eth));
-        if(!eth) {
-                fprintf(stderr,"Out of memory \n");
-                exit(480);
-        }
-        memset(eth,0,sizeof(NS9750eth));
+        NS9750eth *eth = sg_new(NS9750eth);
 	eth->ether_fd = Net_CreateInterface(devname);
         fcntl(eth->ether_fd,F_SETFL,O_NONBLOCK);
 
@@ -2332,10 +2326,9 @@ NS9750_EthInit(const char *devname) {
         eth->bdev.owner=eth;
         eth->bdev.hw_flags=MEM_FLAG_WRITABLE|MEM_FLAG_READABLE;
 
-	sprintf(nodename,"%s.dataendian",devname);
-        eth->dataEndianNode = SigNode_New(nodename);
+        eth->dataEndianNode = SigNode_New("%s.dataendian",devname);
         if(!eth->dataEndianNode) {
-                fprintf(stderr,"Can not create Ser. EndianNode %s\n",nodename);
+                fprintf(stderr,"Can not create Ser. EndianNode\n");
                 exit(3429);
         }
         eth->dataEndianTrace = SigNode_Trace(eth->dataEndianNode,change_dataendian,eth);
