@@ -33,12 +33,13 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <compiler_extensions.h>
-#include <configfile.h>
-#include <arm9cpu.h>
-#include <ns9750_mem.h>
-#include <bus.h>
-#include <signode.h>
+#include "compiler_extensions.h"
+#include "configfile.h"
+#include "arm9cpu.h"
+#include "ns9750_mem.h"
+#include "bus.h"
+#include "signode.h"
+#include "sgstring.h"
 
 /*
  * ------------------------------------------------------------
@@ -93,7 +94,7 @@ mctrl_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 		NS9750_rebuild_map(memco);
 	}
 	if(!(memco->mctrl & MCTRL_MCEN)) {
-		fprintf(stderr,"Disabling memory controller at %08x\n",GET_REG_PC-8);
+		fprintf(stderr,"Disabling memory controller at %08x\n",ARM_GET_NNIA-8);
 	}
 	return;
 }
@@ -825,15 +826,9 @@ NS9750_MemController *
 NS9750_MemCoInit(const char *name) 
 {
 	NS9750_MemController *memco;
-	char *nodename  = alloca(strlen(name)+30);
 	uint32_t i;
 	int result;
-	memco=malloc(sizeof(NS9750_MemController));	
-	if(!memco) {
-		perror("Error allocating mem context\n");
-		exit(48);
-	}
-	memset(memco,0,sizeof(*memco));
+	memco = sg_new(NS9750_MemController);	
 	/* Default register state */
 	for(i=0;i<8;i++) {
 		int cs=(i+4)&7;
@@ -977,10 +972,9 @@ NS9750_MemCoInit(const char *name)
 	if(!result) {
 		memco->mconfig |= 1;
 	}
-	sprintf(nodename,"%s.big_endian",name);
-	memco->big_endianNode = SigNode_New(nodename);
+	memco->big_endianNode = SigNode_New("%s.big_endian",name);
         if(!memco->big_endianNode) {
-                fprintf(stderr,"Can not create big_endian Node %s\n",nodename);
+                fprintf(stderr,"Can not create big_endian Node\n");
                 exit(3427);
         }
 	if(memco->mconfig & MCONFIG_END) {
