@@ -32,12 +32,13 @@
 #include "bus.h"
 #include "byteorder.h"
 #include "signode.h"
+#include "sgstring.h"
 
 
 #if 0
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 
 /* temporary hack */
@@ -106,7 +107,7 @@ typedef struct PCI_Bridge {
  *      Invoked when the Endian Signal Line changes
  * -------------------------------------------------------
  */
-static int
+static void 
 change_endian(SigNode *node,int value,void *clientData)
 {
         PCI_Bridge *br = clientData;
@@ -128,7 +129,6 @@ change_endian(SigNode *node,int value,void *clientData)
 			cursor->Map(cursor->owner,cursor->hw_flags);
 		}
 	}
-        return 0;
 }
 
 
@@ -222,12 +222,12 @@ pci_config_data_write(void *cd,uint32_t value,uint32_t address,int rqlen) {
 	}
 
 	if(((bus==0) && (type!=0)) || ((bus!=0) && (type==0))) {
-		dprintf("wrong access type %d for bus %d, config_addr %08x\n",type,bus,config_addr);
+		dbgprintf("wrong access type %d for bus %d, config_addr %08x\n",type,bus,config_addr);
 		return;
 	}
 	pcifunc=pci_find_function(br,bus,device,function);
 	if(!pcifunc) {
-//		dprintf("device not found,b %d,d %d,f %d\n",bus,device,function);
+//		dbgprintf("device not found,b %d,d %d,f %d\n",bus,device,function);
 		return;
 	}
 	if(pcifunc->configWrite) {
@@ -262,12 +262,12 @@ pci_config_data_read(void *cd,uint32_t address,int rqlen) {
 		}
 	}
 	if(((bus==0) && (type!=0)) || ((bus!=0) && (type==0))) {
-		dprintf("wrong access type %d for bus %d, config_addr %08x\n",type,bus,config_addr);
+		dbgprintf("wrong access type %d for bus %d, config_addr %08x\n",type,bus,config_addr);
 		return 0;
 	}
 	pcifunc=pci_find_function(br,bus,device,function);
 	if(!pcifunc) {
-//		dprintf("device not found,b %d,d %d,f %d\n",bus,device,function);
+//		dbgprintf("device not found,b %d,d %d,f %d\n",bus,device,function);
 		return 0;
 	}
 	if(pcifunc->configRead) {
@@ -373,7 +373,7 @@ BR_ConfigRead(uint32_t *value,uint8_t reg,PCI_Function *pcifunc)
 			break;
 				
 	}
-//	dprintf("BR-Config Read reg %d val %08x\n",reg*4, *value);
+//	dbgprintf("BR-Config Read reg %d val %08x\n",reg*4, *value);
 	return 0;
 }
 
@@ -399,7 +399,7 @@ BR_ConfigWrite(uint32_t value,uint32_t mask,uint8_t reg,PCI_Function *pcifunc)
 				value=value & 0xf0000000;
 				MODIFY_MASKED(br->bar0,value,mask);
 			} else {
-				dprintf("NS9750_Pci: Write to disabled BAR0\n");
+				dbgprintf("NS9750_Pci: Write to disabled BAR0\n");
 			}
 			break;
 		case 0x5:
@@ -407,7 +407,7 @@ BR_ConfigWrite(uint32_t value,uint32_t mask,uint8_t reg,PCI_Function *pcifunc)
 				value=value & 0xfc000000;
 				MODIFY_MASKED(br->bar1,value,mask);
 			} else {
-				dprintf("NS9750_Pci: Write to disabled BAR1\n");
+				dbgprintf("NS9750_Pci: Write to disabled BAR1\n");
 			}
 			break;
 		case 0x6:
@@ -415,7 +415,7 @@ BR_ConfigWrite(uint32_t value,uint32_t mask,uint8_t reg,PCI_Function *pcifunc)
 				value=value & 0xff000000;
 				MODIFY_MASKED(br->bar2,value,mask);
 			} else {
-				dprintf("NS9750_Pci: Write to disabled BAR2\n");
+				dbgprintf("NS9750_Pci: Write to disabled BAR2\n");
 			}
 			break;
 		case 0x7:
@@ -423,7 +423,7 @@ BR_ConfigWrite(uint32_t value,uint32_t mask,uint8_t reg,PCI_Function *pcifunc)
 				value=value & 0xffc00000;
 				MODIFY_MASKED(br->bar3,value,mask);
 			} else {
-				dprintf("NS9750_Pci: Write to disabled BAR3\n");
+				dbgprintf("NS9750_Pci: Write to disabled BAR3\n");
 			}
 			break;
 		case 0x8:
@@ -431,7 +431,7 @@ BR_ConfigWrite(uint32_t value,uint32_t mask,uint8_t reg,PCI_Function *pcifunc)
 				value=value & 0xfff00000;
 				MODIFY_MASKED(br->bar4,value,mask);
 			} else {
-				dprintf("NS9750_Pci: Write to disabled BAR4\n");
+				dbgprintf("NS9750_Pci: Write to disabled BAR4\n");
 			}
 			break;
 		case 0x9:
@@ -439,7 +439,7 @@ BR_ConfigWrite(uint32_t value,uint32_t mask,uint8_t reg,PCI_Function *pcifunc)
 				value=value & 0xfffc0000;
 				MODIFY_MASKED(br->bar5,value,mask);
 			} else {
-				dprintf("NS9750_Pci: Write to disabled BAR5\n");
+				dbgprintf("NS9750_Pci: Write to disabled BAR5\n");
 			}
 			break;
 		case 0xf:
@@ -465,14 +465,8 @@ NS9750_PCIUnPostIRQ(BusDevice *dev,int irq) {
 
 PCI_Function *
 BR_New(const char *name,int dev_nr) {
-	char *nodename = alloca(strlen(name)+30);
 	PCI_Bridge *br;
-	br=malloc(sizeof(PCI_Bridge));	
-	if(!br) {
-		fprintf(stderr,"Out of memory\n");	
-		exit(2434);
-	}
-	memset(br,0,sizeof(PCI_Bridge));
+	br = sg_new(PCI_Bridge);	
 	br->device_id = PCI_DEVICE_ID_DIGI_NS9750;
 	br->vendor_id = PCI_VENDOR_ID_DIGI;
 	br->class_code=0x060000;
@@ -526,10 +520,9 @@ BR_New(const char *name,int dev_nr) {
 	br->csktpst=0;
 	br->csktfev=0;
 	br->csktctl=0;
-	sprintf(nodename,"%s.cpu_endian",name);
-        br->endianNode = SigNode_New(nodename);
+        br->endianNode = SigNode_New("%s.cpu_endian",name);
         if(!br->endianNode) {
-                fprintf(stderr,"Can not create Host. EndianNode %s\n",nodename);
+                fprintf(stderr,"Can not create Host. EndianNode\n");
                 exit(3429);
         }
 	br->endianTrace = SigNode_Trace(br->endianNode,change_endian,br);
