@@ -25,7 +25,6 @@
  */
 
 
-#include <bus.h>
 #include <string.h>
 #include <fcntl.h>
 #include <stdint.h>
@@ -37,20 +36,21 @@
 #include <signal.h>
 #include <termios.h>
 #include <sys/ioctl.h>
-#include <arm9cpu.h>
-#include <cycletimer.h>
-#include <ns9750_serial.h>
-#include <ns9750_timer.h>
-#include <ns9750_bbus.h>
-#include <ns9750_bbdma.h>
-#include <fio.h>
-#include <configfile.h>
-#include <signode.h>
+#include "bus.h"
+#include "cycletimer.h"
+#include "ns9750_serial.h"
+#include "ns9750_timer.h"
+#include "ns9750_bbus.h"
+#include "ns9750_bbdma.h"
+#include "fio.h"
+#include "configfile.h"
+#include "signode.h"
+#include "sgstring.h"
 
 #if 0
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 
 #define RX_FIFO_SIZE (32)
@@ -176,7 +176,7 @@ serial_close(Serial *ser) {
  * 	Invoked when the Endian Signal Line changes
  * -------------------------------------------------------
  */
-static int
+static void 
 change_endian(SigNode *node,int value,void *clientData)
 {
 	Serial *ser = clientData;
@@ -189,7 +189,6 @@ change_endian(SigNode *node,int value,void *clientData)
 		fprintf(stderr,"NS9750 Serial: Endian is neither Little nor Big\n");
 		exit(3424);
 	}
-	return 0;
 }
 
 static void
@@ -512,7 +511,7 @@ update_serconfig(Serial *ser)
 		fprintf(stderr,"NS9750-Serial emulator: Can not handle Baudrate %d\n",rx_baudrate);
 		ser->emu_baudrate=B0;
 	}
-	dprintf("NS9750 %s: baudrate %d\n",ser->name,rx_baudrate);
+	dbgprintf("NS9750 %s: baudrate %d\n",ser->name,rx_baudrate);
 	if(tcgetattr(ser->fd,&termios)<0) {
 		fprintf(stderr,"Can not  get terminal attributes\n");
 		return;
@@ -583,7 +582,7 @@ ser_ctrla_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 		update_txdma_request(ser);
 	}
 	update_tx_interrupt(ser);
-	dprintf("New config %08x\n",value);
+	dbgprintf("New config %08x\n",value);
 	return;
 }
 
@@ -1014,13 +1013,7 @@ NS9750_SerialNew(BBusDMACtrl *bbdma,int index) {
 	int txdma_chan = serials[index].txdma_chan;
 	char *serial_name=serials[index].name;	
 	char *filename;
-	char *nodename=alloca(strlen(serial_name)+20);
-	Serial *ser = malloc(sizeof(Serial));
-	if(!ser) {
-		fprintf(stderr,"Out of memory\n");
-		exit(5462);
-	}
-	memset(ser,0,sizeof(Serial));
+	Serial *ser = sg_new(Serial);
 	ser->fd=-1;
 	ser->rx_irq_no=rx_irq;
 	ser->tx_irq_no=tx_irq;
@@ -1029,26 +1022,23 @@ NS9750_SerialNew(BBusDMACtrl *bbdma,int index) {
 	ser->tx_dmachan = BBDMA_Connect(bbdma,txdma_chan) ;
 	ser->name = serial_name; 
 
-	sprintf(nodename,"%s.RxDmaGnt",serial_name);
-	ser->RxDmaGnt = SigNode_New(nodename);
+	ser->RxDmaGnt = SigNode_New("%s.RxDmaGnt",serial_name);
 	if(!ser->RxDmaGnt) {
-		fprintf(stderr,"Can not create Ser. RxDmaGntNode %s\n",nodename);
+		fprintf(stderr,"Can not create Ser. RxDmaGntNode\n");
 		exit(3427);
 	}
 	SigNode_Set(ser->RxDmaGnt,SIG_OPEN); /* controller sets RxDMAGnt  */
 
-	sprintf(nodename,"%s.TxDmaReq",serial_name);
-	ser->TxDmaReq = SigNode_New(nodename);
+	ser->TxDmaReq = SigNode_New("%s.TxDmaReq",serial_name);
 	if(!ser->TxDmaReq) {
-		fprintf(stderr,"Can not create Ser. TxDmaReqNode %s\n",nodename);
+		fprintf(stderr,"Can not create Ser. TxDmaReqNode\n");
 		exit(3429);
 	}
 	SigNode_Set(ser->TxDmaReq,SIG_LOW); /* no request from me */
 
-	sprintf(nodename,"%s.endian",serial_name);
-	ser->endianNode = SigNode_New(nodename);
+	ser->endianNode = SigNode_New("%s.endian",serial_name);
 	if(!ser->endianNode) {
-		fprintf(stderr,"Can not create Ser. EndianNode %s\n",nodename);
+		fprintf(stderr,"Can not create Ser. EndianNode\n");
 		exit(3429);
 	}
 	ser->endianTrace = SigNode_Trace(ser->endianNode,change_endian,ser);
