@@ -35,10 +35,11 @@
 #include "configfile.h"
 #include "signode.h"
 #include "senseless.h"
+#include "sgstring.h"
 #if 0 
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 
 #define clearbit(bitnr,x) ((x)=(x)&~(1<<(bitnr)))
@@ -173,7 +174,7 @@ Sysco_PostIrq(int isource_nr) {
 	intid=sysco->intid_map[isource_nr];	
 	setbit(intid,sysco->sys_israw);
 	sysco->srcirq_is_posted[isource_nr]=1;
-	dprintf("Posting an irq\n");
+	dbgprintf("Posting an irq\n");
 	if(sysco->sys_israw & sysco->out_ienmask & sysco->irqmask) {
 		if(!sysco->irq_posted) {
 			ARM_PostIrq();	
@@ -197,7 +198,7 @@ Sysco_UnPostIrq(int isource_nr) {
 	intid=sysco->intid_map[isource_nr];	
 	clearbit(intid,sysco->sys_israw);
 	sysco->srcirq_is_posted[isource_nr]=0;
-	dprintf("Unposte einen irq\n");
+	dbgprintf("Unposte einen irq\n");
 	if(!(sysco->sys_israw & sysco->out_ienmask & sysco->irqmask)) {
 		if(sysco->irq_posted) {
 			ARM_UnPostIrq();	
@@ -305,14 +306,14 @@ timer_trcv_write(void *cd,uint32_t value,uint32_t address,int rqlen) {
 	Timer *timer=(Timer*)cd;
 	update_trr(timer);
 	timer->trcv=value;
-	dprintf("New reload count of %08x\n",value);
+	dbgprintf("New reload count of %08x\n",value);
 	return;
 }
 
 static uint32_t 
 timer_trcv_read(void *cd,uint32_t address,int rqlen) {
 	Timer *timer=(Timer*)cd;
-	dprintf("Read reload count %08x\n",timer->trcv);
+	dbgprintf("Read reload count %08x\n",timer->trcv);
 	return timer->trcv;
 }
 /*
@@ -356,7 +357,7 @@ timer_trr_read(void *cd,uint32_t address,int rqlen) {
 	Timer *timer=(Timer*)cd;
 	update_trr(timer);
 	Senseless_Report(200);
-	dprintf("Timer Read Register %08x\n",timer->trr);
+	dbgprintf("Timer Read Register %08x\n",timer->trr);
 	return timer->trr;
 }
 
@@ -375,7 +376,7 @@ timer_irq(void *cd) {
 	Timer *timer=(Timer*)cd;
 	uint32_t tcr=timer->tcr;
 	uint32_t reload=tcr&SYS_TCR_REN;
-	dprintf("Ein Timer IRQ\n");
+	dbgprintf("Ein Timer IRQ\n");
 	if(reload) {
 		update_trr(timer);
 	} else {
@@ -396,7 +397,7 @@ timer_tcr_write(void *cd,uint32_t new_tcr,uint32_t address,int rqlen) {
 	uint32_t diff_tcr = old_tcr ^ new_tcr;
 	int new_tlcs;
 
-	dprintf("New Timer Control of %08x\n",new_tcr);
+	dbgprintf("New Timer Control of %08x\n",new_tcr);
 
 	if((diff_tcr & TCR_NEED_UPDATE_MASK)) { 
 		update_trr(timer);
@@ -414,10 +415,10 @@ timer_tcr_write(void *cd,uint32_t new_tcr,uint32_t address,int rqlen) {
 			remaining=calculate_remaining(timer);
 			CycleTimer_Remove(&timer->irq_timer);
 			CycleTimer_Add(&timer->irq_timer,remaining,timer_irq,timer);
-			dprintf("Started Timer, remaining %08llx\n",remaining);
+			dbgprintf("Started Timer, remaining %08llx\n",remaining);
 		}
 	} else {
-		dprintf("Removed Timer, remaining\n");
+		dbgprintf("Removed Timer, remaining\n");
 		CycleTimer_Remove(&timer->irq_timer);
 	}
 	new_tlcs = ((new_tcr & SYS_TCR_TLCS) >> SYS_TCR_TLCS_SHIFT);
@@ -438,7 +439,7 @@ timer_tcr_read(void *cd,uint32_t address,int rqlen) {
 /* Timer Interrupt Status Register Read */
 static uint32_t 
 timer_tis_read(void *cd,uint32_t address,int rqlen) {
-	dprintf("Read TIS %08x\n",sysco->sys_tis);
+	dbgprintf("Read TIS %08x\n",sysco->sys_tis);
 	return sysco->sys_tis;
 }
 
@@ -602,7 +603,7 @@ static uint32_t
 sys_isra_read(void *cd,uint32_t address,int rqlen) {
 	uint32_t pending = sysco->sys_israw & (sysco->out_ienmask|sysco->out_fienmask) & sysco->irqmask;
 	int intid;
-	dprintf("Read ISRA\n");
+	dbgprintf("Read ISRA\n");
 	if(!pending) {
 		//fprintf(stderr,"Warning: Read ISRA without pending interrupts\n");
 		return 0;
@@ -648,7 +649,7 @@ sys_isra_write(void *cd,uint32_t value,uint32_t address,int rqlen) {
 			fprintf(stderr,"Warning irq acknowledged but not pending\n");
 		}
 	} else {
-		fprintf(stderr,"Ack Interrupt but none pending %08x\n",REG_PC);
+		fprintf(stderr,"Ack Interrupt but none pending %08x\n",ARM_NIA);
 	}
 	return;
 }
@@ -762,7 +763,7 @@ sys_icfg_write(void *cd,uint32_t value,uint32_t address,int rqlen) {
 		} else {
 			if(value&SYS_ICFG_IE_MASK) {
 				if(!testbit(intid,sysco->irqmask)) {
-					fprintf(stderr,"Unmaske IRQ while not acknowledged at %08x\n",REG_PC);
+					fprintf(stderr,"Unmaske IRQ while not acknowledged at %08x\n",ARM_NIA);
 				}
 				setbit(source_irq,sysco->src_ienmask);
 				setbit(intid,sysco->out_ienmask);
@@ -1219,7 +1220,7 @@ init_pll_config(Sysco *sc)
  * Handle changes in extirq line 
  * -----------------------------------------
  */
-static int
+static void 
 extirq_trace_proc(struct SigNode * node,int value, void *clientData)
 {
 	Irq_TraceInfo *ti = clientData;
@@ -1284,7 +1285,6 @@ extirq_trace_proc(struct SigNode * node,int value, void *clientData)
 		}
 	}
 	sysco->extint_oldsts[index]=value;
-	return 0;     
 }
 
 static  void
@@ -1332,22 +1332,12 @@ SyscoMap(Sysco *sc) {
 void
 NS9750_TimerInit(const char *name) {
 	int i;
-	Sysco *sc = malloc(sizeof(Sysco));
-        if(!sc) {
-                fprintf(stderr,"Out of memory \n");
-                exit(0);
-        }
-	memset(sc,0,sizeof(Sysco));
+	Sysco *sc = sg_new(Sysco);
         sysco=sc;
 	sysco->irqmask=~0;
 
 	for(i=0;i<16;i++) {
-		Timer *timer = malloc(sizeof(Timer));
-		if(!timer) {
-			perror("Can not allocate timers\n");
-			exit(30);
-		}
-		memset(timer,0,sizeof(*timer));
+		Timer *timer = sg_new(Timer);
 		timer->nr=i;
 		sc->timer[i]=timer;
 	}
@@ -1369,12 +1359,10 @@ NS9750_TimerInit(const char *name) {
 	init_pll_config(sc);
 	init_genid(sc);
 	for(i=0;i<4;i++) {
-		char str[60];
 		Irq_TraceInfo *ti = &sc->extIrqTraceInfo[i];
-		sprintf(str,"ns9750sysco.extirq%d",i);
-		sc->extIrqNode[i] = SigNode_New(str);
+		sc->extIrqNode[i] = SigNode_New("ns9750sysco.extirq%d",i);
 		if(!sc->extIrqNode[i]) {
-			fprintf(stderr,"Can not create node %s\n",str);
+			fprintf(stderr,"Can not create sysco extirq node\n");
 			exit(3490);
 		}
 		SigNode_Set(sc->extIrqNode[i],SIG_PULLUP); /* ???? */
