@@ -37,11 +37,12 @@
 #include <ns9750_usb.h>
 #include <usb_ohci.h>
 #include "signode.h"
+#include "sgstring.h"
 
 #if 0
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 
 typedef struct NS9750Usb NS9750Usb; 
@@ -86,7 +87,7 @@ usb_update_interrupts(NS9750Usb *usb) {
         }
 }
 
-static int 
+static void 
 irq_change(SigNode *node,int value,void *clientData)
 {
 	IrqTraceInfo *ti = (IrqTraceInfo *) clientData;
@@ -98,7 +99,6 @@ irq_change(SigNode *node,int value,void *clientData)
 		usb->gist &= ~(1<<irq);
 	}
 	usb_update_interrupts(usb);
-	return 0;
 }
 
 
@@ -121,7 +121,7 @@ static void
 usb_gctrl_write(void *clientData,uint32_t value,uint32_t address,int rqlen)  {
 	NS9750Usb *usb=clientData;
 	uint32_t diff=value^usb->gctrl;
-	dprintf("GCTRL Write value %08x\n",value);
+	dbgprintf("GCTRL Write value %08x\n",value);
 	usb->gctrl=value;
 	if(diff & (NS9750_USB_GCTRL_HSTDEV )) {
 		if(((value&NS9750_USB_GCTRL_HRST) == 0)) {
@@ -224,12 +224,7 @@ NS9750Usb_New(const char *name) {
 	NS9750Usb *usb;
 	char *str;
 	int i;
-	usb=malloc(sizeof(NS9750Usb));
-	if(!usb) {
-		fprintf(stderr,"Out of memory\n");
-		exit(5769);
-	}
-	memset(usb,0,sizeof(NS9750Usb));
+	usb=sg_new(NS9750Usb);
 	for(i=0;i<32;i++) {
 		IrqTraceInfo *ti = &usb->irq_trace_info[i];
 		usb->irqIn[i] = SigNode_New("%s.irq%d",name,i);
