@@ -42,11 +42,12 @@
 #include "cycletimer.h"
 #include "ns9xxx_i2c.h"
 #include "ns9750_timer.h" /* should be removed, required for irq */
+#include "sgstring.h"
 
 #if 0
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 
 #define NS9xxx_I2C_BASE 0x90500000
@@ -236,7 +237,7 @@ update_clock(NS_I2C *i2c,unsigned int clref,int speed)
 		return;
 	}
 	period = 1000000000/i2c_bus_clock;
-	dprintf("New period: %d ns, clock %d, scl_delay %d\n",period,i2c_bus_clock,scl_delay);
+	dbgprintf("New period: %d ns, clock %d, scl_delay %d\n",period,i2c_bus_clock,scl_delay);
        	timing->speed = speed;
 	/* 
  	 * ----------------------------------------------------------------------
@@ -272,13 +273,13 @@ update_clock(NS_I2C *i2c,unsigned int clref,int speed)
 		timing->t_susto = period * 600 / 2500;
 		timing->t_buf = period * 1300 / 2500;
 	}
-	dprintf("t_hdsta %d\n",timing->t_hdsta);
-	dprintf("t_low %d\n",timing->t_low);
-	dprintf("t_high %d\n",timing->t_high);
-	dprintf("t_susta %d\n",timing->t_susta);
-	dprintf("t_sudat %d\n",timing->t_sudat);
-	dprintf("t_susto %d\n",timing->t_susto);
-	dprintf("t_buf %d\n",timing->t_buf);
+	dbgprintf("t_hdsta %d\n",timing->t_hdsta);
+	dbgprintf("t_low %d\n",timing->t_low);
+	dbgprintf("t_high %d\n",timing->t_high);
+	dbgprintf("t_susta %d\n",timing->t_susta);
+	dbgprintf("t_sudat %d\n",timing->t_sudat);
+	dbgprintf("t_susto %d\n",timing->t_susto);
+	dbgprintf("t_buf %d\n",timing->t_buf);
 }
 
 /*
@@ -297,14 +298,14 @@ update_interrupts(NS_I2C *i2c)
 
 	if(irq_dis || (irqcd == NO_IRQ)) {
 		if(i2c->interrupt_posted) {
-			dprintf("UnPost irq\n");
+			dbgprintf("UnPost irq\n");
 			SigNode_Set(i2c->irqNode,SIG_HIGH);
 			Sysco_UnPostIrq(IRQ_I2C); // should be removed when done clean
 			i2c->interrupt_posted = 0;
 		}
 	} else {
 		if(!i2c->interrupt_posted) {
-			dprintf("Post irq (irqcd %d)\n",irqcd);
+			dbgprintf("Post irq (irqcd %d)\n",irqcd);
 			SigNode_Set(i2c->irqNode,SIG_LOW);
 			Sysco_PostIrq(IRQ_I2C); // should be removed when done clean
 			i2c->interrupt_posted = 1;
@@ -373,7 +374,7 @@ static void run_interpreter(void *clientData);
  * 	remove the timer and continue running the interpreter
  * -------------------------------------------------------------------
  */
-static int 
+static void 
 scl_released(SigNode *node,int value,void *clientData) 
 {
 	NS_I2C *i2c = (NS_I2C *)clientData;
@@ -382,7 +383,6 @@ scl_released(SigNode *node,int value,void *clientData)
 		CycleTimer_Remove(&i2c->ndelayTimer);
 		run_interpreter(clientData);
 	}
-	return 0;
 }
 
 static inline int
@@ -410,24 +410,24 @@ execute_instruction(NS_I2C *i2c) {
 	icode = i2c->code[i2c->ip++];
 	switch(icode&0xff000000) {
 		case INSTR_SDA_H:
-			dprintf("SDA_H %08x\n",icode);
+			dbgprintf("SDA_H %08x\n",icode);
 			SigNode_Set(i2c->sdaNode,SIG_OPEN);
 			break;
 		case INSTR_SDA_L:
-			dprintf("SDA_L %08x\n",icode);
+			dbgprintf("SDA_L %08x\n",icode);
 			SigNode_Set(i2c->sdaNode,SIG_LOW);
 			break;
 		case INSTR_SCL_H:
-			dprintf("SCL_H %08x\n",icode);
+			dbgprintf("SCL_H %08x\n",icode);
 			SigNode_Set(i2c->sclNode,SIG_OPEN);
 			break;
 		case INSTR_SCL_L:
-			dprintf("SCL_L %08x\n",icode);
+			dbgprintf("SCL_L %08x\n",icode);
 			SigNode_Set(i2c->sclNode,SIG_LOW);
 			break;
 
 		case INSTR_CHECK_ARB:
-			dprintf("CHECK_ARB %08x\n",icode);
+			dbgprintf("CHECK_ARB %08x\n",icode);
 			if(SigNode_Val(i2c->sdaNode)==SIG_LOW) {
 				SigNode *node;
 				uint32_t irqcd = icode & 0xffffff; 
@@ -448,13 +448,13 @@ execute_instruction(NS_I2C *i2c) {
 				uint32_t nsecs = icode & 0xffffff;
 				int64_t cycles = NanosecondsToCycles(nsecs);
 				CycleTimer_Add(&i2c->ndelayTimer,cycles,run_interpreter,i2c);
-				dprintf("NDELAY %08x\n",icode);
+				dbgprintf("NDELAY %08x\n",icode);
 			}
 			return RET_DONE;
 			break;
 
 		case INSTR_SYNC:
-			dprintf("SYNC %08x\n",icode);
+			dbgprintf("SYNC %08x\n",icode);
 			if(SigNode_Val(i2c->sclNode)==SIG_LOW) {
 				uint32_t msecs = 200; 
 				int64_t cycles =  MillisecondsToCycles(msecs);
@@ -470,11 +470,11 @@ execute_instruction(NS_I2C *i2c) {
 			} else {
 				i2c->rxdata = (i2c->rxdata<<1) | 1;
 			}
-			dprintf("READ_SDA %02x %08x\n",i2c->rxdata,icode);
+			dbgprintf("READ_SDA %02x %08x\n",i2c->rxdata,icode);
 			break;
 
 		case INSTR_READ_ACK:
-			dprintf("READ_ACK %08x\n",icode);
+			dbgprintf("READ_ACK %08x\n",icode);
 			if(SigNode_Val(i2c->sdaNode) == SIG_LOW) {
 				i2c->ack = ACK;
 			} else {
@@ -483,12 +483,12 @@ execute_instruction(NS_I2C *i2c) {
 			break;
 
 		case  INSTR_ENDSCRIPT:
-			dprintf("ENDSCRIPT %08x\n",icode);
+			dbgprintf("ENDSCRIPT %08x\n",icode);
 			return RET_DONE;
 			break;
 
 		case  INSTR_CHECK_ACK:
-			dprintf("CHECK_ACK %08x\n",icode);
+			dbgprintf("CHECK_ACK %08x\n",icode);
 			if(i2c->ack == NACK) {
 				int irqcd = (icode & 0xf);
 				queue_irqcd(i2c,irqcd);
@@ -498,7 +498,7 @@ execute_instruction(NS_I2C *i2c) {
 			break;
 
 		case  INSTR_INTERRUPT:
-			dprintf("INTERRUPT %08x\n",icode);
+			dbgprintf("INTERRUPT %08x\n",icode);
 			{
 				int irqcd = icode & 0xf;
 				queue_irqcd(i2c,irqcd);
@@ -508,13 +508,13 @@ execute_instruction(NS_I2C *i2c) {
 			break;
 
 		case  INSTR_RXDATA_AVAIL:
-			dprintf("RXDATA_AVAIL %02x %08x\n",i2c->rxdata,icode);
+			dbgprintf("RXDATA_AVAIL %02x %08x\n",i2c->rxdata,icode);
 			i2c->strdr = (i2c->strdr & ~0xff) | STRDR_RDE | i2c->rxdata;;
 			break;
 
 		case INSTR_WAIT_BUS_FREE:
 			if(!bus_is_free(i2c)) { 
-				dprintf("Bus not free, waiting !\n"); 
+				dbgprintf("Bus not free, waiting !\n"); 
 				i2c->wait_bus_free = 1;
 				return RET_DONE;
 			}
@@ -927,7 +927,7 @@ strdr_read(void *clientData,uint32_t address,int rqlen)
 		i2c->queued_irqcd = NO_IRQ;
 		CycleTimer_Add(&i2c->updateInterruptsTimer,1,update_interrupts_timer_handler,i2c);
 	}
-	dprintf("strdr read\n");
+	dbgprintf("strdr read\n");
 	return value;
 }
 
@@ -943,7 +943,7 @@ static void
 slave_cmd_nop(NS_I2C *i2c,uint8_t txdata,int do_tx) 
 {
 
-	dprintf("Slave cmd NOP in state %d, scl_stretched %d\n",i2c->sstate,i2c->scl_stretched);
+	dbgprintf("Slave cmd NOP in state %d, scl_stretched %d\n",i2c->sstate,i2c->scl_stretched);
 	if(i2c->sstate == SSTATE_IDLE) {
 		fprintf(stderr,"Slave cmd NOP in IDLE state !!\n");
 		return;
@@ -1062,7 +1062,7 @@ static uint32_t
 mar_read(void *clientData,uint32_t address,int rqlen) 
 {
 	NS_I2C *i2c = (NS_I2C *) clientData;
-	dprintf("mar read\n");
+	dbgprintf("mar read\n");
 	return i2c->mar;
 }
 
@@ -1085,7 +1085,7 @@ static uint32_t
 sar_read(void *clientData,uint32_t address,int rqlen) 
 {
 	NS_I2C *i2c = (NS_I2C *) clientData;
-	dprintf("sar read\n");
+	dbgprintf("sar read\n");
 	return i2c->sar;
 }
 
@@ -1100,7 +1100,7 @@ static uint32_t
 cfg_read(void *clientData,uint32_t address,int rqlen) 
 {
 	NS_I2C *i2c = (NS_I2C*)clientData;
-	dprintf("cfg read\n");
+	dbgprintf("cfg read\n");
 	return i2c->cfg;
 }
 
@@ -1128,10 +1128,10 @@ cfg_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 	update_interrupts(i2c);
 }
 
-static int
+static void 
 scl_trace_proc(struct SigNode * node,int value, void *clientData)
 {
-	return 0;
+	return;
 }
 
 /*
@@ -1139,7 +1139,7 @@ scl_trace_proc(struct SigNode * node,int value, void *clientData)
  * Monitor bus Status (start/stop conditions)
  * --------------------------------------------------------
  */
-static int
+static void 
 sda_trace_proc(struct SigNode * node,int value, void *clientData)
 {
 	NS_I2C *i2c = (NS_I2C*)clientData;
@@ -1162,7 +1162,7 @@ sda_trace_proc(struct SigNode * node,int value, void *clientData)
 			}
 		}
 	}
-	return 0;
+	return;
 }
 
 /*
@@ -1173,12 +1173,12 @@ sda_trace_proc(struct SigNode * node,int value, void *clientData)
  * 	is active hight
  * ----------------------------------------------------
  */ 
-static int
+static void 
 reset_trace_proc(struct SigNode * node,int value, void *clientData)
 {
 	NS_I2C *i2c = (NS_I2C *) clientData;
 	if(value == SIG_HIGH) {
-		dprintf("NS9xxx I2C reset\n");
+		dbgprintf("NS9xxx I2C reset\n");
 		I2C_SerDesDetachSlave(i2c->serdes,&i2c->i2c_slave);
 		Mem_AreaDeleteMappings(&i2c->bdev);
 		i2c->ctdr=0;
@@ -1197,7 +1197,7 @@ reset_trace_proc(struct SigNode * node,int value, void *clientData)
 		I2C_SerDesAddSlave(i2c->serdes,&i2c->i2c_slave,addr);
 		Mem_AreaAddMapping(&i2c->bdev,NS9xxx_I2C_BASE,0x10,MEM_FLAG_WRITABLE|MEM_FLAG_READABLE);
 	}	
-	return 0;
+	return;
 }
 
 /* 
@@ -1240,7 +1240,7 @@ nsslave_write(void *dev,uint8_t data)
 	NS_I2C * i2c = (NS_I2C *) dev;
 	if(i2c->sstate == SSTATE_RX_FIRST) {
 		i2c->strdr = (i2c->strdr & ~0xff) | data | STRDR_RDE;
-		dprintf("Slave RX_DATA_1ST_IRQ\n");
+		dbgprintf("Slave RX_DATA_1ST_IRQ\n");
 		queue_irqcd(i2c,S_RX_DATA_1ST_IRQ);
 		i2c->sstate = SSTATE_RX;
 		return I2C_ACK;
@@ -1279,7 +1279,7 @@ nsslave_read(void *dev,uint8_t *data)
 		return I2C_STRETCH_SCL;
 	} else {
 		*data = ctdr & 0xff;
-		dprintf("slave read commit value 0x%02x\n",*data);
+		dbgprintf("slave read commit value 0x%02x\n",*data);
 		i2c->ctdr &= ~CTDR_TXVAL;
 		return I2C_DONE;
 	}
@@ -1302,7 +1302,7 @@ nsslave_start(void *dev,int i2c_addr,int operation)
 		return I2C_ACK;
 	} else if(operation == I2C_READ) {
 		i2c->sstate = SSTATE_TX;
-		dprintf("Slave start READ operation: TX_DATA_IRQ\n");
+		dbgprintf("Slave start READ operation: TX_DATA_IRQ\n");
 		queue_irqcd(i2c,S_TX_DATA_1ST_IRQ);
 		return I2C_ACK;
 	} else {
@@ -1334,10 +1334,10 @@ nsslave_read_ack(void *dev, int ack)
 {
 	NS_I2C *i2c = (NS_I2C *) dev;
 	if(ack == I2C_ACK) {
-		dprintf("Slave TX_DATA_IRQ\n");
+		dbgprintf("Slave TX_DATA_IRQ\n");
 		queue_irqcd(i2c,S_TX_DATA_IRQ);
 	} else if (ack == I2C_NACK) {
-		dprintf("Slave NO_ACK_IRQ\n");
+		dbgprintf("Slave NO_ACK_IRQ\n");
 		queue_irqcd(i2c,S_NO_ACK_IRQ);
 	}
 }
@@ -1360,12 +1360,7 @@ NS9xxx_I2CNew(const char *name) {
 	char *nodename1 = (char*)alloca(strlen(name)+50);
 	char *nodename2 = (char*)alloca(strlen(name)+50);
 	I2C_Slave *i2c_slave;
-	NS_I2C *i2c = (NS_I2C *)malloc(sizeof(NS_I2C));
-	if(!i2c) {
-		fprintf(stderr,"Out of memory for I2C-Controller\n");
-		exit(342);
-	}
-	memset(i2c,0,sizeof(NS_I2C));
+	NS_I2C *i2c = sg_new(NS_I2C);
 	i2c->ctdr=0;
 	i2c->strdr=0;
 	i2c->mar=0;
@@ -1385,12 +1380,12 @@ NS9xxx_I2CNew(const char *name) {
 	/* Create the SDA/SCL nodes and connect with slave */
 	sprintf(nodename1,"%s.scl",name);
 	sprintf(nodename2,"%s.slave.scl",name);
-	i2c->sclNode = SigNode_New(nodename1);
+	i2c->sclNode = SigNode_New("%s",nodename1);
 	SigName_Link(nodename2,nodename1);
 
 	sprintf(nodename1,"%s.sda",name);
 	sprintf(nodename2,"%s.slave.sda",name);
-	i2c->sdaNode = SigNode_New(nodename1);
+	i2c->sdaNode = SigNode_New("%s",nodename1);
 	SigName_Link(nodename2,nodename1);
 
 
@@ -1398,10 +1393,8 @@ NS9xxx_I2CNew(const char *name) {
 		fprintf(stderr,"Can not create I2C signal nodes\n");
 		exit(342);
 	}
-	sprintf(nodename1,"%s.reset",name);
-	i2c->resetNode = SigNode_New(nodename1);
-	sprintf(nodename1,"%s.irq",name);
-	i2c->irqNode = SigNode_New(nodename1);
+	i2c->resetNode = SigNode_New("%s.reset",name);
+	i2c->irqNode = SigNode_New("%s.irq",name);
 
 	if(!i2c->irqNode || !i2c->resetNode) {
 		fprintf(stderr,"Can not create I2C irq or reset node\n");
