@@ -1,8 +1,8 @@
 /*
- * ----------------------------------------------------------------------
+ ******************************************************************************* 
  * Emulation of National semiconductor PC16550D Universal asynchronous
  * Receiver/Transmitter with Fifo
- * (C) 2005 Jochen Karrer 
+ * (C) 2005  2009 Jochen Karrer 
  *   Author: Jochen Karrer
  *
  * state: never used, totally untested
@@ -20,20 +20,21 @@
  *  with this program; if not, write to the Free Software Foundation, Inc.,
  *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
  *
- * ----------------------------------------------------
+ ******************************************************************************* 
  */
 
 #include <errno.h>
 #include <stdint.h>
 #include <string.h>
-#include <termios.h>
 #include <unistd.h>
-#include <termios.h>
 #include <sys/ioctl.h>
 
 #include "bus.h"
+#include "sgstring.h"
 #include "fio.h"
 #include "signode.h"
+#include "serial.h"
+#include "clock.h"
 #include "pc16550.h"
 
 #define RBR(pc)	(0) 	/* Read  */
@@ -53,6 +54,12 @@
 #define			IIR_TYPE_THRE 	(1)
 #define			IIR_TYPE_MST	(0)
 #define FCR(pc)	(2)	/* Write */
+#define FCR_FIFO_ENABLE		(1<<0)
+#define FCR_RCVR_FIFO_RESET	(1<<1)
+#define FCR_XMIT_FIFO_RESET	(1<<2)
+#define FCR_DMA_MODE_SEL	(1<<3)
+#define FCR_RCVR_TRIGGER_SHIFT	(6)	
+#define FCR_RCVR_TRIGGER_MASK	(3<<6)	
 
 #define LCR(pc)	(3)
 #define		LCR_BITS_MASK (3)
@@ -97,19 +104,29 @@
 
 #define DLAB(pc) (LCR(pc) & (1<<7))
 
-#define RX_FIFO_SIZE (16)
-#define RX_FIFO_MASK (RX_FIFO_SIZE-1)
-#define RX_FIFO_COUNT(ser) (((RX_FIFO_SIZE + (ser)->rxfifo_wp - (ser)->rxfifo_rp)) & RX_FIFO_MASK)
-#define RX_FIFO_ROOM(ser) (RX_FIFO_SIZE - RX_FIFO_COUNT(ser) - 1)
+#define RX_FIFO_SIZE_MAX (16)
+#define RX_FIFO_SIZE(ser) (16)
+#define RX_FIFO_MASK(ser) (RX_FIFO_SIZE(ser) - 1)
+#define RX_FIFO_COUNT(ser) ((ser)->rxfifo_wp - (ser)->rxfifo_rp)
+#define RX_FIFO_ROOM(ser) (RX_FIFO_SIZE(ser) - RX_FIFO_COUNT(ser))
+#define RX_FIFO_WP(ser)	(ser->rxfifo_wp & RX_FIFO_MASK(ser))
+#define RX_FIFO_RP(ser)	(ser->rxfifo_rp & RX_FIFO_MASK(ser))
 
-#define TX_FIFO_SIZE (16)
-#define TX_FIFO_MASK (TX_FIFO_SIZE-1)
-#define TX_FIFO_COUNT(ser) ((((ser)->txfifo_wp-(ser)->txfifo_rp)+TX_FIFO_SIZE)&TX_FIFO_MASK)
-#define TX_FIFO_ROOM(ser) (TX_FIFO_SIZE-TX_FIFO_COUNT(ser)-1)
+#define TX_FIFO_SIZE_MAX (16)
+#define TX_FIFO_SIZE(ser) (16)
+#define TX_FIFO_MASK(ser) (TX_FIFO_SIZE(ser) - 1)
+#define TX_FIFO_COUNT(ser) ((ser)->txfifo_wp - (ser)->txfifo_rp)
+#define TX_FIFO_ROOM(ser) (TX_FIFO_SIZE(ser) - TX_FIFO_COUNT(ser))
+#define TX_FIFO_WP(ser)	(ser->txfifo_wp & TX_FIFO_MASK(ser))
+#define TX_FIFO_RP(ser)	(ser->txfifo_rp & TX_FIFO_MASK(ser))
 
 
 typedef struct PC16550 {
 	BusDevice bdev;	
+	UartPort *backend;
+	Clock_t *clk_in;
+	Clock_t *clk_baud;
+
 	int register_shift;
 
 	int baudrate;
@@ -120,43 +137,23 @@ typedef struct PC16550 {
 	uint8_t dll,dlm;
 	uint8_t ier;
 	uint8_t iir;
+	uint8_t fcr;
 	uint8_t scr;
 
-	uint8_t rx_fifo[RX_FIFO_SIZE];
-        int rxfifo_wp;
-        int rxfifo_rp;
+	uint8_t rx_fifo[RX_FIFO_SIZE_MAX];
+        uint64_t rxfifo_wp;
+        uint64_t rxfifo_rp;
+	uint32_t rx_fifo_size;
 
-        uint8_t tx_fifo[TX_FIFO_SIZE];
-        unsigned int txfifo_wp;
-        unsigned int txfifo_rp;
-
-	int fd;
-	FIO_FileHandler input_fh;
-        FIO_FileHandler output_fh;
-        int ifh_is_active;
-        int ofh_is_active;
+        uint8_t tx_fifo[TX_FIFO_SIZE_MAX];
+        uint64_t txfifo_wp;
+        uint64_t txfifo_rp;
+	uint32_t tx_fifo_size;
 
 	int interrupt_posted;
 	SigNode *irqNode;
 } PC16550;
 
-static inline void enable_rx(PC16550 *pc);
-static inline void disable_rx(PC16550 *pc);
-static inline void enable_tx(PC16550 *pc);
-static inline void disable_tx(PC16550 *pc);
-
-static void
-serial_close(PC16550 *pc) {
-        if(pc->fd<0) {
-                return;
-        }
-        disable_rx(pc);
-        disable_tx(pc);
-        close(pc->fd);
-        pc->fd = -1;
-        return;
-}
-
 
 static void
 update_interrupts(PC16550 *pc) 
@@ -212,252 +209,197 @@ update_interrupts(PC16550 *pc)
 	
 }
 
-/*
- * --------------------------------------------------------
- * Filehandler for TX-Fifo
- *      Write chars from TX-Fifo to file as long
- *      as TX-Fifo is not empty and write would not block
- * --------------------------------------------------------
- */
-static int
-serial_output(void *cd,int mask) {
-        PC16550 *pc=cd;
-        int fill;
-        while(pc->txfifo_rp!=pc->txfifo_wp) {
-                int count,len;
-                fill=TX_FIFO_COUNT(pc);
-                len=fill;
-                if((pc->txfifo_rp+fill)>TX_FIFO_SIZE) {
-                        len = TX_FIFO_SIZE - pc->txfifo_rp;
-                }
-                count=write(pc->fd,&pc->tx_fifo[pc->txfifo_rp],len);
-                if(count>0) {
-                        pc->txfifo_rp = (pc->txfifo_rp+count)%TX_FIFO_SIZE;
-                } else if (count==0) { // EOF
-                        fprintf(stderr,"EOF on output\n");
-                        serial_close(pc);
-                        break;
-                } else if ((count<0) && (errno==EAGAIN)) {
-                        break;
-                } else {
-                        fprintf(stderr,"Error on output\n");
-                        serial_close(pc);
-                        break;
-                }
-        }
-        if(TX_FIFO_COUNT(pc)==0) {
-		pc->lsr |= LSR_THRE | LSR_TEMT;
-                update_interrupts(pc);
-        } else if(TX_FIFO_ROOM(pc)>0) {
-		pc->lsr |= LSR_THRE;
-                update_interrupts(pc);
-        } 
-        return 0;
+static void
+update_clock(PC16550 *pc) {
+	uint16_t divisor;
+        divisor= pc->dll + (pc->dlm << 8);
+	if(divisor) {
+		Clock_MakeDerived(pc->clk_baud,pc->clk_in,1,divisor);
+	} else {
+		Clock_MakeDerived(pc->clk_baud,pc->clk_in,0,1);
+	}
 }
 
 /*
- * ------------------------------------
+ **********************************************************************
+ * baud_clock_trace
+ * Eventhandler called whenever the baud rate changes 
+ **********************************************************************
+ */
+
+static void
+baud_clock_trace(Clock_t *clock,void *clientData) 
+{
+	PC16550 *pc = (PC16550 *) clientData;
+        UartCmd cmd;
+        cmd.opcode = UART_OPC_SET_BAUDRATE;
+        cmd.arg = Clock_Freq(clock);
+	SerialDevice_Cmd(pc->backend,&cmd);
+	fprintf(stderr,"****************The Baudrate is now %d\n",(uint32_t)Clock_Freq(clock));
+}
+
+/*
+ ***************************************************************
+ * Filehandler for TX-Fifo
+ *      Write chars from TX-Fifo to backend. 
+ ***************************************************************
+ */
+static void 
+serial_output(void *cd) {
+        PC16550 *pc = cd;
+        while(TX_FIFO_COUNT(pc) > 0) {
+                int count;
+		count=SerialDevice_Write(pc->backend,&pc->tx_fifo[TX_FIFO_RP(pc)],1);
+                if(count > 0) {
+                        pc->txfifo_rp++;
+                } else {
+			break;
+		} 
+        }
+        if(TX_FIFO_COUNT(pc) == 0) {
+		pc->lsr |= LSR_THRE | LSR_TEMT;
+                update_interrupts(pc);
+		SerialDevice_StopTx(pc->backend);
+        } else if(TX_FIFO_ROOM(pc) > 0) {
+		pc->lsr &= ~LSR_THRE; // ??????????????????????Check this */
+                update_interrupts(pc);
+        } 
+        return;
+}
+
+/*
+ ************************************************************
  * Put one byte to the rxfifo
- * ------------------------------------
+ ************************************************************
  */
 static inline int
-serial_rx_char(PC16550 *pc,uint8_t c) {
+serial_put_rx_fifo(PC16550 *pc,uint8_t c) {
         int room = RX_FIFO_ROOM(pc);
         if(room<1) {
                 return -1;
         }
-        pc->rx_fifo[pc->rxfifo_wp]=c;
-        pc->rxfifo_wp=(pc->rxfifo_wp+1) & RX_FIFO_MASK;
+        pc->rx_fifo[RX_FIFO_WP(pc)]=c;
+        pc->rxfifo_wp++;
         if(room==1) {
-                disable_rx(pc);
+		SerialDevice_StopRx(pc->backend);
                 return 0;
         }
         return 1;
 }
 
 
-static int
-serial_input(void *cd,int mask) {
+static void 
+serial_input(void *cd) {
         PC16550 *pc = cd;
         int fifocount;
-        if(pc->fd<0) {
-                fprintf(stderr,"Serial input with illegal fd %d\n",pc->fd);
-                return -1;
-        }
         while(1) {
-                char c;
-                int count=read(pc->fd,&c,1);
-                if(count==1) {
-                        serial_rx_char(pc,c);
-                        //fprintf(stdout,"Console got %c\n",c);
-                } else if (count==0) { // EOF
-                        fprintf(stderr,"EOF reading from serial\n");
-                        serial_close(pc);
-                        break;
-                } else if ((count<0) && (errno==EAGAIN)) {
-                        break;
+                uint8_t c;
+		int count = SerialDevice_Read(pc->backend,&c,1);
+                if(count == 1) {
+			if(serial_put_rx_fifo(pc,c) < 1) {
+                                break;
+                        }
                 } else {
-                        fprintf(stderr,"Error on input\n");
-                        serial_close(pc);
-                        break;
-                }
+			break;
+		} 
         }
         fifocount = RX_FIFO_COUNT(pc);
         if(fifocount) {
 		pc->lsr |= LSR_DR;
                 update_interrupts(pc);
         }
-        return 0;
 }
 
-/*
- * ------------------------------------------------------
- * Flow control for receiver
- * ------------------------------------------------------
- */
-static inline void
-enable_rx(PC16550 *pc) {
-        if((pc->fd >= 0) && !(pc->ifh_is_active)) {
-                FIO_AddFileHandler(&pc->input_fh,pc->fd,FIO_READABLE,serial_input,pc);
-        	pc->ifh_is_active=1;
-        }
+
+static void
+reset_rx_fifo(PC16550 *pc) 
+{
+	pc->rxfifo_rp = pc->rxfifo_wp = 0;
+	pc->lsr &= ~LSR_DR;
+	update_interrupts(pc);
 }
 
-static inline void
-disable_rx(PC16550 *pc) {
-        if(pc->ifh_is_active) {
-                FIO_RemoveFileHandler(&pc->input_fh);
-        }
-        pc->ifh_is_active=0;
-}
-static inline void
-enable_tx(PC16550 *pc) {
-        if((pc->fd>=0) && !(pc->ofh_is_active)) {
-                FIO_AddFileHandler(&pc->output_fh,pc->fd,FIO_WRITABLE,serial_output,pc);
-        	pc->ofh_is_active=1;
-        }
-}
-static inline void
-disable_tx(PC16550 *pc) {
-        if(pc->ofh_is_active) {
-                FIO_RemoveFileHandler(&pc->output_fh);
-        }
-        pc->ofh_is_active=0;
+static void
+reset_tx_fifo(PC16550 *pc) 
+{
+	pc->txfifo_rp = pc->txfifo_wp = 0;
+	pc->lsr |= LSR_THRE;
+	/* 
+ 	 ****************************************************
+	 * TEMT is not really emptied by reset but 
+	 * a separeate shift register is not implemented
+ 	 ****************************************************
+	 */
+	pc->lsr |= LSR_TEMT;
+	update_interrupts(pc);
 }
 
+
 static void
 update_serconfig(PC16550 *pc)
 {
-        int baudrate;
-        uint32_t clk_rate=0;
-        int divisor;
+        UartCmd cmd;
         tcflag_t bits;
         tcflag_t parodd;
         tcflag_t parenb;
         tcflag_t crtscts;
-        struct termios termios;
-        if(pc->fd<0) {
-                return;
-        }
-        if(!isatty(pc->fd)) {
-                return;
-        }
 	/* Does 16550 really have no automatic rtscts handshaking ? */
 	crtscts=0; 
         if(pc->lcr & LCR_EPS) {
                 parodd=0;
         } else {
-                parodd=PARODD;
+                parodd=1;
         }
-        if(pc->lcr & LCR_EPS) {
-                parenb=PARENB;
+        if(pc->lcr & LCR_PEN) {
+                parenb=1;
         } else {
                 parenb=0;
         }
         switch(pc->lcr & LCR_BITS_MASK) {
                 case LCR_5BITS:
-                        bits=CS5; break;
+                        bits=5; break;
                 case LCR_6BITS:
-                        bits=CS6; break;
+                        bits=6; break;
                 case LCR_7BITS:
-                        bits=CS7; break;
+                        bits=7; break;
                 case LCR_8BITS:
-                        bits=CS8; break;
+                        bits=8; break;
                 /* Can not be reached */
                 default:
-                        bits=CS8; break;
+                        bits=8; break;
         }
-        clk_rate = 1.843200;
-        divisor=pc->dll+256*pc->dlm;
-	if(divisor) {
-        	baudrate=clk_rate/divisor;
-	} else {
-		baudrate = 0;
-	}
-	if((baudrate > 440000) && (baudrate < 480000)) {
-                pc->baudrate=B460800;
-        } else if((baudrate > 220000) && (baudrate < 240000)) {
-                pc->baudrate=B230400;
-        } else if((baudrate > 110000) && (baudrate < 120000)) {
-                pc->baudrate=B115200;
-        } else if((baudrate > 55000) && (baudrate < 60000)) {
-                pc->baudrate=B57600;
-        } else if((baudrate > 36571 ) && (baudrate < 40320)) {
-                pc->baudrate=B38400;
-        } else if((baudrate > 18285 ) && (baudrate < 20160)) {
-                pc->baudrate=B19200;
-        } else if((baudrate > 9142 ) && (baudrate < 10080)) {
-                pc->baudrate=B9600;
-        } else if((baudrate > 4571 ) && (baudrate < 5040)) {
-                pc->baudrate=B4800;
-        } else if((baudrate > 2285 ) && (baudrate < 2520)) {
-                pc->baudrate=B2400;
-        } else if((baudrate > 1890 ) && (baudrate < 1714)) {
-                pc->baudrate=B1800;
-        } else if((baudrate > 1142 ) && (baudrate < 1260)) {
-                pc->baudrate=B1200;
-        } else if((baudrate > 570 ) && (baudrate < 630)) {
-                pc->baudrate=B600;
-        } else if((baudrate > 285 ) && (baudrate < 315)) {
-                pc->baudrate=B300;
-        } else if((baudrate > 190 ) && (baudrate < 210)) {
-                pc->baudrate=B200;
-        } else if((baudrate > 142 ) && (baudrate < 158)) {
-                pc->baudrate=B150;
-        } else if((baudrate > 128 ) && (baudrate < 141)) {
-                pc->baudrate=B134;
-        } else if((baudrate > 105 ) && (baudrate < 116)) {
-                pc->baudrate=B110;
-        } else if((baudrate > 71 ) && (baudrate < 79)) {
-                pc->baudrate=B75;
-        } else if((baudrate > 47 ) && (baudrate < 53)) {
-                pc->baudrate=B50;
-        } else {
-                fprintf(stderr,"PC16550-Serial emulator: Can not handle Baudrate %d\n",baudrate);
-                pc->baudrate=B0;
-        }
-        
-        if(tcgetattr(pc->fd,&termios)<0) {
-                fprintf(stderr,"Can not  get terminal attributes\n");
-                return;
-        }
-        if(pc->fd) {
-                cfmakeraw(&termios);
-        } else {
-                termios.c_lflag &= ~(ISIG);
-        }
-        if(cfsetispeed ( &termios, pc->baudrate)<0) {
-                fprintf(stderr,"Can not change Baudrate\n");
-        }
-        termios.c_cflag &= ~(CSIZE|PARENB|PARODD|CRTSCTS);
-        termios.c_cflag |= bits | parenb | parodd | crtscts;
 
-        /* Shit: TCSADRAIN would be better but tcsettattr seems to block */
-        if(tcsetattr(pc->fd,TCSANOW,&termios)<0) {
-                fprintf(stderr,"Can not  set terminal attributes\n");
-                return;
+	if(crtscts) {
+                cmd.opcode = UART_OPC_CRTSCTS;
+                cmd.arg = 1;
+                SerialDevice_Cmd(pc->backend,&cmd);
+        } else {
+                cmd.opcode = UART_OPC_CRTSCTS;
+                cmd.arg = 0;
+                SerialDevice_Cmd(pc->backend,&cmd);
+
+                cmd.opcode = UART_OPC_SET_RTS;
+		/* Set the initial state of RTS */
+#if 0
+                if(iuart->ucr2 & UCR2_CTS) {
+                        cmd.arg = UART_RTS_ACT;
+                } else {
+                        cmd.arg = UART_RTS_INACT;
+                }
+                SerialDevice_Cmd(pc->backend,&cmd);
+#endif
         }
-        return;
+        cmd.opcode = UART_OPC_PAREN;
+	cmd.arg = parenb;
+        SerialDevice_Cmd(pc->backend,&cmd);
+
+        cmd.opcode = UART_OPC_PARODD;
+	cmd.arg = parodd;
+        SerialDevice_Cmd(pc->backend,&cmd);
+
+        cmd.opcode = UART_OPC_SET_CSIZE;
+	cmd.arg = bits;
+        SerialDevice_Cmd(pc->backend,&cmd);
 
 }
 
@@ -470,12 +412,14 @@ static inline void
 thr_write(PC16550 *pc,uint8_t c) {
         int room;
 	uint8_t old_lsr;
-        pc->tx_fifo[pc->txfifo_wp] = c;
-        pc->txfifo_wp = (pc->txfifo_wp+1) % TX_FIFO_SIZE;
-	old_lsr=pc->lsr;	
+	if(TX_FIFO_ROOM(pc) > 0) {
+		pc->tx_fifo[TX_FIFO_WP(pc)] = c;
+		pc->txfifo_wp++;
+	}
+	old_lsr = pc->lsr;	
         pc->lsr &= ~LSR_TEMT;
-        room=TX_FIFO_ROOM(pc);
-        if(room>0) {
+        room = TX_FIFO_ROOM(pc);
+        if(room > 0) {
 		pc->lsr |= LSR_THRE;
         } else {
 		pc->lsr &= ~LSR_THRE;
@@ -483,7 +427,7 @@ thr_write(PC16550 *pc,uint8_t c) {
 	if(pc->lsr != old_lsr) {
 		update_interrupts(pc);
 	}
-        enable_tx(pc);
+	SerialDevice_StartTx(pc->backend);
 }
 
 /*
@@ -495,9 +439,9 @@ static inline uint8_t
 rbr_read(PC16550 *pc)  
 {
         uint8_t data=0;
-	if(RX_FIFO_COUNT(pc)>0) {
-		data = pc->rx_fifo[pc->rxfifo_rp];
-        	pc->rxfifo_rp = (pc->rxfifo_rp+1) & RX_FIFO_MASK;
+	if(RX_FIFO_COUNT(pc) > 0) {
+		data = pc->rx_fifo[RX_FIFO_RP(pc)];
+		pc->rxfifo_rp++;
 	}
 	if(RX_FIFO_COUNT(pc) == 0) {
 		pc->lsr &= ~LSR_DR;
@@ -507,19 +451,22 @@ rbr_read(PC16550 *pc)
 }
 
 /* 
+ **************************************************
  * Divisor latch 
+ **************************************************
  */
 static inline void 
 dll_write(PC16550 *pc,uint8_t value)  
 {
 	pc->dll=value;
-	update_serconfig(pc);
+	update_clock(pc);
 }
+
 static inline void 
 dlm_write(PC16550 *pc,uint8_t value)  
 {
 	pc->dlm=value;
-	update_serconfig(pc);
+	update_clock(pc);
 
 }
 
@@ -621,6 +568,29 @@ iir_read(void *clientData,uint32_t address,int rqlen)
 static void
 fcr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
 {
+	PC16550 *pc = (PC16550*) clientData;
+	uint8_t diff = value ^ pc->fcr;
+	if(diff & FCR_FIFO_ENABLE) {
+		reset_rx_fifo(pc);
+		reset_tx_fifo(pc);
+		if(value & FCR_FIFO_ENABLE) {
+			pc->tx_fifo_size = 16;
+			pc->rx_fifo_size = 16;
+		} else {
+			pc->tx_fifo_size = 1;
+			pc->rx_fifo_size = 1;
+		}
+	} 
+	/* If fifo enable not 1 the other bits will not be programmed */
+	if(!(value & FCR_FIFO_ENABLE)) {
+		return;
+	}
+	if(value & FCR_RCVR_FIFO_RESET) {
+        	pc->rxfifo_wp = pc->rxfifo_rp = 0;
+	} else if(value & FCR_RCVR_FIFO_RESET) {
+        	pc->txfifo_wp = pc->txfifo_rp = 0;
+	}
+	pc->fcr = value;	
         return;
 }
 
@@ -633,7 +603,7 @@ fcr_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
  * 	Bit  4: EPS 1 = Even Parity
  *	Bit  5: Stick parity
  *	Bit  6: Break control
- *	Bit  7: DLAB Divisor Latch Access Bit
+ *	Bit  7: DLAB Divisor Latch Access Bit (bank switch for registers)
  * ---------------------------------------------------------------------------
  */
 static uint32_t
@@ -673,23 +643,26 @@ mcr_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 {
 	PC16550 *pc = clientData;
 	uint8_t diff = pc->mcr ^ value;
+        UartCmd cmd;
+
 	if(diff & (MCR_DTR | MCR_RTS)) {
-		unsigned int clear=0;
-                unsigned int set=0;
-                if(!(value & MCR_DTR)) {
-                        set |= TIOCM_DTR;
+		
+		cmd.opcode = UART_OPC_SET_DTR;
+                if(value & MCR_DTR) {
+			cmd.arg = 1;
                 } else {
-                        clear |= TIOCM_DTR;
+			cmd.arg = 0;
                 }
-                if(!(value & MCR_RTS)) {
-                        set |= TIOCM_RTS;
+		SerialDevice_Cmd(pc->backend,&cmd);
+
+
+		cmd.opcode = UART_OPC_SET_RTS;
+                if(value & MCR_RTS) {
+                        cmd.arg = 1;
                 } else {
-                        clear |= TIOCM_RTS;
+                        cmd.arg = 0;
                 }
-                if(isatty(pc->fd) && clear)
-                        ioctl(pc->fd,TIOCMBIC,clear);
-                if(isatty(pc->fd) && set)
-                        ioctl(pc->fd,TIOCMSET,set);
+		SerialDevice_Cmd(pc->backend,&cmd);
 
 	}
         return;
@@ -737,23 +710,37 @@ msr_read(void *clientData,uint32_t address,int rqlen)
 {
 	PC16550 *pc = clientData;
 	uint8_t msr;
-	unsigned int host_status;
+        UartCmd cmd;
 	msr = pc->msr;	
-	if(isatty(pc->fd) && (ioctl(pc->fd,TIOCMGET,&host_status)>=0)) {
-                msr=msr & ~(MSR_DCD | MSR_RI | MSR_DSR | MSR_CTS);
-                if(!(host_status & TIOCM_CAR)) {
-                        msr |= MSR_DCD;
-                }
-                if(!(host_status & TIOCM_RNG)) {
-                        msr |= MSR_RI;
-                }
-                if(!(host_status & TIOCM_DSR)) {
-                        msr |= MSR_DSR;
-                }
-                if(!(host_status & TIOCM_CTS)) {
-                        msr |= MSR_CTS;
-                }
-        }
+        msr = msr & ~(MSR_DCD | MSR_RI | MSR_DSR | MSR_CTS);
+
+	/* Ask the backend */
+	cmd.arg = 0;
+	cmd.opcode = UART_OPC_GET_DCD;
+	SerialDevice_Cmd(pc->backend,&cmd);
+	if(cmd.retval) {
+		msr |= MSR_DCD;
+	}
+
+        cmd.opcode = UART_OPC_GET_RI;
+	SerialDevice_Cmd(pc->backend,&cmd);
+	if(cmd.retval) {
+		msr |= MSR_RI;
+	}
+
+	cmd.opcode = UART_OPC_GET_DSR;
+	SerialDevice_Cmd(pc->backend,&cmd);
+	if(cmd.retval) {
+		msr |= MSR_DSR;
+	}	
+
+	cmd.opcode = UART_OPC_GET_CTS;
+	SerialDevice_Cmd(pc->backend,&cmd);
+	if(cmd.retval) {
+		msr |= MSR_CTS;
+	}
+
+	/* Now determine the deltas */
 	if((pc->msr ^ msr) & MSR_DCD) {
                 msr |= MSR_DDCD;
 	}
@@ -830,29 +817,33 @@ PC16550_UnMap(void *owner,uint32_t base,uint32_t mask)
  * PC16550_New
  * 	Create a new National semicondcutor PC16650
  *      register_shift defines the space between
- *	to registers of 16550 in powers of two
+ *	to registers of 16550 in powers of two.
  * ---------------------------------------------------
  */
 BusDevice *
 PC16550_New(char *devname,int register_shift) {
-	char *nodename = alloca(strlen(devname)+30);
-        PC16550 *pc = malloc(sizeof(PC16550));
-        if(!pc) {
-                fprintf(stderr,"Out of memory \n");
-                exit(0);
-        }
-        memset(pc,0,sizeof(PC16550));
-	pc->fd=-1;
+        PC16550 *pc = sg_new(PC16550);
 	pc->register_shift = register_shift;
         pc->bdev.first_mapping=NULL;
         pc->bdev.Map=PC16550_Map;
         pc->bdev.UnMap=PC16550_UnMap;
         pc->bdev.owner=pc;
         pc->bdev.hw_flags=MEM_FLAG_WRITABLE|MEM_FLAG_READABLE;
-	sprintf(nodename,"%s.irq",devname);
-	pc->irqNode = SigNode_New(nodename);	
+	pc->backend = Uart_New(devname,serial_input,serial_output,NULL,pc);
+	pc->rx_fifo_size = pc->tx_fifo_size = 1;
+	pc->clk_in = Clock_New("%s.clk",devname);
+	pc->clk_baud = Clock_New("%s.baud_clk",devname);
+	if(!pc->clk_in || !pc->clk_baud) {
+		fprintf(stderr,"Can not create baud clocks for \"%s\"\n",devname);
+		exit(1);
+	}
+	pc->irqNode = SigNode_New("%s.irq",devname);	
+	if(!pc->irqNode) {
+		fprintf(stderr,"Can not create interrupt signal for \"%s\"\n",devname);
+	}
 	SigNode_Set(pc->irqNode,SIG_HIGH); /* No request on startup */	
 	pc->interrupt_posted=0;
+	Clock_Trace(pc->clk_baud,baud_clock_trace,pc);
 	fprintf(stderr,"Created 16550 UART \"%s\"\n",devname);
         return &pc->bdev;
 }
