@@ -35,11 +35,12 @@
 #include "pcf8563.h"
 #include "configfile.h"
 #include "diskimage.h"
+#include "sgstring.h"
 
 #if 0
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 
 #define PCF_STATE_ADDR (0)
@@ -64,11 +65,11 @@ static int
 pcf8563_write(void *dev,uint8_t data) {
 	PCF8563 *pcf = dev;
 	if(pcf->state==PCF_STATE_ADDR) {
-		dprintf("PCF8563 Addr 0x%02x\n",data);
+		dbgprintf("PCF8563 Addr 0x%02x\n",data);
 		pcf->reg_address = data & 0xf;
 		pcf->state = PCF_STATE_DATA;
 	} else if(pcf->state==PCF_STATE_DATA) {
-		dprintf("PCF8563 Write 0x%02x to %04x\n",data,pcf->reg_address);
+		dbgprintf("PCF8563 Write 0x%02x to %04x\n",data,pcf->reg_address);
 		pcf->data[pcf->reg_address]=data;
 		pcf->reg_address=((pcf->reg_address+1)&0xf) | (pcf->reg_address & ~0xf);
 	}
@@ -80,7 +81,7 @@ pcf8563_read(void *dev,uint8_t *data)
 {
 	PCF8563 *pcf = dev;
 	*data=pcf->data[pcf->reg_address];
-	dprintf("PCF8563 read 0x%02x from %04x\n",*data,pcf->reg_address);
+	dbgprintf("PCF8563 read 0x%02x from %04x\n",*data,pcf->reg_address);
 	pcf->reg_address=(pcf->reg_address+1)&0xf;
 	return I2C_DONE;
 };
@@ -177,7 +178,7 @@ diff_systime(PCF8563 *pcf) {
 	sys_utc_time = tv.tv_sec;
 
 	offset = rtc_time-sys_utc_time;
-	dprintf("diff %d, sys %d, rtc %d\n",offset,sys_utc_time,rtc_time);
+	dbgprintf("diff %d, sys %d, rtc %d\n",offset,sys_utc_time,rtc_time);
 	return offset;
 }
 
@@ -185,7 +186,7 @@ diff_systime(PCF8563 *pcf) {
 static int
 pcf8563_start(void *dev,int i2c_addr,int operation) {
 	PCF8563 *pcf = dev;
-	dprintf("pcf8563 start\n");
+	dbgprintf("pcf8563 start\n");
 	pcf->state = PCF_STATE_ADDR;
 	pcf->direction = operation;
 	if(operation == I2C_READ) {
@@ -200,7 +201,7 @@ pcf8563_start(void *dev,int i2c_addr,int operation) {
 static void 
 pcf8563_stop(void *dev) {
 	PCF8563 *pcf = dev;
-	dprintf("pcf8563 stop\n");
+	dbgprintf("pcf8563 stop\n");
 	if(pcf->direction == I2C_WRITE) {
 		// hack ! should be only written if some registers are touched
 		int32_t offset = diff_systime(pcf);
@@ -228,15 +229,10 @@ static I2C_SlaveOps pcf8563_ops = {
 
 I2C_Slave *
 PCF8563_New(char *name) {
-	PCF8563 *pcf = malloc(sizeof(PCF8563)); 
+	PCF8563 *pcf = sg_new(PCF8563); 
 	char *dirname,*imagename;
 	
 	I2C_Slave *i2c_slave;
-	if(!pcf) {
-		fprintf(stderr,"Out of memory\n");
-		exit(8152);
-	}
-	memset(pcf,0,sizeof(PCF8563));
 
 	dirname=Config_ReadVar("global","imagedir");
 	if(dirname) {
@@ -245,12 +241,12 @@ PCF8563_New(char *name) {
 		pcf->disk_image = DiskImage_Open(imagename,4,DI_RDWR | DI_CREAT_00);
 		if(!pcf->disk_image) {
 			fprintf(stderr,"Failed to open PCF8563 time offset file\n");
-			pcf->time_offset = malloc(4);
+			pcf->time_offset = sg_calloc(4);
 		} else {
 			pcf->time_offset = DiskImage_Mmap(pcf->disk_image);
 		}
 	} else {
-		pcf->time_offset = malloc(4);
+		pcf->time_offset = sg_calloc(4);
 	}
 	i2c_slave = &pcf->i2c_slave;
 	i2c_slave->devops = &pcf8563_ops; 
