@@ -37,6 +37,7 @@
 #include "bus.h"
 #include "signode.h"
 #include "pl190_irq.h"
+#include "sgstring.h"
 
 #define VICIRQSTATUS	(0)
 #define VICFIQSTATUS	(4)
@@ -62,9 +63,17 @@
 #define VICITOP1	(0x30c)
 #define VICITOP2	(0x310)
 
+#define PRIV_LEVEL_INVALID (255)
+
 typedef struct IrqTraceInfo IrqTraceInfo;
 typedef struct PL190 {
 	BusDevice bdev;
+	/* Interrupt priority logic */
+	int required_pl; /* only Interrupt with pl <= required_pl will be triggered */
+	int vectaddr_pl;  /* The privilege level of the interrupt currently in vectaddr */
+	int pl_stack[16];
+	unsigned int pl_stackp;
+
 	uint32_t irqstatus;
 	uint32_t fiqstatus;
 
@@ -99,7 +108,6 @@ static void
 update_interrupts(PL190 *pl) 
 {
 	uint32_t  intstatus;
-	uint16_t vecints = 0;
 	int first_vecint = -1;
 	int i;
 	pl->rawintr = pl->softint | pl->hardintr;
@@ -113,7 +121,7 @@ update_interrupts(PL190 *pl)
  	}
 	if(pl->irqstatus) {
 		/* Vectored interrupt logic use irqstatus as input */
-		for(i=0;i<16;i++) {
+		for(i=0;i <= pl->required_pl;i++) {
 			int srcirq;
 			uint32_t veccntl = pl->veccntl[i];
 			if(!(veccntl & (VECCNTL_ENA))) {
@@ -121,17 +129,18 @@ update_interrupts(PL190 *pl)
 			}
 			srcirq = veccntl & 0x1f;		
 			if(pl->irqstatus & (1<<srcirq)) {
-				vecints |= (1<<i);
-				if(first_vecint<0) {
+				if(first_vecint < 0) {
 					first_vecint = i;
 				}
 				break;
 			}
 		}
-		if(first_vecint>=0) {
+		if(first_vecint >= 0) {
 			pl->vectaddr = pl->vectaddrn[first_vecint];
+			pl->vectaddr_pl = first_vecint;
 		} else {
 			pl->vectaddr = pl->defvectaddr;
+			pl->vectaddr_pl = PRIV_LEVEL_INVALID;
 		}	
 		SigNode_Set(pl->irqNode,SIG_LOW);
 	} else {
@@ -145,7 +154,7 @@ update_interrupts(PL190 *pl)
  * 	irq line trace, called whenever a change occurs
  * -----------------------------------------------------------
  */
-static int
+static void 
 int_source_change(SigNode *node,int value,void *clientData)
 {
 	IrqTraceInfo *ti = (IrqTraceInfo *) clientData;
@@ -157,7 +166,6 @@ int_source_change(SigNode *node,int value,void *clientData)
 		pl->hardintr &= ~(1<<irq);
 	}
 	update_interrupts(pl);
-	return 0;
 }
 
 uint32_t
@@ -170,7 +178,7 @@ irqstatus_read(void *clientData,uint32_t address,int rqlen)
 static void
 irqstatus_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
 {
-        fprintf(stderr,"IRQstatus i not writable\n");
+        fprintf(stderr,"PL190 warning: IRQstatus is not writable\n");
         return;
 }
 
@@ -304,18 +312,56 @@ protection_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
         return;
 }
 
+static void
+push_privilege_level(PL190 *pl) {
+	if(pl->vectaddr_pl != PRIV_LEVEL_INVALID) {
+		if(pl->pl_stackp < 16) {
+			pl->pl_stack[pl->pl_stackp] = pl->required_pl;
+			pl->pl_stackp++;
+		} else {
+			fprintf(stderr,"PL190 Bug: privilege level stack overflow\n");
+			exit(1);
+		}
+		/* This may result in -1 ! This is Ok and intended */
+		pl->required_pl = pl->vectaddr_pl - 1; 
+	}
+}
+
+static void
+pop_privilege_level(PL190 *pl) {
+	if(pl->pl_stackp > 0) {
+		pl->pl_stackp--;
+		pl->required_pl = pl->pl_stack[pl->pl_stackp];
+	}
+}
+/*
+ *******************************************************************************
+ * When reading the vectaddr the interrupt priority logic is updated.
+ * The vectaddr disappears after reading (Tested with LPC2106)
+ *******************************************************************************
+ */
 uint32_t
 vectaddr_read(void *clientData,uint32_t address,int rqlen) 
 {
 	PL190 *pl = (PL190 *) clientData;
-	return pl->vectaddr;
+	uint32_t retval = pl->vectaddr;
+	push_privilege_level(pl);
+	update_interrupts(pl);
+	return retval;
 }
 
+/*
+ **************************************************************************************
+ * When writing the vectaddr the old interrupt privilege level comes into effect
+ * by popping from the stack 
+ **************************************************************************************
+ */
 static void
 vectaddr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
 {
-	// clear interrupt
-        fprintf(stderr,"VIC register %08x is not implemented\n",address);
+	PL190 *pl = (PL190 *) clientData;
+	pop_privilege_level(pl);
+	update_interrupts(pl);
         return;
 }
 
@@ -343,11 +389,7 @@ uint32_t
 vectaddrn_read(void *clientData,uint32_t address,int rqlen) 
 {
 	PL190 *pl = (PL190 *)clientData;
-	unsigned int index = (address - VICVECTADDRN(0))>>2;
-	if(index > 15) {
-		fprintf(stderr,"emulator bug: wrong vectadrrn_index %d\n",index);
-		return 0;
-	}
+	unsigned int index = ((address - VICVECTADDRN(0)) & 0x3f) >> 2;
 	return pl->vectaddrn[index];
 }
 
@@ -355,11 +397,7 @@ static void
 vectaddrn_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
 {
 	PL190 *pl = (PL190 *)clientData;
-	unsigned int index = (address - VICVECTADDRN(0))>>2;
-	if(index > 15) {
-		fprintf(stderr,"emulator bug: wrong vectadrrn_index %d\n",index);
-		return;
-	}
+	unsigned int index = ((address - VICVECTADDRN(0)) & 0x3f) >> 2;
 	pl->vectaddrn[index] = value;
         return;
 }
@@ -368,11 +406,7 @@ uint32_t
 veccntl_read(void *clientData,uint32_t address,int rqlen) 
 {
 	PL190 *pl = (PL190 *)clientData;
-	unsigned int index = (address - VICVECCNTL(0))>>2;
-	if(index > 15) {
-		fprintf(stderr,"emulator bug: wrong veccntl_index %d\n",index);
-		return 0;
-	}
+	unsigned int index = ((address - VICVECCNTL(0)) & 0x3f) >> 2;
 	return pl->veccntl[index];
 }
 
@@ -380,12 +414,8 @@ static void
 veccntl_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
 {
 	PL190 *pl = (PL190 *)clientData;
-	unsigned int index = (address - VICVECCNTL(0))>>2;
+	unsigned int index = ((address - VICVECCNTL(0)) & 0x3f) >> 2;
 	uint32_t oldval;
-	if(index > 15) {
-		fprintf(stderr,"emulator bug: wrong vectcntl_index %d\n",index);
-		return;
-	}
 	oldval = pl->veccntl[index];
 	pl->veccntl[index]=value;
 
@@ -404,7 +434,7 @@ periphid_read(void *clientData,uint32_t address,int rqlen)
 static void
 periphid_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
 {
-        fprintf(stderr,"VIC register %08x is not implemented\n",address);
+        fprintf(stderr,"VIC register PERIPHID %08x is not writable\n",address);
         return;
 }
 uint32_t
@@ -418,7 +448,76 @@ pcellid_read(void *clientData,uint32_t address,int rqlen)
 static void
 pcellid_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
 {
-        fprintf(stderr,"VIC pcellid register at %08x is now writable\n",address);
+        fprintf(stderr,"VIC pcellid register at %08x is not writable\n",address);
+        return;
+}
+
+uint32_t
+vicitcr_read(void *clientData,uint32_t address,int rqlen) 
+{
+        fprintf(stderr,"PL190: Read Test mode register\n");
+	return 0; 
+}
+
+static void
+vicitcr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
+{
+        fprintf(stderr,"PL190: Write Test mode register VICITRC: 0x%08x\n",value);
+        return;
+}
+
+uint32_t
+vicitip1_read(void *clientData,uint32_t address,int rqlen) 
+{
+        fprintf(stderr,"PL190: Read Test input register VICITIP1\n");
+	return 0; 
+}
+
+static void
+vicitip1_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
+{
+        fprintf(stderr,"PL190: Write Test mode register VICITIP1: 0x%08x\n",value);
+        return;
+}
+
+uint32_t
+vicitip2_read(void *clientData,uint32_t address,int rqlen) 
+{
+        fprintf(stderr,"PL190: Read Test input register VICITIP2\n");
+	return 0; 
+}
+
+static void
+vicitip2_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
+{
+        fprintf(stderr,"PL190: Write Test mode register VICITIP2: 0x%08x\n",value);
+        return;
+}
+
+uint32_t
+vicitop1_read(void *clientData,uint32_t address,int rqlen) 
+{
+        fprintf(stderr,"PL190: Read Test input register VICITOP1\n");
+	return 0; 
+}
+
+static void
+vicitop1_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
+{
+        fprintf(stderr,"PL190: Write Test mode register VICITOP1: 0x%08x\n",value);
+        return;
+}
+uint32_t
+vicitop2_read(void *clientData,uint32_t address,int rqlen) 
+{
+        fprintf(stderr,"PL190: Read Test input register VICITOP2\n");
+	return 0; 
+}
+
+static void
+vicitop2_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
+{
+        fprintf(stderr,"PL190: Write Test mode register VICITOP2: 0x%08x\n",value);
         return;
 }
 
@@ -446,6 +545,11 @@ PL190_Map(void *owner,uint32_t base,uint32_t mask,uint32_t flags)
 		IOH_New32(base+VICPERIPHID(i),periphid_read,periphid_write,pl);
 		IOH_New32(base+VICPCELLID(i),pcellid_read,pcellid_write,pl);
 	}
+	IOH_New32(base + VICITCR,vicitcr_read,vicitcr_write,pl);
+	IOH_New32(base + VICITIP1,vicitip1_read,vicitip1_write,pl);
+	IOH_New32(base + VICITIP2,vicitip2_read,vicitip2_write,pl);
+	IOH_New32(base + VICITOP1,vicitop1_read,vicitop1_write,pl);
+	IOH_New32(base + VICITOP2,vicitop2_read,vicitop2_write,pl);
 }
 
 static void
@@ -475,22 +579,13 @@ PL190_UnMap(void *owner,uint32_t base,uint32_t mask)
 }
 
 BusDevice *
-PL190_New(char *name) 
+PL190_New(const char *name) 
 {
 	PL190 *pl;
 	int i;
-	pl = (PL190*)malloc(sizeof(PL190));
-	if(!pl) {
-		fprintf(stderr,"Out of memory allocating PL190 VIC\n");
-		exit(429);
-	}
-	memset(pl,0,sizeof(PL190));
+	pl = sg_new(PL190);
 	for(i=0;i<32;i++) {
-		IrqTraceInfo *ti = (IrqTraceInfo *)malloc(sizeof(IrqTraceInfo));
-		if(!ti) {
-			fprintf(stderr,"out of memory allocating IrqTrace\n");
-			exit(342);
-		}
+		IrqTraceInfo *ti = sg_new(IrqTraceInfo);
 		ti->nr = i;
 		ti->pl190 = pl;
 		pl->traceInfo[i] = ti;
@@ -509,6 +604,12 @@ PL190_New(char *name)
 	} 
 	SigNode_Set(pl->irqNode,SIG_HIGH);	
 	SigNode_Set(pl->fiqNode,SIG_HIGH);	
+
+	/* Initialize priority encoder */
+	pl->required_pl = 15; 
+	pl->vectaddr_pl = PRIV_LEVEL_INVALID;
+	pl->pl_stackp = 0;
+
 	pl->bdev.first_mapping=NULL;
         pl->bdev.Map=PL190_Map;
         pl->bdev.UnMap=PL190_UnMap;
