@@ -12,7 +12,7 @@ typedef struct Instruction {
         InstructionProc *proc;
 } Instruction;
 
-void IDecoder_New(int cpu_type);
+void PPCIDecoder_New(int cpu_type);
 static inline InstructionProc *
 InstructionProcFind(uint32_t icode) {
         int index=INSTR_INDEX(icode);
