@@ -6,7 +6,8 @@
  * (C) 2006 Jochen Karrer
  *
  * State:
- *	Works with RAW encoding.
+ *	Works with RAW and ZRLE encoding with True color and
+ *	8 Bit colormap. Recommended viewer is realvnc
  *
  *  This program is free software; you can distribute it and/or modify it
  *  under the terms of the GNU General Public License (Version 2) as
@@ -44,6 +45,10 @@
 #include <fbdisplay.h>
 #include <keyboard.h>
 #include <zlib.h>
+#include <sys/types.h>
+#include <signal.h>
+#include <time.h>
+#include "sgstring.h"
 
 #if 0
 #define dbgprintf(x...) { fprintf(stderr,x); }
@@ -84,7 +89,6 @@ typedef struct PixelFormat {
 	uint8_t green_shift;
 	uint8_t blue_shift;
 	uint8_t padding[3];
-
 } PixelFormat;
 
 #define MAX_NAME_LEN (80)
@@ -126,7 +130,13 @@ typedef struct RfbConnection {
 	int state;
 	int current_encoding;
 	FIO_FileHandler ifh;
+
 	PixelFormat pixfmt;
+	/* translation table belongs to the pixel format */
+	uint32_t trans_red[256];
+	uint32_t trans_green[256];
+	uint32_t trans_blue[256];
+
 	FrameBufferInfo *fbi; /* points to fbi of RfbServer */
 	int ifh_is_active;
 	struct RfbConnection *next;
@@ -154,6 +164,7 @@ struct RfbServer {
 	RfbConnection *con_head;
 	/* Servers native FBI (window info & Pixelformat */
 	FrameBufferInfo fbi;
+	uint32_t exit_on_close;
 };
 
 int
@@ -181,6 +192,33 @@ pixfmt_update_bits(PixelFormat *pixfmt)
 	pixfmt->red_bits = onecount(pixfmt->red_max);		
 	pixfmt->green_bits = onecount(pixfmt->green_max);		
 	pixfmt->blue_bits = onecount(pixfmt->blue_max);		
+
+}
+
+static void
+pixfmt_update_translation(RfbConnection * rcon) {
+	uint32_t red,green,blue;
+	FrameBufferInfo *fbi = rcon->fbi;
+	PixelFormat *pixf = &rcon->pixfmt;
+	PixelFormat *fbpixf = &fbi->pixfmt;
+	for(red = 0; red <= fbpixf->red_max; red++) {
+		uint32_t trans;
+		trans = red * pixf->red_max / fbpixf->red_max; 
+		trans <<= pixf->red_shift;
+		rcon->trans_red[red] = trans;
+	}
+	for(green = 0; green <= fbpixf->green_max; green++) {
+		uint32_t trans;
+		trans = green * pixf->green_max / fbpixf->green_max; 
+		trans <<= pixf->green_shift;
+		rcon->trans_green[green] = trans;
+	}
+	for(blue = 0; blue <= fbpixf->blue_max; blue++) {
+		uint32_t trans;
+		trans = blue * pixf->blue_max / fbpixf->blue_max; 
+		trans <<= pixf->blue_shift;
+		rcon->trans_blue[blue] = trans;
+	}
 }
 /*
  * --------------------------------------------------------------------------
@@ -221,6 +259,10 @@ rfbsrv_disconnect(RfbConnection *rcon)
 		free(rcon->obuf);
 	}
 	free(rcon);
+	if(rfbserv->exit_on_close && !rfbserv->con_head) {
+		fprintf(stderr,"Exiting after termination of last VNC connection\n");
+		exit(0);
+	}
 }
 
 /*
@@ -301,7 +343,7 @@ static int
 Msg_ServerInitialisation(RfbConnection *rcon) 
 {
 	RfbServer *rfbserv = rcon->rfbserv;
-	char *msg = malloc(sizeof(FrameBufferInfo));
+	char *msg = sg_calloc(sizeof(FrameBufferInfo));
 	char *p = msg;
 	FrameBufferInfo *fbi = &rfbserv->fbi;
 	PixelFormat *pixfmt = &fbi->pixfmt;
@@ -376,12 +418,14 @@ Msg_ServerInitialisation(RfbConnection *rcon)
  */
 
 static inline uint32_t 
-encode_pixval(void *src,PixelFormat *pixf,PixelFormat *fbpixf) {
+encode_pixval(RfbConnection *rcon,void *src,PixelFormat *pixf,PixelFormat *fbpixf) {
 	uint32_t pixval;
-	int red,green,blue;
+	uint8_t red,green,blue;
+
+#if __BYTE_ORDER == __BIG_ENDIAN
 	switch(fbpixf->bits_per_pixel) {
 		case 32: 
-			pixval = le32_to_host(*(uint16_t*)src); 
+			pixval = le32_to_host(*(uint32_t*)src); 
 			break;
 		case 16: 
 			pixval = le16_to_host(*(uint16_t*)src); 
@@ -392,30 +436,20 @@ encode_pixval(void *src,PixelFormat *pixf,PixelFormat *fbpixf) {
 		default:
 			pixval = 0;
 	}
+#else 
+	pixval = le32_to_host(*(uint32_t*)src); 
+#endif
 	red = (pixval >> fbpixf->red_shift) & fbpixf->red_max;
 	green = (pixval >> fbpixf->green_shift) & fbpixf->green_max;
 	blue = (pixval >> fbpixf->blue_shift) & fbpixf->blue_max;
-	if(fbpixf->red_bits > pixf->red_bits) {
-		red = red >> (fbpixf->red_bits - pixf->red_bits); 
-	} else {
-		red = red << (pixf->red_bits - fbpixf->red_bits); 
-	}
-	if(fbpixf->green_bits > pixf->green_bits) {
-		green = green >> (fbpixf->green_bits - pixf->green_bits);
-	} else {
-		green = green << (pixf->green_bits - fbpixf->green_bits);
-	}
-	if(fbpixf->blue_bits > pixf->blue_bits) {
-		blue = blue >> (fbpixf->blue_bits - pixf->blue_bits);
-	} else {
-		blue = blue << (pixf->blue_bits - fbpixf->blue_bits);
-	}
-	pixval = (red << pixf->red_shift) | (green <<  pixf->green_shift) |
-		(blue << pixf->blue_shift);
-
-	return pixval;
+	return rcon->trans_red[red] | rcon->trans_green[green] | rcon->trans_blue[blue];
 }
 
+/*
+ * ---------------------------------------------------------------
+ * write a pixel value into a buffer
+ * ---------------------------------------------------------------
+ */
 static inline int 
 write_pixel(int con_bypp,int swap,uint8_t *dst,uint32_t pixval) 
 {
@@ -457,8 +491,13 @@ write_pixel(int con_bypp,int swap,uint8_t *dst,uint32_t pixval)
 	}
 }
 
+/*
+ * -----------------------------------------------------------------------------
+ * Writes an update message header into the header of the current message
+ * -----------------------------------------------------------------------------
+ */
 static inline int 
-fill_update_header(uint8_t *hdr,int rects) 
+add_update_header(uint8_t *hdr,int rects) 
 {
 	hdr[0] = SRV_FB_UPDATE;
 	hdr[1] = 0;
@@ -472,6 +511,12 @@ rle_init(RLEncoder *rle)
 	rle->run_length = 0;
 }
 
+/*
+ * --------------------------------------------------------
+ * rle_add_pixval
+ *	Add a pixel value to a rle encoding
+ * --------------------------------------------------------
+ */
 static inline int
 rle_add_pixval(RLEncoder *rle,int con_bypp,int swap,uint8_t *dst,uint32_t pixval) 
 {
@@ -487,16 +532,13 @@ rle_add_pixval(RLEncoder *rle,int con_bypp,int swap,uint8_t *dst,uint32_t pixval
 			int count;
 			count = write_pixel(con_bypp,swap,dst,rle->pixval);
 			dst+=count;
-			//fprintf(stderr,"Pixel %08x, rl %d cnt ",rle->pixval,rle->run_length);
 			rle->run_length-=1;
 			while(rle->run_length >= 255) {
 				*dst++ = 255;
-				//fprintf(stderr,"%d %02x ",count,255);
 				count++;
 				rle->run_length-=255;
 			}
 			*dst = rle->run_length;
-			//fprintf(stderr,"%d %02x ",count,*dst);
 			count++;
 			rle->pixval = pixval;
 			rle->run_length = 1;
@@ -505,38 +547,59 @@ rle_add_pixval(RLEncoder *rle,int con_bypp,int swap,uint8_t *dst,uint32_t pixval
 	}
 }
 
+/*
+ * --------------------------------------------------------------
+ * rle_flush
+ *	Flush the last value in the rle buffer to the encoding
+ *      destination, Then reset the run length to zero.
+ * --------------------------------------------------------------
+ */
 static inline int
 rle_flush(RLEncoder *rle,int con_bypp,int swap,uint8_t *dst) 
 {
 	int retval;
 	retval = rle_add_pixval(rle,con_bypp,swap,dst,rle->pixval ^ 0xffffffff);
 	rle->run_length = 0;
-	//fprintf(stderr,"Flushed RLE\n");
 	return retval;
 }
 
+/*
+ * ------------------------------------------------------------------------
+ * srv_encode_update_raw
+ *	Make a raw update request from the rectangles in the fifo
+ * ------------------------------------------------------------------------
+ */
 static inline void 
-srv_fb_encode_update_raw(RfbConnection *rcon,UpdateRectangle *udrect) 
+srv_fb_encode_update_raw(RfbConnection *rcon) 
 {
 	FrameBufferInfo *fbi = rcon->fbi;
 	PixelFormat *pixf = &rcon->pixfmt;
 	PixelFormat *fbpixf = &fbi->pixfmt;
 	int con_bypp = pixf->bits_per_pixel >> 3;
 	int fb_bypp = fbi->pixfmt.bits_per_pixel >> 3;
-	int memsize = udrect->width * udrect->height * con_bypp + 16;
 	uint8_t *reply; 
 	uint8_t *data;
 	int x,y;
-	if(rcon->obuf_size < memsize) {
-		rcon->obuf_size = memsize;
-		rcon->obuf = realloc(rcon->obuf,rcon->obuf_size);
-	}
 	reply = rcon->obuf;
 	data = reply;
-	data += fill_update_header(data,rcon->udrect_wp-rcon->udrect_rp);
+	data += add_update_header(data,rcon->udrect_wp-rcon->udrect_rp);
 	while(rcon->udrect_rp < rcon->udrect_wp) {	
-		udrect = &rcon->udrect_fifo[rcon->udrect_rp % UDRECT_FIFOSIZE];
+		int memsize;
+		UpdateRectangle *udrect = &rcon->udrect_fifo[rcon->udrect_rp % UDRECT_FIFOSIZE];
+
 		rcon->udrect_rp++;
+		memsize = udrect->width * udrect->height * con_bypp + 16;
+		if(rcon->obuf_size < memsize) {
+			rcon->obuf_size = memsize;
+			reply = realloc(rcon->obuf,rcon->obuf_size);
+			if(!reply) {
+				fprintf(stderr,"RFB-Server: No memory\n"); 
+				return;
+			}
+			data = reply + (data-rcon->obuf);
+			rcon->obuf = reply;
+		}
+		
 		write16be(data,udrect->x);
 		write16be(data+2,udrect->y);
 		write16be(data+4,udrect->width);
@@ -548,7 +611,7 @@ srv_fb_encode_update_raw(RfbConnection *rcon,UpdateRectangle *udrect)
 		for(y = udrect->y; y < (udrect->y+udrect->height);y++) {
 			int ofs = (y * fbi->fb_width + udrect->x) * fb_bypp;
 			for(x=udrect->x; x < (udrect->x + udrect->width);x++) {
-				uint32_t pixval = encode_pixval(&fbi->framebuffer[ofs],pixf,fbpixf); 
+				uint32_t pixval = encode_pixval(rcon,&fbi->framebuffer[ofs],pixf,fbpixf); 
 				data += write_pixel(con_bypp,swap,data,pixval);
 				ofs += fb_bypp;
 			}
@@ -559,50 +622,14 @@ srv_fb_encode_update_raw(RfbConnection *rcon,UpdateRectangle *udrect)
 	return;
 }
 
-#if 0
-static void
-zrle_tile(RfbConnection *rcon,UpdateRectangle *udrect,int x0,int y0) {
-	RLEncoder *rle = &rcon->rle;
-	uint8_t *tileBuf = alloca(1 + 64*64*5);
-	uint8_t *tileBufEnd;
-	int x1,y1,y,ofs;
-	int fb_bypp = rcon->fbi->pixfmt.bits_per_pixel >> 3;
-	tileBufEnd = tileBuf;
-	*tileBufEnd++ = 128;  /* Plain RLE */
-	rle_init(rle);
-	for(y1=0;(y1 < 64) && ((y0+y1) < udrect->height);y1++) { 
-		y = y0 + y1 + udrect->y;
-		ofs = (y * rcon->fbi->fb_width + (x0+udrect->x)) * fb_bypp;
-		for(x1=0;(x1 < 64) && ((x1+x0) < udrect->width);x1++) {
-			uint32_t pixval = encode_pixval(&rcon->fbi->framebuffer[ofs],pixf,fbpixf); 
-			tileBufEnd += rle_add_pixval(rle,con_bypp,swap,tileBufEnd,pixval);
-			ofs += fb_bypp;
-		}	
-	}
-	//fprintf(stderr,"Tile at %d %d, %d %d\n",x0+udrect->x,y0+udrect->y,x1,y1);
-	tileBufEnd += rle_flush(rle,con_bypp,swap,tileBufEnd);
-	zs->next_in = tileBuf;  
-	zs->avail_in = tileBufEnd - tileBuf;  
-	deflate(zs,Z_SYNC_FLUSH);
-	if(zs->avail_out < 32768) {
-		rcon->obuf_size *= 2;
-		rcon->obuf = realloc(rcon->obuf,rcon->obuf_size);
-		zs->next_out = rcon->obuf + rcon->obuf_wp + zs->total_out;
-		zs->avail_out = rcon->obuf_size - (rcon->obuf_wp + zs->total_out);
-		deflate(zs,Z_SYNC_FLUSH);
-	}
-}
-#endif
-
 /*
  * ----------------------------------------------------------------------------
  * srv_fb_encode_update_zrle
- * 	create zrle encoded update request. Currently not working because 
- * 	compression is missing
+ * 	create zrle encoded update request. 
  * ----------------------------------------------------------------------------
  */
 static inline void 
-srv_fb_encode_update_zrle(RfbConnection *rcon,UpdateRectangle *udrect) 
+srv_fb_encode_update_zrle(RfbConnection *rcon) 
 {
 	FrameBufferInfo *fbi = rcon->fbi;
 	PixelFormat *pixf = &rcon->pixfmt;
@@ -616,9 +643,10 @@ srv_fb_encode_update_zrle(RfbConnection *rcon,UpdateRectangle *udrect)
 	int x0,y0,x1,y1,y;
 	RLEncoder *rle = &rcon->rle;
 	z_stream *zs = &rcon->zs;
-	rcon->obuf_wp = fill_update_header(rcon->obuf,rcon->udrect_wp - rcon->udrect_rp);
+	rcon->obuf_wp = add_update_header(rcon->obuf,rcon->udrect_wp - rcon->udrect_rp);
 	while(rcon->udrect_rp < rcon->udrect_wp) {	
-		udrect = &rcon->udrect_fifo[rcon->udrect_rp % UDRECT_FIFOSIZE];
+		UpdateRectangle *udrect = &rcon->udrect_fifo[rcon->udrect_rp % UDRECT_FIFOSIZE];
+		/* fprintf(stderr,"Udrect height %d, width %d\n",udrect->height,udrect->width); */
 		rcon->udrect_rp++;
 		write16be(rcon->obuf+rcon->obuf_wp,udrect->x);
 		write16be(rcon->obuf+rcon->obuf_wp+2,udrect->y);
@@ -645,7 +673,7 @@ srv_fb_encode_update_zrle(RfbConnection *rcon,UpdateRectangle *udrect)
 					y = y0 + y1 + udrect->y;
 					ofs = (y * fbi->fb_width + (x0+udrect->x)) * fb_bypp;
 					for(x1=0;(x1 < 64) && ((x1+x0) < udrect->width);x1++) {
-						uint32_t pixval = encode_pixval(&fbi->framebuffer[ofs],pixf,fbpixf); 
+						uint32_t pixval = encode_pixval(rcon,&fbi->framebuffer[ofs],pixf,fbpixf); 
 						tileBufEnd += rle_add_pixval(rle,con_bypp,swap,tileBufEnd,pixval);
 						ofs += fb_bypp;
 					}	
@@ -656,8 +684,14 @@ srv_fb_encode_update_zrle(RfbConnection *rcon,UpdateRectangle *udrect)
 				zs->avail_in = tileBufEnd - tileBuf;  
 				deflate(zs,Z_SYNC_FLUSH);
 				if(zs->avail_out < 32768) {
+					uint8_t *obuf;
 					rcon->obuf_size *= 2;
-					rcon->obuf = realloc(rcon->obuf,rcon->obuf_size);
+					obuf = realloc(rcon->obuf,rcon->obuf_size);
+					if(!obuf) {
+						fprintf(stderr,"RFB: Not enough memory\n");
+						return;
+					}
+					rcon->obuf = obuf; 
 					zs->next_out = rcon->obuf + rcon->obuf_wp + zs->total_out;
 					zs->avail_out = rcon->obuf_size - (rcon->obuf_wp + zs->total_out);
 				}
@@ -672,26 +706,26 @@ srv_fb_encode_update_zrle(RfbConnection *rcon,UpdateRectangle *udrect)
 	return;
 }
 
+/*
+ * ----------------------------------------------------------------
+ * srv_fb_update
+ *	Calls the encoding specific framebuffer update proc
+ * ----------------------------------------------------------------
+ */
 
 static inline void 
-srv_fb_update(RfbConnection *rcon,UpdateRectangle *udrect) 
+srv_fb_update(RfbConnection *rcon) 
 {
-	FrameBufferInfo *fbi = rcon->fbi;
 	if((rcon->state != CONSTAT_IDLE)) {
 		return; 
 	}
-	if(((udrect->y + udrect->height) > fbi->fb_height) || 
-	   ((udrect->x + udrect->width) > fbi->fb_width)) {
-	   fprintf(stderr,"RFB-Server received update outside of window\n");
-	   return;
-	}  
-	//fprintf(stderr,"SRV update display\n"); // jk
+	//fprintf(stderr,"SRV update display enc %d\n",rcon->current_encoding); // jk
 	switch(rcon->current_encoding) {
 		case ENC_RAW:
-			srv_fb_encode_update_raw(rcon,udrect);
+			srv_fb_encode_update_raw(rcon);
 			break;
 		case ENC_ZRLE:
-			srv_fb_encode_update_zrle(rcon,udrect);
+			srv_fb_encode_update_zrle(rcon);
 			break;
 		case ENC_HEXTILE:
 		default:
@@ -699,14 +733,18 @@ srv_fb_update(RfbConnection *rcon,UpdateRectangle *udrect)
 	}	
 }
 
+/*
+ * -------------------------------------------------------------------------
+ * trigger_fb_update
+ *	Start an update if there is something in the rectangle fifo
+ * -------------------------------------------------------------------------
+ */
 static inline void 
 trigger_fb_update(RfbConnection *rcon) 
 {
 	if((rcon->udrect_wp != rcon->udrect_rp) && rcon->update_outstanding) {
-		UpdateRectangle * udrect;
-		udrect = &rcon->udrect_fifo[rcon->udrect_rp % UDRECT_FIFOSIZE];
 		rcon->update_outstanding = 0;
-		srv_fb_update(rcon,udrect);
+		srv_fb_update(rcon);
 	}
 }
 
@@ -736,7 +774,12 @@ expand_color_to_16(int color, int bits)
 }
 
 /*
- * For 8 Bit Pseudo color
+ * ---------------------------------------------------------------------
+ * srv_set_8Bit_color_map_entries
+ * 	For 8 Bit Pseudo color a color map has to be sent to the client. 
+ *	I'm to lazy to find the colors in
+ *	the framebuffer. So a rgb332 pixel encoding is used
+ * ---------------------------------------------------------------------
  */
 static void
 srv_set_8bit_color_map_entries(RfbConnection *rcon,PixelFormat *pixf)
@@ -751,6 +794,7 @@ srv_set_8bit_color_map_entries(RfbConnection *rcon,PixelFormat *pixf)
         pixf->blue_max = 3;
         pixf->blue_shift = 0;
         pixfmt_update_bits(pixf);
+	pixfmt_update_translation(rcon); 
         wp = reply;
         *wp++=SRV_SET_COLOUR_MAP_ENTRIES;
         *wp++=0;
@@ -790,6 +834,7 @@ clnt_set_pixel_format(RfbConnection *rcon,uint8_t *data,int len)
         if((!pixf->true_color_flag) && (pixf->bits_per_pixel == 8)) {
                 srv_set_8bit_color_map_entries(rcon,pixf);
         }
+	pixfmt_update_translation(rcon); 
 #if 1
 	dbgprintf("got set pixelformat message\n");
 	dbgprintf("msgtype %02x\n",data[0]);
@@ -847,9 +892,15 @@ static void
 write_udrect_to_fifo(RfbConnection *rcon,UpdateRectangle *udrect) 
 {
 	UpdateRectangle *u;
+	FrameBufferInfo *fbi = rcon->fbi;
+	if(((udrect->y + udrect->height) > fbi->fb_height) ||
+		((udrect->x + udrect->width) > fbi->fb_width)) {
+		fprintf(stderr,"RFB-Server received update outside of window\n");
+		return;
+	}
+
 	if((rcon->udrect_wp - rcon->udrect_rp + 1) >= UDRECT_FIFOSIZE) {
 		/* Fifo full, queue a complete screen update instead */
-		FrameBufferInfo *fbi = rcon->fbi;
 		rcon->udrect_wp = rcon->udrect_rp;
 		u = &rcon->udrect_fifo[rcon->udrect_wp % UDRECT_FIFOSIZE];
 		u->x = 0; u->y = 0;
@@ -874,7 +925,7 @@ clnt_fb_update_req(RfbConnection *rcon,uint8_t *data,int len)
 	udrect.y=read16be(data+4);
 	udrect.width = read16be(data+6);
 	udrect.height = read16be(data+8);
-	//fprintf(stderr,"Got updaterequest from Client\n");
+	/* fprintf(stderr,"Got updaterequest from Client\n"); */
 	if(!incremental) {
 		write_udrect_to_fifo(rcon,&udrect);
 		rcon->update_outstanding = 1;
@@ -894,7 +945,7 @@ clnt_key_event(RfbConnection *rcon,uint8_t *data,int len)
 	kev.down = data[1];
 	kev.key = read32be(data+4);
 	Keyboard_SendEvent(&rfbserv->keyboard,&kev);
-	//fprintf(stderr,"Got key event %08x\n",key);
+	//fprintf(stderr,"Got key event %08x\n",kev.key);
 }
 
 static void
@@ -1090,24 +1141,14 @@ rfbsrv_accept(int fd,char *host,unsigned short port, void *clientData)
 	RfbServer *rfbserv = (RfbServer *) clientData;   
 	int on;
 	RfbConnection *rcon;
-	rcon = malloc(sizeof(RfbConnection));
-	if(!rcon) {
-		fprintf(stderr,"Out of memory allocating rfb connection\n");
-		close(fd);
-		return;
-	}
-	memset(rcon,0,sizeof(RfbConnection));
+	rcon = sg_new(RfbConnection);
 	rcon->current_encoding = -1; /* Hope this doesnt exist */
 	rcon->sock_fd = fd;
 	rcon->rfbserv = rfbserv;
 	rcon->next = rfbserv->con_head;
 	rcon->fbi = &rfbserv->fbi;
 	rcon->obuf_size = 65536;
-	rcon->obuf = malloc(rcon->obuf_size);
-	if(!rcon->obuf) {
-		fprintf(stderr,"RFB: Out of memory allocating output buffer\n");
-		exit(1);
-	}
+	rcon->obuf = sg_calloc(rcon->obuf_size);
 
 	rcon->zs.zalloc = Z_NULL;
 	rcon->zs.zfree = Z_NULL;
@@ -1118,6 +1159,7 @@ rfbsrv_accept(int fd,char *host,unsigned short port, void *clientData)
 
 	/* Default Pixel format is framebuffers pixelformat */
 	memcpy(&rcon->pixfmt,&rfbserv->fbi.pixfmt,sizeof(PixelFormat));
+	pixfmt_update_translation(rcon); 
 	rfbserv->con_head = rcon;
 	rcon->current_encoding = ENC_RAW;
 	fcntl(rcon->sock_fd,F_SETFL,O_NONBLOCK);
@@ -1149,7 +1191,10 @@ rfbserv_set_fbformat(struct FbDisplay *fbdisp,FbFormat *fbf)
 	RfbServer *rfbserv = fbdisp->owner;
 	FrameBufferInfo *fbi = &rfbserv->fbi;	
 	PixelFormat *pixf = &fbi->pixfmt;
-
+	if((fbf->red_bits > 8) || (fbf->green_bits > 8) || (fbf->blue_bits > 8)) {
+		fprintf(stderr,"Framebuffer format with more than 8 Bit per color not supported\n");
+		exit(1);
+	}
 	pixf->red_max = (1<<fbf->red_bits) - 1;
 	pixf->red_bits = fbf->red_bits;
 	pixf->red_shift = fbf->red_shift;
@@ -1168,6 +1213,7 @@ rfbserv_set_fbformat(struct FbDisplay *fbdisp,FbFormat *fbf)
 	pixf->depth = fbf->depth;
 	pixf->true_color_flag = 1;
 	fb_size = fbi->fb_width*fbi->fb_height*pixf->bypp;
+	fbi->fb_linebytes = fbi->fb_width * pixf->bypp;
 	if(fb_size != fbi->fb_size) {
 		fbi->fb_size = fb_size;
 		fbi->framebuffer = realloc(fbi->framebuffer,fbi->fb_size);
@@ -1176,6 +1222,10 @@ rfbserv_set_fbformat(struct FbDisplay *fbdisp,FbFormat *fbf)
 			exit(1);
 		}
 	}
+	RfbConnection *rcon;
+	for(rcon = rfbserv->con_head;rcon;rcon=rcon->next) {
+		pixfmt_update_translation(rcon); 
+	}
 
 }
 
@@ -1187,14 +1237,14 @@ rfbserv_set_fbformat(struct FbDisplay *fbdisp,FbFormat *fbf)
  * ----------------------------------------------------------------
  */
 static int 
-rfbserv_update_display(struct FbDisplay *fbdisp,FbUpdateRequest *fbudrect) 
+rfbserv_update_display(struct FbDisplay *fbdisp,FbUpdateRequest *fbudreq) 
 {
 	RfbServer *rfbserv = fbdisp->owner;
 	RfbConnection *rcon;
 	UpdateRectangle udrect;
 	FrameBufferInfo *fbi = &rfbserv->fbi;
-	unsigned int start = fbudrect->offset;
-	unsigned int count = fbudrect->count;
+	unsigned int start = fbudreq->offset;
+	unsigned int count = fbudreq->count;
 	if(start > fbi->fb_size) {
 		return -1;
 	} 
@@ -1206,20 +1256,23 @@ rfbserv_update_display(struct FbDisplay *fbdisp,FbUpdateRequest *fbudrect)
 	 * also comparing differences should be delayed to time of sending the 
 	 * the data in a fileevent handler
  	 */
-	memcpy(fbi->framebuffer+start,fbudrect->fbdata,count);
+	memcpy(fbi->framebuffer+start,fbudreq->fbdata,count);
 	udrect.x = 0;
-	udrect.y = ((start / fbi->fb_width)>>1);
+	udrect.y = (start / fbi->fb_linebytes);
 	udrect.width = fbi->fb_width; 
 	udrect.height = (count+fbi->fb_linebytes-1)/(fbi->fb_linebytes); 
 	if(udrect.height+udrect.y > fbi->fb_height) {
 		udrect.height =  fbi->fb_height - udrect.y;	
 	}
-	//fprintf(stderr,"Got update request from LCD controller\n");
+#if 0
+	fprintf(stderr,"Got update request from LCD controller x %d ,y %d, w %d h %d\n",udrect.x,udrect.y,udrect.width,udrect.height);
+	fprintf(stderr,"linebytes %d count %d\n",fbi->fb_linebytes,count);
+#endif
 	for(rcon = rfbserv->con_head;rcon;rcon=rcon->next) {
 		write_udrect_to_fifo(rcon,&udrect);
 		trigger_fb_update(rcon);
 	}
-	return -1;
+	return 0;
 }
 
 /*
@@ -1233,6 +1286,7 @@ void
 RfbServer_New(const char *name,FbDisplay **display,Keyboard **keyboard,void **mouse)
 {
         int fd;
+	int softgunpid;
         RfbServer *rfbserv;
 	FrameBufferInfo *fbi;
 	PixelFormat *pixf;
@@ -1256,21 +1310,16 @@ RfbServer_New(const char *name,FbDisplay **display,Keyboard **keyboard,void **mo
 	}
 	/* Sanity check for width/height is missing here */
 
-        rfbserv=malloc(sizeof(RfbServer));
-        if(!rfbserv)
-                return;
-	memset(rfbserv,0,sizeof(RfbServer));
+        rfbserv = sg_new(RfbServer); 
+	rfbserv->exit_on_close = 0;
+	Config_ReadUInt32(&rfbserv->exit_on_close,name,"exit_on_close"); 
 	fbi = &rfbserv->fbi;	
 
 	fbi->fb_width = width;	
 	fbi->fb_height = height;
 	fbi->fb_linebytes = fbi->fb_width * 2;
 	fbi->fb_size = fbi->fb_width*fbi->fb_height*2;
-	fbi->framebuffer = malloc(fbi->fb_size);
-	if(!fbi->framebuffer) {
-		fprintf(stderr,"Out of memory allocating framebuffer\n");
-		exit(1);
-	}
+	fbi->framebuffer = sg_calloc(fbi->fb_size);
 	sprintf(fbi->name_string,"%s %s","softgun",name);
 
 	pixf = &fbi->pixfmt;
@@ -1304,22 +1353,47 @@ RfbServer_New(const char *name,FbDisplay **display,Keyboard **keyboard,void **mo
 	rfbserv->display.owner = rfbserv;
 	rfbserv->display.fbUpdateRequest = rfbserv_update_display;
 	rfbserv->display.setFbFormat = rfbserv_set_fbformat;
+	rfbserv->display.name = sg_strdup(name);
+	rfbserv->display.width = width;
+	rfbserv->display.height = height;
 	*keyboard = &rfbserv->keyboard;
         *display =  &rfbserv->display;
+	softgunpid = getpid();
 	if(startcmd && !(rfbserv->viewerpid = fork())) {
+		/* Shit this inherits the atexit commands currently */
 		int maxfd = getdtablesize();
+		int retcode;
 		int fd;
-    		for (fd = 0; fd < maxfd; fd++ ) {
+		struct timespec ts_start;
+		struct timespec ts_exit;
+
+    		for (fd = 3; fd < maxfd; fd++ ) {
         		close( fd );
     		}
+		dup(2); /* keep stderr as fd 3 */
+		close(0); close(1); close(2);
 		if(open("/dev/null",O_RDWR)!=0) {
 			/* Error message does not work because stderr is closed */
-			/* perror("VNC-viewer fork: Can not open /dev/null");   */
 			exit(1);
 		}
 		dup(0);
 		dup(0);
-		system(startcmd);	
+        	clock_gettime(CLOCK_MONOTONIC, &ts_start);
+		retcode = system(startcmd);	
+        	clock_gettime(CLOCK_MONOTONIC, &ts_exit);
+		/* restore stderr */
+		close(2);
+		dup(3);
+		if(retcode != 0) {
+			fprintf(stderr,"*** VNC server exited with code: %d\n",retcode);
+			fprintf(stderr,"*** Please check your config file:\n");
+			fprintf(stderr,"*** \"%s\"\n",startcmd);
+		}
+		close(2);
+		/* If an error happens during the first 1-2 seconds exit */
+		if((retcode != 0) && (ts_exit.tv_sec <= ts_start.tv_sec + 1)) {
+			kill(softgunpid,SIGTERM);
+		}
 		exit(0);
 	}
 }
