@@ -27,7 +27,6 @@
 #include <stdint.h>
 #include <unistd.h>
 #include <string.h>
-#include <sgstring.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -35,9 +34,10 @@
 #include <termios.h>
 #include <sys/ioctl.h>
 
-#include <serial.h>
-#include <fio.h>
-#include <configfile.h>
+#include "sgstring.h"
+#include "serial.h"
+#include "fio.h"
+#include "configfile.h"
 
 #if 0
 #define dbgprintf(x...) { fprintf(stderr,x); }
@@ -287,6 +287,8 @@ file_uart_cmd(SerialDevice *serial_device,UartCmd *cmd)
 		case UART_OPC_GET_CTS:
 			if(isatty(fuart->fd) && (ioctl(fuart->fd,TIOCMGET,&host_status)>=0)) {
 				cmd->retval = !!(host_status & TIOCM_CTS);
+			} else {
+				cmd->retval = 1;
 			}
 			break;
 
@@ -427,7 +429,7 @@ static SerialDevice file_uart = {
 
 SerialDevice *
 FileUart_New(const char *uart_name,const char *filename)  {
-	FileUart *fiua = sg_calloc(FileUart);
+	FileUart *fiua = sg_new(FileUart);
 	char *logfilename;
 	fiua->serdev = file_uart; /* Copy from template */
 	fiua->serdev.owner = fiua;
@@ -492,7 +494,7 @@ static SerialDevice null_uart =  {
 
 static SerialDevice * 
 NullUart_New(const char *uart_name,const char *filename)  {
-	NullUart *nua = sg_calloc(NullUart);
+	NullUart *nua = sg_new(NullUart);
 	nua->serdev = null_uart; /* copy from template */
 	nua->serdev.owner = nua;
 	nua->tx_enabled = 0;
@@ -514,7 +516,7 @@ static SerialModule_ListEntry *modListHead = NULL;
 void
 SerialModule_Register(const char *modname,SerialDevice_Constructor *newSerdev) 
 {	
-	SerialModule_ListEntry *le = sg_calloc(*le);
+	SerialModule_ListEntry *le = sg_new(*le);
 	le->type = sg_strdup(modname);
 	le->next = modListHead;
 	le->constructor = newSerdev;
@@ -526,7 +528,7 @@ Uart_New(const char *uart_name,UartRxEventProc *rxEventProc,UartTxEventProc *txE
 {
 	const char *filename=Config_ReadVar(uart_name,"file");
 	const char *type=Config_ReadVar(uart_name,"type");
-	UartPort *port = (UartPort*) malloc(sizeof(UartPort));
+	UartPort *port = sg_new(UartPort);
 	port->owner = owner;
 	port->rxEventProc = rxEventProc;
         port->txEventProc = txEventProc;
