@@ -31,9 +31,10 @@
 #include <math.h>
 #include <unistd.h>
 #include <string.h>
-#include <signode.h>
-#include <cycletimer.h>
-#include <sht71.h>
+#include "signode.h"
+#include "cycletimer.h"
+#include "sht71.h"
+#include "sgstring.h"
 
 #if 0
 #define dbgprintf(x...) { fprintf(stderr,x); }
@@ -113,18 +114,15 @@ CRC8(uint8_t val,uint8_t *crc)
  * Bitwise reverse a byte
  * ---------------------------------------
  */
-uint8_t 
-reverse_b(uint8_t value) {
-	uint8_t reverse = 0;
-	int i;
-	for(i=0;i<7;i++) {
-		if(value & (1<<i)) {
-			reverse |= (1<<(7-i));
-		}	
-	}
-	return reverse;	
 
+static uint8_t
+bitreverse8(uint8_t x) {
+        x = (x >> 4 & 0x0f) | (x << 4 & 0xf0);
+        x = (x >> 2 & 0x33) | (x << 2 & 0xcc);
+        x = (x >> 1 & 0x55) | (x << 1 & 0xaa);
+        return x;
 }
+
 /*
  * ---------------------------------------------------------------------------
  * Add "len" bytes from the reply to the CRC8 
@@ -171,7 +169,7 @@ decode_cmd(SHT71 *sht) {
 		case CMD_READSTAT:
 			sht->reply[0] = sht->statreg;
 			sht_add_reply_to_crc(sht,1);
-			sht->reply[1] = reverse_b(sht->crc);
+			sht->reply[1] = bitreverse8(sht->crc);
 			sht->replylen = 2;
 			sht->bitcnt = 1;
 			sht->bytecnt = 0;
@@ -220,7 +218,7 @@ meas_done(void *clientData)
 				sht->reply[0] = temp >> 8;
 				sht->reply[1] = temp & 0xff;
 				sht_add_reply_to_crc(sht,2);
-				sht->reply[2] = reverse_b(sht->crc);
+				sht->reply[2] = bitreverse8(sht->crc);
 				sht->state = STATE_REPLY;	
 
 				sht->replylen = 3;
@@ -267,7 +265,7 @@ meas_done(void *clientData)
 				sht->reply[0] = hum >> 8;
 				sht->reply[1] = hum & 0xff;
 				sht_add_reply_to_crc(sht,2);
-				sht->reply[2] = reverse_b(sht->crc);
+				sht->reply[2] = bitreverse8(sht->crc);
 				sht->state = STATE_REPLY;	
 
 				sht->replylen = 3;
@@ -283,7 +281,7 @@ meas_done(void *clientData)
 	}
 }
 
-static int
+static void 
 SHT_SckChange(SigNode *node,int value,void *clientData)
 {
 	SHT71 *sht = (SHT71*) clientData;
@@ -489,10 +487,9 @@ SHT_SckChange(SigNode *node,int value,void *clientData)
 			}
 			break;
 	}
-	return 0;
 }
 
-static int
+static void 
 SHT_DataChange(SigNode *node,int value,void *clientData)
 {
 	SHT71 *sht = (SHT71*) clientData;
@@ -547,7 +544,6 @@ SHT_DataChange(SigNode *node,int value,void *clientData)
 			} 
 			break;
 	}
-	return 0;
 }
 
 /*
@@ -561,7 +557,7 @@ void
 SHT71_New(const char *name) 
 {
 	SHT71 *sht;
-	sht = malloc(sizeof(*sht));
+	sht = sg_new(SHT71);
 	if(!sht) {
 		fprintf(stderr,"Out of memory allocating SHT71\n");
 		exit(1);
