@@ -1,8 +1,9 @@
 /*
  * -------------------------------------------------------------
  *
- * Emulation on Logical Signal Level 
+ * Emulation of logical signals. 
  * (C) 2004  Lightmaze Solutions AG
+ * (C) 2009  Jochen Karrer
  *   Author: Jochen Karrer
  *
  * Status:
@@ -36,10 +37,34 @@
 #include "compiler_extensions.h"
 #include <stdarg.h>
 
-static XY_HashTable signode_hash;
+static SHashTable signode_hash;
 static SigStamp g_stamp=0;
 static int active_trace_deleted_flag=0;
+static SigConflictProc *g_conflictProc = NULL;
 
+
+static char *
+SigVal_String(int sigval) {
+	switch (sigval) {
+		case SIG_LOW:
+			return "Low";
+		case SIG_HIGH:
+			return "High";
+		case SIG_OPEN:
+			return "Open";
+		case SIG_PULLDOWN:
+			return "Pulldown";
+		case SIG_PULLUP:
+			return "Pullup";
+		case SIG_WEAK_PULLDOWN:
+			return "Weak Pulldown";
+		case SIG_WEAK_PULLUP:
+			return "Weak Pullup";
+		default:
+			return "Undefined";
+	}
+};
+	
 /*
  * ----------------------------------------------------
  * SigNode_Find
@@ -51,16 +76,16 @@ SigNode *
 SigNode_Find(const char *format,...) {
 	char name[512];
 	va_list ap;
-	XY_HashEntry *entryPtr;
+	SHashEntry *entryPtr;
 	SigNode *signode;
 	va_start (ap, format);	
-	vsnprintf(name,512,format,ap);
+	vsnprintf(name,sizeof(name),format,ap);
 	va_end(ap);
-	entryPtr = XY_FindHashEntry(&signode_hash,name);
+	entryPtr = SHash_FindEntry(&signode_hash,name);
 	if(!entryPtr) {
 		return NULL;
 	}
-	signode = XY_GetHashValue(entryPtr);	
+	signode = SHash_GetValue(entryPtr);	
 	return signode;
 }
 
@@ -75,29 +100,21 @@ SigNode_New(const char *format,...) {
 	char name[512];
 	va_list ap;
 	SigNode *signode;
-	int isnew;
 	va_start (ap, format);	
 	vsnprintf(name,512,format,ap);
 	va_end(ap);
-	if((signode=SigNode_Find(name))) {
+	if((signode = SigNode_Find(name))) {
 		fprintf(stderr,"Node \"%s\" already exists\n",name);
 		return NULL;
 	}	
-	signode=sg_calloc(SigNode);
-	memset(signode,0,sizeof(SigNode));
-	signode->name=strdup(name);
-	if(!signode->name) {
-		fprintf(stderr,"Can not allocate name \"%s\"\n",name);
-		exit(3245);
-	}
-	signode->hash_entry=XY_CreateHashEntry(&signode_hash,signode->name,&isnew);
+	signode = sg_new(SigNode);
+	signode->hash_entry = SHash_CreateEntry(&signode_hash,name);
 	if(!signode->hash_entry) {
 		fprintf(stderr,"Can not create hash entry for %s\n",name);
-		free(signode->name);
-		free(signode);
+		sg_free(signode);
 		return NULL;
 	}
-	XY_SetHashValue(signode->hash_entry,signode);
+	SHash_SetValue(signode->hash_entry,signode);
 	signode->propval = signode->selfval = SIG_OPEN;
 	return signode;
 }
@@ -108,7 +125,7 @@ SigNode_New(const char *format,...) {
  * Calculate one Value from two signal values
  * ---------------------------------------------
  */
-static int8_t 
+static uint32_t 
 calculate_sigval(int val1,int val2) 
 {
 	if(val1 == SIG_OPEN) {
@@ -134,6 +151,10 @@ calculate_sigval(int val1,int val2)
 			return SIG_PULLDOWN;
 		} else if (val2==SIG_PULLUP) {
 			return SIG_PULLUP;
+		} else if (val2 == SIG_WEAK_PULLDOWN) {
+			return SIG_PULLUP;
+		} else if (val2 == SIG_WEAK_PULLUP) {
+			return SIG_PULLUP;
 		} 
 	} else if(val1 == SIG_PULLDOWN) {
 		if (val2==SIG_LOW) {
@@ -144,12 +165,45 @@ calculate_sigval(int val1,int val2)
 			return SIG_PULLDOWN;
 		} else if (val2==SIG_PULLUP) {
 			return SIG_PULLUP;
+		} else if (val2 == SIG_WEAK_PULLDOWN) {
+			return SIG_PULLDOWN;
+		} else if (val2 == SIG_WEAK_PULLUP) {
+			return SIG_PULLDOWN;
 		} 
+	} else if(val1 == SIG_WEAK_PULLUP) {
+		if(val2 == SIG_LOW) {
+			return SIG_LOW;
+		} else if(val2 == SIG_HIGH) {
+			return SIG_HIGH;
+		} else if(val2 == SIG_PULLDOWN) {
+			return SIG_PULLDOWN;
+		} else if(val2 == SIG_PULLUP) {
+			return SIG_PULLUP;
+		} else if(val2 == SIG_WEAK_PULLDOWN) {
+			return SIG_WEAK_PULLDOWN;
+		} else if(val2 == SIG_WEAK_PULLUP) {
+			return SIG_WEAK_PULLUP;
+		}
+	} else if(val1 == SIG_WEAK_PULLDOWN) {
+		if(val2 == SIG_LOW) {
+			return SIG_LOW;
+		} else if(val2 == SIG_HIGH) {
+			return SIG_HIGH;
+		} else if(val2 == SIG_PULLDOWN) {
+			return SIG_PULLDOWN;
+		} else if(val2 == SIG_PULLUP) {
+			return SIG_PULLUP;
+		} else if(val2 == SIG_WEAK_PULLDOWN) {
+			return SIG_WEAK_PULLDOWN;
+		} else if(val2 == SIG_WEAK_PULLUP) {
+			return SIG_WEAK_PULLUP;
+		}
 	}
 	return SIG_OPEN;
 }
 
-static int8_t sigval_tab[256];
+static uint32_t sigval_tab[256];
+static uint32_t sigmeassure_tab[16];
 
 static void 
 init_sigval_tab(void) 
@@ -160,13 +214,20 @@ init_sigval_tab(void)
 		int val2 = (i>>4)&0xf;
 		sigval_tab[i]=calculate_sigval(val1,val2);
 	}
+	sigmeassure_tab[SIG_LOW] = SIG_LOW;
+	sigmeassure_tab[SIG_HIGH] = SIG_HIGH;
+	sigmeassure_tab[SIG_OPEN] = SIG_OPEN; /* Should be made random */ 
+	sigmeassure_tab[SIG_PULLDOWN] = SIG_LOW;
+	sigmeassure_tab[SIG_PULLUP] = SIG_HIGH;
+	sigmeassure_tab[SIG_WEAK_PULLDOWN] = SIG_LOW;
+	sigmeassure_tab[SIG_WEAK_PULLUP] = SIG_HIGH;
 }
 
 static inline int 
 lookup_sigval(int val1,int val2) 
 {
-	uint8_t index = (val1 & 0xf) | (val2<<4);
-	int8_t result = sigval_tab[index];
+	uint8_t index = (val1 & 0xf) | (val2 << 4);
+	uint32_t result = sigval_tab[index];
 	return result;
 }
 
@@ -176,6 +237,9 @@ lookup_sigval(int val1,int val2)
  * -----------------------------------------------
  */
 
+/* avoid alloca */
+static char conflict_msg[100];
+
 static int
 _SigMeassure(SigNode *sig,SigStamp stamp,int sigval) 
 {
@@ -183,14 +247,19 @@ _SigMeassure(SigNode *sig,SigStamp stamp,int sigval)
 	sig->stamp = stamp;
 	for(cursor=sig->linkList;cursor;cursor=cursor->next) {
 		SigNode *partner = cursor->partner;
-		if(partner->stamp!=stamp) {
+		if(likely(partner->stamp!=stamp)) {
 			int oldsigval = sigval;
 			sigval = lookup_sigval(sigval,partner->selfval);
 			if((sigval==SIG_LOW) || (sigval==SIG_HIGH)) {
 				return sigval;
-			} else if(sigval & SIG_ILLEGAL) {
-				fprintf(stderr,"************ Signal conflict between %s(%d) and %s(%d) test %d\n",
-					sig->name,oldsigval,partner->name,partner->selfval,lookup_sigval(SIG_OPEN,SIG_LOW));
+			} else if(unlikely(sigval & SIG_ILLEGAL)) {
+				snprintf(conflict_msg,sizeof(conflict_msg),"************ Signal conflict between %s:(%s) and %s:(%s) ",
+					SHash_GetKey(sig->hash_entry),SigVal_String(oldsigval),SHash_GetKey(partner->hash_entry),SigVal_String(partner->selfval));
+				if(g_conflictProc) { 
+					g_conflictProc(conflict_msg);	
+				} else {
+					fprintf(stderr,"%s\n",conflict_msg);
+				}
 				return sigval;
 			}
 			sigval = _SigMeassure(partner,stamp,sigval);
@@ -199,18 +268,12 @@ _SigMeassure(SigNode *sig,SigStamp stamp,int sigval)
 	return sigval;
 }
 
-static int
+static inline int
 SigMeassure(SigNode *sig) 
 {
 
-	int measval = _SigMeassure(sig,++g_stamp,sig->selfval);
-	if(measval == SIG_PULLUP) {
-		return SIG_HIGH;
-	} else if(measval == SIG_PULLDOWN) {
-		return SIG_LOW;
-	}
-	return measval;
-
+	int propval = _SigMeassure(sig,++g_stamp,sig->selfval);
+	return sigmeassure_tab[propval & 0xf];
 }
 /*
  * -------------------------------------------------
@@ -284,7 +347,7 @@ update_sigval(SigNode *signode)
 int 
 SigNode_Set(SigNode *signode,int sigval) 
 {
-	if(sigval==signode->selfval && !(signode->propval & SIG_ILLEGAL)) {
+	if(unlikely(sigval==signode->selfval && !(signode->propval & SIG_ILLEGAL))) {
 		return signode->propval;
 	}
 	//fprintf(stderr,"Propagate new %d, old %d ",sigval,signode->selfval); //jk
@@ -302,8 +365,8 @@ int
 SigNode_Link(SigNode *sig1,SigNode *sig2) 
 {
 	SigLink *link1,*link2;
-	link1 = sg_calloc(SigLink);
-	link2 = sg_calloc(SigLink);
+	link1 = sg_new(SigLink);
+	link2 = sg_new(SigLink);
 
 	link1->partner = sig2;
 	link1->next=sig1->linkList;
@@ -358,7 +421,7 @@ Sig_RemoveLink(SigNode *sig,SigLink *link)
 	} else {
 		sig->linkList = cursor->next;
 	}	
-	free(cursor);
+	sg_free(cursor);
 	prev=NULL;
 	for(cursor=partner->linkList;cursor;prev=cursor, cursor=cursor->next) {
 		if(cursor->partner == sig) {
@@ -376,7 +439,7 @@ Sig_RemoveLink(SigNode *sig,SigLink *link)
 	}	
 	update_sigval(sig);
 	update_sigval(partner);
-	free(cursor);
+	sg_free(cursor);
 	return 1;
 }
 
@@ -475,16 +538,15 @@ void
 SigNode_Delete(SigNode *signode) {
 	
 	SigNode_UnLink(signode);
-	XY_DeleteHashEntry(&signode_hash,signode->hash_entry);
-	free(signode->name);
-	free(signode);
+	SHash_DeleteEntry(&signode_hash,signode->hash_entry);
+	sg_free(signode);
 }
 
  
 SigTrace *
 SigNode_Trace(SigNode *node,SigTraceProc *proc,void *clientData) {
 	SigTrace *trace;
-	trace=sg_calloc(SigTrace);
+	trace=sg_new(SigTrace);
 	memset(trace,0,sizeof(SigTrace));
 	trace->clientData = clientData;
 	trace->proc = proc;
@@ -511,7 +573,7 @@ SigNode_Untrace(SigNode *node,SigTrace *trace) {
 		if(cursor->isactive) {
 			active_trace_deleted_flag = 1;
 		}
-		free(cursor);
+		sg_free(cursor);
 		return 0;
 	} else {
 		fprintf(stderr,"Bug: Deleting non existing trace\n");
@@ -548,6 +610,7 @@ SigNode_FindDominant(SigNode *sig)
 {
 	return _SigNode_FindDominant(sig,++g_stamp);
 }
+
 static void 
 _SigNode_Dump(SigNode *sig,SigStamp stamp) 
 {
@@ -570,6 +633,11 @@ SigNode_Dump(SigNode *sig)
 	_SigNode_Dump(sig,++g_stamp);
 }
 
+void
+Signodes_SetConflictProc(SigConflictProc *proc)
+{
+	g_conflictProc = proc;
+}
 /*
  * --------------------------------------------
  * Setup hash tables for the signal table 
@@ -579,7 +647,7 @@ SigNode_Dump(SigNode *sig)
 void
 SignodesInit() {
 	SigNode *node;
-	XY_InitHashTable(&signode_hash,XY_STRING_KEYS,1024);
+	SHash_InitTable(&signode_hash);
 
 	init_sigval_tab();
 	node=SigNode_New("GND");
