@@ -26,7 +26,7 @@
 #define SIGNODE_H
 #include <stdint.h>
 #include <stdlib.h>
-#include <xy_hash.h>
+#include <strhash.h>
 
 /*
  * ----------------------------------------------------------------------
@@ -34,20 +34,22 @@
  * A meassured value can only be SIG_HIGH, SIG_LOW, or (currently) OPEN
  * ----------------------------------------------------------------------
  */
-#define SIG_LOW       (0)
-#define SIG_HIGH      (1)
-#define SIG_OPEN      (2)
-#define SIG_PULLDOWN  (3)
-#define SIG_PULLUP    (4)
-#define SIG_ILLEGAL   (0x40)
+#define SIG_LOW            (0)
+#define SIG_HIGH           (1)
+#define SIG_OPEN           (2)
+#define SIG_PULLDOWN  	   (3)
+#define SIG_PULLUP    	   (4)
+#define SIG_WEAK_PULLDOWN  (5)
+#define SIG_WEAK_PULLUP    (6)
+#define SIG_ILLEGAL   (0x100)
 
 typedef uint64_t SigStamp;
 
 struct SigTrace;
 struct SigLink;
 typedef struct SigNode {
-	char *name;
-	XY_HashEntry *hash_entry;
+	uint32_t magic;
+	SHashEntry *hash_entry;
 	int selfval;
 	int propval;
 	SigStamp stamp;
@@ -55,15 +57,16 @@ typedef struct SigNode {
 	struct SigTrace *sigTraceList;
 } SigNode;
 
-static inline char * SigName(SigNode *node) {
-	return(node->name);
+static inline const char * SigName(SigNode *node) {
+	return SHash_GetKey(node->hash_entry);
 }
 typedef struct SigLink {
 	struct SigLink * next;
 	SigNode *partner;
 } SigLink;
 
-typedef int SigTraceProc(struct SigNode *,int value,void *clientData);
+typedef void SigTraceProc(struct SigNode *,int value,void *clientData);
+typedef void SigConflictProc(const char *msg);
 
 typedef struct SigTrace {
 	SigTraceProc *proc;
@@ -89,5 +92,11 @@ SigNode * SigNode_FindDominant(SigNode *sig);
 void SigNode_Dump(SigNode *sig); 
 SigNode * SigNode_Find(const char *format,...)  __attribute__ ((format (printf, 1, 2)));; 
 int SigEdge(SigNode *node); 
+/*
+ **************************************************************************
+ * Procedure to print an error message if a signal level conflict occurs
+ **************************************************************************
+ */
+void Signodes_SetConflictProc(SigConflictProc *proc);
 
 #endif
