@@ -25,19 +25,19 @@
 #include <stdint.h>
 #include <stdio.h>
 #include <string.h>
-#include <malloc.h>
-#include <unistd.h>
 #include <stdlib.h>
-#include <bus.h>
-#include <sja1000.h>
-#include <ns9750_timer.h> // should be removed with interrupt ops pointer
-#include <socket_can.h>
+#include <unistd.h>
+#include "bus.h"
+#include "sja1000.h"
+#include "ns9750_timer.h" // should be removed with interrupt ops pointer
+#include "socket_can.h"
 #include "signode.h"
+#include "sgstring.h"
 
 #if 0
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 
 #define SJA_BASE  (0x70000000)
@@ -169,7 +169,7 @@ unhandled_read(void *clientData,uint32_t address,int rqlen) {
 }
 static void 
 unhandled_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-        dprintf("sja1000 unhandled write of addr %08x, value %08x,rqlen %d\n",address,value,rqlen);
+        dbgprintf("sja1000 unhandled write of addr %08x, value %08x,rqlen %d\n",address,value,rqlen);
 	return;
 }
 /*
@@ -243,7 +243,7 @@ sja_pc_check_acceptance(SJA1000 *sja,CAN_MSG *msg) {
 		if(((value ^ code) | mask)==0xffffffff) {
 			return 1;
 		}
-		dprintf("v %08x mask %08x c %08x\n",value,mask,code);
+		dbgprintf("v %08x mask %08x c %08x\n",value,mask,code);
 		return 0;	
 	} else { 
 
@@ -269,7 +269,7 @@ sja_pc_check_acceptance(SJA1000 *sja,CAN_MSG *msg) {
 			value1 = msg->id>>13;
 			value2 = msg->id>>13;
 		} else {
-			dprintf("Unknown message type, not accepted\n"); // maybe we should handle Error Messages
+			dbgprintf("Unknown message type, not accepted\n"); // maybe we should handle Error Messages
 			return 0;
 		}
 		if(msg->typ == CAN_MSG_T_11_RTR) {
@@ -282,8 +282,8 @@ sja_pc_check_acceptance(SJA1000 *sja,CAN_MSG *msg) {
 		if(((value2 ^ code2) | mask2)==0xffffffff) {
 			return 1;
 		}
-		dprintf("v1 %08x mask1 %08x c1 %08x\n",value1,mask1,code1);
-		dprintf("v2 %08x mask2 %08x c2 %08x\n",value2,mask2,code2);
+		dbgprintf("v1 %08x mask1 %08x c1 %08x\n",value1,mask1,code1);
+		dbgprintf("v2 %08x mask2 %08x c2 %08x\n",value2,mask2,code2);
 		return 0;	
 	}
 
@@ -302,7 +302,7 @@ sja_pc_check_acceptance(SJA1000 *sja,CAN_MSG *msg) {
 static void
 sja_pelican_receive (SJA1000 *sja,CAN_MSG *msg) {
 	int i;
-	dprintf("Received a PeliCAN message\n");
+	dbgprintf("Received a PeliCAN message\n");
 	if(msg->len>8) {
 		fprintf(stderr,"CAN: Illegal Message length %d, message ignored\n",msg->len);
 		return;
@@ -312,7 +312,7 @@ sja_pelican_receive (SJA1000 *sja,CAN_MSG *msg) {
 		return;
 	}
 	if(msg->typ==CAN_MSG_T_11) {
-		dprintf("Typ 11\n");
+		dbgprintf("Typ 11\n");
 		INFIFO_PUT(sja,msg->len | 0);
 		INFIFO_PUT(sja,msg->id>>3);
 		INFIFO_PUT(sja,((msg->id&7)<<5) | 0  );
@@ -321,7 +321,7 @@ sja_pelican_receive (SJA1000 *sja,CAN_MSG *msg) {
 		}
 		sja->pc_rxcnt++;
 	} else if(msg->typ==CAN_MSG_T_11_RTR) {
-		dprintf("Typ 11 RTR\n");
+		dbgprintf("Typ 11 RTR\n");
 		INFIFO_PUT(sja,msg->len | 0x40);
 		INFIFO_PUT(sja,msg->id>>3);
 		INFIFO_PUT(sja,((msg->id&7)<<5) | 0x10 );
@@ -330,7 +330,7 @@ sja_pelican_receive (SJA1000 *sja,CAN_MSG *msg) {
 		}
 		sja->pc_rxcnt++;
 	} else if(msg->typ==CAN_MSG_T_29) {
-		dprintf("Typ 29 \n");
+		dbgprintf("Typ 29 \n");
 		INFIFO_PUT(sja,msg->len | 0x80);
 		INFIFO_PUT(sja,msg->id>>21);
 		INFIFO_PUT(sja,msg->id>>13);
@@ -341,7 +341,7 @@ sja_pelican_receive (SJA1000 *sja,CAN_MSG *msg) {
 		}
 		sja->pc_rxcnt++;
 	} else if(msg->typ==CAN_MSG_T_29_RTR) {
-		dprintf("Typ 29 RTR\n");
+		dbgprintf("Typ 29 RTR\n");
 		INFIFO_PUT(sja,msg->len | 0xc0);
 		INFIFO_PUT(sja,msg->id>>21);
 		INFIFO_PUT(sja,msg->id>>13);
@@ -370,7 +370,7 @@ sja_pelican_receive (SJA1000 *sja,CAN_MSG *msg) {
 		}
 	}
 	sja_update_interrupts(sja);
-	dprintf("SJA1000 got a new CAN-message\n");
+	dbgprintf("SJA1000 got a new CAN-message\n");
 }
 /*
  * ----------------------------------------------------
@@ -434,11 +434,11 @@ sja_pc_cmr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 		CanSend(sja->interface,&msg);
 		sja->ir |= SJA_PC_IR_TI & sja->pc_ier;
 		sja_update_interrupts(sja);
-		dprintf("SJA1000: Transmit request\n");
+		dbgprintf("SJA1000: Transmit request\n");
 	} else if((value & SJA_PC_CMR_SRR)  && !(sja->mode & SJA_PC_MOD_LOM)) {
 		/* Setting SRR and TR simultanously ignores SRR */
 		CAN_MSG msg;
-		dprintf("Self reception request\n");
+		dbgprintf("Self reception request\n");
 		sja_assemble_tx_message(sja,&msg);
 		CanSend(sja->interface,&msg);
 		sja->ir |= SJA_PC_IR_TI & sja->pc_ier;
@@ -457,7 +457,7 @@ sja_pc_cmr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
  	 */
 	if(value & SJA_PC_CMR_RRB) {
 		if(!sja->sr & SJA_SR_RBS) {
-			dprintf("SJA1000: Warning, try to release empty receive buffer\n");
+			dbgprintf("SJA1000: Warning, try to release empty receive buffer\n");
 			return;
 		}
 		if(IS_PELICAN(sja)) {
@@ -665,7 +665,7 @@ sja_pcr_acm_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	SJA1000 *sja=clientData;
 	uint32_t idx=(address&0x7f) - SJA_REG(SJA_PCR_ACM0);
 	if(idx<4) {
-		dprintf("setze acm %d to %08x\n",idx,value);
+		dbgprintf("setze acm %d to %08x\n",idx,value);
 		sja->pcr_acm[idx]=value;
 	} else {
 		fprintf(stderr,"illegal index %d\n",idx);
@@ -693,7 +693,7 @@ sja_pcr_acc_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	SJA1000 *sja=clientData;
 	uint32_t idx=(address &0x7f) - SJA_REG(SJA_PCR_ACC0);
 	if(idx<4) {
-		dprintf("setze acc %d to %08x\n",idx,value);
+		dbgprintf("setze acc %d to %08x\n",idx,value);
 		sja->pcr_acc[idx]=value;
 	} else {
 		fprintf(stderr,"illegal index %d\n",idx);
@@ -841,7 +841,7 @@ sja_pcr_rbsa_write (void *clientData,uint32_t value,uint32_t address,int rqlen)
 static uint32_t 
 sja_clkdiv_read(void *clientData,uint32_t address,int rqlen) {
 	SJA1000 *sja=clientData;
-        dprintf("SJA1000 clkdiv read 0x%02x\n",sja->clkdiv);
+        dbgprintf("SJA1000 clkdiv read 0x%02x\n",sja->clkdiv);
         return sja->clkdiv;
 }
 
@@ -856,7 +856,7 @@ sja_clkdiv_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	if(sja->layout!=SJA_LOUT_PCR && sja->layout!=SJA_LOUT_BCR) {
 		fprintf(stderr,"SJA1000: Warning, CLKDIV write in  mem layout %08x\n",sja->layout);
 	}
-        dprintf("SJA1000 clkdiv write 0x%02x\n",value);
+        dbgprintf("SJA1000 clkdiv write 0x%02x\n",value);
         return;
 }
 
@@ -1052,7 +1052,7 @@ sja_receive (void *clientData,CAN_MSG *msg) {
 	} else if(sja->layout==SJA_LOUT_BC) {
 		sja_basic_receive(sja,msg);
 	} else {
-		dprintf("received was in lout %08x\n",sja->layout);
+		dbgprintf("received was in lout %08x\n",sja->layout);
 	}
 }
 
@@ -1063,12 +1063,7 @@ static CanChipOperations sja_iface_ops = {
 SJA1000 *
 SJA1000_New(BusDevice *bus,const char *name) {
 	SJA1000 *sja;
-	sja=malloc(sizeof(SJA1000));
-	if(!sja) {
-		fprintf(stderr,"Out of memory\n");		
-		exit(574);
-	}
-	memset(sja,0,sizeof(SJA1000));
+	sja = sg_new(SJA1000);
 	sja->irqNode = SigNode_New("%s.irq",name);
 	if(!sja->irqNode) {
 		exit(3242);
