@@ -34,13 +34,18 @@
 #include "signode.h"
 #include "clock.h"
 #include "mmu.h"
-#include "gdebug.h"
+#include "debugger.h"
 #include "loader.h"
 #include "configfile.h"
 #include "shlib.h"
 #include "boards/boards.h"
 #include "senseless.h"
 #include "version.h"
+#include "cliserver.h"
+#include "debugvars.h"
+#include "interpreter.h"
+#include "sgstring.h"
+#include "sglib.h"
 
 typedef struct LoadChainEntry {
 	struct LoadChainEntry *next;
@@ -58,22 +63,18 @@ static void
 LoadChain_Append(const char *addr_string,const char *filename)
 {
 	struct stat stat_buf;
-	LoadChainEntry *lce = malloc(sizeof(*lce));
+	LoadChainEntry *lce = sg_new(LoadChainEntry);
 	if(!addr_string) {
 		addr_string = "flash";
 	}
-	if(!lce) {
-		fprintf(stderr,"Out of memory allocating LoadChainEntry\n");
-		exit(1);
-	}
 	fprintf(stderr,"LCA \"%s\" \"%s\"\n",addr_string,filename);
 	if(stat(filename,&stat_buf) <0) {
 		fprintf(stderr,"stat on file \"%s\" failed ",filename);
 		perror("");
 		exit(1);
 	}
-	lce->filename = strdup(filename);
-	lce->addr_string = strdup(addr_string);
+	lce->filename = sg_strdup(filename);
+	lce->addr_string = sg_strdup(addr_string);
 	if(!loadChainHead) {
 		lce->next = NULL;
 		loadChainHead = lce;
@@ -186,8 +187,8 @@ parse_commandline(int argc,char *argv[]) {
 					if((argc > 2) && (argv[1][0] != '-') && (argv[2][0] != '-')) {
 						LoadChain_Append(argv[1],argv[2]);
 						argc-= 2; argv+=2;
-#if 0
-					} else if((argc > 1) && (argv[1][0] != '-')) {
+#if 1
+					} else if((argc > 1)) {
 						LoadChain_Append(NULL,argv[1]);
 						argc-= 1; argv+=1;
 #endif
@@ -199,7 +200,7 @@ parse_commandline(int argc,char *argv[]) {
 					break;
 
 				case 'd':
-					Config_AddString("\n[global]\ngdbwait: 1\n");
+					Config_AddString("\n[global]\ndbgwait: 1\n");
 					break;
 					
 				case 'g':
@@ -237,8 +238,11 @@ int
 main(int argc,char *argv[]) {
 	char *boardname;
 	Board *board;
+	SGLib_Init();
 	signal(SIGPIPE, SIG_IGN);
 	parse_commandline(argc-1,argv+1);
+	CmdRegistry_Init();
+	DbgVars_Init();
 	read_configfile();
 	FIO_Init();
 	SignodesInit();
@@ -259,6 +263,7 @@ main(int argc,char *argv[]) {
 		Config_AddString(Board_DefaultConfig(board));
 	}
 	Board_Create(board);
+	CliServer_New("cli");	
 	LoadChain_Resolve();
 	if(LoadChain_Load() < 0) {
 		fprintf(stderr,"Loading failed\n");
