@@ -111,11 +111,11 @@ parse_srec_data(uint8_t *buf,const char *line,int count,uint32_t sum)
  */
 static int 
 read_record(FILE *file,uint32_t *addr,uint8_t *buf) {
-	char line[100];
+	char line[257];
 	int len;
 	unsigned int sum;
 	uint8_t count;
-	if(fgets(line,80,file)!=line) {
+	if(fgets(line,256,file)!=line) {
 		if(feof(file)) {
 			return 0;
 		}
@@ -233,7 +233,7 @@ XY_LoadSRecordFile(char *filename,XY_SRecCallback *callback,void *clientData)
 		if((count=read_record(file,&addr,buf))<0) {
 			break;
 		}
-		if(callback) { 
+		if(callback && (count > 0)) { 
 			result=callback(addr,buf,count,clientData);
 			if(result<0) {
 				return result;
