@@ -40,17 +40,18 @@
 #include <unistd.h>
 #include <errno.h>
 #include <stdio.h>
-#include <signode.h>
-#include <m93c46.h>
-#include <ste10_100.h>
-#include <fio.h>
-#include <linux-tap.h>
-#include <cycletimer.h>
+#include "signode.h"
+#include "m93c46.h"
+#include "ste10_100.h"
+#include "fio.h"
+#include "linux-tap.h"
+#include "cycletimer.h"
+#include "sgstring.h"
 
 #if 0 
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 /*
  * -----------------------------------------
@@ -330,25 +331,25 @@ update_interrupts(STE10_100 *ste,int line)
 	uint32_t ier=IER(ste);	
 	uint32_t sr=SR(ste); 
 	uint32_t pending = sr&ier;
-	dprintf("Update interrupts %08x, posted %d\n",pending,ste->interrupt_posted);
+	dbgprintf("Update interrupts %08x, posted %d\n",pending,ste->interrupt_posted);
 	if(
 		((pending & NORMAL_INT_MASK) && (ier&IER_NIE))
 		||((pending & ABNORMAL_INT_MASK) && (ier&IER_AIE))
 	) {
 		if(!ste->interrupt_posted) {
-			dprintf("Poste Interrupt\n");
+			dbgprintf("Poste Interrupt\n");
 			PCI_PostIRQ(ste->bridge,ste->bus_irq);
 			ste->interrupt_posted=1;
 		} else {
-			dprintf("already posted\n");
+			dbgprintf("already posted\n");
 		}	
 	} else {
 		if(ste->interrupt_posted) {
-			dprintf("Un Interrupt\n");
+			dbgprintf("Un Interrupt\n");
 			PCI_UnPostIRQ(ste->bridge,ste->bus_irq);
 			ste->interrupt_posted=0;
 		} else {
-			dprintf("already UNposted\n");
+			dbgprintf("already UNposted\n");
 		}
 	} 
 }
@@ -416,7 +417,7 @@ gptmr_timeout(void *clientData)
 
 static void
 disable_receiver(STE10_100 *ste) {
-	dprintf("ste10/100: disable receiver\n");
+	dbgprintf("ste10/100: disable receiver\n");
 	if(ste->receiver_is_enabled) {
  		FIO_RemoveFileHandler(&ste->input_fh);
 		ste->receiver_is_enabled=0;
@@ -440,7 +441,7 @@ fill_rxfifo(STE10_100 *ste) {
 	int result;
 	char buf[2048];
 	if(ste->rxfifo_count) {
-		dprintf("Paket loss, rxfifo is full !\n");
+		dbgprintf("Paket loss, rxfifo is full !\n");
 		return read(ste->ether_fd,buf,2048);
 	}
 	result=read(ste->ether_fd,ste->rx_fifo,2048);
@@ -568,7 +569,7 @@ receive(STE10_100 *ste) {
 		rdes0 |= RDES0_FS | RDES0_LS;
 		pci_descriptor_write32(ste,rdes0,ste->int_rxbufp);
 	} else {
-		dprintf("ste10/100 Warning: Rx Buffer smaller than paket\n");
+		dbgprintf("ste10/100 Warning: Rx Buffer smaller than paket\n");
 	}
 
 	if(rdes1 & RDES1_RER) {
@@ -603,7 +604,7 @@ input_event(void *cd,int mask) {
 static void
 enable_receiver(STE10_100 *ste) {
 	if(!ste->receiver_is_enabled && (ste->ether_fd >=0)) {
-		dprintf("ste10/100: enable receiver\n");
+		dbgprintf("ste10/100: enable receiver\n");
 		FIO_AddFileHandler(&ste->input_fh,ste->ether_fd,FIO_READABLE,input_event,ste);
 		ste->receiver_is_enabled=1;
 	}
@@ -634,7 +635,7 @@ transmit(STE10_100 *ste) {
 		tdes1=pci_descriptor_read32(ste,ste->int_txbufp+4);
 		buf1=tdes2=pci_descriptor_read32(ste,ste->int_txbufp+8);
 		buf2=tdes3=pci_descriptor_read32(ste,ste->int_txbufp+12);
-		dprintf("ste10/100 Transmit called\n");
+		dbgprintf("ste10/100 Transmit called\n");
 		if(!(tdes0 & TDES0_OWN))  {
 			SR(ste) |= SR_TDU; 
 			SR(ste) = (SR(ste) & ~SR_TS_MASK) | SR_TS_SUSPENDED;
@@ -651,11 +652,11 @@ transmit(STE10_100 *ste) {
 		chain=tdes1 & TDES1_CHAIN;
 		len1=tdes1&0x7ff;
 		len2=(tdes1>>11)&0x7ff;
-		dprintf("Transmit called\n");	
-		dprintf("buffer1 address %08x, len %d\n",tdes2,len1);	
-		dprintf("buffer2 address %08x, len %d\n",tdes3,len2);	
+		dbgprintf("Transmit called\n");	
+		dbgprintf("buffer1 address %08x, len %d\n",tdes2,len1);	
+		dbgprintf("buffer2 address %08x, len %d\n",tdes3,len2);	
 		if(len1) {
-			dprintf("\n");
+			dbgprintf("\n");
 			pci_data_read(ste,buf1,data,len1);
 			count=0;
 			do  {
@@ -692,7 +693,7 @@ transmit(STE10_100 *ste) {
 		pci_descriptor_write32(ste,tdes0,ste->int_txbufp);
 		if(tdes1 & TDES1_IC) {
 			SR(ste) |= SR_TCI;	
-			dprintf("ste10/100: Poste einen TX-Interrupt, csr5 %08x ier %08x \n",SR(ste),IER(ste));
+			dbgprintf("ste10/100: Poste einen TX-Interrupt, csr5 %08x ier %08x \n",SR(ste),IER(ste));
 			update_interrupts(ste,__LINE__);
 		}
 		if(chain) {
@@ -723,7 +724,7 @@ ser_port_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 {
 	STE10_100 *ste=clientData;
 	if((value&0x4800)!=0x4800) {
-		//dprintf("access to serial eeprom not enabled (%04x)\n",value);	
+		//dbgprintf("access to serial eeprom not enabled (%04x)\n",value);	
 		CSR9(ste)=value;
 		return;
 	}
@@ -773,7 +774,7 @@ sr_read(void *clientData,uint32_t address,int rqlen) {
 	if(sr & (SR_TPS |  SR_TJT | SR_TUF | SR_RDU | SR_RPS | SR_RWT | SR_GPTT | SR_FBE)) {
 		sr=sr | SR_AISS;
 	}
-	dprintf("ste10/100 SR read %08x, value %08x\n",address,SR(ste));
+	dbgprintf("ste10/100 SR read %08x, value %08x\n",address,SR(ste));
 	return sr;
 }
 
@@ -790,7 +791,7 @@ sr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 		update_gptimer(ste);
 	}
 	update_interrupts(ste,__LINE__);
-	dprintf("ste10/100 SR of addr %08x, value %08x, mask %08x, new sr: %08x\n",address,value,mask,SR(ste));
+	dbgprintf("ste10/100 SR of addr %08x, value %08x, mask %08x, new sr: %08x\n",address,value,mask,SR(ste));
 	return;
 }
 /*
@@ -801,7 +802,7 @@ sr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 static uint32_t 
 nar_read(void *clientData,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
-	dprintf("ste10/100 NAR read %08x, value %08x\n",address,CSR6(ste));
+	dbgprintf("ste10/100 NAR read %08x, value %08x\n",address,CSR6(ste));
 	return CSR6(ste);
 }
 
@@ -845,7 +846,7 @@ nar_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 		/* should try this with real device */
 		SR(ste) = (SR(ste) & ~SR_TS_MASK) | SR_TS_SUSPENDED;
 	}
-	dprintf("ste10/100 NAR of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 NAR of addr %08x, value %08x\n",address,value);
 	return;
 }
 
@@ -856,7 +857,7 @@ nar_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 static uint32_t 
 ier_read(void *clientData,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
-	dprintf("ste10/100 IER read %08x, value %08x\n",address,IER(ste));
+	dbgprintf("ste10/100 IER read %08x, value %08x\n",address,IER(ste));
 	return IER(ste);   
 }
 
@@ -871,7 +872,7 @@ ier_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 		fprintf(stderr,"Warning AISS not set but some abnormal interrupts\n");	
 	}
 	update_interrupts(ste,__LINE__);
-	dprintf("ste10/100 IER write %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 IER write %08x, value %08x\n",address,value);
 	return;
 }
 
@@ -884,12 +885,12 @@ ier_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 static uint32_t 
 lpc_read(void *clientData,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
-	dprintf("ste10/100 LPC read %08x, value %08x\n",address,CSR8(ste));
+	dbgprintf("ste10/100 LPC read %08x, value %08x\n",address,CSR8(ste));
 	return CSR8(ste);
 }
 static void 
 lpc_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled LPC of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 unhandled LPC of addr %08x, value %08x\n",address,value);
 	return;
 }
 
@@ -901,14 +902,14 @@ lpc_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 static uint32_t 
 par_read(void *clientData,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
-	dprintf("ste10/100 PAR read %08x, value %08x\n",address,PAR(ste) & ~(uint32_t)1);
+	dbgprintf("ste10/100 PAR read %08x, value %08x\n",address,PAR(ste) & ~(uint32_t)1);
 	return PAR(ste) & ~(uint32_t)1; // clear reste flag   
 }
 
 static void 
 par_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
-	dprintf("ste10/100 PAR write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 PAR write of addr %08x, value %08x\n",address,value);
 	PAR(ste)=value;
 	if(value&1) {
 		STE_Reset(ste);	
@@ -929,14 +930,14 @@ par_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 static uint32_t 
 tdr_read(void *clientData,uint32_t address,int rqlen) {
 	//STE10_100 *ste=clientData;
-	dprintf("ste10/100 TDR read %08x, value %08x\n",address,0xffffffff);
+	dbgprintf("ste10/100 TDR read %08x, value %08x\n",address,0xffffffff);
 	return 0xffffffff; // ??? written value of always 0xffffffff ???
 }
 
 static void 
 tdr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
-	dprintf("ste10/100 TDR write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 TDR write of addr %08x, value %08x\n",address,value);
 	CSR1(ste)=value;
 	transmit(ste);	
 	return;
@@ -950,14 +951,14 @@ tdr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
  */
 static uint32_t 
 rdr_read(void *clientData,uint32_t address,int rqlen) {
-	dprintf("ste10/100 RDR read %08x, value %08x\n",address,0xffffffff);
+	dbgprintf("ste10/100 RDR read %08x, value %08x\n",address,0xffffffff);
 	return 0xffffffff; // ??? written value of always 0xffffffff ???
 }
 
 static void 
 rdr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
-	dprintf("ste10/100 RDR write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 RDR write of addr %08x, value %08x\n",address,value);
 	CSR2(ste)=value;
 	SR(ste) = (SR(ste) & ~SR_RS_MASK) | SR_RS_WAIT;
 	enable_receiver(ste);	
@@ -972,14 +973,14 @@ rdr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 static uint32_t 
 rdb_read(void *clientData,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
-	dprintf("ste10/100 RDB read %08x, value %08x\n",address,CSR3(ste));
+	dbgprintf("ste10/100 RDB read %08x, value %08x\n",address,CSR3(ste));
 	return CSR3(ste);
 }
 
 static void 
 rdb_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
-	dprintf("ste10/100 RDB write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 RDB write of addr %08x, value %08x\n",address,value);
 	CSR3(ste)=value;
 	ste->int_rxbufp=value;
 	return;
@@ -992,14 +993,14 @@ rdb_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 static uint32_t 
 tdb_read(void *clientData,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
-	dprintf("ste10/100 TDB read %08x, value %08x\n",address,CSR4(ste));
+	dbgprintf("ste10/100 TDB read %08x, value %08x\n",address,CSR4(ste));
 	return CSR4(ste);
 }
 
 static void 
 tdb_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
-	dprintf("ste10/100 TDB write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 TDB write of addr %08x, value %08x\n",address,value);
 	CSR4(ste)=value;
 	ste->int_txbufp=value;
 	return;
@@ -1021,7 +1022,7 @@ par0_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	ste->mac[1]=(value>>8)&0xff;
 	ste->mac[2]=(value>>16)&0xff;
 	ste->mac[3]=(value>>24)&0xff;
-	dprintf("ste10/100 PAR0 (csr25) write addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 PAR0 (csr25) write addr %08x, value %08x\n",address,value);
 	return;
 }
 static uint32_t 
@@ -1029,7 +1030,7 @@ par0_read(void *clientData,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
 	uint8_t *mac = ste->mac;
 	uint32_t par0=mac[0] | (mac[1]<<8) | (mac[2]<<16) | (mac[3]<<24);
-//	dprintf("ste10/100 csr25 read %08x, value %08x\n",address,par0);
+//	dbgprintf("ste10/100 csr25 read %08x, value %08x\n",address,par0);
 	return par0;
 }
 
@@ -1039,7 +1040,7 @@ par1_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	uint8_t *mac = ste->mac;
 	mac[4]= value & 0xff;	
 	mac[5]= (value>>8) & 0xff;	
-	dprintf("ste10/100 PAR1 (csr26) write addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 PAR1 (csr26) write addr %08x, value %08x\n",address,value);
 	return;
 }
 static uint32_t 
@@ -1047,7 +1048,7 @@ par1_read(void *clientData,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
 	uint8_t *mac = ste->mac;
 	uint32_t par1;
-//	dprintf("ste10/100 csr26 read %08x, value %08x\n",address,par1);
+//	dbgprintf("ste10/100 csr26 read %08x, value %08x\n",address,par1);
 	par1=mac[4] | (mac[5]<<8);
 	return par1;   
 }
@@ -1061,36 +1062,36 @@ static void
 mar0_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
 	MAR0(ste)=value;
-	dprintf("ste10/100 MAR0 write addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 MAR0 write addr %08x, value %08x\n",address,value);
 	return;
 }
 static uint32_t 
 mar0_read(void *clientData,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
-	dprintf("ste10/100 MAR0 read %08x, value %08x\n",address,MAR0(ste));
+	dbgprintf("ste10/100 MAR0 read %08x, value %08x\n",address,MAR0(ste));
 	return MAR0(ste);   
 }
 static void 
 mar1_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
 	MAR1(ste)=value;
-	dprintf("ste10/100 MAR1 write addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 MAR1 write addr %08x, value %08x\n",address,value);
 	return;
 }
 static uint32_t 
 mar1_read(void *clientData,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
-	dprintf("ste10/100 MAR1 read %08x, value %08x\n",address,MAR1(ste));
+	dbgprintf("ste10/100 MAR1 read %08x, value %08x\n",address,MAR1(ste));
 	return MAR1(ste);
 }
 static uint32_t 
 unhandled_read(void *clientData,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled read of addr %08x\n",address);
+	dbgprintf("ste10/100 unhandled read of addr %08x\n",address);
 	return 0;
 }
 static void 
 unhandled_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
 	return;
 }
 
@@ -1113,7 +1114,7 @@ tmr_read(void *clientData,uint32_t address,int rqlen) {
 		ste->gptmr_stopped=1;
 		CycleTimer_Remove(&ste->gptmr);
 	}
-	dprintf("STE: GP-Timer read\n");
+	dbgprintf("STE: GP-Timer read\n");
 	return ste->gptmr_count | 0xfffe0000 | periodic;
 }
 
@@ -1125,58 +1126,58 @@ tmr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	ste->gptmr_last_update = CycleCounter_Get();
 	ste->gptmr_stopped=0;
 	update_gptimer(ste);
-	dprintf("STE: GP-Timer write\n");
+	dbgprintf("STE: GP-Timer write\n");
 	return;
 }
 
 static uint32_t 
 wcsr_read(void *clientData,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled read of addr %08x\n",address);
+	dbgprintf("ste10/100 unhandled read of addr %08x\n",address);
 	return 0;
 }
 static void 
 wcsr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
 	return;
 }
 static uint32_t 
 wpdr_read(void *clientData,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled read of addr %08x\n",address);
+	dbgprintf("ste10/100 unhandled read of addr %08x\n",address);
 	return 0;
 }
 static void 
 wpdr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
 	return;
 }
 static uint32_t 
 wtmr_read(void *clientData,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled read of addr %08x\n",address);
+	dbgprintf("ste10/100 unhandled read of addr %08x\n",address);
 	return 0;
 }
 static void 
 wtmr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
 	return;
 }
 static uint32_t 
 acsr5_read(void *clientData,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled read of addr %08x\n",address);
+	dbgprintf("ste10/100 unhandled read of addr %08x\n",address);
 	return 0;
 }
 static void 
 acsr5_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
 	return;
 }
 static uint32_t 
 acsr7_read(void *clientData,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled read of addr %08x\n",address);
+	dbgprintf("ste10/100 unhandled read of addr %08x\n",address);
 	return 0;
 }
 static void 
 acsr7_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
 	return;
 }
 /*
@@ -1187,56 +1188,56 @@ acsr7_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 static uint32_t 
 cr_read(void *clientData,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
-	dprintf("ste10/100 CR read %08x, value %08x\n",address,CSR18(ste));
+	dbgprintf("ste10/100 CR read %08x, value %08x\n",address,CSR18(ste));
 	return CSR18(ste);
 }
 
 static void 
 cr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
-	dprintf("ste10/100 CR write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 CR write of addr %08x, value %08x\n",address,value);
 	CSR18(ste)=value;
 	return;
 }
 
 static uint32_t 
 pcic_read(void *clientData,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled read of addr %08x\n",address);
+	dbgprintf("ste10/100 unhandled read of addr %08x\n",address);
 	return 0;
 }
 static void 
 pcic_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
 	return;
 }
 static uint32_t 
 pmcsr_read(void *clientData,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled read of addr %08x\n",address);
+	dbgprintf("ste10/100 unhandled read of addr %08x\n",address);
 	return 0;
 }
 static void 
 pmcsr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
 	return;
 }
 static uint32_t 
 txbr_read(void *clientData,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled read of addr %08x\n",address);
+	dbgprintf("ste10/100 unhandled read of addr %08x\n",address);
 	return 0;
 }
 static void 
 txbr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
 	return;
 }
 static uint32_t 
 from_read(void *clientData,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled read of addr %08x\n",address);
+	dbgprintf("ste10/100 unhandled read of addr %08x\n",address);
 	return 0;
 }
 static void 
 from_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
 	return;
 }
 
@@ -1268,7 +1269,7 @@ pid1_read(void *clientData,uint32_t address,int rqlen) {
 }
 static void 
 pid1_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-	dprintf("ste10/100 PID1 not writable\n");
+	dbgprintf("ste10/100 PID1 not writable\n");
 	return;
 }
 static uint32_t 
@@ -1278,20 +1279,20 @@ pid2_read(void *clientData,uint32_t address,int rqlen) {
 }
 static void 
 pid2_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-	dprintf("ste10/100 PID2 not writable\n");
+	dbgprintf("ste10/100 PID2 not writable\n");
 	return;
 }
 static uint32_t 
 ana_read(void *clientData,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
-	dprintf("ste10/100 unhandled read of addr %08x\n",address);
+	dbgprintf("ste10/100 unhandled read of addr %08x\n",address);
 	return ANA(ste);
 }
 static void 
 ana_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	STE10_100 *ste=clientData;
 	ANA(ste)=(value & 0x25e1) | 1;
-	dprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
 	return;
 }
 
@@ -1303,7 +1304,7 @@ anlpa_read(void *clientData,uint32_t address,int rqlen) {
 
 static void 
 anlpa_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
 	return;
 }
 
@@ -1326,7 +1327,7 @@ xmc_read(void *clientData,uint32_t address,int rqlen) {
 }
 static void 
 xmc_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
 	return;
 }
 static uint32_t 
@@ -1336,17 +1337,17 @@ xciis_read(void *clientData,uint32_t address,int rqlen) {
 }
 static void 
 xciis_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-	dprintf("ste10/100: Register not writable: addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100: Register not writable: addr %08x, value %08x\n",address,value);
 	return;
 }
 static uint32_t 
 xie_read(void *clientData,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled read of addr %08x\n",address);
+	dbgprintf("ste10/100 unhandled read of addr %08x\n",address);
 	return 0;
 }
 static void 
 xie_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-	dprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 unhandled write of addr %08x, value %08x\n",address,value);
 	return;
 }
 static uint32_t 
@@ -1366,7 +1367,7 @@ ctr100_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	if(diff & value & CTR100_DISMLT) {
 		fprintf(stderr,"*** STE10/100: Possible driver bug ! MLT3 is disabled\n");
 	}
-	dprintf("ste10/100 missing functionality: write of addr %08x, value %08x\n",address,value);
+	dbgprintf("ste10/100 missing functionality: write of addr %08x, value %08x\n",address,value);
 	return;
 }
 
@@ -1494,7 +1495,7 @@ STE_UnMap(void *owner)
  */ 
 static void
 change_pci_command(STE10_100 *ste,uint16_t value,uint16_t mask) {
-	dprintf("Change PCI command from %04x to %04x, mask %04x\n",ste->command,value,mask);
+	dbgprintf("Change PCI command from %04x to %04x, mask %04x\n",ste->command,value,mask);
 	MODIFY_MASKED(ste->command,value,mask);
 	// register/register iospace and mmio
 	ste_unmap_io(ste);
@@ -1521,7 +1522,7 @@ STE_ConfigWrite(uint32_t value,uint32_t mask,uint8_t reg,PCI_Function *pcifunc)
         switch(reg) {
                 case 1:
                         // PCI Command, PCI Status
-			//dprintf("Got write %08x to %08x, mask %08x\n",value,reg,mask);
+			//dbgprintf("Got write %08x to %08x, mask %08x\n",value,reg,mask);
 			change_pci_command(ste,value,mask);	
                         break;
 
@@ -1539,7 +1540,7 @@ STE_ConfigWrite(uint32_t value,uint32_t mask,uint8_t reg,PCI_Function *pcifunc)
                         }
                         break;
                 case 0x4:
-	//		dprintf("Write bar 0 to %08x\n",value);
+	//		dbgprintf("Write bar 0 to %08x\n",value);
 			ste_unmap_io(ste);
                         MODIFY_MASKED(ste->bar0,value,mask);
 			ste->bar0 = ste->bar0 & ~0xff;
@@ -1548,7 +1549,7 @@ STE_ConfigWrite(uint32_t value,uint32_t mask,uint8_t reg,PCI_Function *pcifunc)
 			}
                         break;
                 case 0x5:
-	//		dprintf("Write bar 1 to %08x\n",value);
+	//		dbgprintf("Write bar 1 to %08x\n",value);
 			ste_unmap_mmio(ste);
                         MODIFY_MASKED(ste->bar1,value,mask);
 			ste->bar1 = ste->bar1 & ~0xff;
@@ -1625,13 +1626,13 @@ STE_ConfigRead(uint32_t *value,uint8_t reg,PCI_Function *pcifunc)
                 case 4:
 			// IO-Type Base Address with Window of 256 bytes
                         *value = ste->bar0 | 1; 
-//			dprintf("read bar 0 %08x\n",*value);
+//			dbgprintf("read bar 0 %08x\n",*value);
                         break;
 
                 case 5:
 			// MMIO Type
                         *value = ste->bar1;
-//			dprintf("read bar 1 %08x\n",*value);
+//			dbgprintf("read bar 1 %08x\n",*value);
                         break;
 
                 case 6:
@@ -1675,7 +1676,7 @@ STE_ConfigRead(uint32_t *value,uint8_t reg,PCI_Function *pcifunc)
                         break;
 
         }
-//      dprintf("STE-Config Read reg %d val %08x\n",reg*4, *value);
+//      dbgprintf("STE-Config Read reg %d val %08x\n",reg*4, *value);
         return 0;
 }
 
@@ -1801,12 +1802,7 @@ STE_New(const char *devname,PCI_Function *bridge,int dev_nr,int bus_irq) {
 	SigNode *mw_sdi;
 	SigNode *mw_sdo;
 	SigNode *mw_csel;
-        ste=malloc(sizeof(*ste));
-        if(!ste) {
-                dprintf("Out of memory\n");
-                exit(2434);
-        }
-        memset(ste,0,sizeof(*ste));
+        ste = sg_new(STE10_100);
 	ste->xr=&ste->csr[0xb4>>2];
 	ste->dev_nr=dev_nr;
 
