@@ -34,11 +34,12 @@
 #include "usb_ohci.h"
 #include "cycletimer.h"
 #include "signode.h"
+#include "sgstring.h"
 
 #if 0
-#define dprintf(x...) { fprintf(stderr,x); }
+#define dbgprintf(x...) { fprintf(stderr,x); }
 #else
-#define dprintf(x...)
+#define dbgprintf(x...)
 #endif
 
 #define MAX_NDP 15
@@ -602,12 +603,14 @@ bug:
 	fprintf(stderr,"BUG To many transferdescriptors \n");
 }
 
+#if 0
 static void 
 HcFrameTimerProc(void *cd) {
 	OhciHC *hc=cd;
-	CycleTimer_Add(&hc->timer,CycleTimerRate/1000,HcFrameTimerProc,hc);
+	CycleTimer_Mod(&hc->timer,CycleTimerRate/1000,HcFrameTimerProc,hc);
 	return HcOneFrame(hc);
 }
+#endif
 
 static void
 HcSoftReset(OhciHC *hc) {
@@ -677,18 +680,18 @@ HcHardReset(OhciHC *hc) {
  */
 static uint32_t 
 HcRevision_read(void *clientData,uint32_t address,int rqlen) {
-        dprintf("HcRevision read\n");
+        dbgprintf("HcRevision read\n");
         return 0x10;
 }
 static void 
 HcRevision_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-        dprintf("HcRevision not writable\n");
+        dbgprintf("HcRevision not writable\n");
         return;
 }
 static uint32_t
 HcControl_read(void *clientData,uint32_t address,int rqlen) {
 	OhciHC *ohci=clientData;
-        dprintf("HcControl: unhandled read of addr %08x\n",address);
+        dbgprintf("HcControl: unhandled read of addr %08x\n",address);
         return ohci->HcControl;
 }
 
@@ -704,7 +707,7 @@ HcControl_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	OhciHC *hc=clientData;
 	uint32_t fState,oldFState;
 	//uint32_t diff=hc->HcControl^value;;
-        dprintf("HcControl unhandled write of addr %08x, value %08x\n",address,value);
+        dbgprintf("HcControl unhandled write of addr %08x, value %08x\n",address,value);
 	fState=(value & HcControl_HCFS_MASK)>>HcControl_HCFS_SHIFT;
 	oldFState=(hc->HcControl & HcControl_HCFS_MASK)>>HcControl_HCFS_SHIFT;
 
@@ -731,7 +734,7 @@ HcControl_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 		if(!hc->timer_is_active) {
 #if 0
 			hc->timer_is_active=1;
-			CycleTimer_Add(&hc->timer,CycleTimerRate/1000,HcFrameTimerProc,hc);	
+			CycleTimer_Mod(&hc->timer,CycleTimerRate/1000,HcFrameTimerProc,hc);	
 #endif
 		}
 	} else {
@@ -752,14 +755,14 @@ HcControl_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 static uint32_t 
 HcCommandStatus_read(void *clientData,uint32_t address,int rqlen) {
 	OhciHC *hc=clientData;
-        dprintf("HcCommandStatus: unhandled read of addr %08x\n",address);
+        dbgprintf("HcCommandStatus: unhandled read of addr %08x\n",address);
         return hc->HcCommandStatus;
 }
 
 static void 
 HcCommandStatus_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	OhciHC *hc=clientData;
-        dprintf("HcCommandStatus unhandled write of addr %08x, value %08x\n",address,value);
+        dbgprintf("HcCommandStatus unhandled write of addr %08x, value %08x\n",address,value);
 	hc->HcCommandStatus=value;
 	if(value & HcCommandStatus_HCR) {
 		HcSoftReset(hc); 
@@ -770,14 +773,14 @@ HcCommandStatus_write(void *clientData,uint32_t value,uint32_t address,int rqlen
 static uint32_t 
 HcInterruptStatus_read(void *clientData,uint32_t address,int rqlen) {
 	OhciHC *hc=clientData;
-        dprintf("HcInterruptStatus: read of value %08x\n",hc->HcInterruptStatus);
+        dbgprintf("HcInterruptStatus: read of value %08x\n",hc->HcInterruptStatus);
         return hc->HcInterruptStatus;
 }
 
 static void 
 HcInterruptStatus_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	OhciHC *hc=clientData;
-        dprintf("HcInterruptStatus write value %08x\n",value);
+        dbgprintf("HcInterruptStatus write value %08x\n",value);
 	hc->HcInterruptStatus &=  ~value;
 	update_interrupts(hc);
         return;
@@ -798,7 +801,7 @@ HcInterruptEnable_read(void *clientData,uint32_t address,int rqlen) {
 static void 
 HcInterruptEnable_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	OhciHC *hc=clientData;
-        dprintf("HcInterruptEnable unhandled write of addr %08x, value %08x\n",address,value);
+        dbgprintf("HcInterruptEnable unhandled write of addr %08x, value %08x\n",address,value);
 	hc->HcInterruptEnable |= value;
 	update_interrupts(hc);
         return;
@@ -811,7 +814,7 @@ HcInterruptDisable_read(void *clientData,uint32_t address,int rqlen) {
 static void 
 HcInterruptDisable_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	OhciHC *hc=clientData;
-        dprintf("HcInterruptEnable unhandled write of addr %08x, value %08x\n",address,value);
+        dbgprintf("HcInterruptEnable unhandled write of addr %08x, value %08x\n",address,value);
 	hc->HcInterruptEnable &= ~value;
 	update_interrupts(hc);
         return;
@@ -825,7 +828,7 @@ HcInterruptDisable_write(void *clientData,uint32_t value,uint32_t address,int rq
 static uint32_t 
 HcHCCA_read(void *clientData,uint32_t address,int rqlen) {
 	OhciHC *hc=clientData;
-        dprintf("HcHCCA: read value %08x\n",*value);
+        dbgprintf("HcHCCA: read value %08x\n",*value);
         return hc->HcHCCA;
 }
 
@@ -921,14 +924,14 @@ static void
 HcFmInterval_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	OhciHC *hc=clientData;
 	hc->HcFmInterval=value;
-	dprintf("FSLargestDataPacket: %d\n",HcFmInterval_FSMPS(value));
+	dbgprintf("FSLargestDataPacket: %d\n",HcFmInterval_FSMPS(value));
         return;
 }
 
 static uint32_t 
 HcFmRemaining_read(void *clientData,uint32_t address,int rqlen) {
 	// calculate remaining from CPU-cycles
-        dprintf("HcFmRemaining: unhandled read of addr %08x\n",address);
+        dbgprintf("HcFmRemaining: unhandled read of addr %08x\n",address);
         return 0;
 }
 
@@ -941,20 +944,20 @@ HcFmRemaining_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 static uint32_t 
 HcFmNumber_read(void *clientData,uint32_t address,int rqlen) {
 	OhciHC *hc=clientData;
-        dprintf("HcFmNumber: read value %08x\n",*value);
+        dbgprintf("HcFmNumber: read value %08x\n",*value);
         return hc->HcFmNumber;
 }
 
 static void
 HcFmNumber_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
-        dprintf("HcFmNumber: Not writable by HCD\n");
+        dbgprintf("HcFmNumber: Not writable by HCD\n");
         return;
 }
 
 static uint32_t 
 HcPeriodicStart_read(void *clientData,uint32_t address,int rqlen) {
 	OhciHC *hc=clientData;
-        dprintf("HcPeriodicStart: read value %08x\n",*value);
+        dbgprintf("HcPeriodicStart: read value %08x\n",*value);
         return hc->HcPeriodicStart;
 }
 
@@ -962,21 +965,21 @@ static void
 HcPeriodicStart_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	OhciHC *hc=clientData;
 	hc->HcPeriodicStart=value;
-        dprintf("HcPeriodicStart: unhandled write of addr %08x, value %08x\n",address,value);
+        dbgprintf("HcPeriodicStart: unhandled write of addr %08x, value %08x\n",address,value);
         //printf("HcPeriodicStart:  write of addr %08x, value %08x\n",address,value);
         return;
 }
 static uint32_t 
 HcLSThreshold_read(void *clientData,uint32_t address,int rqlen) {
 	OhciHC *hc=clientData;
-        dprintf("HcLSThreshold: unhandled read of addr %08x\n",address);
+        dbgprintf("HcLSThreshold: unhandled read of addr %08x\n",address);
         return hc->HcLSThreshold;
 }
 static void 
 HcLSThreshold_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	OhciHC *hc=clientData;
 	hc->HcLSThreshold=value;
-        dprintf("HcLSThreshold:  write of value %08x\n",value);
+        dbgprintf("HcLSThreshold:  write of value %08x\n",value);
         return;
 }
 
@@ -988,7 +991,7 @@ HcLSThreshold_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 static uint32_t 
 HcRhDescriptorA_read(void *clientData,uint32_t address,int rqlen) {
 	OhciHC *hc=clientData;
-    //    dprintf("HcRhDescriptorA: read of value %08x\n",*value);
+    //    dbgprintf("HcRhDescriptorA: read of value %08x\n",*value);
         return hc->HcRhDescriptorA;
 }
 static void 
@@ -997,32 +1000,32 @@ HcRhDescriptorA_write(void *clientData,uint32_t value,uint32_t address,int rqlen
 	uint32_t writemask = HcRhDescriptorA_NPS | HcRhDescriptorA_PSM | 
 		HcRhDescriptorA_OCPM | HcRhDescriptorA_POTPGT_MASK;
 	hc->HcRhDescriptorA=(value&writemask) |(hc->HcRhDescriptorA & ~writemask);
-        dprintf("HcRhDescriptorA: write of value %08x\n",value);
+        dbgprintf("HcRhDescriptorA: write of value %08x\n",value);
         return;
 }
 static uint32_t 
 HcRhDescriptorB_read(void *clientData,uint32_t address,int rqlen) {
 	OhciHC *hc=clientData;
-        dprintf("HcRhDescriptorB: raad value %08x\n",*value);
+        dbgprintf("HcRhDescriptorB: raad value %08x\n",*value);
         return hc->HcRhDescriptorB;
 }
 static void 
 HcRhDescriptorB_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	OhciHC *hc=clientData;
-        dprintf("HcRhDescriptorB: write value %08x\n",value);
+        dbgprintf("HcRhDescriptorB: write value %08x\n",value);
 	hc->HcRhDescriptorB=value;	
         return;
 }
 static uint32_t 
 HcRhStatus_read(void *clientData,uint32_t address,int rqlen) {
 	OhciHC *hc=clientData;
-     //   dprintf("HcRhStatus: read value %08x\n",*value);
+     //   dbgprintf("HcRhStatus: read value %08x\n",*value);
         return hc->HcRhStatus;
 }
 static void 
 HcRhStatus_write(void *clientData,uint32_t value,uint32_t address,int rqlen) {
 	OhciHC *hc=clientData;
-        dprintf("HcRhDescriptorB: write value %08x\n",value);
+        dbgprintf("HcRhDescriptorB: write value %08x\n",value);
 	hc->HcRhStatus=value;	
         return;
 }
@@ -1034,7 +1037,7 @@ HcRhPortStatus_read(void *clientData,uint32_t address,int rqlen) {
 		fprintf(stderr,"Error Trying to read Port Status of port %d\n",port); 
 		return -1;
 	}
-        dprintf("HcRhPortStatus:  read port %d value %08x\n",port,*value);
+        dbgprintf("HcRhPortStatus:  read port %d value %08x\n",port,*value);
         return hc->HcRhPortStatus[port];
 }
 /*
@@ -1092,7 +1095,7 @@ HcRhPortStatus_write(void *clientData,uint32_t value,uint32_t address,int rqlen)
 	if(value & HcRhPortStatus_LSDA) {
 		tc|= HcRhPortStatus_PPS;
 	}
-        dprintf("HcRhPortStatus: write of value %08x to port %d\n",value,port);
+        dbgprintf("HcRhPortStatus: write of value %08x to port %d\n",value,port);
 	hc->HcRhPortStatus[port] = (hc->HcRhPortStatus[port] & ~tc) |ts;
 //	fprintf(stderr,"new RhPortStatus %08x\n",hc->HcRhPortStatus[port]); 
         return;
@@ -1189,7 +1192,7 @@ OhciHC_Enable(BusDevice *dev) {
  *	No clue what the real device does here
  * -------------------------------------------------------
  */
-static int
+static void 
 change_endian(SigNode *node,int value,void *clientData)
 {
         OhciHC *hc = clientData;
@@ -1202,7 +1205,6 @@ change_endian(SigNode *node,int value,void *clientData)
                 fprintf(stderr,"OHCI: Endian is neither Little nor Big\n");
                 exit(3424);
         }
-        return 0;
 }
 
 /*
@@ -1216,12 +1218,7 @@ change_endian(SigNode *node,int value,void *clientData)
  */
 BusDevice *
 OhciHC_New(char *name,Bus *bus) {
-	OhciHC *hc=malloc(sizeof(OhciHC));	
-	if(!hc) {
-		fprintf(stderr,"Out of memory\n");
-		exit(3456);
-	}
-	memset(hc,0,sizeof(OhciHC));
+	OhciHC *hc = sg_new(OhciHC);	
 	hc->irqNode = SigNode_New("%s.irq",name);
 	if(!hc->irqNode) {
 		fprintf(stderr,"Can not create irq signal\n");
