@@ -71,7 +71,7 @@ UsbDev_RegisterEndpoint(UsbDevice *udev,int epaddr,int eptype,int maxpacket,UsbT
 	UsbEndpoint *ep;
 	int epnum = epaddr & 0xf; 
 	int in =  !!(epaddr & 0x80);
-	ep = sg_calloc(UsbEndpoint);
+	ep = sg_new(UsbEndpoint);
 	memset(ep,0,sizeof(UsbEndpoint));
 	ep->status = 0;
 	ep->doTransaction = proc;
@@ -82,7 +82,7 @@ UsbDev_RegisterEndpoint(UsbDevice *udev,int epaddr,int eptype,int maxpacket,UsbT
 	if((eptype == EPNT_TYPE_CONTROL)) {
 		udev->in_endpnt[epnum] = ep;	
 		udev->out_endpnt[epnum] = ep;	
-		ep->setup_buf = sg_calloc(*ep->setup_buf);
+		ep->setup_buf = sg_new(*ep->setup_buf);
 		XY_InitHashTable(&ep->requestHash,XY_ONE_WORD_KEYS,32);
         	XY_InitHashTable(&ep->descriptorHash,XY_ONE_WORD_KEYS,32);
 	} else if(in) {
@@ -298,7 +298,7 @@ pid_sof(UsbDevice *udev,const UsbPacket *packet)
  * To lazy in first version, return always true
  */
 static int 
-check_toggle() {
+check_toggle(UsbEndpoint *epnt,int toggle) {
 	return 1;
 }
 
@@ -533,7 +533,7 @@ UsbDev_Feed(void *dev,const UsbPacket *packet)
 UsbDevice *
 UsbDev_New(void *owner,usb_device_speed speed) 
 {
-	UsbDevice *udev = sg_calloc(UsbDevice);
+	UsbDevice *udev = sg_new(UsbDevice);
 	if(!udev) {
 		fprintf(stderr,"Out of memory allocating usb device\n");
 		exit(1);
@@ -683,7 +683,7 @@ UsbDev_RegisterDescriptorHandler(UsbEndpoint *ep,int dt,int index,UsbDescriptorP
 {
 	XY_HashEntry *entry;
 	unsigned long key = (dt << 8) | index;
-	UsbDescriptorHandler *dh = (UsbDescriptorHandler *)sg_calloc(*dh);
+	UsbDescriptorHandler *dh = (UsbDescriptorHandler *)sg_new(*dh);
 	int isnew;
 	dh->getDescriptor = get; dh->setDescriptor = set;
 	if(!dh) {
@@ -709,7 +709,7 @@ find_request_handler(UsbEndpoint *ep,UsbSetupBuffer *sb)
 	return NULL;
 }
 
-UNUSED void
+__UNUSED__ void
 dump_setup_buffer(UsbSetupBuffer *sb) 
 {
 	int i;
