@@ -26,6 +26,7 @@
 #include <string.h>
 #include <stdint.h>
 #include <xy_hash.h>
+#include "sgstring.h"
 
 static unsigned int
 hashkey_one_word32(const void *data) 
@@ -66,7 +67,8 @@ string_isequal(const void *d1,const void *d2) {
 	return (strcmp(str1,str2)==0);
 }
 
-int hash_string2(const char *s) {
+static int 
+hash_string2(const char *s) {
 	unsigned int hash=0;
 	while(*s) {
 		hash = *s + (hash << 6) + (hash << 16) - hash;
@@ -200,6 +202,45 @@ XY_ClearHashTable(XY_HashTable *hash) {
 	free(hash->table);
 }
 
+XY_HashEntry *
+XY_AddHashEntry(XY_HashTable *hash,XY_HashEntry *newentry,const void *key)
+{
+        int h = hash->hashfunc(key);
+        XY_HashEntry **first=&hash->table[h];
+        XY_HashEntry *cursor;
+
+        for(cursor=*first;cursor;cursor=cursor->next) {
+                if(hash->isequal(cursor->key,key)) {
+                        break;
+                }
+        }
+        if(cursor) {
+                return NULL;
+        }
+        newentry->next = *first;
+        newentry->prev = NULL;
+        if(*first)
+                (*first)->prev = newentry;
+        *first = newentry;
+        newentry->key = key;
+        return newentry;
+}
+
+void
+XY_RemoveHashEntry(XY_HashTable *hash,XY_HashEntry *entry) {
+        XY_HashEntry *prev = entry->prev;
+        XY_HashEntry *next = entry->next;
+        if(prev) {
+                prev->next = next;
+        } else {
+                int h = hash->hashfunc(entry->key);
+                XY_HashEntry **first=&hash->table[h];
+                *first = next;
+        }
+        if(next)
+                next->prev = prev;
+}
+
 XY_HashEntry *
 XY_CreateHashEntry(XY_HashTable *hash,void *key,int *newptr) 
 {
@@ -240,14 +281,10 @@ XY_CreateHashEntry(XY_HashTable *hash,void *key,int *newptr)
 int
 XY_InitHashTable(XY_HashTable *hash,int keytype,int size) {
 	int n_bytes = size * sizeof(void*);
-	hash->table = (XY_HashEntry **)malloc(n_bytes);
-	if(hash->table == NULL) {
-		exit(5243);
-	}
+	hash->table = (XY_HashEntry **)sg_calloc(n_bytes);
 #if 0
 	printf("Table at %p, len %d\n",hash->table,n_bytes);
 #endif
-	memset(hash->table,0,n_bytes);
 	switch(keytype) {
 		case XY_USER_HASHFUNC:
 			break;
@@ -267,7 +304,7 @@ XY_InitHashTable(XY_HashTable *hash,int keytype,int size) {
 					hash->hashfunc = hashkey_one_word8192;
 					break;
 				default:
-					free(hash->table);
+					sg_free(hash->table);
 					return -1;
 			}
 			break;
@@ -290,13 +327,13 @@ XY_InitHashTable(XY_HashTable *hash,int keytype,int size) {
 					hash->hashfunc = hashkey_string65536;
 					break;
 				default:
-					free(hash->table);
+					sg_free(hash->table);
 					return -1;
 
 			}		
 			break;
 		default:
-			free(hash->table);
+			sg_free(hash->table);
 			return -1;
 			break;
 	}
