@@ -6,6 +6,7 @@
  * modified for use with xy_lib by Jochen Karrer 02/05/2002 
  * -----------------------------------------------------------------
  */
+#include "sgstring.h"
 #include <xy_tree.h> 
 
 #ifdef TREE_DEBUG
@@ -309,7 +310,7 @@ XY_AddTreeNode(XY_Tree* tree, xy_node *x,void* key, void* value) {
 }
 xy_node * 
 XY_CreateTreeNode(XY_Tree* tree, void* key, void* value) {
-	xy_node *node =(xy_node*) malloc(sizeof(xy_node));
+	xy_node *node =(xy_node*) sg_new(xy_node);
 	if(!node)
 		return NULL;
 	XY_AddTreeNode(tree,node,key,value);
